@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see http://gnu.org/licenses
  */
+
 ////////////////////////////////////////////////////////////////////
 /**
  * \file CBot.h
@@ -164,7 +165,7 @@ public:
      * \param [in] bModif Not used. Probably need to be removed
      * \return Found variable
      */
-    CBotVar* FindVar(CBotToken& Token, bool bUpdate = false,
+    CBotVar* FindVar(CBotToken& pToken, bool bUpdate = false,
                                               bool bModif  = false);
 
     /**
@@ -189,20 +190,20 @@ public:
      * \brief Find variable by its token and returns a copy of it.
      * \param Token Token upon which search is performed
      * \param bUpdate Not used.
-     * \return Found variable, NULL if not found
+     * \return Found variable, nullptr if not found
      */
     CBotVar*        CopyVar(CBotToken& Token, bool bUpdate = false);
 
 
-    CBotStack*        AddStack(CBotInstr* instr = NULL, bool bBlock = false);    // extends the stack
-    CBotStack*        AddStackEOX(CBotCall* instr = NULL, bool bBlock = false);    // extends the stack
-    CBotStack*        RestoreStack(CBotInstr* instr = NULL);
-    CBotStack*        RestoreStackEOX(CBotCall* instr = NULL);
+    CBotStack*        AddStack(CBotInstr* instr = nullptr, bool bBlock = false);    // extends the stack
+    CBotStack*        AddStackEOX(CBotCall* instr = nullptr, bool bBlock = false);    // extends the stack
+    CBotStack*        RestoreStack(CBotInstr* instr = nullptr);
+    CBotStack*        RestoreStackEOX(CBotCall* instr = nullptr);
 
     CBotStack*        AddStack2(bool bBlock = false);                        // extends the stack
     bool            Return(CBotStack* pFils);                            // transmits the result over
     bool            ReturnKeep(CBotStack* pFils);                        // transmits the result without reducing the stack
-    bool            BreakReturn(CBotStack* pfils, const char* name = NULL);
+    bool            BreakReturn(CBotStack* pfils, const char* name = nullptr);
                                                                         // in case of eventual break
     bool            IfContinue(int state, const char* name);
                                                                         // or "continue"
@@ -224,7 +225,7 @@ public:
     long            GetVal();
 
     void            SetStartError(int pos);
-    void            SetError(int n, CBotToken* token = NULL);
+    void            SetError(int n, CBotToken* token = nullptr);
     void            SetPosError(CBotToken* token);
     void            ResetError(int n, int start, int end);
     void            SetBreak(int val, const char* name);
@@ -353,7 +354,7 @@ public:
     bool            CheckVarLocal(CBotToken* &pToken);
     CBotVar*        CopyVar(CBotToken& Token);    // finds and makes a copy
 
-    CBotCStack*        TokenStack(CBotToken* pToken = NULL, bool bBlock = false);
+    CBotCStack*        TokenStack(CBotToken* pToken = nullptr, bool bBlock = false);
     CBotInstr*        Return(CBotInstr* p, CBotCStack* pParent);    // transmits the result upper
     CBotFunction*    ReturnFunc(CBotFunction* p, CBotCStack* pParent);    // transmits the result upper
 
@@ -483,8 +484,8 @@ public:
                 ~CBotWhile();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotDo : public CBotInstr
@@ -499,8 +500,8 @@ public:
                 ~CBotDo();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotFor : public CBotInstr
@@ -517,8 +518,8 @@ public:
                 ~CBotFor();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotBreak : public CBotInstr
@@ -531,8 +532,8 @@ public:
                 ~CBotBreak();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotReturn : public CBotInstr
@@ -545,8 +546,8 @@ public:
                 ~CBotReturn();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -561,8 +562,8 @@ public:
                 ~CBotSwitch();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -576,9 +577,9 @@ public:
                 ~CBotCase();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
-    bool        CompCase(CBotStack* &pj, int val);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
+    bool        CompCase(CBotStack* &pj, int val) override;
 };
 
 class CBotCatch : public CBotInstr
@@ -595,8 +596,8 @@ public:
     static
     CBotCatch*    Compile(CBotToken* &p, CBotCStack* pStack);
     bool        TestCatch(CBotStack* &pj, int val);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
     void        RestoreCondState(CBotStack* &pj, bool bMain);
 };
 
@@ -612,8 +613,8 @@ public:
                 ~CBotTry();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotThrow : public CBotInstr
@@ -626,8 +627,8 @@ public:
                 ~CBotThrow();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -640,7 +641,7 @@ public:
                 ~CBotStartDebugDD();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
+    bool        Execute(CBotStack* &pj) override;
 };
 
 
@@ -656,8 +657,8 @@ public:
                 ~CBotIf();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -675,8 +676,8 @@ public:
                 ~CBotInt();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, bool cont = false, bool noskip = false);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 // definition of an array
@@ -695,8 +696,8 @@ public:
                 ~CBotInstArray();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, CBotTypResult type);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -713,15 +714,15 @@ public:
                 ~CBotListArray();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, CBotTypResult type);
-    bool        Execute(CBotStack* &pj, CBotVar* pVar);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj, CBotVar* pVar) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
 class CBotEmpty : public CBotInstr
 {
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 // defininition of a boolean
@@ -737,8 +738,8 @@ public:
                 ~CBotBoolean();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, bool cont = false, bool noskip=false);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -755,8 +756,8 @@ public:
                 ~CBotFloat();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, bool cont = false, bool noskip=false);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 // definition of an element string
@@ -772,8 +773,8 @@ public:
                 ~CBotIString();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, bool cont = false, bool noskip=false);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 // definition of an element of any class
@@ -792,9 +793,9 @@ public:
                 CBotClassInst();
                 ~CBotClassInst();
     static
-    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, CBotClass* pClass = NULL);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, CBotClass* pClass = nullptr);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotCondition : public CBotInstr
@@ -823,9 +824,9 @@ public:
     CBotLeftExpr*    Compile(CBotToken* &p, CBotCStack* pStack);
     bool        Execute(CBotStack* &pStack, CBotStack* array);
 
-    bool        ExecuteVar(CBotVar* &pVar, CBotCStack* &pile);
+    bool        ExecuteVar(CBotVar* &pVar, CBotCStack* &pile) override;
     bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pile, CBotToken* prevToken, bool bStep);
-    void        RestoreStateVar(CBotStack* &pile, bool bMain);
+    void        RestoreStateVar(CBotStack* &pile, bool bMain) override;
 };
 
 
@@ -843,9 +844,9 @@ public:
     void        SetUniqNum(int num);
 //    static
 //    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        ExecuteVar(CBotVar* &pVar, CBotCStack* &pile);
-    bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pile, CBotToken* prevToken, bool bStep, bool bExtend);
-    void        RestoreStateVar(CBotStack* &pj, bool bMain);
+    bool        ExecuteVar(CBotVar* &pVar, CBotCStack* &pile) override;
+    bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pile, CBotToken* prevToken, bool bStep, bool bExtend) override;
+    void        RestoreStateVar(CBotStack* &pj, bool bMain) override;
 };
 
 // management of indices of the tables
@@ -862,9 +863,9 @@ public:
                 ~CBotIndexExpr();
 //    static
 //    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        ExecuteVar(CBotVar* &pVar, CBotCStack* &pile);
-    bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pile, CBotToken* prevToken, bool bStep, bool bExtend);
-    void        RestoreStateVar(CBotStack* &pj, bool bMain);
+    bool        ExecuteVar(CBotVar* &pVar, CBotCStack* &pile) override;
+    bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pile, CBotToken* prevToken, bool bStep, bool bExtend) override;
+    void        RestoreStateVar(CBotStack* &pj, bool bMain) override;
 };
 
 // expressions like
@@ -882,8 +883,8 @@ public:
                 ~CBotExpression();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pStack);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pStack) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotListExpression : public CBotInstr
@@ -896,8 +897,8 @@ public:
                 ~CBotListExpression();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pStack);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pStack) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotLogicExpr : public CBotInstr
@@ -913,8 +914,8 @@ public:
                 ~CBotLogicExpr();
 //    static
 //    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pStack);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pStack) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -952,8 +953,8 @@ public:
                 ~CBotExprUnaire();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pStack);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pStack) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 // all operations with two operands
@@ -967,9 +968,9 @@ public:
                 CBotTwoOpExpr();
                 ~CBotTwoOpExpr();
     static
-    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, int* pOperations = NULL);
-    bool        Execute(CBotStack* &pStack);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, int* pOperations = nullptr);
+    bool        Execute(CBotStack* &pStack) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -1000,8 +1001,8 @@ public:
                 ~CBotListInstr();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, bool bLocal = true);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -1020,8 +1021,8 @@ public:
                 ~CBotInstrCall();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 // a call of method
@@ -1045,9 +1046,9 @@ public:
                 ~CBotInstrMethode();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack, CBotVar* pVar);
-    bool        Execute(CBotStack* &pj);
-    bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pj, CBotToken* prevToken, bool bStep, bool bExtend);
-    void        RestoreStateVar(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pj, CBotToken* prevToken, bool bStep, bool bExtend) override;
+    void        RestoreStateVar(CBotStack* &pj, bool bMain) override;
 };
 
 // expression for the variable name
@@ -1067,11 +1068,11 @@ public:
     static
     CBotInstr*    CompileMethode(CBotToken* &p, CBotCStack* pStack);
 
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
     bool        ExecuteVar(CBotVar* &pVar, CBotStack* &pile, CBotToken* prevToken, bool bStep);
     bool        Execute2Var(CBotVar* &pVar, CBotStack* &pj, CBotToken* prevToken, bool bStep);
-    void        RestoreStateVar(CBotStack* &pj, bool bMain);
+    void        RestoreStateVar(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotPostIncExpr : public CBotInstr
@@ -1085,8 +1086,8 @@ public:
                 ~CBotPostIncExpr();
 //    static
 //    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotPreIncExpr : public CBotInstr
@@ -1100,8 +1101,8 @@ public:
                 ~CBotPreIncExpr();
 //    static
 //    CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -1118,8 +1119,8 @@ public:
                 ~CBotLeftExprVar();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -1133,8 +1134,8 @@ public:
 
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -1146,8 +1147,8 @@ public:
                 CBotExprNull();
                 ~CBotExprNull();
 
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotExprNan : public CBotInstr
@@ -1158,8 +1159,8 @@ public:
                 CBotExprNan();
                 ~CBotExprNan();
 
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 class CBotNew : public CBotInstr
@@ -1176,8 +1177,8 @@ public:
 
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 // expression representing a number
@@ -1194,8 +1195,8 @@ public:
                 ~CBotExprNum();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -1211,8 +1212,8 @@ public:
                 ~CBotExprAlpha();
     static
     CBotInstr*    Compile(CBotToken* &p, CBotCStack* pStack);
-    bool        Execute(CBotStack* &pj);
-    void        RestoreState(CBotStack* &pj, bool bMain);
+    bool        Execute(CBotStack* &pj) override;
+    void        RestoreState(CBotStack* &pj, bool bMain) override;
 };
 
 
@@ -1231,44 +1232,44 @@ public:
                 CBotVarInt( const CBotToken* name );
 //                ~CBotVarInt();
 
-    void        SetValInt(int val, const char* s = NULL);
-    void        SetValFloat(float val);
-    int            GetValInt();
-    float        GetValFloat();
-    CBotString    GetValString();
+    void        SetValInt(int val, const char* s = nullptr) override;
+    void        SetValFloat(float val) override;
+    int            GetValInt() override;
+    float        GetValFloat() override;
+    CBotString    GetValString() override;
 
-    void        Copy(CBotVar* pSrc, bool bName=true);
+    void        Copy(CBotVar* pSrc, bool bName=true) override;
 
 
-    void        Add(CBotVar* left, CBotVar* right);    // addition
-    void        Sub(CBotVar* left, CBotVar* right);    // substraction
-    void        Mul(CBotVar* left, CBotVar* right);    // multiplication
-    int            Div(CBotVar* left, CBotVar* right);    // division
-    int            Modulo(CBotVar* left, CBotVar* right);    // remainder of division
-    void        Power(CBotVar* left, CBotVar* right);    // power
+    void        Add(CBotVar* left, CBotVar* right) override;    // addition
+    void        Sub(CBotVar* left, CBotVar* right) override;    // substraction
+    void        Mul(CBotVar* left, CBotVar* right) override;    // multiplication
+    int            Div(CBotVar* left, CBotVar* right) override;    // division
+    int            Modulo(CBotVar* left, CBotVar* right) override;    // remainder of division
+    void        Power(CBotVar* left, CBotVar* right) override;    // power
 
-    bool        Lo(CBotVar* left, CBotVar* right);
-    bool        Hi(CBotVar* left, CBotVar* right);
-    bool        Ls(CBotVar* left, CBotVar* right);
-    bool        Hs(CBotVar* left, CBotVar* right);
-    bool        Eq(CBotVar* left, CBotVar* right);
-    bool        Ne(CBotVar* left, CBotVar* right);
+    bool        Lo(CBotVar* left, CBotVar* right) override;
+    bool        Hi(CBotVar* left, CBotVar* right) override;
+    bool        Ls(CBotVar* left, CBotVar* right) override;
+    bool        Hs(CBotVar* left, CBotVar* right) override;
+    bool        Eq(CBotVar* left, CBotVar* right) override;
+    bool        Ne(CBotVar* left, CBotVar* right) override;
 
-    void        XOr(CBotVar* left, CBotVar* right);
-    void        Or(CBotVar* left, CBotVar* right);
-    void        And(CBotVar* left, CBotVar* right);
+    void        XOr(CBotVar* left, CBotVar* right) override;
+    void        Or(CBotVar* left, CBotVar* right) override;
+    void        And(CBotVar* left, CBotVar* right) override;
 
-    void        SL(CBotVar* left, CBotVar* right);
-    void        SR(CBotVar* left, CBotVar* right);
-    void        ASR(CBotVar* left, CBotVar* right);
+    void        SL(CBotVar* left, CBotVar* right) override;
+    void        SR(CBotVar* left, CBotVar* right) override;
+    void        ASR(CBotVar* left, CBotVar* right) override;
 
-    void        Neg();
-    void        Not();
-    void        Inc();
-    void        Dec();
+    void        Neg() override;
+    void        Not() override;
+    void        Inc() override;
+    void        Dec() override;
 
-    bool        Save0State(FILE* pf);
-    bool        Save1State(FILE* pf);
+    bool        Save0State(FILE* pf) override;
+    bool        Save1State(FILE* pf) override;
 
 };
 
@@ -1282,34 +1283,34 @@ public:
                 CBotVarFloat( const CBotToken* name );
 //                ~CBotVarFloat();
 
-    void        SetValInt(int val, const char* s = NULL);
-    void        SetValFloat(float val);
-    int            GetValInt();
-    float        GetValFloat();
-    CBotString    GetValString();
+    void        SetValInt(int val, const char* s = nullptr) override;
+    void        SetValFloat(float val) override;
+    int            GetValInt() override;
+    float        GetValFloat() override;
+    CBotString    GetValString() override;
 
-    void        Copy(CBotVar* pSrc, bool bName=true);
+    void        Copy(CBotVar* pSrc, bool bName=true) override;
 
 
-    void        Add(CBotVar* left, CBotVar* right);    // addition
-    void        Sub(CBotVar* left, CBotVar* right);    // substraction
-    void        Mul(CBotVar* left, CBotVar* right);    // multiplication
-    int         Div(CBotVar* left, CBotVar* right);    // division
-    int            Modulo(CBotVar* left, CBotVar* right);    // remainder of division
-    void        Power(CBotVar* left, CBotVar* right);    // power
+    void        Add(CBotVar* left, CBotVar* right) override;    // addition
+    void        Sub(CBotVar* left, CBotVar* right) override;    // substraction
+    void        Mul(CBotVar* left, CBotVar* right) override;    // multiplication
+    int         Div(CBotVar* left, CBotVar* right) override;    // division
+    int            Modulo(CBotVar* left, CBotVar* right) override;    // remainder of division
+    void        Power(CBotVar* left, CBotVar* right) override;    // power
 
-    bool        Lo(CBotVar* left, CBotVar* right);
-    bool        Hi(CBotVar* left, CBotVar* right);
-    bool        Ls(CBotVar* left, CBotVar* right);
-    bool        Hs(CBotVar* left, CBotVar* right);
-    bool        Eq(CBotVar* left, CBotVar* right);
-    bool        Ne(CBotVar* left, CBotVar* right);
+    bool        Lo(CBotVar* left, CBotVar* right) override;
+    bool        Hi(CBotVar* left, CBotVar* right) override;
+    bool        Ls(CBotVar* left, CBotVar* right) override;
+    bool        Hs(CBotVar* left, CBotVar* right) override;
+    bool        Eq(CBotVar* left, CBotVar* right) override;
+    bool        Ne(CBotVar* left, CBotVar* right) override;
 
-    void        Neg();
-    void        Inc();
-    void        Dec();
+    void        Neg() override;
+    void        Inc() override;
+    void        Dec() override;
 
-    bool        Save1State(FILE* pf);
+    bool        Save1State(FILE* pf) override;
 };
 
 
@@ -1323,21 +1324,21 @@ public:
                 CBotVarString( const CBotToken* name );
 //                ~CBotVarString();
 
-    void        SetValString(const char* p);
-    CBotString    GetValString();
+    void        SetValString(const char* p) override;
+    CBotString    GetValString() override;
 
-    void        Copy(CBotVar* pSrc, bool bName=true);
+    void        Copy(CBotVar* pSrc, bool bName=true) override;
 
-    void        Add(CBotVar* left, CBotVar* right);    // addition
+    void        Add(CBotVar* left, CBotVar* right) override;    // addition
 
-    bool        Lo(CBotVar* left, CBotVar* right);
-    bool        Hi(CBotVar* left, CBotVar* right);
-    bool        Ls(CBotVar* left, CBotVar* right);
-    bool        Hs(CBotVar* left, CBotVar* right);
-    bool        Eq(CBotVar* left, CBotVar* right);
-    bool        Ne(CBotVar* left, CBotVar* right);
+    bool        Lo(CBotVar* left, CBotVar* right) override;
+    bool        Hi(CBotVar* left, CBotVar* right) override;
+    bool        Ls(CBotVar* left, CBotVar* right) override;
+    bool        Hs(CBotVar* left, CBotVar* right) override;
+    bool        Eq(CBotVar* left, CBotVar* right) override;
+    bool        Ne(CBotVar* left, CBotVar* right) override;
 
-    bool        Save1State(FILE* pf);
+    bool        Save1State(FILE* pf) override;
 };
 
 // class for the management of boolean
@@ -1350,22 +1351,22 @@ public:
                 CBotVarBoolean( const CBotToken* name );
 //                ~CBotVarBoolean();
 
-    void        SetValInt(int val, const char* s = NULL);
-    void        SetValFloat(float val);
-    int            GetValInt();
-    float        GetValFloat();
-    CBotString    GetValString();
+    void        SetValInt(int val, const char* s = nullptr) override;
+    void        SetValFloat(float val) override;
+    int            GetValInt() override;
+    float        GetValFloat() override;
+    CBotString    GetValString() override;
 
-    void        Copy(CBotVar* pSrc, bool bName=true);
+    void        Copy(CBotVar* pSrc, bool bName=true) override;
 
-    void        And(CBotVar* left, CBotVar* right);
-    void        Or(CBotVar* left, CBotVar* right);
-    void        XOr(CBotVar* left, CBotVar* right);
-    void        Not();
-    bool        Eq(CBotVar* left, CBotVar* right);
-    bool        Ne(CBotVar* left, CBotVar* right);
+    void        And(CBotVar* left, CBotVar* right) override;
+    void        Or(CBotVar* left, CBotVar* right) override;
+    void        XOr(CBotVar* left, CBotVar* right) override;
+    void        Not() override;
+    bool        Eq(CBotVar* left, CBotVar* right) override;
+    bool        Ne(CBotVar* left, CBotVar* right) override;
 
-    bool        Save1State(FILE* pf);
+    bool        Save1State(FILE* pf) override;
 };
 
 
@@ -1394,38 +1395,38 @@ public:
                 ~CBotVarClass();
 //    void        InitCBotVarClass( const CBotToken* name, CBotTypResult& type, int &nIdent );
 
-    void        Copy(CBotVar* pSrc, bool bName=true);
-    void        SetClass(CBotClass* pClass); //, int &nIdent);
-    CBotClass*    GetClass();
-    CBotVar*    GetItem(const char* name);    // return an element of a class according to its name (*)
-    CBotVar*    GetItemRef(int nIdent);
+    void        Copy(CBotVar* pSrc, bool bName=true) override;
+    void        SetClass(CBotClass* pClass) override; //, int &nIdent);
+    CBotClass*    GetClass() override;
+    CBotVar*    GetItem(const char* name) override;    // return an element of a class according to its name (*)
+    CBotVar*    GetItemRef(int nIdent) override;
 
-    CBotVar*    GetItem(int n, bool bExtend);
-    CBotVar*    GetItemList();
+    CBotVar*    GetItem(int n, bool bExtend) override;
+    CBotVar*    GetItemList() override;
 
-    CBotString    GetValString();
+    CBotString    GetValString() override;
 
-    bool        Save1State(FILE* pf);
-    void        Maj(void* pUser, bool bContinue);
+    bool        Save1State(FILE* pf) override;
+    void        Maj(void* pUser, bool bContinue) override;
 
     void        IncrementUse();                // a reference to incrementation
     void        DecrementUse();                // a reference to decrementation
 
     CBotVarClass*
-                GetPointer();
+                GetPointer() override;
     void        SetItemList(CBotVar* pVar);
 
-    void        SetIdent(long n);
+    void        SetIdent(long n) override;
 
     static CBotVarClass* Find(long id);
 
 
 //    CBotVar*    GetMyThis();
 
-    bool        Eq(CBotVar* left, CBotVar* right);
-    bool        Ne(CBotVar* left, CBotVar* right);
+    bool        Eq(CBotVar* left, CBotVar* right) override;
+    bool        Ne(CBotVar* left, CBotVar* right) override;
 
-    void        ConstructorSet();
+    void        ConstructorSet() override;
 };
 
 
@@ -1442,27 +1443,27 @@ public:
                 CBotVarPointer( const CBotToken* name, CBotTypResult& type );
                 ~CBotVarPointer();
 
-    void        Copy(CBotVar* pSrc, bool bName=true);
-    void        SetClass(CBotClass* pClass);
-    CBotClass*    GetClass();
-    CBotVar*    GetItem(const char* name);    // return an element of a class according to its name (*)
-    CBotVar*    GetItemRef(int nIdent);
-    CBotVar*    GetItemList();
+    void        Copy(CBotVar* pSrc, bool bName=true) override;
+    void        SetClass(CBotClass* pClass) override;
+    CBotClass*    GetClass() override;
+    CBotVar*    GetItem(const char* name) override;    // return an element of a class according to its name (*)
+    CBotVar*    GetItemRef(int nIdent) override;
+    CBotVar*    GetItemList() override;
 
-    CBotString    GetValString();
-    void        SetPointer(CBotVar* p);
+    CBotString    GetValString() override;
+    void        SetPointer(CBotVar* p) override;
     CBotVarClass*
-                GetPointer();
+                GetPointer() override;
 
-    void        SetIdent(long n);            // associates an identification number (unique)
+    void        SetIdent(long n) override;            // associates an identification number (unique)
     long        GetIdent();                    // gives the identification number associated with
-    void        ConstructorSet();
+    void        ConstructorSet() override;
 
-    bool        Save1State(FILE* pf);
-    void        Maj(void* pUser, bool bContinue);
+    bool        Save1State(FILE* pf) override;
+    void        Maj(void* pUser, bool bContinue) override;
 
-    bool        Eq(CBotVar* left, CBotVar* right);
-    bool        Ne(CBotVar* left, CBotVar* right);
+    bool        Eq(CBotVar* left, CBotVar* right) override;
+    bool        Ne(CBotVar* left, CBotVar* right) override;
 };
 
 
@@ -1482,19 +1483,19 @@ public:
                 CBotVarArray( const CBotToken* name, CBotTypResult& type );
                 ~CBotVarArray();
 
-    void        SetPointer(CBotVar* p);
+    void        SetPointer(CBotVar* p) override;
     CBotVarClass*
-                GetPointer();
+                GetPointer() override;
 
-    void        Copy(CBotVar* pSrc, bool bName=true);
-    CBotVar*    GetItem(int n, bool bGrow=false);    // makes an element according to its numeric index
+    void        Copy(CBotVar* pSrc, bool bName=true) override;
+    CBotVar*    GetItem(int n, bool bGrow=false) override;    // makes an element according to its numeric index
                                                 // enlarged the table if necessary if bExtend
 //    CBotVar*    GetItem(const char* name);        // makes a element by  literal index
-    CBotVar*    GetItemList();                    // gives the first item in the list
+    CBotVar*    GetItemList() override;                    // gives the first item in the list
 
-    CBotString    GetValString();                    // gets the contents of the array into a string
+    CBotString    GetValString() override;                    // gets the contents of the array into a string
 
-    bool        Save1State(FILE* pf);
+    bool        Save1State(FILE* pf) override;
 };
 
 
@@ -1579,7 +1580,7 @@ class CBotCallMethode
 {
 private:
     CBotString    m_name;
-    bool        (*m_rExec) (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception);
+    bool        (*m_rExec) (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception, void* user);
     CBotTypResult
                 (*m_rComp) (CBotVar* pThis, CBotVar* &pVar);
     CBotCallMethode*    m_next;
@@ -1588,7 +1589,7 @@ private:
 
 public:
                 CBotCallMethode(const char* name,
-                         bool rExec (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception),
+                         bool rExec (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception, void* user),
                          CBotTypResult rCompile (CBotVar* pThis, CBotVar* &pVar));
                 ~CBotCallMethode();
 
@@ -1674,8 +1675,8 @@ public:
     CBotFunction*    Compile(CBotToken* &p, CBotCStack* pStack, CBotFunction* pFunc, bool bLocal = true);
     static
     CBotFunction*    Compile1(CBotToken* &p, CBotCStack* pStack, CBotClass*    pClass);
-    bool            Execute(CBotVar** ppVars, CBotStack* &pj, CBotVar* pInstance = NULL);
-    void            RestoreState(CBotVar** ppVars, CBotStack* &pj, CBotVar* pInstance = NULL);
+    bool            Execute(CBotVar** ppVars, CBotStack* &pj, CBotVar* pInstance = nullptr);
+    void            RestoreState(CBotVar** ppVars, CBotStack* &pj, CBotVar* pInstance = nullptr);
 
     void            AddNext(CBotFunction* p);
     CBotTypResult    CompileCall(const char* name, CBotVar** ppVars, long& nIdent);
@@ -1698,4 +1699,3 @@ public:
 
     bool            GetPosition(int& start, int& stop, CBotGet modestart, CBotGet modestop);
 };
-
