@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see http://gnu.org/licenses
  */
+
 ////////////////////////////////////////////////////////////////////////
 
 /**
@@ -399,7 +400,7 @@ public:
                     CBotProgram(CBotVar* pInstance);
                     ~CBotProgram();
 
-    bool            Compile( const char* program, CBotStringArray& ListFonctions, void* pUser = NULL);
+    bool            Compile( const char* program, CBotStringArray& ListFonctions, void* pUser = nullptr);
     //                compiles the program given in text
     //                returns false if an error at compile
     //                see GetCompileError () to retrieve the error
@@ -428,7 +429,7 @@ public:
     //                returns false if the funtion name is not found
     //                the program does nothing, we must call Run () for this
 
-    bool            Run(void* pUser = NULL, int timer = -1);
+    bool            Run(void* pUser = nullptr, int timer = -1);
     //                executes the program
     //                returns false if the program was suspended
     //                returns true if the program ended with or without error
@@ -443,10 +444,10 @@ public:
     CBotVar*        GetStackVars(const char* &FunctionName, int level);
     //                provides the pointer to the variables on the execution stack
     //                level is an input parameter,  0 for the last level, -1, -2, etc. for the other levels
-    //                the return value (CBotVar *) is a variable list (or NULL)
+    //                the return value (CBotVar *) is a variable list (or nullptr)
     //                    that can be processed as the list of parameters received by a routine
     //                FunctionName gives the name of the function where are these variables
-    //                FunctionName == NULL means that is more in a program (depending on  level)
+    //                FunctionName == nullptr means that is more in a program (depending on  level)
 
     void            Stop();
     //                stops execution of the program
@@ -512,9 +513,9 @@ public:
 
 int    cMean(CBotVar* &pVar, CBotString& ClassName)
 {
-    if ( pVar == NULL ) return 6001;    // there is no parameter!
+    if ( pVar == nullptr ) return 6001;    // there is no parameter!
 
-    while ( pVar != NULL )
+    while ( pVar != nullptr )
     {
         if ( pVar->GetType() > CBotTypDouble ) return 6002;        // this is not a number
         pVar = pVar -> GetNext();
@@ -528,7 +529,7 @@ bool rMean(CBotVar* pVar, CBotVar* pResult, int& Exception)
 {
     float total = 0;
     int   nb      = 0;
-    while (pVar != NULL)
+    while (pVar != nullptr)
     {
         total += pVar->GetValFloat();
         pVar = pVar->GetNext();
@@ -547,11 +548,6 @@ bool rMean(CBotVar* pVar, CBotVar* pResult, int& Exception)
 // may be useful to the outside of the module
 // ( it is currently not expected to be able to create these objects in outer )
 
-// results of GetInit()
-#define    IS_UNDEF    0        // undefined variable
-#define IS_DEF        1        // variable defined
-#define    IS_NAN        999        // variable defined as not a number
-
 // variable type SetPrivate / IsPrivate
 #define PR_PUBLIC    0        // public variable
 #define    PR_READ        1        // read only
@@ -560,6 +556,10 @@ bool rMean(CBotVar* pVar, CBotVar* pResult, int& Exception)
 
 class CBotVar
 {
+public:
+    // results of GetInit()
+    enum class InitType : int { UNDEF = 0, DEF = 1, IS_POINTER = 2, IS_NAN = 999 };
+
 protected:
     CBotToken*        m_token;                    // the corresponding token
 
@@ -571,7 +571,7 @@ protected:
 
     CBotTypResult    m_type;                        // type of value
 
-    int                m_binit;                    // not initialized?
+    InitType         m_binit;                    // not initialized?
     CBotVarClass*    m_pMyThis;                    // ^ corresponding this element
     void*            m_pUserPtr;                    // ^user data if necessary
     bool            m_bStatic;                    // static element (in class)
@@ -635,9 +635,11 @@ virtual                ~CBotVar( );                        // destructor
     CBotToken*        GetToken();
     void            SetType(CBotTypResult& type);
 
-    void            SetInit(int bInit);            // is the variable in the state IS_UNDEF, IS_DEF, IS_NAN
-
-    int                GetInit();                    // gives the state of the variable
+    void            SetInit(InitType initType);            // is the variable in the state UNDEF, DEF, NAN
+    InitType        GetInit() const;                       // gives the state of the variable
+    bool IsUndefined() const { return GetInit() == InitType::UNDEF; }
+    bool IsDefined() const { return GetInit() == InitType::DEF; }
+    bool IsNAN() const { return GetInit() == InitType::IS_NAN; }
 
     void            SetStatic(bool bStatic);
     bool            IsStatic();
@@ -676,7 +678,7 @@ virtual                ~CBotVar( );                        // destructor
     virtual
     void            Copy(CBotVar* pSrc, bool bName = true);    // makes a copy of the variable
 
-    virtual void    SetValInt(int val, const char* name = NULL);
+    virtual void    SetValInt(int val, const char* name = nullptr);
                                                 // initialized with an integer value (#)
     /////////////////////////////////////////////////////////////////////////////////
 
@@ -742,7 +744,7 @@ virtual                ~CBotVar( );                        // destructor
 //    CBotVar*        GetMyThis();
 
     virtual
-    void            Maj(void* pUser = NULL, bool bContinue = true);
+    void            Maj(void* pUser = nullptr, bool bContinue = true);
 
     void            SetUniqNum(long n);
     long            GetUniqNum();
@@ -806,7 +808,7 @@ public:
                     ~CBotClass( );                            // destructor
 
     bool            AddFunction(const char* name,
-                                bool rExec (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception),
+                                bool rExec (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception, void* user),
                                 CBotTypResult rCompile (CBotVar* pThis, CBotVar* &pVar));
     //                this call allows to add as external (**)
     //                new method used by the objects of this class
@@ -828,7 +830,7 @@ public:
     void            AddNext(CBotClass* pClass);
 
     CBotString        GetName();                    // gives the name of the class
-    CBotClass*        GetParent();                // gives the parent class (or NULL)
+    CBotClass*        GetParent();                // gives the parent class (or nullptr)
 
     // true if a class is derived (Extends) of another
     // return true also if the classes are identical
@@ -894,7 +896,10 @@ public:
 #define TokenKeyVal            2200                // keywords representing the value (true, false, null, nan)
 #define TokenKeyOp            2300                // operators
 
-/** \class Responsible for token management */
+/**
+ * \class CBotToken
+ * Responsible for token management
+ */
 class CBotToken
 {
 private:
@@ -942,7 +947,7 @@ public:
                     CBotToken();
                     CBotToken(const CBotToken* pSrc);
                     CBotToken(const CBotString& mot, const CBotString& sep, int start=0, int end=0);
-                    CBotToken(const char* mot, const char* sep = NULL);
+                    CBotToken(const char* mot, const char* sep = nullptr);
 
     /**
      * \brief Destructor
@@ -1021,7 +1026,7 @@ public:
 
 
 
-#if 0
+/*
 ////////////////////////////////////////////////////////////////////////
 // Examples of use
 // Definition classes and functions
@@ -1029,7 +1034,7 @@ public:
 
 // define the global class CPoint
 // --------------------------------
-    m_pClassPoint    = new CBotClass("CPoint", NULL);
+    m_pClassPoint    = new CBotClass("CPoint", nullptr);
     // adds the component ".x"
     m_pClassPoint->AddItem("x", CBotTypResult(CBotTypFloat));
     // adds the component ".y"
@@ -1077,7 +1082,7 @@ public:
 
 // build a stack for interpreter
 // --------------------------------------
-    CBotStack*    pStack = new CBotStack(NULL);
+    CBotStack*    pStack = new CBotStack(nullptr);
 
 // executes the main program
 // -------------------------
@@ -1104,16 +1109,15 @@ bool rDoGoto( CBotVar* pVar, CBotVar* pResult, int& exception )
     // or so
     CBotVar*    temp;
     temp = pVar->GetItem("x");            // is necessary for the object of type CPoint
-    ASSERT (temp != NULL && temp->GetType() == CBotTypFloat);
+    ASSERT (temp != nullptr && temp->GetType() == CBotTypFloat);
     m_PosToGo.x = temp->GetValFloat();
 
     temp = pVar->GetItem("y");            // is necessary for the object of type CPoint
-    ASSERT (temp != NULL && temp->GetType() == CBotTypFloat);
+    ASSERT (temp != nullptr && temp->GetType() == CBotTypFloat);
     m_PosToGo.y = temp->GetValFloat();
 
     return (m_CurentPos == m_PosToGo);    // makes true if the position is reached
                                         // returns false if one had wait yet
 }
 
-#endif
-
+*/
