@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,11 +25,11 @@
 
 CBotProgram::CBotProgram()
 {
-    m_Prog      = NULL;
-    m_pRun      = NULL;
-    m_pClass    = NULL;
-    m_pStack    = NULL;
-    m_pInstance = NULL;
+    m_Prog      = nullptr;
+    m_pRun      = nullptr;
+    m_pClass    = nullptr;
+    m_pStack    = nullptr;
+    m_pInstance = nullptr;
 
     m_ErrorCode = 0;
     m_Ident     = 0;
@@ -38,10 +38,10 @@ CBotProgram::CBotProgram()
 
 CBotProgram::CBotProgram(CBotVar* pInstance)
 {
-    m_Prog      = NULL;
-    m_pRun      = NULL;
-    m_pClass    = NULL;
-    m_pStack    = NULL;
+    m_Prog      = nullptr;
+    m_pRun      = nullptr;
+    m_pClass    = nullptr;
+    m_pStack    = nullptr;
     m_pInstance = pInstance;
 
     m_ErrorCode = 0;
@@ -54,7 +54,7 @@ CBotProgram::~CBotProgram()
 {
 //  delete  m_pClass;
     m_pClass->Purge();
-    m_pClass    = NULL;
+    m_pClass    = nullptr;
 
     CBotClass::FreeLock(this);
 
@@ -75,28 +75,25 @@ bool CBotProgram::Compile( const char* program, CBotStringArray& ListFonctions,
 //  delete      m_pClass;
     m_pClass->Purge();      // purge the old definitions of classes
                             // but without destroying the object
-    m_pClass    = NULL;
-    delete      m_Prog;     m_Prog= NULL;
+    m_pClass    = nullptr;
+    delete      m_Prog;     m_Prog= nullptr;
 
     ListFonctions.SetSize(0);
     m_ErrorCode = 0;
 
-    if (m_pInstance != NULL && m_pInstance->m_pUserPtr != NULL)
-        pUser = m_pInstance->m_pUserPtr;
-
     // transforms the program in Tokens
     CBotToken*  pBaseToken = CBotToken::CompileTokens(program, error);
-    if ( pBaseToken == NULL ) return false;
+    if ( pBaseToken == nullptr ) return false;
 
 
-    CBotCStack* pStack      = new CBotCStack(NULL);
+    CBotCStack* pStack      = new CBotCStack(nullptr);
     CBotToken*  p  = pBaseToken->GetNext();                 // skips the first token (separator)
 
     pStack->SetBotCall(this);                               // defined used routines
     CBotCall::SetPUser(pUser);
 
     // first made a quick pass just to take the headers of routines and classes
-    while ( pStack->IsOk() && p != NULL && p->GetType() != 0)
+    while ( pStack->IsOk() && p != nullptr && p->GetType() != 0)
     {
         if ( IsOfType(p, ID_SEP) ) continue;                // semicolons lurking
 
@@ -104,13 +101,13 @@ bool CBotProgram::Compile( const char* program, CBotStringArray& ListFonctions,
             ( p->GetType() == ID_PUBLIC && p->GetNext()->GetType() == ID_CLASS ))
         {
             CBotClass*  nxt = CBotClass::Compile1(p, pStack);
-            if (m_pClass == NULL ) m_pClass = nxt;
+            if (m_pClass == nullptr ) m_pClass = nxt;
             else m_pClass->AddNext(nxt);
         }
         else
         {
-            CBotFunction*   next = CBotFunction::Compile1(p, pStack, NULL);
-            if (m_Prog == NULL ) m_Prog = next;
+            CBotFunction*   next = CBotFunction::Compile1(p, pStack, nullptr);
+            if (m_Prog == nullptr ) m_Prog = next;
             else m_Prog->AddNext(next);
         }
     }
@@ -118,17 +115,17 @@ bool CBotProgram::Compile( const char* program, CBotStringArray& ListFonctions,
     {
         m_ErrorCode = pStack->GetError(m_ErrorStart, m_ErrorEnd);
         delete m_Prog;
-        m_Prog = NULL;
+        m_Prog = nullptr;
         delete pBaseToken;
         return false;
     }
 
-//  CBotFunction*   temp = NULL;
+//  CBotFunction*   temp = nullptr;
     CBotFunction*   next = m_Prog;      // rewind the list
 
     p  = pBaseToken->GetNext();                             // returns to the beginning
 
-    while ( pStack->IsOk() && p != NULL && p->GetType() != 0 )
+    while ( pStack->IsOk() && p != nullptr && p->GetType() != 0 )
     {
         if ( IsOfType(p, ID_SEP) ) continue;                // semicolons lurking
 
@@ -155,13 +152,13 @@ bool CBotProgram::Compile( const char* program, CBotStringArray& ListFonctions,
     {
         m_ErrorCode = pStack->GetError(m_ErrorStart, m_ErrorEnd);
         delete m_Prog;
-        m_Prog = NULL;
+        m_Prog = nullptr;
     }
 
     delete pBaseToken;
     delete pStack;
 
-    return (m_Prog != NULL);
+    return (m_Prog != nullptr);
 }
 
 
@@ -172,16 +169,16 @@ bool CBotProgram::Start(const char* name)
 #else
     delete m_pStack;
 #endif
-    m_pStack = NULL;
+    m_pStack = nullptr;
 
     m_pRun = m_Prog;
-    while (m_pRun != NULL)
+    while (m_pRun != nullptr)
     {
         if ( m_pRun->GetName() == name ) break;
         m_pRun = m_pRun->m_next;
     }
 
-    if ( m_pRun == NULL )
+    if ( m_pRun == nullptr )
     {
         m_ErrorCode = TX_NORUN;
         return false;
@@ -190,7 +187,7 @@ bool CBotProgram::Start(const char* name)
 #if STACKMEM
     m_pStack = CBotStack::FirstStack();
 #else
-    m_pStack = new CBotStack(NULL);                 // creates an execution stack
+    m_pStack = new CBotStack(nullptr);                 // creates an execution stack
 #endif
 
     m_pStack->SetBotCall(this);                     // bases for routines
@@ -201,13 +198,13 @@ bool CBotProgram::Start(const char* name)
 bool CBotProgram::GetPosition(const char* name, int& start, int& stop, CBotGet modestart, CBotGet modestop)
 {
     CBotFunction* p = m_Prog;
-    while (p != NULL)
+    while (p != nullptr)
     {
         if ( p->GetName() == name ) break;
         p = p->m_next;
     }
 
-    if ( p == NULL ) return false;
+    if ( p == nullptr ) return false;
 
     p->GetPosition(start, stop, modestart, modestop);
     return true;
@@ -217,11 +214,9 @@ bool CBotProgram::Run(void* pUser, int timer)
 {
     bool    ok;
 
-    if (m_pStack == NULL || m_pRun == NULL) goto error;
+    if (m_pStack == nullptr || m_pRun == nullptr) goto error;
 
     m_ErrorCode = 0;
-    if (m_pInstance != NULL && m_pInstance->m_pUserPtr != NULL)
-        pUser = m_pInstance->m_pUserPtr;
 
     m_pStack->Reset(pUser);                         // empty the possible previous error, and resets the timer
     if ( timer >= 0 ) m_pStack->SetTimer(timer);
@@ -237,15 +232,15 @@ bool CBotProgram::Run(void* pUser, int timer)
         CBotVar*    ppVar[3];
         ppVar[0] = CBotVar::Create("aa", CBotTypInt);
         ppVar[1] = CBotVar::Create("bb", CBotTypInt);
-        ppVar[2] = NULL;
+        ppVar[2] = nullptr;
         ok = m_pRun->Execute(ppVar, m_pStack, m_pInstance);
 #else
         // returns to normal execution
-        ok = m_pRun->Execute(NULL, m_pStack, m_pInstance);
+        ok = m_pRun->Execute(nullptr, m_pStack, m_pInstance);
 #endif
     }
 #else
-    ok = m_pRun->Execute(NULL, m_pStack, m_pInstance);
+    ok = m_pRun->Execute(nullptr, m_pStack, m_pInstance);
 #endif
 
     // completed on a mistake?
@@ -257,11 +252,11 @@ bool CBotProgram::Run(void* pUser, int timer)
 #else
         delete m_pStack;
 #endif
-        m_pStack = NULL;
+        m_pStack = nullptr;
         return true;                                // execution is finished!
     }
 
-    if ( ok ) m_pRun = NULL;                        // more function in execution
+    if ( ok ) m_pRun = nullptr;                        // more function in execution
     return ok;
 
 error:
@@ -276,17 +271,17 @@ void CBotProgram::Stop()
 #else
     delete m_pStack;
 #endif
-    m_pStack = NULL;
-    m_pRun = NULL;
+    m_pStack = nullptr;
+    m_pRun = nullptr;
 }
 
 
 
 bool CBotProgram::GetRunPos(const char* &FunctionName, int &start, int &end)
 {
-    FunctionName = NULL;
+    FunctionName = nullptr;
     start = end = 0;
-    if (m_pStack == NULL) return false;
+    if (m_pStack == nullptr) return false;
 
     m_pStack->GetRunPos(FunctionName, start, end);
     return true;
@@ -294,8 +289,8 @@ bool CBotProgram::GetRunPos(const char* &FunctionName, int &start, int &end)
 
 CBotVar* CBotProgram::GetStackVars(const char* &FunctionName, int level)
 {
-    FunctionName = NULL;
-    if (m_pStack == NULL) return NULL;
+    FunctionName = nullptr;
+    if (m_pStack == nullptr) return nullptr;
 
     return m_pStack->GetStackVars(FunctionName, level);
 }
@@ -507,7 +502,7 @@ bool CBotProgram::SaveState(FILE* pf)
     if (!WriteWord( pf, CBOTVERSION)) return false;
 
 
-    if ( m_pStack != NULL )
+    if ( m_pStack != nullptr )
     {
         if (!WriteWord( pf, 1)) return false;
         if (!WriteString( pf, m_pRun->GetName() )) return false;
@@ -542,15 +537,15 @@ bool CBotProgram::RestoreState(FILE* pf)
 #else
     delete m_pStack;
 #endif
-    m_pStack = NULL;
+    m_pStack = nullptr;
 
     // retrieves the stack from the memory
-    // uses a NULL pointer (m_pStack) but it's ok like that
+    // uses a nullptr pointer (m_pStack) but it's ok like that
     if (!m_pStack->RestoreState(pf, m_pStack)) return false;
     m_pStack->SetBotCall(this);                     // bases for routines
 
     // restored some states in the stack according to the structure
-    m_pRun->RestoreState(NULL, m_pStack, m_pInstance);
+    m_pRun->RestoreState(nullptr, m_pStack, m_pInstance);
     return true;
 }
 
@@ -562,7 +557,7 @@ int CBotProgram::GetVersion()
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////
 
-CBotCall* CBotCall::m_ListCalls = NULL;
+CBotCall* CBotCall::m_ListCalls = nullptr;
 
 CBotCall::CBotCall(const char* name,
                    bool rExec (CBotVar* pVar, CBotVar* pResult, int& Exception, void* pUser),
@@ -571,14 +566,14 @@ CBotCall::CBotCall(const char* name,
     m_name       = name;
     m_rExec      = rExec;
     m_rComp      = rCompile;
-    m_next       = NULL;
+    m_next       = nullptr;
     m_nFuncIdent = CBotVar::NextUniqNum();
 }
 
 CBotCall::~CBotCall()
 {
     if (m_next) delete m_next;
-    m_next = NULL;
+    m_next = nullptr;
 }
 
 void CBotCall::Free()
@@ -591,9 +586,9 @@ bool CBotCall::AddFunction(const char* name,
                            CBotTypResult rCompile (CBotVar* &pVar, void* pUser))
 {
     CBotCall*   p = m_ListCalls;
-    CBotCall*   pp = NULL;
+    CBotCall*   pp = nullptr;
 
-    if ( p != NULL ) while ( p->m_next != NULL )
+    if ( p != nullptr ) while ( p->m_next != nullptr )
     {
         if ( p->GetName() == name )
         {
@@ -602,7 +597,7 @@ bool CBotCall::AddFunction(const char* name,
             else      m_ListCalls = p->m_next;
             pp = p;
             p = p->m_next;
-            pp->m_next = NULL;  // not to destroy the following list
+            pp->m_next = nullptr;  // not to destroy the following list
             delete pp;
             continue;
         }
@@ -624,12 +619,12 @@ bool CBotCall::AddFunction(const char* name,
 CBotVar* MakeListVars(CBotVar** ppVars, bool bSetVal=false)
 {
     int     i = 0;
-    CBotVar*    pVar = NULL;
+    CBotVar*    pVar = nullptr;
 
     while( true )
     {
 //        ppVars[i];
-        if ( ppVars[i] == NULL ) break;
+        if ( ppVars[i] == nullptr ) break;
 
         CBotVar*    pp = CBotVar::Create(ppVars[i]);
         if (bSetVal) pp->Copy(ppVars[i]);
@@ -637,7 +632,7 @@ CBotVar* MakeListVars(CBotVar** ppVars, bool bSetVal=false)
             if ( ppVars[i]->GetType() == CBotTypPointer )
                 pp->SetClass( ppVars[i]->GetClass());
 // copy the pointer according to indirections
-        if (pVar == NULL) pVar = pp;
+        if (pVar == nullptr) pVar = pp;
         else pVar->AddNext(pp);
         i++;
     }
@@ -653,7 +648,7 @@ CBotTypResult CBotCall::CompileCall(CBotToken* &p, CBotVar** ppVar, CBotCStack*
     CBotCall*   pt = m_ListCalls;
     CBotString  name = p->GetString();
 
-    while ( pt != NULL )
+    while ( pt != nullptr )
     {
         if ( pt->m_name == name )
         {
@@ -678,7 +673,7 @@ CBotTypResult CBotCall::CompileCall(CBotToken* &p, CBotVar** ppVar, CBotCStack*
     return -1;
 }
 
-void* CBotCall::m_pUser = NULL;
+void* CBotCall::m_pUser = nullptr;
 
 void CBotCall::SetPUser(void* pUser)
 {
@@ -689,7 +684,7 @@ bool CBotCall::CheckCall(const char* name)
 {
     CBotCall* p = m_ListCalls;
 
-    while ( p != NULL )
+    while ( p != nullptr )
     {
         if ( name == p->GetName() ) return true;
         p = p->m_next;
@@ -714,7 +709,7 @@ int CBotCall::DoCall(long& nIdent, CBotToken* token, CBotVar** ppVar, CBotStack*
 {
     CBotCall*   pt = m_ListCalls;
 
-    if ( nIdent ) while ( pt != NULL )
+    if ( nIdent ) while ( pt != nullptr )
     {
         if ( pt->m_nFuncIdent == nIdent )
         {
@@ -725,10 +720,10 @@ int CBotCall::DoCall(long& nIdent, CBotToken* token, CBotVar** ppVar, CBotStack*
 
     pt = m_ListCalls;
 
-    if ( token != NULL )
+    if ( token != nullptr )
     {
         CBotString name = token->GetString();
-        while ( pt != NULL )
+        while ( pt != nullptr )
         {
             if ( pt->m_name == name )
             {
@@ -749,7 +744,7 @@ fund:
     CBotVar*    pVarToDelete = pVar;
 
     // creates a variable to the result
-    CBotVar*    pResult = rettype.Eq(0) ? NULL : CBotVar::Create("", rettype);
+    CBotVar*    pResult = rettype.Eq(0) ? nullptr : CBotVar::Create("", rettype);
 
     CBotVar*    pRes = pResult;
     int         Exception = 0;
@@ -769,7 +764,7 @@ fund:
     }
     pStack->SetVar(pResult);
 
-    if ( rettype.GetType() > 0 && pResult == NULL )
+    if ( rettype.GetType() > 0 && pResult == nullptr )
     {
         pStack->SetError(TX_NORETVAL, token);
     }
@@ -787,7 +782,7 @@ fund:
 //    CBotVar*    pVarToDelete = pVar;
 
     // creates a variable to the result
-    CBotVar*    pResult = rettype.Eq(0) ? NULL : CBotVar::Create("", rettype);
+    CBotVar*    pResult = rettype.Eq(0) ? nullptr : CBotVar::Create("", rettype);
 
     pile->SetVar( pVar );
 
@@ -809,14 +804,14 @@ bool CBotCall::RestoreCall(long& nIdent, CBotToken* token, CBotVar** ppVar, CBot
 
     {
         CBotString name = token->GetString();
-        while ( pt != NULL )
+        while ( pt != nullptr )
         {
             if ( pt->m_name == name )
             {
                 nIdent = pt->m_nFuncIdent;
 
                 CBotStack*  pile = pStack->RestoreStackEOX(pt);
-                if ( pile == NULL ) return true;
+                if ( pile == nullptr ) return true;
 
  //               CBotStack*  pile2 = pile->RestoreStack();
                 pile->RestoreStack();
@@ -852,7 +847,7 @@ bool CBotCall::Run(CBotStack* pStack)
         return false;
     }
 
-    if ( pResult != NULL ) pStack->SetCopyVar( pResult );
+    if ( pResult != nullptr ) pStack->SetCopyVar( pResult );
     if ( pResult != pRes ) delete pResult;  // different result if made
 
     return true;
@@ -863,20 +858,20 @@ bool CBotCall::Run(CBotStack* pStack)
 ///////////////////////////////////////////////////////////////////////////////////////
 
 CBotCallMethode::CBotCallMethode(const char* name,
-                   bool rExec (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception),
+                   bool rExec (CBotVar* pThis, CBotVar* pVar, CBotVar* pResult, int& Exception, void* user),
                    CBotTypResult rCompile (CBotVar* pThis, CBotVar* &pVar))
 {
     m_name       = name;
     m_rExec      = rExec;
     m_rComp      = rCompile;
-    m_next       = NULL;
+    m_next       = nullptr;
     m_nFuncIdent = CBotVar::NextUniqNum();
 }
 
 CBotCallMethode::~CBotCallMethode()
 {
     delete m_next;
-    m_next = NULL;
+    m_next = nullptr;
 }
 
 // is acceptable by a call procedure name
@@ -889,7 +884,7 @@ CBotTypResult CBotCallMethode::CompileCall(const char* name, CBotVar* pThis,
     CBotCallMethode*    pt = this;
     nIdent = 0;
 
-    while ( pt != NULL )
+    while ( pt != nullptr )
     {
         if ( pt->m_name == name )
         {
@@ -924,7 +919,7 @@ CBotCallMethode* CBotCallMethode::Next()
 void CBotCallMethode::AddNext(CBotCallMethode* pt)
 {
     CBotCallMethode* p = this;
-    while ( p->m_next != NULL ) p = p->m_next;
+    while ( p->m_next != nullptr ) p = p->m_next;
 
     p->m_next = pt;
 }
@@ -936,7 +931,7 @@ int CBotCallMethode::DoCall(long& nIdent, const char* name, CBotVar* pThis, CBot
 
     // search by the identifier
 
-    if ( nIdent ) while ( pt != NULL )
+    if ( nIdent ) while ( pt != nullptr )
     {
         if ( pt->m_nFuncIdent == nIdent )
         {
@@ -948,7 +943,7 @@ int CBotCallMethode::DoCall(long& nIdent, const char* name, CBotVar* pThis, CBot
             // then calls the routine external to the module
 
             int         Exception = 0;
-            int res = pt->m_rExec(pThis, pVar, pResult, Exception);
+            int res = pt->m_rExec(pThis, pVar, pResult, Exception, pStack->GetPUser());
             pStack->SetVar(pResult);
 
             if (res == false)
@@ -969,7 +964,7 @@ int CBotCallMethode::DoCall(long& nIdent, const char* name, CBotVar* pThis, CBot
 
     // search by name
 
-    while ( pt != NULL )
+    while ( pt != nullptr )
     {
         if ( pt->m_name == name )
         {
@@ -979,7 +974,7 @@ int CBotCallMethode::DoCall(long& nIdent, const char* name, CBotVar* pThis, CBot
             CBotVar*    pVarToDelete = pVar;
 
             int         Exception = 0;
-            int res = pt->m_rExec(pThis, pVar, pResult, Exception);
+            int res = pt->m_rExec(pThis, pVar, pResult, Exception, pStack->GetPUser());
             pStack->SetVar(pResult);
 
             if (res == false)
@@ -1004,12 +999,12 @@ int CBotCallMethode::DoCall(long& nIdent, const char* name, CBotVar* pThis, CBot
 
 bool rSizeOf( CBotVar* pVar, CBotVar* pResult, int& ex, void* pUser )
 {
-    if ( pVar == NULL ) return TX_LOWPARAM;
+    if ( pVar == nullptr ) return TX_LOWPARAM;
 
     int i = 0;
     pVar = pVar->GetItemList();
 
-    while ( pVar != NULL )
+    while ( pVar != nullptr )
     {
         i++;
         pVar = pVar->GetNext();
@@ -1021,7 +1016,7 @@ bool rSizeOf( CBotVar* pVar, CBotVar* pResult, int& ex, void* pUser )
 
 CBotTypResult cSizeOf( CBotVar* &pVar, void* pUser )
 {
-    if ( pVar == NULL ) return CBotTypResult( TX_LOWPARAM );
+    if ( pVar == nullptr ) return CBotTypResult( TX_LOWPARAM );
     if ( pVar->GetType() != CBotTypArrayPointer )
                         return CBotTypResult( TX_BADPARAM );
     return CBotTypResult( CBotTypInt );
@@ -1040,7 +1035,7 @@ bool rCBotDebug( CBotVar* pVar, CBotVar* pResult, int& ex, void* pUser )
 CBotTypResult cCBotDebug( CBotVar* &pVar, void* pUser )
 {
     // no parameter
-    if ( pVar != NULL ) return CBotTypResult( TX_OVERPARAM );
+    if ( pVar != nullptr ) return CBotTypResult( TX_OVERPARAM );
 
     // function returns a result "string"
     return CBotTypResult( CBotTypString );
@@ -1114,4 +1109,3 @@ void CBotProgram::Free()
     CBotCall ::Free() ;
     CBotClass::Free() ;
 }
-
