@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,9 +24,9 @@
 
 #pragma once
 
-#include "common/singleton.h"
-
+#include <memory>
 #include <string>
+#include <vector>
 
 /**
  * \enum SystemDialogType
@@ -88,11 +88,13 @@ struct SystemTimeStamp;
  * This class provides system-specific utilities like displaying user dialogs and
  * querying system timers for exact timestamps.
  */
-class CSystemUtils : public CSingleton<CSystemUtils>
+class CSystemUtils
 {
 public:
+    virtual ~CSystemUtils();
+
     //! Creates system utils for specific platform
-    static CSystemUtils* Create();
+    static std::unique_ptr<CSystemUtils> Create();
 
     //! Performs platform-specific initialization
     virtual void Init() = 0;
@@ -134,11 +136,7 @@ public:
 
     //! Sleep for given amount of microseconds
     virtual void Usleep(int usecs) = 0;
+
+private:
+    std::vector<std::unique_ptr<SystemTimeStamp>> m_timeStamps;
 };
-
-//! Global function to get CSystemUtils instance
-inline CSystemUtils* GetSystemUtils()
-{
-    return CSystemUtils::GetInstancePointer();
-}
-
