@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,13 +28,13 @@
 class CSystemUtilsMacOSX : public CSystemUtilsOther
 {
 public:
-    virtual void Init() OVERRIDE;
+    void Init() override;
 
-    virtual std::string GetDataPath() OVERRIDE;
-    virtual std::string GetLangPath() OVERRIDE;
-    virtual std::string GetSaveDir() OVERRIDE;
+    std::string GetDataPath() override;
+    std::string GetLangPath() override;
+    std::string GetSaveDir() override;
 
-    virtual void Usleep(int usec) OVERRIDE;
+    void Usleep(int usec) override;
 
 private:
     std::string m_ASPath;
