@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,17 +22,19 @@
 
 #include "common/logger.h"
 
+#include <boost/lexical_cast.hpp>
+
 namespace
 {
-static EventType UNIQUE_EVENT_TYPE = EVENT_USER;
 const char* EVENT_TYPE_TEXT[EVENT_STD_MAX];
 }
 
 EventType GetUniqueEventType()
 {
-    int i = static_cast<int>(UNIQUE_EVENT_TYPE+1);
-    UNIQUE_EVENT_TYPE = static_cast<EventType>(i);
-    return UNIQUE_EVENT_TYPE;
+    static EventType uniqueEventType = EVENT_USER;
+    int nextUniqueEventType = static_cast<int>(uniqueEventType+1);
+    uniqueEventType = static_cast<EventType>(nextUniqueEventType);
+    return uniqueEventType;
 }
 
 void InitializeEventTypeTexts()
@@ -146,6 +148,8 @@ void InitializeEventTypeTexts()
     EVENT_TYPE_TEXT[EVENT_LIST8]             = "EVENT_LIST8";
     EVENT_TYPE_TEXT[EVENT_LIST9]             = "EVENT_LIST9";
 
+    EVENT_TYPE_TEXT[EVENT_LOADING]           = "EVENT_LOADING";
+
     EVENT_TYPE_TEXT[EVENT_TOOLTIP]           = "EVENT_TOOLTIP";
 
     EVENT_TYPE_TEXT[EVENT_DIALOG_OK]         = "EVENT_DIALOG_OK";
@@ -163,6 +167,7 @@ void InitializeEventTypeTexts()
     EVENT_TYPE_TEXT[EVENT_INTERFACE_DEFI]    = "EVENT_INTERFACE_DEFI";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_MISSION] = "EVENT_INTERFACE_MISSION";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_FREE]    = "EVENT_INTERFACE_FREE";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_CODE_BATTLES] = "EVENT_INTERFACE_CODE_BATTLES";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_NAME]    = "EVENT_INTERFACE_NAME";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_SETUP]   = "EVENT_INTERFACE_SETUP";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_QUIT]    = "EVENT_INTERFACE_QUIT";
@@ -189,33 +194,35 @@ void InitializeEventTypeTexts()
     EVENT_TYPE_TEXT[EVENT_INTERFACE_FULL]    = "EVENT_INTERFACE_FULL";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_APPLY]   = "EVENT_INTERFACE_APPLY";
 
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_TOTO]    = "EVENT_INTERFACE_TOTO";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_SHADOW]  = "EVENT_INTERFACE_SHADOW";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_DIRTY]   = "EVENT_INTERFACE_DIRTY";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_LENS]    = "EVENT_INTERFACE_LENS";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_SKY]     = "EVENT_INTERFACE_SKY";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_PLANET]  = "EVENT_INTERFACE_PLANET";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_LIGHT]   = "EVENT_INTERFACE_LIGHT";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_PARTI]   = "EVENT_INTERFACE_PARTI";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_CLIP]    = "EVENT_INTERFACE_CLIP";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_DETAIL]  = "EVENT_INTERFACE_DETAIL";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_TEXTURE] = "EVENT_INTERFACE_TEXTURE";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_RAIN]    = "EVENT_INTERFACE_RAIN";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_GLINT]   = "EVENT_INTERFACE_GLINT";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_TOOLTIP] = "EVENT_INTERFACE_TOOLTIP";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_MOVIES]  = "EVENT_INTERFACE_MOVIES";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_NICERST] = "EVENT_INTERFACE_NICERST";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_SCROLL]  = "EVENT_INTERFACE_SCROLL";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_INVERTX] = "EVENT_INTERFACE_INVERTX";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_INVERTY] = "EVENT_INTERFACE_INVERTY";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_EFFECT]  = "EVENT_INTERFACE_EFFECT";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_GROUND]  = "EVENT_INTERFACE_GROUND";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_GADGET]  = "EVENT_INTERFACE_GADGET";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_MOUSE]   = "EVENT_INTERFACE_MOUSE";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_FOG]     = "EVENT_INTERFACE_FOG";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_HIMSELF] = "EVENT_INTERFACE_HIMSELF";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_EDITMODE]= "EVENT_INTERFACE_EDITMODE";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_EDITVALUE]= "EVENT_INTERFACE_EDITVALUE";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_SOLUCE4] = "EVENT_INTERFACE_SOLUCE4";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_BLOOD]   = "EVENT_INTERFACE_BLOOD";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_AUTOSAVE_ENABLE]    = "EVENT_INTERFACE_AUTOSAVE_ENABLE";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_AUTOSAVE_INTERVAL]  = "EVENT_INTERFACE_AUTOSAVE_INTERVAL";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_AUTOSAVE_SLOTS]     = "EVENT_INTERFACE_AUTOSAVE_SLOTS";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_TEXTURE_FILTER]     = "EVENT_INTERFACE_TEXTURE_FILTER";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_TEXTURE_MIPMAP]     = "EVENT_INTERFACE_TEXTURE_MIPMAP";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_TEXTURE_ANISOTROPY] = "EVENT_INTERFACE_TEXTURE_ANISOTROPY";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_MSAA]               = "EVENT_INTERFACE_MSAA";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_SHADOW_SPOTS]       = "EVENT_INTERFACE_SHADOW_SPOTS";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_SHADOW_MAPPING]     = "EVENT_INTERFACE_SHADOW_MAPPING";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_SHADOW_MAPPING_QUALITY]     = "EVENT_INTERFACE_SHADOW_MAPPING_QUALITY";
+    EVENT_TYPE_TEXT[EVENT_INTERFACE_SHADOW_MAPPING_BUFFER]      = "EVENT_INTERFACE_SHADOW_MAPPING_BUFFER";
 
     EVENT_TYPE_TEXT[EVENT_INTERFACE_KINFO1]  = "EVENT_INTERFACE_KINFO1";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_KINFO2]  = "EVENT_INTERFACE_KINFO2";
@@ -244,7 +251,6 @@ void InitializeEventTypeTexts()
     EVENT_TYPE_TEXT[EVENT_INTERFACE_NEDIT]   = "EVENT_INTERFACE_NEDIT";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_NLIST]   = "EVENT_INTERFACE_NLIST";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_NOK]     = "EVENT_INTERFACE_NOK";
-    EVENT_TYPE_TEXT[EVENT_INTERFACE_NCANCEL] = "EVENT_INTERFACE_NCANCEL";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_NDELETE] = "EVENT_INTERFACE_NDELETE";
     EVENT_TYPE_TEXT[EVENT_INTERFACE_NLABEL]  = "EVENT_INTERFACE_NLABEL";
 
@@ -436,7 +442,6 @@ void InitializeEventTypeTexts()
     EVENT_TYPE_TEXT[EVENT_OBJECT_GENERGY]    = "EVENT_OBJECT_GENERGY";
     EVENT_TYPE_TEXT[EVENT_OBJECT_GSHIELD]    = "EVENT_OBJECT_GSHIELD";
     EVENT_TYPE_TEXT[EVENT_OBJECT_GRANGE]     = "EVENT_OBJECT_GRANGE";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_COMPASS]    = "EVENT_OBJECT_COMPASS";
     EVENT_TYPE_TEXT[EVENT_OBJECT_MAP]        = "EVENT_OBJECT_MAP";
     EVENT_TYPE_TEXT[EVENT_OBJECT_MAPZOOM]    = "EVENT_OBJECT_MAPZOOM";
     EVENT_TYPE_TEXT[EVENT_OBJECT_GPROGRESS]  = "EVENT_OBJECT_GPROGRESS";
@@ -457,28 +462,10 @@ void InitializeEventTypeTexts()
     EVENT_TYPE_TEXT[EVENT_OBJECT_CAMERAright]= "EVENT_OBJECT_CAMERAright";
     EVENT_TYPE_TEXT[EVENT_OBJECT_CAMERAnear] = "EVENT_OBJECT_CAMERAnear";
     EVENT_TYPE_TEXT[EVENT_OBJECT_CAMERAaway] = "EVENT_OBJECT_CAMERAaway";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT00] = "EVENT_OBJECT_SHORTCUT00";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT01] = "EVENT_OBJECT_SHORTCUT01";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT02] = "EVENT_OBJECT_SHORTCUT02";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT03] = "EVENT_OBJECT_SHORTCUT03";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT04] = "EVENT_OBJECT_SHORTCUT04";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT05] = "EVENT_OBJECT_SHORTCUT05";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT06] = "EVENT_OBJECT_SHORTCUT06";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT07] = "EVENT_OBJECT_SHORTCUT07";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT08] = "EVENT_OBJECT_SHORTCUT08";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT09] = "EVENT_OBJECT_SHORTCUT09";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT10] = "EVENT_OBJECT_SHORTCUT10";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT11] = "EVENT_OBJECT_SHORTCUT11";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT12] = "EVENT_OBJECT_SHORTCUT12";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT13] = "EVENT_OBJECT_SHORTCUT13";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT14] = "EVENT_OBJECT_SHORTCUT14";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT15] = "EVENT_OBJECT_SHORTCUT15";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT16] = "EVENT_OBJECT_SHORTCUT16";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT17] = "EVENT_OBJECT_SHORTCUT17";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT18] = "EVENT_OBJECT_SHORTCUT18";
-    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT19] = "EVENT_OBJECT_SHORTCUT19";
+    EVENT_TYPE_TEXT[EVENT_OBJECT_SHORTCUT_MODE] = "EVENT_OBJECT_SHORTCUT_MODE";
     EVENT_TYPE_TEXT[EVENT_OBJECT_MOVIELOCK]  = "EVENT_OBJECT_MOVIELOCK";
     EVENT_TYPE_TEXT[EVENT_OBJECT_EDITLOCK]   = "EVENT_OBJECT_EDITLOCK";
+    EVENT_TYPE_TEXT[EVENT_OBJECT_SAVING]     = "EVENT_OBJECT_SAVING";
     EVENT_TYPE_TEXT[EVENT_OBJECT_LIMIT]      = "EVENT_OBJECT_LIMIT";
 
     EVENT_TYPE_TEXT[EVENT_OBJECT_PEN0]       = "EVENT_OBJECT_PEN0";
@@ -512,6 +499,11 @@ void InitializeEventTypeTexts()
     EVENT_TYPE_TEXT[EVENT_STUDIO_RUN]        = "EVENT_STUDIO_RUN";
     EVENT_TYPE_TEXT[EVENT_STUDIO_REALTIME]   = "EVENT_STUDIO_REALTIME";
     EVENT_TYPE_TEXT[EVENT_STUDIO_STEP]       = "EVENT_STUDIO_STEP";
+
+    EVENT_TYPE_TEXT[EVENT_WRITE_SCENE_FINISHED] = "EVENT_WRITE_SCENE_FINISHED";
+
+    EVENT_TYPE_TEXT[EVENT_CODE_BATTLE_START] = "EVENT_CODE_BATTLE_START";
+    EVENT_TYPE_TEXT[EVENT_CODE_BATTLE_SPECTATOR] = "EVENT_CODE_BATTLE_SPECTATOR";
 }
 
 std::string ParseEventType(EventType eventType)
@@ -525,20 +517,27 @@ std::string ParseEventType(EventType eventType)
 
     if (eventType < EVENT_STD_MAX)
     {
+        if(eventType >= EVENT_INTERFACE_KEY && eventType <= EVENT_INTERFACE_KEY_END)
+        {
+            return "EVENT_INTERFACE_KEY"+boost::lexical_cast<std::string>(eventType-EVENT_INTERFACE_KEY);
+        }
+
+        if(eventType >= EVENT_OBJECT_SHORTCUT && eventType <= EVENT_OBJECT_SHORTCUT_MAX)
+        {
+            return "EVENT_OBJECT_SHORTCUT"+boost::lexical_cast<std::string>(eventType-EVENT_OBJECT_SHORTCUT);
+        }
+
         const char* stdEvent = EVENT_TYPE_TEXT[eventType];
         if (stdEvent[0] == 0)
             return Other("STD_UNDEFINED");
 
         return stdEvent;
     }
-    
-    if(eventType >= EVENT_INTERFACE_KEY && eventType <= EVENT_INTERFACE_KEY_END)
-    {
-        return Other("EVENT_INTERFACE_KEY - EVENT_INTERFACE_KEY_END ");
-    }
 
     if (eventType >= EVENT_USER)
+    {
         return Other("USER_EVENT");
+    }
 
     return Other("UNDEFINED");
 }
@@ -546,48 +545,74 @@ std::string ParseEventType(EventType eventType)
 
 
 CEventQueue::CEventQueue()
-{
-    Flush();
-}
+    : m_mutex{},
+      m_fifo(),
+      m_head{0},
+      m_tail{0},
+      m_total{0}
+{}
 
 CEventQueue::~CEventQueue()
-{
-}
+{}
 
-void CEventQueue::Flush()
+bool CEventQueue::IsEmpty()
 {
-    m_head = 0;
-    m_tail = 0;
-    m_total = 0;
+    return m_head == m_tail;
 }
 
 /** If the maximum size of queue has been reached, returns \c false.
     Else, adds the event to the queue and returns \c true. */
-bool CEventQueue::AddEvent(const Event &event)
+bool CEventQueue::AddEvent(Event&& event)
 {
-    if ( m_total >= MAX_EVENT_QUEUE )
+    bool result{};
+
+    SDL_LockMutex(*m_mutex);
+
+    if (m_total >= MAX_EVENT_QUEUE)
     {
         GetLogger()->Warn("Event queue flood!\n");
-        return false;
+
+        result = false;
+    }
+    else
+    {
+        m_fifo[m_head++] = std::move(event);
+
+        if (m_head >= MAX_EVENT_QUEUE)
+            m_head = 0;
+
+        m_total++;
+
+        result = true;
     }
 
-    m_fifo[m_head++] = event;
-    if ( m_head >= MAX_EVENT_QUEUE )  m_head = 0;
-    m_total ++;
+    SDL_UnlockMutex(*m_mutex);
 
-    return true;
+    return result;
 }
 
-/** If the queue is empty, returns \c false.
-    Else, gets the event from the front, puts it into \a event and returns \c true. */
-bool CEventQueue::GetEvent(Event &event)
+Event CEventQueue::GetEvent()
 {
-    if ( m_head == m_tail )  return false;
+    Event event;
 
-    event = m_fifo[m_tail++];
-    if ( m_tail >= MAX_EVENT_QUEUE )  m_tail = 0;
-    m_total --;
+    SDL_LockMutex(*m_mutex);
 
-    return true;
+    if (m_head == m_tail)
+    {
+        event.type = EVENT_NULL;
+    }
+    else
+    {
+        event = std::move(m_fifo[m_tail++]);
+
+        if (m_tail >= MAX_EVENT_QUEUE)
+            m_tail = 0;
+
+        m_total--;
+
+    }
+
+    SDL_UnlockMutex(*m_mutex);
+
+    return event;
 }
-
