@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,15 @@
 
 #pragma once
 
-
 #include "common/key.h"
+#include "common/make_unique.h"
+
+#include "common/thread/sdl_mutex_wrapper.h"
 
 #include "math/point.h"
 #include "math/vector.h"
 
+#include <memory>
 
 /**
   \enum EventType
@@ -126,16 +129,16 @@ enum EventType
     EVENT_EDIT8             = 78,
     EVENT_EDIT9             = 79,
 
-    EVENT_WINDOW0           = 80,   // the bottom panel
-    EVENT_WINDOW1           = 81,   // map
-    EVENT_WINDOW2           = 82,   // CDisplayText
-    EVENT_WINDOW3           = 83,   // CStudio
-    EVENT_WINDOW4           = 84,   // DisplayInfo
-    EVENT_WINDOW5           = 85,   // setup
-    EVENT_WINDOW6           = 86,
-    EVENT_WINDOW7           = 87,
-    EVENT_WINDOW8           = 88,
-    EVENT_WINDOW9           = 89,   // dialogue
+    EVENT_WINDOW0           = 80,   //!< object interface (CObjectInterface + CAuto classes)
+    EVENT_WINDOW1           = 81,   //!< CMainMap
+    EVENT_WINDOW2           = 82,   //!< CDisplayText
+    EVENT_WINDOW3           = 83,   //!< CStudio
+    EVENT_WINDOW4           = 84,   //!< CDisplayInfo
+    EVENT_WINDOW5           = 85,   //!< all menu windows
+    EVENT_WINDOW6           = 86,   //!< code battle interface
+    EVENT_WINDOW7           = 87,   //!< (unused)
+    EVENT_WINDOW8           = 88,   //!< (unused)
+    EVENT_WINDOW9           = 89,   //!< CMainDialog and CStudio file selector
 
     EVENT_LABEL0            = 90,
     EVENT_LABEL1            = 91,
@@ -169,6 +172,8 @@ enum EventType
     EVENT_LIST8             = 118,
     EVENT_LIST9             = 119,
 
+    EVENT_LOADING           = 120,
+
     EVENT_TOOLTIP           = 200,
 
     EVENT_DIALOG_OK         = 300,
@@ -186,6 +191,7 @@ enum EventType
     EVENT_INTERFACE_DEFI    = 401,
     EVENT_INTERFACE_MISSION = 402,
     EVENT_INTERFACE_FREE    = 403,
+    EVENT_INTERFACE_CODE_BATTLES = 404,
     EVENT_INTERFACE_NAME    = 405,
     EVENT_INTERFACE_SETUP   = 406,
     EVENT_INTERFACE_QUIT    = 407,
@@ -212,30 +218,21 @@ enum EventType
     EVENT_INTERFACE_FULL    = 442,
     EVENT_INTERFACE_APPLY   = 443,
 
-    EVENT_INTERFACE_TOTO    = 450,
-    EVENT_INTERFACE_SHADOW  = 451,
+    EVENT_INTERFACE_SHADOW_SPOTS = 451,
     EVENT_INTERFACE_DIRTY   = 452,
-    EVENT_INTERFACE_LENS    = 453,
-    EVENT_INTERFACE_SKY     = 454,
-    EVENT_INTERFACE_PLANET  = 456,
     EVENT_INTERFACE_LIGHT   = 457,
     EVENT_INTERFACE_PARTI   = 458,
     EVENT_INTERFACE_CLIP    = 459,
-    EVENT_INTERFACE_DETAIL  = 460,
-    EVENT_INTERFACE_TEXTURE = 461,
     EVENT_INTERFACE_RAIN    = 462,
     EVENT_INTERFACE_GLINT   = 463,
     EVENT_INTERFACE_TOOLTIP = 464,
     EVENT_INTERFACE_MOVIES  = 465,
-    EVENT_INTERFACE_NICERST = 466,
     EVENT_INTERFACE_SCROLL  = 467,
     EVENT_INTERFACE_INVERTX = 468,
     EVENT_INTERFACE_INVERTY = 469,
     EVENT_INTERFACE_EFFECT  = 470,
-    EVENT_INTERFACE_GROUND  = 472,
-    EVENT_INTERFACE_GADGET  = 473,
+    EVENT_INTERFACE_MOUSE   = 471,
     EVENT_INTERFACE_FOG     = 474,
-    EVENT_INTERFACE_HIMSELF = 475,
     EVENT_INTERFACE_EDITMODE= 476,
     EVENT_INTERFACE_EDITVALUE= 477,
     EVENT_INTERFACE_SOLUCE4 = 478,
@@ -243,6 +240,13 @@ enum EventType
     EVENT_INTERFACE_AUTOSAVE_ENABLE = 780,
     EVENT_INTERFACE_AUTOSAVE_INTERVAL = 781,
     EVENT_INTERFACE_AUTOSAVE_SLOTS = 782,
+    EVENT_INTERFACE_TEXTURE_FILTER = 783,
+    EVENT_INTERFACE_TEXTURE_MIPMAP = 784,
+    EVENT_INTERFACE_TEXTURE_ANISOTROPY = 785,
+    EVENT_INTERFACE_MSAA = 786,
+    EVENT_INTERFACE_SHADOW_MAPPING = 787,
+    EVENT_INTERFACE_SHADOW_MAPPING_QUALITY = 788,
+    EVENT_INTERFACE_SHADOW_MAPPING_BUFFER = 789,
 
     EVENT_INTERFACE_KINFO1  = 500,
     EVENT_INTERFACE_KINFO2  = 501,
@@ -257,7 +261,7 @@ enum EventType
     EVENT_INTERFACE_MIN     = 540,
     EVENT_INTERFACE_NORM    = 541,
     EVENT_INTERFACE_MAX     = 542,
-    
+
     EVENT_INTERFACE_VOLSOUND= 550,
     EVENT_INTERFACE_VOLMUSIC= 551,
     EVENT_INTERFACE_SILENT  = 552,
@@ -274,7 +278,6 @@ enum EventType
     EVENT_INTERFACE_NEDIT   = 580,
     EVENT_INTERFACE_NLIST   = 581,
     EVENT_INTERFACE_NOK     = 582,
-    EVENT_INTERFACE_NCANCEL = 583,
     EVENT_INTERFACE_NDELETE = 584,
     EVENT_INTERFACE_NLABEL  = 585,
 
@@ -467,7 +470,6 @@ enum EventType
     EVENT_OBJECT_GENERGY    = 1360,
     EVENT_OBJECT_GSHIELD    = 1361,
     EVENT_OBJECT_GRANGE     = 1362,
-    EVENT_OBJECT_COMPASS    = 1363,
     EVENT_OBJECT_MAP        = 1364,
     EVENT_OBJECT_MAPZOOM    = 1365,
     EVENT_OBJECT_GPROGRESS  = 1366,
@@ -488,28 +490,12 @@ enum EventType
     EVENT_OBJECT_CAMERAright= 1404,
     EVENT_OBJECT_CAMERAnear = 1405,
     EVENT_OBJECT_CAMERAaway = 1406,
-    EVENT_OBJECT_SHORTCUT00 = 1500,
-    EVENT_OBJECT_SHORTCUT01 = 1501,
-    EVENT_OBJECT_SHORTCUT02 = 1502,
-    EVENT_OBJECT_SHORTCUT03 = 1503,
-    EVENT_OBJECT_SHORTCUT04 = 1504,
-    EVENT_OBJECT_SHORTCUT05 = 1505,
-    EVENT_OBJECT_SHORTCUT06 = 1506,
-    EVENT_OBJECT_SHORTCUT07 = 1507,
-    EVENT_OBJECT_SHORTCUT08 = 1508,
-    EVENT_OBJECT_SHORTCUT09 = 1509,
-    EVENT_OBJECT_SHORTCUT10 = 1510,
-    EVENT_OBJECT_SHORTCUT11 = 1511,
-    EVENT_OBJECT_SHORTCUT12 = 1512,
-    EVENT_OBJECT_SHORTCUT13 = 1513,
-    EVENT_OBJECT_SHORTCUT14 = 1514,
-    EVENT_OBJECT_SHORTCUT15 = 1515,
-    EVENT_OBJECT_SHORTCUT16 = 1516,
-    EVENT_OBJECT_SHORTCUT17 = 1517,
-    EVENT_OBJECT_SHORTCUT18 = 1518,
-    EVENT_OBJECT_SHORTCUT19 = 1519,
+    EVENT_OBJECT_SHORTCUT_MODE = 1500,
+    EVENT_OBJECT_SHORTCUT   = 1501,
+    EVENT_OBJECT_SHORTCUT_MAX = 1549,
     EVENT_OBJECT_MOVIELOCK  = 1550,
     EVENT_OBJECT_EDITLOCK   = 1551,
+    EVENT_OBJECT_SAVING     = 1552,
     EVENT_OBJECT_LIMIT      = 1560,
 
     EVENT_OBJECT_PEN0       = 1570,
@@ -544,6 +530,11 @@ enum EventType
     EVENT_STUDIO_REALTIME   = 2052,
     EVENT_STUDIO_STEP       = 2053,
 
+    EVENT_WRITE_SCENE_FINISHED = 2100, //!< indicates end of writing scene (writing screenshot image)
+
+    EVENT_CODE_BATTLE_START = 2200, //!< button that starts the code battle
+    EVENT_CODE_BATTLE_SPECTATOR = 2201, //!< button that controls the code battle spectator camera
+
     //! Maximum value of standard events
     EVENT_STD_MAX,
 
@@ -551,22 +542,38 @@ enum EventType
     EVENT_FORCE_LONG        = 0x7fffffff
 };
 
+/**
+ * \struct EventData
+ * \brief Base class for additional event data
+ */
+struct EventData
+{
+    virtual ~EventData()
+    {}
+
+    virtual std::unique_ptr<EventData> Clone() const = 0;
+};
 
 /**
  * \struct KeyEventData
  * \brief Additional data for keyboard event
  */
-struct KeyEventData
+struct KeyEventData : public EventData
 {
+    std::unique_ptr<EventData> Clone() const override
+    {
+        return MakeUnique<KeyEventData>(*this);
+    }
+
     //! If true, the key is a virtual code generated by certain key modifiers or joystick buttons
-    bool virt;
+    bool virt = false;
     //! Key symbol: KEY(...) macro value or virtual key VIRTUAL_... (from common/key.h)
-    unsigned int key;
+    unsigned int key = 0;
     //! Unicode character
     //! NOTE: applicable only to EVENT_KEY_DOWN events!
-    unsigned int unicode;
+    unsigned int unicode = 0;
     //! Input binding slot for this key
-    InputSlot slot;
+    InputSlot slot = INPUT_SLOT_LEFT;
 };
 
 /**
@@ -588,10 +595,15 @@ enum MouseButton
  * \struct MouseButtonEventData
  * \brief Additional data mouse button event
  */
-struct MouseButtonEventData
+struct MouseButtonEventData : public EventData
 {
+    std::unique_ptr<EventData> Clone() const override
+    {
+        return MakeUnique<MouseButtonEventData>(*this);
+    }
+
     //! The mouse button
-    MouseButton button;
+    MouseButton button = MOUSE_BUTTON_LEFT;
 };
 
 /**
@@ -608,32 +620,47 @@ enum WheelDirection
  * \struct MouseWheelEventData
  * \brief Additional data for mouse wheel event.
  */
-struct MouseWheelEventData
+struct MouseWheelEventData : public EventData
 {
+    std::unique_ptr<EventData> Clone() const override
+    {
+        return MakeUnique<MouseWheelEventData>(*this);
+    }
+
     //! Wheel direction
-    WheelDirection dir;
+    WheelDirection dir = WHEEL_UP;
 };
 
 /**
  * \struct JoyAxisEventData
  * \brief Additional data for joystick axis event
  */
-struct JoyAxisEventData
+struct JoyAxisEventData : public EventData
 {
+    std::unique_ptr<EventData> Clone() const override
+    {
+        return MakeUnique<JoyAxisEventData>(*this);
+    }
+
     //! The joystick axis index
-    unsigned char axis;
+    unsigned char axis = 0;
     //! The axis value (range: -32768 to 32767)
-    int value;
+    int value = 0;
 };
 
 /**
  * \struct JoyButtonEventData
  * \brief Additional data for joystick button event
  */
-struct JoyButtonEventData
+struct JoyButtonEventData : public EventData
 {
+    std::unique_ptr<EventData> Clone() const override
+    {
+        return MakeUnique<JoyButtonEventData>(*this);
+    }
+
     //! The joystick button index
-    unsigned char button;
+    unsigned char button = 0;
 };
 
 /**
@@ -655,14 +682,18 @@ enum ActiveEventFlags
  * \struct ActiveEventData
  * \brief Additional data for active event
  */
-struct ActiveEventData
+struct ActiveEventData : public EventData
 {
-    //! Flags (bitmask of enum values ActiveEventFlags)
-    unsigned char flags;
-    //! True if the focus was gained; false otherwise
-    bool gain;
-};
+    std::unique_ptr<EventData> Clone() const override
+    {
+        return MakeUnique<ActiveEventData>(*this);
+    }
 
+    //! Flags (bitmask of enum values ActiveEventFlags)
+    unsigned char flags = 0;
+    //! True if the focus was gained; false otherwise
+    bool gain = false;
+};
 
 /**
  * \struct Event
@@ -677,6 +708,80 @@ struct ActiveEventData
  **/
 struct Event
 {
+    explicit Event(EventType type = EVENT_NULL)
+     : type(type),
+       rTime(0.0f),
+       kmodState(0),
+       mouseButtonsState(0),
+       customParam(0)
+    {}
+
+    Event(const Event&) = delete;
+    Event& operator=(const Event&) = delete;
+
+    // Workaround for MSVC2013
+    Event(Event&& other)
+        : type(std::move(other.type)),
+          rTime(std::move(other.rTime)),
+          motionInput(std::move(other.motionInput)),
+          cameraInput(std::move(other.cameraInput)),
+          kmodState(std::move(other.kmodState)),
+          mousePos(std::move(other.mousePos)),
+          mouseButtonsState(std::move(other.mouseButtonsState)),
+          customParam(std::move(other.customParam)),
+          data(std::move(other.data))
+    {}
+
+    Event& operator=(Event&& other)
+    {
+        type = std::move(other.type);
+        rTime = std::move(other.rTime);
+        motionInput = std::move(other.motionInput);
+        cameraInput = std::move(other.cameraInput);
+        kmodState = std::move(other.kmodState);
+        mousePos = std::move(other.mousePos);
+        mouseButtonsState = std::move(other.mouseButtonsState);
+        customParam = std::move(other.customParam);
+        data = std::move(other.data);
+        return *this;
+    }
+
+    //! Convenience function for getting appropriate EventData subclass
+    template<typename EventDataSubclass>
+    EventDataSubclass* GetData()
+    {
+        return static_cast<EventDataSubclass*>(data.get());
+    }
+
+    //! Convenience function for getting appropriate EventData subclass
+    template<typename EventDataSubclass>
+    const EventDataSubclass* GetData() const
+    {
+        return static_cast<EventDataSubclass*>(data.get());
+    }
+
+    //! Returns a clone of this event
+    Event Clone() const
+    {
+        Event clone;
+
+        clone.type = type;
+        clone.rTime = rTime;
+        clone.motionInput = motionInput;
+        clone.cameraInput = cameraInput;
+        clone.kmodState = kmodState;
+        clone.mousePos = mousePos;
+        clone.mouseButtonsState = mouseButtonsState;
+        clone.customParam = customParam;
+
+        if (data != nullptr)
+        {
+            clone.data = data->Clone();
+        }
+
+        return clone;
+    }
+
     //! Type of event
     EventType type;
 
@@ -689,6 +794,10 @@ struct Event
     //! Scope: all system events
     Math::Vector motionInput;
 
+    //! Motion vector set by numeric keyboard (managed by CInput)
+    //! Scope: all system events
+    Math::Vector cameraInput;
+
     //! Current state of keyboard modifier keys: bitmask made of KEY_MOD(...) macro values (from common/key.h)
     //! Scope: all system events
     unsigned int kmodState;
@@ -705,30 +814,8 @@ struct Event
     //! Scope: some interface events
     long         customParam;
 
-    //! Union with additional data, applicable only to some events
-    union
-    {
-        //! Additional data for EVENT_KEY_DOWN and EVENT_KEY_UP
-        KeyEventData key;
-        //! Additional data for EVENT_MOUSE_BUTTON_DOWN and EVENT_MOUSE_BUTTON_UP
-        MouseButtonEventData mouseButton;
-        //! Additional data for EVENT_MOUSE_WHEEL
-        MouseWheelEventData mouseWheel;
-        //! Additional data for EVENT_JOY
-        JoyAxisEventData joyAxis;
-        //! Additional data for EVENT_JOY_AXIS
-        JoyButtonEventData joyButton;
-        //! Additional data for EVENT_ACTIVE
-        ActiveEventData active;
-    };
-
-    explicit Event(EventType _type = EVENT_NULL)
-     : type(_type)
-     , rTime(0.0f)
-     , kmodState(0)
-     , mouseButtonsState(0)
-     , customParam(0)
-    {}
+    //! Additional data for some events
+    std::unique_ptr<EventData> data;
 };
 
 
@@ -747,6 +834,8 @@ std::string ParseEventType(EventType eventType);
  *
  * Provides an interface to a global FIFO queue with events (both system- and user-generated).
  * The queue has a fixed maximum size but it should not be a problem.
+ *
+ * This class is thread-safe
  */
 class CEventQueue
 {
@@ -760,17 +849,17 @@ public:
     //! Object's destructor
     ~CEventQueue();
 
-    //! Empties the FIFO of events
-    void    Flush();
+    //! Checks if queue is empty
+    bool IsEmpty();
     //! Adds an event to the queue
-    bool    AddEvent(const Event &event);
-    //! Removes and returns an event from queue front
-    bool    GetEvent(Event &event);
+    bool AddEvent(Event&& event);
+    //! Removes and returns an event from queue front; if queue is empty, returns event of type EVENT_NULL
+    Event GetEvent();
 
 protected:
+    CSDLMutexWrapper m_mutex;
     Event        m_fifo[MAX_EVENT_QUEUE];
     int          m_head;
     int          m_tail;
     int          m_total;
 };
-
