@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,11 @@
 
 #pragma once
 
-
 #include "graphics/core/color.h"
 
 #include "math/intpoint.h"
 
-#include <stddef.h>
+#include <memory>
 #include <string>
 
 
@@ -42,9 +41,7 @@ struct SDL_Surface;
 struct ImageData
 {
     //! SDL surface with image data
-    SDL_Surface* surface;
-
-    ImageData() { surface = NULL; }
+    SDL_Surface* surface = nullptr;
 };
 
 /**
@@ -56,20 +53,17 @@ struct ImageData
   */
 class CImage
 {
-private:
-    //! Blocked!
-    CImage(const CImage &other) {}
-    //! Blocked!
-    void operator=(const CImage &other) {}
-
 public:
-    //! Constructs empty image (with NULL data)
+    //! Constructs empty image (with nullptr data)
     CImage();
     //! Constructs a RGBA image of given size
     CImage(Math::IntPoint size);
     //! Destroys image, calling Free()
     virtual ~CImage();
 
+    CImage(const CImage &other) = delete;
+    void operator=(const CImage &other) = delete;
+
     //! Frees the allocated image data
     void Free();
 
@@ -113,9 +107,9 @@ public:
     std::string GetError();
 
     //! Flips the image vertically
-    void flipVertically();
-    
-    //! sets/replaces the pixels from the surface 
+    void FlipVertically();
+
+    //! sets/replaces the pixels from the surface
     void SetDataPixels(void *pixels);
 
 private:
@@ -125,6 +119,6 @@ private:
     //! Last encountered error
     std::string m_error;
     //! Image data
-    ImageData* m_data;
+    std::unique_ptr<ImageData> m_data;
 };
 
