@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,44 +18,43 @@
  */
 
 #include "common/resources/inputstream.h"
+
 #include "common/resources/inputstreambuffer.h"
 
 
-CInputStream::CInputStream() : std::istream(new CInputStreamBuffer())
+CInputStream::CInputStream()
+    : CInputStreamBufferContainer(),
+      std::istream(&m_buffer)
 {
 }
 
-CInputStream::CInputStream(const std::string& filename) : std::istream(new CInputStreamBuffer())
+CInputStream::CInputStream(const std::string& filename)
+    : CInputStreamBufferContainer(),
+      std::istream(&m_buffer)
 {
     open(filename);
 }
 
-
 CInputStream::~CInputStream()
 {
-    delete rdbuf();
 }
 
-
 void CInputStream::open(const std::string& filename)
 {
-    static_cast<CInputStreamBuffer *>(rdbuf())->open(filename);
+    m_buffer.open(filename);
 }
 
-
 void CInputStream::close()
 {
-    static_cast<CInputStreamBuffer *>(rdbuf())->close();
+    m_buffer.close();
 }
 
-
 bool CInputStream::is_open()
 {
-    return static_cast<CInputStreamBuffer *>(rdbuf())->is_open();
+    return m_buffer.is_open();
 }
 
-
-size_t CInputStream::size()
+std::size_t CInputStream::size()
 {
-    return static_cast<CInputStreamBuffer *>(rdbuf())->size();
+    return m_buffer.size();
 }
