@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,11 +19,21 @@
 
 #pragma once
 
+#include "common/resources/inputstreambuffer.h"
+
+#include <cstddef>
 #include <istream>
 #include <string>
 
 
-class CInputStream : public std::istream
+class CInputStreamBufferContainer
+{
+protected:
+    CInputStreamBuffer m_buffer;
+};
+
+
+class CInputStream : public CInputStreamBufferContainer, public std::istream
 {
 public:
     CInputStream();
@@ -33,5 +43,5 @@ public:
     void open(const std::string& filename);
     void close();
     bool is_open();
-    size_t size();
+    std::size_t size();
 };
