@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,38 +18,38 @@
  */
 
 #include "common/resources/outputstream.h"
+
 #include "common/resources/outputstreambuffer.h"
 
 
-COutputStream::COutputStream() : std::ostream(new COutputStreamBuffer())
+COutputStream::COutputStream()
+    : COutputStreamBufferContainer(),
+      std::ostream(&m_buffer)
 {
 }
 
-COutputStream::COutputStream(const std::string& filename) : std::ostream(new COutputStreamBuffer())
+COutputStream::COutputStream(const std::string& filename)
+    : COutputStreamBufferContainer(),
+      std::ostream(&m_buffer)
 {
     open(filename);
 }
 
-
 COutputStream::~COutputStream()
 {
-    delete rdbuf();
 }
 
-
 void COutputStream::open(const std::string& filename)
 {
-    static_cast<COutputStreamBuffer *>(rdbuf())->open(filename);
+    m_buffer.open(filename);
 }
 
-
 void COutputStream::close()
 {
-    static_cast<COutputStreamBuffer *>(rdbuf())->close();
+    m_buffer.close();
 }
 
-
 bool COutputStream::is_open()
 {
-    return static_cast<COutputStreamBuffer *>(rdbuf())->is_open();
+    return m_buffer.is_open();
 }
