@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,11 +19,19 @@
 
 #pragma once
 
+#include "common/resources/outputstreambuffer.h"
+
 #include <ostream>
 #include <string>
 
 
-class COutputStream : public std::ostream
+class COutputStreamBufferContainer
+{
+protected:
+    COutputStreamBuffer m_buffer;
+};
+
+class COutputStream : public COutputStreamBufferContainer, public std::ostream
 {
 public:
     COutputStream();
