@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,28 +19,30 @@
 
 #pragma once
 
+#include <cstddef>
+#include <memory>
 #include <streambuf>
 #include <string>
+
 #include <physfs.h>
 
 class COutputStreamBuffer : public std::streambuf
 {
 public:
-    COutputStreamBuffer(size_t buffer_size = 512);
+    COutputStreamBuffer(std::size_t bufferSize = 512);
     virtual ~COutputStreamBuffer();
+
+    COutputStreamBuffer(const COutputStreamBuffer &) = delete;
+    COutputStreamBuffer &operator= (const COutputStreamBuffer &) = delete;
+
     void open(const std::string &filename);
     void close();
     bool is_open();
 
 private:
-    int_type overflow(int_type ch);
-    int sync();
+    int_type overflow(int_type ch) override;
+    int sync() override;
 
-    // copy ctor and assignment not implemented;
-    // copying not allowed
-    COutputStreamBuffer(const COutputStreamBuffer &);
-    COutputStreamBuffer &operator= (const COutputStreamBuffer &);
     PHYSFS_File *m_file;
-    char *m_buffer;
-    size_t m_buffer_size;
+    std::unique_ptr<char[]> m_buffer;
 };
