@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,19 +20,18 @@
 
 #include "common/restext.h"
 
-#include "app/input.h"
-
 #include "common/config.h"
 
-#include "common/global.h"
+#include "CBot/resource.h"
+
+#include "app/input.h"
+
+#include "common/error.h"
 #include "common/event.h"
 #include "common/logger.h"
 #include "common/stringutils.h"
 
-#include "CBot/resource.h"
-
-#include "object/object.h"
-#include "object/robotmain.h"
+#include "object/object_type.h"
 
 #include <libintl.h>
 #include <SDL_keyboard.h>
@@ -66,31 +65,28 @@ void InitializeRestext()
     stringsText[RT_IO_NEW]           = TR("New ...");
     stringsText[RT_KEY_OR]           = TR(" or ");
 
-    stringsText[RT_TITLE_BASE]       = TR("Colobot: Gold Edition");
-    stringsText[RT_TITLE_INIT]       = TR("Colobot: Gold Edition");
+    stringsText[RT_TITLE_APPNAME]    = TR("Colobot: Gold Edition");
     stringsText[RT_TITLE_TRAINER]    = TR("Programming exercises");
     stringsText[RT_TITLE_DEFI]       = TR("Challenges");
     stringsText[RT_TITLE_MISSION]    = TR("Missions");
     stringsText[RT_TITLE_FREE]       = TR("Free game");
     stringsText[RT_TITLE_USER]       = TR("User levels");
+    stringsText[RT_TITLE_CODE_BATTLES]=TR("Code battles");
     stringsText[RT_TITLE_SETUP]      = TR("Options");
     stringsText[RT_TITLE_NAME]       = TR("Player's name");
     stringsText[RT_TITLE_PERSO]      = TR("Customize your appearance");
     stringsText[RT_TITLE_WRITE]      = TR("Save the current mission");
     stringsText[RT_TITLE_READ]       = TR("Load a saved mission");
 
-    stringsText[RT_PLAY_CHAPt]       = TR(" Chapters:");
-    stringsText[RT_PLAY_CHAPd]       = TR(" Chapters:");
-    stringsText[RT_PLAY_CHAPm]       = TR(" Planets:");
-    stringsText[RT_PLAY_CHAPf]       = TR(" Planets:");
-    stringsText[RT_PLAY_CHAPu]       = TR(" User levels:");
-    stringsText[RT_PLAY_CHAPte]      = TR(" Chapters:");
-    stringsText[RT_PLAY_LISTt]       = TR(" Exercises in the chapter:");
-    stringsText[RT_PLAY_LISTd]       = TR(" Challenges in the chapter:");
-    stringsText[RT_PLAY_LISTm]       = TR(" Missions on this planet:");
-    stringsText[RT_PLAY_LISTf]       = TR(" Free game on this planet:");
-    stringsText[RT_PLAY_LISTu]       = TR(" Missions on this level:");
-    stringsText[RT_PLAY_RESUME]      = TR(" Summary:");
+    stringsText[RT_PLAY_CHAP_CHAPTERS]   = TR(" Chapters:");
+    stringsText[RT_PLAY_CHAP_PLANETS]    = TR(" Planets:");
+    stringsText[RT_PLAY_CHAP_USERLVL]    = TR(" Custom levels:");
+    stringsText[RT_PLAY_LIST_LEVELS]     = TR(" Levels in this chapter:");
+    stringsText[RT_PLAY_LIST_EXERCISES]  = TR(" Exercises in the chapter:");
+    stringsText[RT_PLAY_LIST_CHALLENGES] = TR(" Challenges in the chapter:");
+    stringsText[RT_PLAY_LIST_MISSIONS]   = TR(" Missions on this planet:");
+    stringsText[RT_PLAY_LIST_FREEGAME]   = TR(" Free game on this planet:");
+    stringsText[RT_PLAY_RESUME]          = TR(" Summary:");
 
     stringsText[RT_SETUP_DEVICE]     = TR(" Drivers:");
     stringsText[RT_SETUP_MODE]       = TR(" Resolution:");
@@ -103,18 +99,16 @@ void InitializeRestext()
     stringsText[RT_PERSO_COMBI]      = TR("Suit color:");
     stringsText[RT_PERSO_BAND]       = TR("Strip color:");
 
-    stringsText[RT_DIALOG_QUIT]      = TR("Do you want to quit Colobot: Gold Edition?");
-    stringsText[RT_DIALOG_TITLE]     = TR("Colobot: Gold Edition");
-    stringsText[RT_DIALOG_YESQUIT]   = TR("Quit\\Quit Colobot: Gold Edition");
-    stringsText[RT_DIALOG_ABORT]     = TR("Quit the mission?");
-    stringsText[RT_DIALOG_YES]       = TR("Abort\\Abort the current mission");
-    stringsText[RT_DIALOG_NO]        = TR("Continue\\Continue the current mission");
-    stringsText[RT_DIALOG_NOQUIT]    = TR("Continue\\Continue the game");
+    stringsText[RT_DIALOG_ABORT]     = TR("Abort\\Abort the current mission");
+    stringsText[RT_DIALOG_CONTINUE]  = TR("Continue\\Continue the current mission");
     stringsText[RT_DIALOG_DELOBJ]    = TR("Do you really want to destroy the selected building?");
     stringsText[RT_DIALOG_DELGAME]   = TR("Do you want to delete %s's saved games? ");
-    stringsText[RT_DIALOG_YESDEL]    = TR("Delete");
-    stringsText[RT_DIALOG_NODEL]     = TR("Cancel");
+    stringsText[RT_DIALOG_YES]       = TR("Yes");
+    stringsText[RT_DIALOG_NO]        = TR("No");
     stringsText[RT_DIALOG_LOADING]   = TR("LOADING");
+    stringsText[RT_DIALOG_OK]        = TR("OK");
+    stringsText[RT_DIALOG_NOUSRLVL_TITLE] = TR("No userlevels installed!");
+    stringsText[RT_DIALOG_NOUSRLVL_TEXT]  = TR("This menu is for userlevels from mods, but you didn't install any");
 
     stringsText[RT_STUDIO_LISTTT]    = TR("Keyword help(\\key cbot;)");
     stringsText[RT_STUDIO_COMPOK]    = TR("Compilation ok (0 errors)");
@@ -148,6 +142,20 @@ void InitializeRestext()
 
     stringsText[RT_INTERFACE_REC]    = TR("Recorder");
 
+    stringsText[RT_LOADING_INIT]           = TR("Starting...");
+    stringsText[RT_LOADING_PROCESSING]     = TR("Processing level file");
+    stringsText[RT_LOADING_LEVEL_SETTINGS] = TR("Loading basic level settings");
+    stringsText[RT_LOADING_MUSIC]          = TR("Loading music");
+    stringsText[RT_LOADING_TERRAIN]        = TR("Loading terrain");
+    stringsText[RT_LOADING_OBJECTS]        = TR("Loading objects");
+    stringsText[RT_LOADING_OBJECTS_SAVED]  = TR("Restoring saved objects");
+    stringsText[RT_LOADING_CBOT_SAVE]      = TR("Restoring CBot execution state");
+    stringsText[RT_LOADING_FINISHED]       = TR("Loading finished!");
+    stringsText[RT_LOADING_TERRAIN_RELIEF] = TR("Terrain relief");
+    stringsText[RT_LOADING_TERRAIN_RES]    = TR("Resources");
+    stringsText[RT_LOADING_TERRAIN_TEX]    = TR("Textures");
+    stringsText[RT_LOADING_TERRAIN_GEN]    = TR("Generating");
+
 
 
     stringsEvent[EVENT_BUTTON_OK]           = TR("OK");
@@ -162,7 +170,8 @@ void InitializeRestext()
     stringsEvent[EVENT_INTERFACE_DEFI]      = TR("Challenges\\Programming challenges");
     stringsEvent[EVENT_INTERFACE_MISSION]   = TR("Missions\\Select mission");
     stringsEvent[EVENT_INTERFACE_FREE]      = TR("Free game\\Free game without a specific goal");
-    stringsEvent[EVENT_INTERFACE_USER]      = TR("User\\User levels");
+    stringsEvent[EVENT_INTERFACE_CODE_BATTLES] = TR("Code battles\\Program your robot to be the best of them all!");
+    stringsEvent[EVENT_INTERFACE_USER]      = TR("Custom levels\\Levels from mods created by the users");
     stringsEvent[EVENT_INTERFACE_NAME]      = TR("Change player\\Change player");
     stringsEvent[EVENT_INTERFACE_SETUP]     = TR("Options\\Preferences");
     stringsEvent[EVENT_INTERFACE_AGAIN]     = TR("Restart\\Restart the mission from the beginning");
@@ -182,37 +191,35 @@ void InitializeRestext()
     stringsEvent[EVENT_INTERFACE_FULL]      = TR("Full screen\\Full screen or window mode");
     stringsEvent[EVENT_INTERFACE_APPLY]     = TR("Apply changes\\Activates the changed settings");
 
-    stringsEvent[EVENT_INTERFACE_TOTO]      = TR("Robbie\\Your assistant");
-    stringsEvent[EVENT_INTERFACE_SHADOW]    = TR("Shadows\\Shadows on the ground");
-    stringsEvent[EVENT_INTERFACE_GROUND]    = TR("Marks on the ground\\Marks on the ground");
     stringsEvent[EVENT_INTERFACE_DIRTY]     = TR("Dust\\Dust and dirt on bots and buildings");
     stringsEvent[EVENT_INTERFACE_FOG]       = TR("Fog\\Fog");
-    stringsEvent[EVENT_INTERFACE_LENS]      = TR("Sunbeams\\Sunbeams in the sky");
-    stringsEvent[EVENT_INTERFACE_SKY]       = TR("Sky\\Clouds and nebulae");
-    stringsEvent[EVENT_INTERFACE_PLANET]    = TR("Planets and stars\\Astronomical objects in the sky");
     stringsEvent[EVENT_INTERFACE_LIGHT]     = TR("Dynamic lighting\\Mobile light sources");
     stringsEvent[EVENT_INTERFACE_PARTI]     = TR("Number of particles\\Explosions, dust, reflections, etc.");
     stringsEvent[EVENT_INTERFACE_CLIP]      = TR("Render distance\\Maximum visibility");
-    stringsEvent[EVENT_INTERFACE_DETAIL]    = TR("Details\\Visual quality of 3D objects");
-    stringsEvent[EVENT_INTERFACE_TEXTURE]   = TR("Textures\\Quality of textures ");
-    stringsEvent[EVENT_INTERFACE_GADGET]    = TR("Num of decorative objects\\Number of purely ornamental objects");
     stringsEvent[EVENT_INTERFACE_RAIN]      = TR("Particles in the interface\\Steam clouds and sparks in the interface");
     stringsEvent[EVENT_INTERFACE_GLINT]     = TR("Reflections on the buttons \\Shiny buttons");
     stringsEvent[EVENT_INTERFACE_TOOLTIP]   = TR("Help balloons\\Explain the function of the buttons");
     stringsEvent[EVENT_INTERFACE_MOVIES]    = TR("Film sequences\\Films before and after the missions");
-    stringsEvent[EVENT_INTERFACE_NICERST]   = TR("Exit film\\Film at the exit of exercises");
-    stringsEvent[EVENT_INTERFACE_HIMSELF]   = TR("Friendly fire\\Your shooting can damage your own objects ");
     stringsEvent[EVENT_INTERFACE_SCROLL]    = TR("Scrolling\\Scrolling when the mouse touches right or left border");
     stringsEvent[EVENT_INTERFACE_INVERTX]   = TR("Mouse inversion X\\Inversion of the scrolling direction on the X axis");
     stringsEvent[EVENT_INTERFACE_INVERTY]   = TR("Mouse inversion Y\\Inversion of the scrolling direction on the Y axis");
     stringsEvent[EVENT_INTERFACE_EFFECT]    = TR("Quake at explosions\\The screen shakes at explosions");
+    stringsEvent[EVENT_INTERFACE_MOUSE]     = TR("System mouse\\Use system mouse cursor");
     stringsEvent[EVENT_INTERFACE_EDITMODE]  = TR("Automatic indent\\When program editing");
     stringsEvent[EVENT_INTERFACE_EDITVALUE] = TR("Big indent\\Indent 2 or 4 spaces per level defined by braces");
     stringsEvent[EVENT_INTERFACE_SOLUCE4]   = TR("Access to solutions\\Show program \"4: Solution\" in the exercises");
-    stringsEvent[EVENT_INTERFACE_BLOOD]     = TR("Blood\\Display blood when the astronaut or the alien queen is hit");
+    stringsEvent[EVENT_INTERFACE_BLOOD]     = TR("Blood\\Display blood when the astronaut is hit");
     stringsEvent[EVENT_INTERFACE_AUTOSAVE_ENABLE]   = TR("Autosave\\Enables autosave");
     stringsEvent[EVENT_INTERFACE_AUTOSAVE_INTERVAL] = TR("Autosave interval\\How often your game will autosave");
     stringsEvent[EVENT_INTERFACE_AUTOSAVE_SLOTS]    = TR("Autosave slots\\How many autosave slots you'll have");
+    stringsEvent[EVENT_INTERFACE_TEXTURE_FILTER]    = TR("Texture filtering\\Texture filtering");
+    stringsEvent[EVENT_INTERFACE_TEXTURE_MIPMAP]    = TR("Mipmap level\\Mipmap level");
+    stringsEvent[EVENT_INTERFACE_TEXTURE_ANISOTROPY]= TR("Anisotropy level\\Anisotropy level");
+    stringsEvent[EVENT_INTERFACE_MSAA]              = TR("MSAA\\Multisample anti-aliasing");
+    stringsEvent[EVENT_INTERFACE_SHADOW_SPOTS]      = TR("Simple shadows\\Shadows spots on the ground");
+    stringsEvent[EVENT_INTERFACE_SHADOW_MAPPING]    = TR("Dynamic shadows\\Beautiful shadows!");
+    stringsEvent[EVENT_INTERFACE_SHADOW_MAPPING_QUALITY]= TR("Quality dynamic shadows\\Enable shadow transparency and self shadowing");
+    stringsEvent[EVENT_INTERFACE_SHADOW_MAPPING_BUFFER] = TR("Shadow resolution\\Higher means better range and quality, but slower");
 
     stringsEvent[EVENT_INTERFACE_KDEF]      = TR("Standard controls\\Standard key functions");
     assert(INPUT_SLOT_MAX < EVENT_INTERFACE_KEY_END-EVENT_INTERFACE_KEY);
@@ -239,6 +246,7 @@ void InitializeRestext()
     stringsEvent[EVENT_INTERFACE_KEY+INPUT_SLOT_SPEED20]     = TR("Speed 2.0x\\Double speed");
     stringsEvent[EVENT_INTERFACE_KEY+INPUT_SLOT_SPEED30]     = TR("Speed 3.0x\\Triple speed");
     stringsEvent[EVENT_INTERFACE_KEY+INPUT_SLOT_SPEED40]     = TR("Speed 4.0x\\Quadruple speed");
+    stringsEvent[EVENT_INTERFACE_KEY+INPUT_SLOT_SPEED60]     = TR("Speed 6.0x\\Sextuple speed");
     stringsEvent[EVENT_INTERFACE_KEY+INPUT_SLOT_CAMERA_UP]   = TR("Camera up\\Increase camera angle while visiting message origin");
     stringsEvent[EVENT_INTERFACE_KEY+INPUT_SLOT_CAMERA_DOWN] = TR("Camera down\\Decrease camera angle while visiting message origin");
     stringsEvent[EVENT_INTERFACE_KEY+INPUT_SLOT_PAUSE]       = TR("Pause\\Pause the game without opening menu");
@@ -258,7 +266,6 @@ void InitializeRestext()
 
     stringsEvent[EVENT_INTERFACE_NEDIT]     = TR("\\New player name");
     stringsEvent[EVENT_INTERFACE_NOK]       = TR("OK\\Choose the selected player");
-    stringsEvent[EVENT_INTERFACE_NCANCEL]   = TR("Cancel\\Keep current player name");
     stringsEvent[EVENT_INTERFACE_NDELETE]   = TR("Delete player\\Deletes the player from the list");
     stringsEvent[EVENT_INTERFACE_NLABEL]    = TR("Player name");
 
@@ -388,7 +395,6 @@ void InitializeRestext()
     stringsEvent[EVENT_OBJECT_GPROGRESS]    = TR("Still working ...");
     stringsEvent[EVENT_OBJECT_GRADAR]       = TR("Number of insects detected");
     stringsEvent[EVENT_OBJECT_GINFO]        = TR("Transmitted information");
-    stringsEvent[EVENT_OBJECT_COMPASS]      = TR("Compass");
     stringsEvent[EVENT_OBJECT_MAPZOOM]      = TR("Zoom mini-map");
     stringsEvent[EVENT_OBJECT_CAMERA]       = TR("Camera (\\key camera;)");
     stringsEvent[EVENT_OBJECT_CAMERAleft]   = TR("Camera to left");
@@ -397,7 +403,7 @@ void InitializeRestext()
     stringsEvent[EVENT_OBJECT_CAMERAaway]   = TR("Camera awayest");
     stringsEvent[EVENT_OBJECT_HELP]         = TR("Help about selected object");
     stringsEvent[EVENT_OBJECT_SOLUCE]       = TR("Show the solution");
-    stringsEvent[EVENT_OBJECT_SHORTCUT00]   = TR("Switch bots <-> buildings");
+    stringsEvent[EVENT_OBJECT_SHORTCUT_MODE]= TR("Switch bots <-> buildings");
     stringsEvent[EVENT_OBJECT_LIMIT]        = TR("Show the range");
     stringsEvent[EVENT_OBJECT_PEN0]         = TR("\\Raise the pencil");
     stringsEvent[EVENT_OBJECT_PEN1]         = TR("\\Use the black pencil");
@@ -561,9 +567,8 @@ void InitializeRestext()
 
 
 
-    stringsErr[ERR_GENERIC]         = TR("Internal error - tell the developers");
+    stringsErr[ERR_UNKNOWN]         = TR("Internal error - tell the developers");
     stringsErr[ERR_CMD]             = TR("Unknown command");
-    stringsErr[ERR_MANIP_VEH]       = TR("Inappropriate bot");
     stringsErr[ERR_MANIP_FLY]       = TR("Impossible when flying");
     stringsErr[ERR_MANIP_BUSY]      = TR("Already carrying something");
     stringsErr[ERR_MANIP_NIL]       = TR("Nothing to grab");
@@ -588,24 +593,19 @@ void InitializeRestext()
     stringsErr[ERR_SEARCH_FLY]      = TR("Impossible when flying");
     stringsErr[ERR_BUILD_DISABLED]  = TR("Can not produce this object in this mission");
     stringsErr[ERR_BUILD_RESEARCH]  = TR("Can not produce not researched object");
-    stringsErr[ERR_SEARCH_VEH]      = TR("Inappropriate bot");
     stringsErr[ERR_SEARCH_MOTOR]    = TR("Impossible when moving");
-    stringsErr[ERR_TERRA_VEH]       = TR("Inappropriate bot");
     stringsErr[ERR_TERRA_ENERGY]    = TR("Not enough energy");
     stringsErr[ERR_TERRA_FLOOR]     = TR("Ground inappropriate");
     stringsErr[ERR_TERRA_BUILDING]  = TR("Building too close");
     stringsErr[ERR_TERRA_OBJECT]    = TR("Object too close");
-    stringsErr[ERR_RECOVER_VEH]     = TR("Inappropriate bot");
     stringsErr[ERR_RECOVER_ENERGY]  = TR("Not enough energy");
     stringsErr[ERR_RECOVER_NULL]    = TR("Nothing to recycle");
-    stringsErr[ERR_SHIELD_VEH]      = TR("Inappropriate bot");
     stringsErr[ERR_SHIELD_ENERGY]   = TR("No more energy");
     stringsErr[ERR_MOVE_IMPOSSIBLE] = TR("Error in instruction move");
     stringsErr[ERR_FIND_IMPOSSIBLE] = TR("Object not found");
     stringsErr[ERR_GOTO_IMPOSSIBLE] = TR("Goto: inaccessible destination");
     stringsErr[ERR_GOTO_ITER]       = TR("Goto: inaccessible destination");
     stringsErr[ERR_GOTO_BUSY]       = TR("Goto: destination occupied");
-    stringsErr[ERR_FIRE_VEH]        = TR("Inappropriate bot");
     stringsErr[ERR_FIRE_ENERGY]     = TR("Not enough energy");
     stringsErr[ERR_FIRE_FLY]        = TR("Impossible when flying");
     stringsErr[ERR_CONVERT_EMPTY]   = TR("No titanium ore to convert");
@@ -651,8 +651,10 @@ void InitializeRestext()
     stringsErr[ERR_DELETEMOBILE]    = TR("Bot destroyed");
     stringsErr[ERR_DELETEBUILDING]  = TR("Building destroyed");
     stringsErr[ERR_TOOMANY]         = TR("Can not create this; there are too many objects");
+    stringsErr[ERR_ENEMY_OBJECT]    = TR("Unable to control enemy objects");
     stringsErr[ERR_OBLIGATORYTOKEN] = TR("\"%s\" missing in this exercise");
     stringsErr[ERR_PROHIBITEDTOKEN] = TR("Do not use in this exercise");
+    stringsErr[ERR_WRONG_BOT]       = TR("Inappropriate bot");
 
     stringsErr[INFO_BUILD]          = TR("Building completed");
     stringsErr[INFO_CONVERT]        = TR("Titanium available");
@@ -753,8 +755,10 @@ void InitializeRestext()
 }
 
 
-
-static char         g_gamerName[100];
+namespace
+{
+char g_gamerName[100];
+} // anonymous namespace
 
 void SetGlobalGamerName(std::string name)
 {
@@ -763,7 +767,10 @@ void SetGlobalGamerName(std::string name)
 
 // Replaces the commands "\key name;" in a text.
 
-static void PutKeyName(std::string& dst, const char* src)
+namespace
+{
+
+void PutKeyName(std::string& dst, const char* src)
 {
     dst.clear();
 
@@ -793,9 +800,9 @@ static void PutKeyName(std::string& dst, const char* src)
 
 // Returns the translated text of a resource that needs key substitution
 
-static const char* GetResourceBase(ResType type, unsigned int num)
+const char* GetResourceBase(ResType type, unsigned int num)
 {
-    const char *str = NULL;
+    const char *str = nullptr;
 
     assert(num >= 0);
 
@@ -838,6 +845,8 @@ static const char* GetResourceBase(ResType type, unsigned int num)
     return gettext(str);
 }
 
+} // anonymous namespace
+
 // Returns the text of a resource.
 
 bool GetResource(ResType type, unsigned int num, std::string& text)
@@ -881,4 +890,3 @@ bool GetResource(ResType type, unsigned int num, std::string& text)
         return true;
     }
 }
-
