@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,18 +27,26 @@
 #include "graphics/engine/terrain.h"
 #include "graphics/engine/water.h"
 
+#include "level/robotmain.h"
+
 #include "math/const.h"
 #include "math/geometry.h"
 
 #include "object/object.h"
-#include "object/objman.h"
-#include "object/robotmain.h"
+#include "object/object_manager.h"
+
+#include "object/interface/carrier_object.h"
+#include "object/interface/controllable_object.h"
+#include "object/interface/movable_object.h"
+#include "object/interface/powered_object.h"
+#include "object/interface/transportable_object.h"
 
 #include "physics/physics.h"
 
 
 // Graphics module namespace
-namespace Gfx {
+namespace Gfx
+{
 
 
 //! Changes the level of transparency of an object and objects transported (battery & cargo)
@@ -46,13 +54,19 @@ void SetTransparency(CObject* obj, float value)
 {
     obj->SetTransparency(value);
 
-    CObject *fret = obj->GetFret();
-    if (fret != NULL)
-        fret->SetTransparency(value);
+    if (obj->Implements(ObjectInterfaceType::Carrier))
+    {
+        CObject* cargo = dynamic_cast<CCarrierObject*>(obj)->GetCargo();
+        if (cargo != nullptr)
+            cargo->SetTransparency(value);
+    }
 
-    fret = obj->GetPower();
-    if (fret != NULL)
-        fret->SetTransparency(value);
+    if (obj->Implements(ObjectInterfaceType::Powered))
+    {
+        CObject* power = dynamic_cast<CPoweredObject*>(obj)->GetPower();
+        if (power != nullptr)
+            power->SetTransparency(value);
+    }
 }
 
 
@@ -64,12 +78,12 @@ CCamera::CCamera()
 
     m_main    = CRobotMain::GetInstancePointer();
     m_terrain = m_main->GetTerrain();
-    
+
     m_input   = CInput::GetInstancePointer();
 
     m_type      = CAM_TYPE_FREE;
     m_smooth    = CAM_SMOOTH_NORM;
-    m_cameraObj = 0;
+    m_cameraObj = nullptr;
 
     m_eyeDistance = 10.0f;
     m_initDelay   =  0.0f;
@@ -137,6 +151,13 @@ CCamera::CCamera()
     m_effectProgress = 0.0f;
     m_effectOffset   = Math::Vector(0.0f, 0.0f, 0.0f);
 
+    m_overType = CAM_OVER_EFFECT_NULL;
+    m_overForce = 0.0f;
+    m_overTime = 0.0f;
+    m_overMode = 0.0f;
+    m_overFadeIn = 0.0f;
+    m_overFadeOut = 0.0f;
+
     m_scriptEye    = Math::Vector(0.0f, 0.0f, 0.0f);
     m_scriptLookat = Math::Vector(0.0f, 0.0f, 0.0f);
 
@@ -156,12 +177,18 @@ void CCamera::SetEffect(bool enable)
     m_effect = enable;
 }
 
+bool CCamera::GetEffect()
+{
+    return m_effect;
+}
+
 void CCamera::SetBlood(bool enable)
 {
     m_blood = enable;
 }
 
-bool CCamera::GetBlood() {
+bool CCamera::GetBlood()
+{
     return m_blood;
 }
 
@@ -170,16 +197,31 @@ void CCamera::SetCameraScroll(bool scroll)
     m_cameraScroll = scroll;
 }
 
+bool CCamera::GetCameraScroll()
+{
+    return m_cameraScroll;
+}
+
 void CCamera::SetCameraInvertX(bool invert)
 {
     m_cameraInvertX = invert;
 }
 
+bool CCamera::GetCameraInvertX()
+{
+    return m_cameraInvertX;
+}
+
 void CCamera::SetCameraInvertY(bool invert)
 {
     m_cameraInvertY = invert;
 }
 
+bool CCamera::GetCameraInvertY()
+{
+    return m_cameraInvertY;
+}
+
 float CCamera::GetMotorTurn()
 {
     if (m_type == CAM_TYPE_BACK)
@@ -243,12 +285,10 @@ void CCamera::SetType(CameraType type)
 
     if ( (m_type == CAM_TYPE_BACK) && m_transparency )
     {
-        for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+        for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
         {
-            CObject* obj = it.second;
-
-            if (obj->GetTruck())
-                continue;  // battery or cargo?
+            if (IsObjectBeingTransported(obj))
+                continue;
 
             SetTransparency(obj, 0.0f);  // opaque object
         }
@@ -310,7 +350,7 @@ void CCamera::SetType(CameraType type)
         m_addDirectionV = -Math::PI*0.05f;
 
         ObjectType oType;
-        if ( m_cameraObj == 0 )  oType = OBJECT_NULL;
+        if ( m_cameraObj == nullptr )  oType = OBJECT_NULL;
         else                     oType = m_cameraObj->GetType();
 
         m_backDist = 30.0f;
@@ -379,16 +419,26 @@ float CCamera::GetDist()
     return m_fixDist;
 }
 
-void CCamera::SetFixDirection(float angle)
+void CCamera::SetFixDirectionH(float angle)
 {
     m_fixDirectionH = angle;
 }
 
-float CCamera::GetFixDirection()
+float CCamera::GetFixDirectionH()
 {
     return m_fixDirectionH;
 }
 
+void CCamera::SetFixDirectionV(float angle)
+{
+    m_fixDirectionV = angle;
+}
+
+float CCamera::GetFixDirectionV()
+{
+    return m_fixDirectionV;
+}
+
 void CCamera::SetRemotePan(float value)
 {
     m_remotePan = value;
@@ -873,7 +923,7 @@ bool CCamera::IsCollision(Math::Vector &eye, Math::Vector lookat)
 bool CCamera::IsCollisionBack(Math::Vector &eye, Math::Vector lookat)
 {
     ObjectType iType;
-    if (m_cameraObj == NULL)
+    if (m_cameraObj == nullptr)
         iType = OBJECT_NULL;
     else
         iType = m_cameraObj->GetType();
@@ -889,12 +939,11 @@ bool CCamera::IsCollisionBack(Math::Vector &eye, Math::Vector lookat)
     max.z = Math::Max(m_actualEye.z, m_actualLookat.z);
 
     m_transparency = false;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
-    {
-        CObject* obj = it.second;
 
-        if (obj->GetTruck()) continue;  // battery or cargo?
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
+    {
+        if (IsObjectBeingTransported(obj))
+            continue;
 
         SetTransparency(obj, 0.0f);  // opaque object
 
@@ -921,16 +970,14 @@ bool CCamera::IsCollisionBack(Math::Vector &eye, Math::Vector lookat)
         if ( oType == OBJECT_HUMAN  ||
              oType == OBJECT_TECH   ||
              oType == OBJECT_TOTO   ||
-             oType == OBJECT_FIX    ||
-             oType == OBJECT_FRET   ||
              oType == OBJECT_ANT    ||
              oType == OBJECT_SPIDER ||
              oType == OBJECT_BEE    ||
              oType == OBJECT_WORM   )  continue;
 
-        Math::Vector oPos;
-        float oRadius = 0.0f;
-        obj->GetGlobalSphere(oPos, oRadius);
+        Math::Sphere objSphere = obj->GetCameraCollisionSphere();
+        Math::Vector oPos = objSphere.pos;
+        float oRadius = objSphere.radius;
         if ( oRadius <= 2.0f )  continue;  // ignores small objects
 
         if ( oPos.x+oRadius < min.x ||
@@ -947,7 +994,7 @@ bool CCamera::IsCollisionBack(Math::Vector &eye, Math::Vector lookat)
         if ( oType == OBJECT_FACTORY )
         {
             float angle = Math::RotateAngle(m_actualEye.x-oPos.x, oPos.z-m_actualEye.z);  // CW !
-            angle = Math::Direction(angle, obj->GetAngleY(0));
+            angle = Math::Direction(angle, obj->GetRotationY());
             if ( fabs(angle) < 30.0f*Math::PI/180.0f )  continue;  // in the gate?
         }
 
@@ -966,15 +1013,12 @@ bool CCamera::IsCollisionBack(Math::Vector &eye, Math::Vector lookat)
 
 bool CCamera::IsCollisionFix(Math::Vector &eye, Math::Vector lookat)
 {
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        CObject* obj = it.second;
-
         if (obj == m_cameraObj) continue;
 
         ObjectType type = obj->GetType();
         if ( type == OBJECT_TOTO    ||
-             type == OBJECT_FRET    ||
              type == OBJECT_STONE   ||
              type == OBJECT_URANIUM ||
              type == OBJECT_METAL   ||
@@ -991,9 +1035,9 @@ bool CCamera::IsCollisionFix(Math::Vector &eye, Math::Vector lookat)
              type == OBJECT_BEE     ||
              type == OBJECT_WORM )  continue;
 
-        Math::Vector objPos;
-        float objRadius = 0.0f;
-        obj->GetGlobalSphere(objPos, objRadius);
+        Math::Sphere objSphere = obj->GetCameraCollisionSphere();
+        Math::Vector objPos = objSphere.pos;
+        float objRadius = objSphere.radius;
         if (objRadius == 0.0f) continue;
 
         float dist = Math::Distance(eye, objPos);
@@ -1021,7 +1065,7 @@ bool CCamera::EventProcess(const Event &event)
             break;
 
         case EVENT_MOUSE_WHEEL:
-            EventMouseWheel(event.mouseWheel.dir);
+            EventMouseWheel(event.GetData<MouseWheelEventData>()->dir);
             break;
 
         default:
@@ -1192,6 +1236,12 @@ EngineMouseType CCamera::GetMouseDef(Math::Point pos)
 
 bool CCamera::EventFrameFree(const Event &event)
 {
+    Math::Vector cameraInput = event.cameraInput;
+    if (m_cameraObj == nullptr)
+    {
+        cameraInput = Math::Clamp(cameraInput + event.motionInput, Math::Vector(-1.0f, -1.0f, -1.0f), Math::Vector(1.0f, 1.0f, 1.0f));
+    }
+
     float factor = m_heightEye * 0.5f + 30.0f;
 
     if ( m_mouseDirH != 0.0f )
@@ -1200,19 +1250,19 @@ bool CCamera::EventFrameFree(const Event &event)
         m_eyePt = Math::LookatPoint(m_eyePt, m_directionH, m_directionV, m_mouseDirV * event.rTime * factor * m_speed);
 
     // Up/Down
-    m_eyePt = Math::LookatPoint(m_eyePt, m_directionH, m_directionV, event.motionInput.y * event.rTime * factor * m_speed);
+    m_eyePt = Math::LookatPoint(m_eyePt, m_directionH, m_directionV, cameraInput.y * event.rTime * factor * m_speed);
 
     // Left/Right
     if ( event.kmodState & KEY_MOD(CTRL) )
     {
-        if ( event.motionInput.x < 0.0f )
-            m_eyePt = Math::LookatPoint(m_eyePt, m_directionH + Math::PI / 2.0f, m_directionV, -event.motionInput.x * event.rTime * factor * m_speed);
-        if ( event.motionInput.x > 0.0f )
-            m_eyePt = Math::LookatPoint(m_eyePt, m_directionH - Math::PI / 2.0f, m_directionV,  event.motionInput.x * event.rTime * factor * m_speed);
+        if ( cameraInput.x < 0.0f )
+            m_eyePt = Math::LookatPoint(m_eyePt, m_directionH + Math::PI / 2.0f, m_directionV, -cameraInput.x * event.rTime * factor * m_speed);
+        if ( cameraInput.x > 0.0f )
+            m_eyePt = Math::LookatPoint(m_eyePt, m_directionH - Math::PI / 2.0f, m_directionV,  cameraInput.x * event.rTime * factor * m_speed);
     }
     else
     {
-        m_directionH -= event.motionInput.x * event.rTime * 0.7f * m_speed;
+        m_directionH -= cameraInput.x * event.rTime * 0.7f * m_speed;
     }
 
     // PageUp/PageDown
@@ -1227,6 +1277,16 @@ bool CCamera::EventFrameFree(const Event &event)
             m_heightEye -= event.rTime * factor * m_speed;
     }
 
+
+    if ( m_input->GetKeyState(INPUT_SLOT_CAMERA_UP) )
+    {
+        m_heightLookat += event.rTime * factor * m_speed;
+    }
+    if ( m_input->GetKeyState(INPUT_SLOT_CAMERA_DOWN) )
+    {
+        m_heightLookat -= event.rTime * factor * m_speed;
+    }
+
     m_terrain->AdjustToBounds(m_eyePt, 10.0f);
 
     if (m_terrain->AdjustToFloor(m_eyePt, true))
@@ -1303,7 +1363,7 @@ bool CCamera::EventFrameDialog(const Event &event)
 bool CCamera::EventFrameBack(const Event &event)
 {
     ObjectType type;
-    if (m_cameraObj == NULL)
+    if (m_cameraObj == nullptr)
         type = OBJECT_NULL;
     else
         type = m_cameraObj->GetType();
@@ -1378,15 +1438,15 @@ bool CCamera::EventFrameBack(const Event &event)
     if (m_centeringAngleV == 99.9f) centeringV = 0.0f;
     if (m_centeringDist   ==  0.0f) centeringD = 0.0f;
 
-    if (m_cameraObj != NULL)
+    if (m_cameraObj != nullptr)
     {
-        Math::Vector lookatPt = m_cameraObj->GetPosition(0);
+        Math::Vector lookatPt = m_cameraObj->GetPosition();
              if (type == OBJECT_BASE ) lookatPt.y += 40.0f;
         else if (type == OBJECT_HUMAN) lookatPt.y +=  1.0f;
         else if (type == OBJECT_TECH ) lookatPt.y +=  1.0f;
         else                           lookatPt.y +=  4.0f;
 
-        float h = -m_cameraObj->GetAngleY(0);  // angle vehicle / building
+        float h = -m_cameraObj->GetRotationY();  // angle vehicle / building
 
         if ( type == OBJECT_DERRICK  ||
              type == OBJECT_FACTORY  ||
@@ -1434,8 +1494,10 @@ bool CCamera::EventFrameBack(const Event &event)
 
         m_eyePt = RotateView(lookatPt, h, v, d);
 
-        CPhysics* physics = m_cameraObj->GetPhysics();
-        if ( (physics != NULL) && physics->GetLand() )  // ground?
+        bool ground = true;
+        if (m_cameraObj->Implements(ObjectInterfaceType::Movable))
+            ground = dynamic_cast<CMovableObject*>(m_cameraObj)->GetPhysics()->GetLand();
+        if ( ground )  // ground?
         {
             Math::Vector pos = lookatPt + (lookatPt - m_eyePt);
             float floor = m_terrain->GetHeightToFloor(pos) - 4.0f;
@@ -1469,19 +1531,24 @@ bool CCamera::EventFrameFix(const Event &event)
         if (m_fixDist > 200.0f) m_fixDist = 200.0f;
     }
 
+    // Left/Right
     if (m_cameraScroll)
     {
-        // Left/Right
         m_fixDirectionH += m_mouseDirH * event.rTime * 1.0f * m_speed;
         m_fixDirectionH = Math::NormAngle(m_fixDirectionH);
     }
+    m_fixDirectionH += event.cameraInput.x * event.rTime * 0.7f * m_speed;
+
+    // Up/Down
+    m_fixDirectionV -= event.cameraInput.y * event.rTime * 0.7f * m_speed;
+    m_fixDirectionV = Math::Min(Math::Max(m_fixDirectionV, -0.5*Math::PI), 0.25*Math::PI);
 
     if ((m_mouseDirH != 0) || (m_mouseDirV != 0))
         AbortCentering();  // special stops framing
 
-    if (m_cameraObj != NULL)
+    if (m_cameraObj != nullptr)
     {
-        Math::Vector lookatPt = m_cameraObj->GetPosition(0);
+        Math::Vector lookatPt = m_cameraObj->GetPosition();
 
         float h = m_fixDirectionH + m_remotePan;
         float v = m_fixDirectionV;
@@ -1534,11 +1601,11 @@ bool CCamera::EventFrameExplo(const Event &event)
 
 bool CCamera::EventFrameOnBoard(const Event &event)
 {
-    if (m_cameraObj != NULL)
+    if (m_cameraObj != nullptr)
     {
+        assert(m_cameraObj->Implements(ObjectInterfaceType::Controllable));
         Math::Vector lookatPt, upVec;
-        m_cameraObj->SetViewFromHere(m_eyePt, m_directionH, m_directionV,
-                                     lookatPt, upVec, m_type);
+        dynamic_cast<CControllableObject*>(m_cameraObj)->AdjustCamera(m_eyePt, m_directionH, m_directionV, lookatPt, upVec, m_type);
         Math::Vector eye    = m_effectOffset * 0.3f + m_eyePt;
         Math::Vector lookat = m_effectOffset * 0.3f + lookatPt;
 
@@ -1658,7 +1725,7 @@ Math::Vector CCamera::ExcludeObject(Math::Vector eye, Math::Vector lookat,
     for (int i = 0; i < 1000000; i++)
     {
         CObject* obj = static_cast<CObject*>( iMan->SearchInstance(CLASS_OBJECT, i) );
-        if (obj == NULL)
+        if (obj == nullptr)
             break;
 
         int j = 0;
@@ -1675,6 +1742,10 @@ Math::Vector CCamera::ExcludeObject(Math::Vector eye, Math::Vector lookat,
     return eye;*/
 }
 
-
+void CCamera::SetCameraSpeed(float speed)
+{
+    m_speed = speed;
 }
 
+
+}
