@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +24,6 @@
 
 #pragma once
 
-
-#include "app/system.h"
-#include "app/pausemanager.h"
-
-#include "common/event.h"
 #include "common/singleton.h"
 
 #include "graphics/core/color.h"
@@ -36,8 +31,6 @@
 #include "graphics/core/texture.h"
 #include "graphics/core/vertex.h"
 
-#include "graphics/engine/modelfile.h"
-
 #include "math/intpoint.h"
 #include "math/matrix.h"
 #include "math/point.h"
@@ -48,19 +41,27 @@
 #include <vector>
 #include <map>
 #include <set>
+#include <memory>
+#include <unordered_map>
 
 
 class CApplication;
 class CObject;
 class CSoundInterface;
 class CImage;
+class CPauseManager;
+class CSystemUtils;
+struct SystemTimeStamp;
+struct Event;
 
 
 // Graphics module namespace
-namespace Gfx {
+namespace Gfx
+{
 
 
 class CDevice;
+class COldModelManager;
 class CLightManager;
 class CText;
 class CParticle;
@@ -69,6 +70,10 @@ class CCloud;
 class CLightning;
 class CPlanet;
 class CTerrain;
+class CPyroManager;
+class CModelMesh;
+struct ModelShadowSpot;
+struct ModelTriangle;
 
 
 /**
@@ -123,7 +128,11 @@ enum EngineRenderState
     //! Only opaque texture, no blending, etc.
     ENG_RSTATE_OPAQUE_TEXTURE   = (1<<19),
     //! Only opaque color, no texture, blending, etc.
-    ENG_RSTATE_OPAQUE_COLOR     = (1<<20)
+    ENG_RSTATE_OPAQUE_COLOR     = (1<<20),
+    //! Texture using alpha channel
+    ENG_RSTATE_TTEXTURE_ALPHA   = (1<<21),
+    //! Color with transparency
+    ENG_RSTATE_TCOLOR_ALPHA     = (1<<22)
 };
 
 
@@ -150,16 +159,11 @@ struct EngineTriangle
     //! Material
     Material       material;
     //! Render state
-    int            state;
+    int            state = ENG_RSTATE_NORMAL;
     //! 1st texture
     std::string    tex1Name;
     //! 2nd texture
     std::string    tex2Name;
-
-    inline EngineTriangle()
-    {
-        state = ENG_RSTATE_NORMAL;
-    }
 };
 
 /**
@@ -186,7 +190,7 @@ enum EngineObjectType
 
 /**
  * \struct EngineBaseObjDataTier
- * \brief Tier 4 of object tree (data)
+ * \brief Tier 3 of object tree (data)
  */
 struct EngineBaseObjDataTier
 {
@@ -197,31 +201,17 @@ struct EngineBaseObjDataTier
     unsigned int            staticBufferId;
     bool                    updateStaticBuffer;
 
-    inline EngineBaseObjDataTier(EngineTriangleType _type = ENG_TRIANGLE_TYPE_TRIANGLES,
-                                 const Material& _material = Material(),
-                                 int _state = ENG_RSTATE_NORMAL)
-     : type(_type)
-     , material(_material)
-     , state(_state)
+    inline EngineBaseObjDataTier(EngineTriangleType type = ENG_TRIANGLE_TYPE_TRIANGLES,
+                                 const Material& material = Material(),
+                                 int state = ENG_RSTATE_NORMAL)
+     : type(type)
+     , material(material)
+     , state(state)
      , staticBufferId(0)
      , updateStaticBuffer(false)
     {}
 };
 
-/**
- * \struct EngineBaseObjLODTier
- * \brief Tier 3 of base object tree (LOD)
- */
-struct EngineBaseObjLODTier
-{
-    LODLevel                            lodLevel;
-    std::vector<EngineBaseObjDataTier>  next;
-
-    inline EngineBaseObjLODTier(LODLevel _lodLevel = LOD_Constant)
-     : lodLevel(_lodLevel)
-    {}
-};
-
 /**
  * \struct EngineBaseObjTexTier
  * \brief Tier 2 of base object tree (textures)
@@ -232,11 +222,12 @@ struct EngineBaseObjTexTier
     Texture                            tex1;
     std::string                        tex2Name;
     Texture                            tex2;
-    std::vector<EngineBaseObjLODTier>  next;
+    std::vector<EngineBaseObjDataTier>  next;
 
-    inline EngineBaseObjTexTier(const std::string& _tex1Name = "", const std::string& _tex2Name = "")
-     : tex1Name(_tex1Name)
-     , tex2Name(_tex2Name)
+    inline EngineBaseObjTexTier(const std::string& tex1Name = "",
+                                const std::string& tex2Name = "")
+     : tex1Name(tex1Name)
+     , tex2Name(tex2Name)
     {}
 };
 
@@ -249,30 +240,21 @@ struct EngineBaseObjTexTier
 struct EngineBaseObject
 {
     //! If true, base object is valid in objects vector
-    bool used;
+    bool used = false;
     //! Number of triangles
-    int                    totalTriangles;
+    int                    totalTriangles = 0;
     //! Bounding box min (origin 0,0,0 always included)
     Math::Vector           bboxMin;
     //! bounding box max (origin 0,0,0 always included)
     Math::Vector           bboxMax;
     //! Radius of the sphere at the origin
-    float                  radius;
-    //! Next tier (LOD)
+    float                  radius = 0.0f;
+    //! Next tier (Tex)
     std::vector<EngineBaseObjTexTier> next;
 
-    inline EngineBaseObject()
-    {
-        LoadDefault();
-    }
-
     inline void LoadDefault()
     {
-        used = false;
-        totalTriangles = 0;
-        bboxMax.LoadZero();
-        bboxMin.LoadZero();
-        radius = 0.0f;
+        *this = EngineBaseObject();
     }
 };
 
@@ -283,45 +265,30 @@ struct EngineBaseObject
 struct EngineObject
 {
     //! If true, object is valid in objects vector
-    bool                   used;
+    bool                   used = false;
     //! Rank of associated base engine object
-    int                    baseObjRank;
+    int                    baseObjRank = -1;
     //! If true, the object is drawn
-    bool                   visible;
+    bool                   visible = false;
     //! If true, object is behind the 2D interface
-    bool                   drawWorld;
+    bool                   drawWorld = false;
     //! If true, the shape is before the 2D interface
-    bool                   drawFront;
+    bool                   drawFront = false;
     //! Type of object
-    EngineObjectType       type;
+    EngineObjectType       type = ENG_OBJTYPE_NULL;
     //! Transformation matrix
     Math::Matrix           transform;
     //! Distance to object from eye point
-    float                  distance;
+    float                  distance = 0.0f;
     //! Rank of the associated shadow
-    int                    shadowRank;
+    int                    shadowRank = -1;
     //! Transparency of the object [0, 1]
-    float                  transparency;
-
-    //! Calls LoadDefault()
-    inline EngineObject()
-    {
-        LoadDefault();
-    }
+    float                  transparency = 0.0f;
 
     //! Loads default values
     inline void LoadDefault()
     {
-        used = false;
-        baseObjRank = -1;
-        visible = false;
-        drawWorld = false;
-        drawFront = false;
-        type = ENG_OBJTYPE_NULL;
-        transform.LoadIdentity();
-        distance = 0.0f;
-        shadowRank = -1;
-        transparency = 0.0f;
+        *this = EngineObject();
     }
 };
 
@@ -344,40 +311,29 @@ enum EngineShadowType
 struct EngineShadow
 {
     //! If true, shadow is valid
-    bool                used;
+    bool                used = false;
     //! If true, shadow is invisible (object being carried for example)
-    bool                hide;
+    bool                hide = false;
     //! Rank of the associated object
-    int                 objRank;
+    int                 objRank = -1;
     //! Type of shadow
-    EngineShadowType type;
+    EngineShadowType type = ENG_SHADOW_NORM;
     //! Position of the shadow
     Math::Vector        pos;
     //! Normal to the terrain
     Math::Vector        normal;
     //! Angle of the shadow
-    float               angle;
+    float               angle = 0.0f;
     //! Radius of the shadow
-    float               radius;
+    float               radius = 0.0f;
     //! Intensity of the shadow
-    float               intensity;
+    float               intensity = 0.0f;
     //! Height from the ground
-    float               height;
+    float               height = 0.0f;
 
-    inline EngineShadow()
+    void LoadDefault()
     {
-        LoadDefault();
-    }
-
-    inline void LoadDefault()
-    {
-        used = false;
-        hide = false;
-        objRank = 0;
-        type = ENG_SHADOW_NORM;
-        pos.LoadZero();
-        normal.LoadZero();
-        angle = radius = intensity = height = 0.0f;
+        *this = EngineShadow();
     }
 };
 
@@ -388,36 +344,27 @@ struct EngineShadow
 struct EngineGroundSpot
 {
     //! If true, ground spot is valid
-    bool            used;
+    bool            used = false;
     //! Color of the shadow
     Color      color;
     //! Min altitude
-    float           min;
+    float           min = 0.0f;
     //! Max altitude
-    float           max;
+    float           max = 0.0f;
     //! Transition area
-    float           smooth;
+    float           smooth = 0.0f;
     //! Position for the shadow
     Math::Vector    pos;
     //! Radius of the shadow
-    float           radius;
+    float           radius = 0.0f;
     //! Position of the shadow drawn
     Math::Vector    drawPos;
     //! Radius of the shadow drawn
-    float           drawRadius;
+    float           drawRadius = 0.0f;
 
-    inline EngineGroundSpot()
+    void LoadDefault()
     {
-        LoadDefault();
-    }
-
-    inline void LoadDefault()
-    {
-        used = false;
-        color = Color();
-        pos.LoadZero();
-        drawPos.LoadZero();
-        min = max = smooth = radius = drawRadius = 0.0f;
+        *this = EngineGroundSpot();
     }
 };
 
@@ -444,47 +391,35 @@ enum EngineGroundMarkPhase
 struct EngineGroundMark
 {
     //! If true, draw mark
-    bool                        draw;
+    bool                        draw = false;
     //! Phase of life
-    EngineGroundMarkPhase  phase;
+    EngineGroundMarkPhase  phase = ENG_GR_MARK_PHASE_NULL;
     //! Times for 3 life phases
-    float                       delay[3];
+    float                       delay[3] = {};
     //! Fixed time
-    float                       fix;
+    float                       fix = 0.0f;
     //! Position for marks
     Math::Vector                pos;
     //! Radius of marks
-    float                       radius;
+    float                       radius = 0.0f;
     //! Color intensity
-    float                       intensity;
+    float                       intensity = 0.0f;
     //! Draw position for marks
     Math::Vector                drawPos;
     //! Radius for  marks
-    float                       drawRadius;
+    float                       drawRadius = 0.0f;
     //! Draw intensity for marks
-    float                       drawIntensity;
+    float                       drawIntensity = 0.0f;
     //! X dimension of table
-    int                         dx;
+    int                         dx = 0;
     //! Y dimension of table
-    int                         dy;
+    int                         dy = 0;
     //! Pointer to the table
-    char*                       table;
+    char*                       table = nullptr;
 
-    inline EngineGroundMark()
+    void LoadDefault()
     {
-        LoadDefault();
-    }
-
-    inline void LoadDefault()
-    {
-        draw = false;
-        phase = ENG_GR_MARK_PHASE_NULL;
-        pos = Math::Vector();
-        drawPos = Math::Vector();
-        delay[0] = delay[1] = delay[2] = 0.0f;
-        fix = radius = intensity = drawRadius = drawIntensity = 0.0f;
-        dx = dy = 0;
-        table = nullptr;
+        *this = EngineGroundMark();
     }
 };
 
@@ -567,16 +502,18 @@ struct EngineMouse
     //! Hot point
     Math::Point hotPoint;
 
-    inline EngineMouse(int _icon1 = -1, int _icon2 = -1, int _iconShadow = -1,
-                       EngineRenderState _mode1 = ENG_RSTATE_NORMAL,
-                       EngineRenderState _mode2 = ENG_RSTATE_NORMAL,
-                       Math::Point _hotPoint = Math::Point())
-     : icon1(_icon1)
-     , icon2(_icon2)
-     , iconShadow(_iconShadow)
-     , mode1(_mode1)
-     , mode2(_mode2)
-     , hotPoint(_hotPoint)
+    EngineMouse(int icon1 = -1,
+                int icon2 = -1,
+                int iconShadow = -1,
+                EngineRenderState mode1 = ENG_RSTATE_NORMAL,
+                EngineRenderState mode2 = ENG_RSTATE_NORMAL,
+                Math::Point hotPoint = Math::Point())
+     : icon1(icon1)
+     , icon2(icon2)
+     , iconShadow(iconShadow)
+     , mode1(mode1)
+     , mode2(mode2)
+     , hotPoint(hotPoint)
     {}
 };
 
@@ -641,8 +578,7 @@ struct EngineMouse
  * The 4 tiers contain the following information:
  *  - level 1 (EngineBaseObject) - geometric statistics
  *  - level 2 (EngineBaseObjTexTier) - two textures (names and structs) applied to triangles,
- *  - level 3 (EngineBaseObjLODTier) - minumum and maximum LOD (=level of detail)
- *  - level 4 (EngineBaseObjDataTier) - type of object*, material, render state and the actual vertex data
+ *  - level 3 (EngineBaseObjDataTier) - type of object*, material, render state and the actual vertex data
  *
  *  *NOTE: type of object in this context means only the internal type in 3D engine. It is not related
  *  to CObject types.
@@ -684,7 +620,7 @@ struct EngineMouse
 class CEngine : public CSingleton<CEngine>
 {
 public:
-    CEngine(CApplication* app);
+    CEngine(CApplication* app, CSystemUtils* systemUtils);
     ~CEngine();
 
     //! Sets the device to be used
@@ -694,6 +630,8 @@ public:
 
     //! Returns the text rendering engine
     CText*          GetText();
+    COldModelManager* GetModelManager();
+    CPyroManager*   GetPyroManager();
     //! Returns the light manager
     CLightManager*  GetLightManager();
     //! Returns the particle manager
@@ -708,6 +646,8 @@ public:
     CPlanet*        GetPlanet();
     //! Returns the fog manager
     CCloud*         GetCloud();
+    //! Returns the pause manager
+    CPauseManager*  GetPauseManager();
 
     //! Sets the terrain object
     void            SetTerrain(CTerrain* terrain);
@@ -734,18 +674,12 @@ public:
 
 
     //! Writes a screenshot containing the current frame
-    bool            WriteScreenShot(const std::string& fileName, int width, int height);
+    void            WriteScreenShot(const std::string& fileName);
 
 
     //! Get pause mode
     TEST_VIRTUAL bool GetPause();
 
-    //@{
-    //! Management of lock for the duration of movie sequence
-    void            SetMovieLock(bool lock);
-    bool            GetMovieLock();
-    //@}
-
     //@{
     //! Management of displaying statistic information
     void            SetShowStats(bool show);
@@ -755,6 +689,15 @@ public:
     //! Enables/disables rendering
     void            SetRenderEnable(bool enable);
 
+    void            SetRenderInterface(bool enable);
+    bool            GetRenderInterface();
+
+    //! Management of "screenshot mode" (disables interface particle rendering)
+    //@{
+    void            SetScreenshotMode(bool screenshotMode);
+    bool            GetScreenshotMode();
+    //@}
+
     //! Returns current size of viewport window
     Math::IntPoint   GetWindowSize();
 
@@ -777,14 +720,39 @@ public:
     void            AddStatisticTriangle(int nb);
     //! Returns the number of triangles in current frame
     int             GetStatisticTriangle();
-    
+
     //! Sets the coordinates to display in stats window
     void            SetStatisticPos(Math::Vector pos);
-    
+
     //! Sets text to display as mission timer
     void            SetTimerDisplay(const std::string& text);
 
 
+    /* *************** New mesh interface *************** */
+    //! Add new instance of static mesh
+    /**
+     * Static meshes never change their geometry or texture mapping,
+     * so specific instances can share mesh data.
+     *
+     * @param mesh mesh data
+     * @param key key unique per object class
+     * @return mesh instance handle
+     */
+    int AddStaticMesh(const std::string& key, const Gfx::CModelMesh* mesh, const Math::Matrix& worldMatrix);
+
+    //! Removes given static mesh
+    void DeleteStaticMesh(int meshHandle);
+
+    //! Adds a shadow spot to mesh
+    void AddStaticMeshShadowSpot(int meshHandle, const Gfx::ModelShadowSpot& shadowSpot);
+
+    //! Returns static mesh world matrix
+    const Math::Matrix& GetStaticMeshWorldMatrix(int meshHandle);
+
+    //! Sets transparency for static mesh
+    void SetStaticMeshTransparency(int meshHandle, float value);
+
+
     /* *************** Object management *************** */
 
     // Base objects
@@ -800,16 +768,12 @@ public:
     void            CopyBaseObject(int sourceBaseObjRank, int destBaseObjRank);
 
     //! Adds triangles to given object with the specified params
-    void            AddBaseObjTriangles(int baseObjRank, const std::vector<VertexTex2>& vertices,
-                                        EngineTriangleType triangleType,
-                                        const Material& material, int state,
-                                        std::string tex1Name, std::string tex2Name,
-                                        LODLevel lodLevel, bool globalUpdate);
+    void AddBaseObjTriangles(int baseObjRank, const std::vector<Gfx::ModelTriangle>& triangles);
 
     //! Adds a tier 4 engine object directly
     void            AddBaseObjQuick(int baseObjRank, const EngineBaseObjDataTier& buffer,
                                     std::string tex1Name, std::string tex2Name,
-                                    LODLevel lodLevel, bool globalUpdate);
+                                    bool globalUpdate);
 
     // Objects
 
@@ -857,11 +821,10 @@ public:
 
     //! Returns the first found tier 4 engine object for the given params or nullptr if not found
     EngineBaseObjDataTier* FindTriangles(int objRank, const Material& material,
-                                         int state, std::string tex1Name, std::string tex2Name,
-                                         int lodLevelMask);
+                                         int state, std::string tex1Name, std::string tex2Name);
 
     //! Returns a partial list of triangles for given object
-    int             GetPartialTriangles(int objRank, int lodLevelMask, float percent, int maxCount,
+    int             GetPartialTriangles(int objRank, float percent, int maxCount,
                                         std::vector<EngineTriangle>& triangles);
 
     //! Changes the 2nd texure for given object
@@ -870,13 +833,13 @@ public:
     //! Changes (recalculates) texture mapping for given object
     void            ChangeTextureMapping(int objRank, const Material& mat, int state,
                                          const std::string& tex1Name, const std::string& tex2Name,
-                                         int lodLevelMask, EngineTextureMapping mode,
+                                         EngineTextureMapping mode,
                                          float au, float bu, float av, float bv);
 
     //! Changes texture mapping for robot tracks
     void            TrackTextureMapping(int objRank, const Material& mat, int state,
                                         const std::string& tex1Name, const std::string& tex2Name,
-                                        int lodLevelMask, EngineTextureMapping mode,
+                                        EngineTextureMapping mode,
                                         float pos, float factor, float tl, float ts, float tt);
 
     //! Detects the target object that is selected with the mouse
@@ -884,21 +847,20 @@ public:
     int             DetectObject(Math::Point mouse);
 
     //! Creates a shadow for the given object
-    void            CreateShadow(int objRank);
+    void            CreateShadowSpot(int objRank);
     //! Deletes the shadow for given object
-    void            DeleteShadow(int objRank);
+    void            DeleteShadowSpot(int objRank);
 
     //@{
     //! Management of different shadow params
-    void            SetObjectShadowHide(int objRank, bool hide);
-    void            SetObjectShadowType(int objRank, EngineShadowType type);
-    void            SetObjectShadowPos(int objRank, const Math::Vector& pos);
-    void            SetObjectShadowNormal(int objRank, const Math::Vector& normal);
-    void            SetObjectShadowAngle(int objRank, float angle);
-    void            SetObjectShadowRadius(int objRank, float radius);
-    void            SetObjectShadowIntensity(int objRank, float intensity);
-    void            SetObjectShadowHeight(int objRank, float height);
-    float           GetObjectShadowRadius(int objRank);
+    void            SetObjectShadowSpotHide(int objRank, bool hide);
+    void            SetObjectShadowSpotType(int objRank, EngineShadowType type);
+    void            SetObjectShadowSpotPos(int objRank, const Math::Vector& pos);
+    void            SetObjectShadowSpotAngle(int objRank, float angle);
+    void            SetObjectShadowSpotRadius(int objRank, float radius);
+    void            SetObjectShadowSpotIntensity(int objRank, float intensity);
+    void            SetObjectShadowSpotHeight(int objRank, float height);
+    void            UpdateObjectShadowSpotNormal(int objRank);
     //@}
 
     //! Lists the ranks of objects and subobjects selected
@@ -955,6 +917,15 @@ public:
     bool            LoadAllTextures();
 
     //! Changes colors in a texture
+    //@{
+    bool            ChangeTextureColor(const std::string& texName,
+                                       const std::string& srcName,
+                                       Color colorRef1, Color colorNew1,
+                                       Color colorRef2, Color colorNew2,
+                                       float tolerance1, float tolerance2,
+                                       Math::Point ts, Math::Point ti,
+                                       Math::Point *exclude = nullptr,
+                                       float shift = 0.0f, bool hsv = false);
     bool            ChangeTextureColor(const std::string& texName,
                                        Color colorRef1, Color colorNew1,
                                        Color colorRef2, Color colorNew2,
@@ -962,6 +933,7 @@ public:
                                        Math::Point ts, Math::Point ti,
                                        Math::Point *exclude = nullptr,
                                        float shift = 0.0f, bool hsv = false);
+    //@}
 
     //! Sets texture for given stage; if not present in cache, the texture is loaded
     /** If loading fails, returns false. */
@@ -989,26 +961,16 @@ public:
     float           GetFocus();
     //@}
 
-    //@{
-    //! Management of the global mode of marking
-    void            SetGroundSpot(bool mode);
-    bool            GetGroundSpot();
-    //@}
-
-    //@{
-    //! Management of the global mode of shading
-    void            SetShadow(bool mode);
-    bool            GetShadow();
-    //@}
-
     //@{
     //! Management of the global mode of contamination
+    // NOTE: This is an user configuration setting
     void            SetDirty(bool mode);
     bool            GetDirty();
     //@}
 
     //@{
     //! Management of the global mode of horizontal fog patches
+    // NOTE: This is an user configuration setting
     void            SetFog(bool mode);
     bool            GetFog();
     //@}
@@ -1070,10 +1032,10 @@ public:
     //! Management of the background image to use
     void            SetBackground(const std::string& name, Color up = Color(), Color down = Color(),
                                   Color cloudUp = Color(), Color cloudDown = Color(),
-                                  bool full = false);
+                                  bool full = false, bool scale = false);
     void            GetBackground(std::string& name, Color& up, Color& down,
                                   Color& cloudUp, Color& cloudDown,
-                                  bool& full);
+                                  bool& full, bool& scale);
     //@}
 
     //! Specifies the name of foreground texture
@@ -1085,6 +1047,7 @@ public:
 
     //@{
     //! Management of the particle density
+    // NOTE: This is an user configuration setting
     void            SetParticleDensity(float value);
     float           GetParticleDensity();
     //@}
@@ -1094,83 +1057,66 @@ public:
 
     //@{
     //! Management of the distance of clipping.
+    // NOTE: This is an user configuration setting
     void            SetClippingDistance(float value);
     float           GetClippingDistance();
     //@}
 
-    //@{
-    //! Management of objects detals.
-    void            SetObjectDetail(float value);
-    float           GetObjectDetail();
-    //@}
-
-    //@{
-    //! The amount of management objects gadgets
-    void            SetGadgetQuantity(float value);
-    float           GetGadgetQuantity();
-    //@}
-
-    //@{
-    //! Management the quality of textures
-    void            SetTextureQuality(int value);
-    int             GetTextureQuality();
-    //@}
-
     //@{
     //! Management the texture filter mode
+    // NOTE: This is an user configuration setting
     void            SetTextureFilterMode(TexFilter value);
     TexFilter       GetTextureFilterMode();
     //@}
 
     //@{
     //! Management the mipmap level for textures
+    // NOTE: This is an user configuration setting
     void            SetTextureMipmapLevel(int value);
     int             GetTextureMipmapLevel();
     //@}
 
     //@{
     //! Management the anisotropy level for textures
+    // NOTE: This is an user configuration setting
     void            SetTextureAnisotropyLevel(int value);
     int             GetTextureAnisotropyLevel();
     //@}
 
     //@{
     //! Management of shadow mapping
+    // NOTE: These are user configuration settings
+    bool            IsShadowMappingSupported();
     void            SetShadowMapping(bool value);
     bool            GetShadowMapping();
+    void            SetShadowMappingOffscreen(bool value);
+    bool            GetShadowMappingOffscreen();
+    void            SetShadowMappingOffscreenResolution(int resolution);
+    int             GetShadowMappingOffscreenResolution();
+    bool            IsShadowMappingQualitySupported();
+    void            SetShadowMappingQuality(bool value);
+    bool            GetShadowMappingQuality();
     //@}
 
     //@{
-    //! Management of shadow mapping
+    //! Management of shadow color
+    // NOTE: This is a setting configurable only in INI file
     void            SetShadowColor(float value);
-    bool            GetShadowColor();
+    float           GetShadowColor();
     //@}
 
     //@{
-    //! Management mode of toto
-    void            SetTotoMode(bool present);
-    bool            GetTotoMode();
+    //! Management of shadow range
+    // NOTE: This is a setting configurable only in INI file
+    void            SetShadowRange(float value);
+    float           GetShadowRange();
     //@}
 
     //@{
-    //! Management the mode of foreground
-    void            SetLensMode(bool present);
-    bool            GetLensMode();
-    //@}
-
-    //@{
-    //! Management the mode of water
-    void            SetWaterMode(bool present);
-    bool            GetWaterMode();
-    //@}
-
-    void            SetLightingMode(bool present);
-    bool            GetLightingMode();
-
-    //@{
-    //! Management the mode of sky
-    void            SetSkyMode(bool present);
-    bool            GetSkyMode();
+    //! Management of shadow range
+    // NOTE: This is an user configuration setting
+    void            SetMultiSample(int value);
+    int             GetMultiSample();
     //@}
 
     //@{
@@ -1179,28 +1125,25 @@ public:
     bool            GetBackForce();
     //@}
 
-    //@{
-    //! Management the mode of planets
-    void            SetPlanetMode(bool present);
-    bool            GetPlanetMode();
-    //@}
-
     //@{
     //! Managing the mode of dynamic lights.
+    // NOTE: This is an user configuration setting
     void            SetLightMode(bool present);
     bool            GetLightMode();
     //@}
 
     //@{
-    // TODO: move to more appropriate class ?
     //! Management of the indentation mode while editing (CEdit)
+    // NOTE: This is an user configuration setting
+    // TODO: Move to CSettings
     void            SetEditIndentMode(bool autoIndent);
     bool            GetEditIndentMode();
     //@}
 
     //@{
-    // TODO: move to more appropriate class ?
     //! Management of tab indent when editing (CEdit)
+    // NOTE: This is an user configuration setting
+    // TODO: Move to CSettings
     void            SetEditIndentValue(int value);
     int             GetEditIndentValue();
     //@}
@@ -1243,6 +1186,8 @@ protected:
     void        RenderShadowMap();
     //! Enables or disables shadow mapping
     void        UseShadowMapping(bool enable);
+    //! Enables or disables MSAA
+    void        UseMSAA(bool enable);
     //! Draw 3D object
     void        DrawObject(const EngineBaseObjDataTier& p4);
     //! Draws the user interface over the scene
@@ -1251,8 +1196,8 @@ protected:
     //! Updates the textures used for drawing ground spot
     void        UpdateGroundSpotTextures();
 
-    //! Draws shadows
-    void        DrawShadow();
+    //! Draws old-style shadow spots
+    void        DrawShadowSpots();
     //! Draws the gradient background
     void        DrawBackground();
     //! Draws the gradient background
@@ -1278,10 +1223,8 @@ protected:
 
     //! Creates a new tier 2 object (texture)
     EngineBaseObjTexTier&  AddLevel2(EngineBaseObject& p1, const std::string& tex1Name, const std::string& tex2Name);
-    //! Creates a new tier 3 object (LOD)
-    EngineBaseObjLODTier&  AddLevel3(EngineBaseObjTexTier &p2, LODLevel lodLevel);
-    //! Creates a new tier 4 object (data)
-    EngineBaseObjDataTier& AddLevel4(EngineBaseObjLODTier &p3, EngineTriangleType type,
+    //! Creates a new tier 3 object (data)
+    EngineBaseObjDataTier& AddLevel3(EngineBaseObjTexTier &p3, EngineTriangleType type,
                                      const Material& mat, int state);
 
     //! Create texture and add it to cache
@@ -1290,9 +1233,6 @@ protected:
     //! Tests whether the given object is visible
     bool        IsVisible(int objRank);
 
-    //! Checks whether the given distance is within LOD min & max limit
-    bool        IsWithinLODLimit(float distance, LODLevel lodLevel);
-
     //! Detects whether an object is affected by the mouse
     bool        DetectBBox(int objRank, Math::Point mouse);
 
@@ -1318,19 +1258,35 @@ protected:
     //! Updates static buffers of changed objects
     void        UpdateStaticBuffers();
 
+    void            AddBaseObjTriangles(int baseObjRank, const std::vector<VertexTex2>& vertices,
+                                        const Material& material, int state,
+                                        std::string tex1Name, std::string tex2Name);
+
+    int GetEngineState(const ModelTriangle& triangle);
+
+    struct WriteScreenShotData
+    {
+        std::unique_ptr<CImage> img;
+        std::string fileName;
+    };
+    static void WriteScreenShotThread(std::unique_ptr<WriteScreenShotData> data);
+
 protected:
     CApplication*     m_app;
+    CSystemUtils*     m_systemUtils;
     CSoundInterface*  m_sound;
     CDevice*          m_device;
-    CText*            m_text;
-    CLightManager*    m_lightMan;
-    CParticle*        m_particle;
-    CWater*           m_water;
-    CCloud*           m_cloud;
-    CLightning*       m_lightning;
-    CPlanet*          m_planet;
     CTerrain*         m_terrain;
-    CPauseManager*    m_pause;
+    std::unique_ptr<COldModelManager> m_modelManager;
+    std::unique_ptr<CText>            m_text;
+    std::unique_ptr<CLightManager>    m_lightMan;
+    std::unique_ptr<CParticle>        m_particle;
+    std::unique_ptr<CWater>           m_water;
+    std::unique_ptr<CCloud>           m_cloud;
+    std::unique_ptr<CLightning>       m_lightning;
+    std::unique_ptr<CPlanet>          m_planet;
+    std::unique_ptr<CPauseManager>    m_pause;
+    std::unique_ptr<CPyroManager> m_pyroManager;
 
     //! Last encountered error
     std::string     m_error;
@@ -1342,11 +1298,13 @@ protected:
 
     //! Whether to show stats (FPS, etc)
     bool            m_showStats;
-    std::string     m_fpsText;
     //! Rendering enabled?
     bool            m_render;
-    //! Lock for duration of movie?
-    bool            m_movieLock;
+    //! Render / hide the UI?
+    bool            m_renderInterface;
+
+    //! Screenshot mode?
+    bool            m_screenshotMode;
 
     //! Projection matrix for 3D scene
     Math::Matrix    m_matProj;
@@ -1379,7 +1337,7 @@ protected:
     //! Object parameters
     std::vector<EngineObject>     m_objects;
     //! Shadow list
-    std::vector<EngineShadow>     m_shadows;
+    std::vector<EngineShadow>     m_shadowSpots;
     //! Ground spot list
     std::vector<EngineGroundSpot> m_groundSpots;
     //! Ground mark
@@ -1402,14 +1360,10 @@ protected:
     Math::Vector    m_statisticPos;
     bool            m_updateGeometry;
     bool            m_updateStaticBuffers;
-    int             m_alphaMode;
-    bool            m_groundSpotVisible;
-    bool            m_shadowVisible;
-    bool            m_dirty;
-    bool            m_fog;
     bool            m_firstGroundSpot;
     std::string     m_secondTex;
     bool            m_backgroundFull;
+    bool            m_backgroundScale;
     std::string     m_backgroundName;
     Texture         m_backgroundTex;
     Color           m_backgroundColorUp;
@@ -1423,23 +1377,18 @@ protected:
     Texture         m_foregroundTex;
     bool            m_drawWorld;
     bool            m_drawFront;
+    float           m_terrainVision;
+    bool            m_backForce;
+    float           m_tracePrecision;
+
+    bool            m_dirty;
+    bool            m_fog;
     float           m_particleDensity;
     float           m_clippingDistance;
     float           m_lastClippingDistance;
-    float           m_objectDetail;
-    float           m_terrainVision;
-    float           m_gadgetQuantity;
-    int             m_textureQuality;
-    bool            m_totoMode;
-    bool            m_lensMode;
-    bool            m_waterMode;
-    bool            m_skyMode;
-    bool            m_backForce;
-    bool            m_planetMode;
     bool            m_lightMode;
     bool            m_editIndentMode;
     int             m_editIndentValue;
-    float           m_tracePrecision;
 
     Texture         m_shadowMap;
 
@@ -1467,10 +1416,16 @@ protected:
     bool m_shadowMapping;
     //! true enables offscreen shadow rendering
     bool m_offscreenShadowRendering;
+    //! Offscreen shadow rendering resolution
+    int m_offscreenShadowRenderingResolution;
     //! true enables higher quality shadows
     bool m_qualityShadows;
     //! Shadow color
     float m_shadowColor;
+    //! Shadow range
+    float m_shadowRange;
+    //! Number of samples for multisample rendering
+    int m_multisample;
 
     //! Map of loaded textures (by name)
     std::map<std::string, Texture> m_texNameMap;
@@ -1504,10 +1459,11 @@ protected:
 
     bool            m_debugLights;
     bool            m_debugDumpLights;
-    
+
     std::string     m_timerText;
+
+    std::unordered_map<std::string, int> m_staticMeshBaseObjects;
 };
 
 
 } // namespace Gfx
-
