@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,28 +22,33 @@
 
 #include "app/app.h"
 
-#include "common/iman.h"
 #include "common/logger.h"
 
 #include "graphics/engine/lightman.h"
 #include "graphics/engine/particle.h"
 #include "graphics/engine/terrain.h"
 
+#include "level/robotmain.h"
+
 #include "math/geometry.h"
 
-#include "object/objman.h"
-#include "object/robotmain.h"
+#include "object/object_manager.h"
+#include "object/old_object.h"
+
 #include "object/motion/motionhuman.h"
 
+#include "object/subclass/shielder.h"
+
+#include "sound/sound.h"
+
 
 // Graphics module namespace
-namespace Gfx {
+namespace Gfx
+{
 
 
 CPyro::CPyro()
 {
-    CInstanceManager::GetInstancePointer()->AddInstance(CLASS_PYRO, this, 100);
-
     m_engine      = CEngine::GetInstancePointer();
     m_main        = CRobotMain::GetInstancePointer();
     m_terrain     = m_main->GetTerrain();
@@ -51,18 +56,10 @@ CPyro::CPyro()
     m_particle    = m_engine->GetParticle();
     m_lightMan    = m_engine->GetLightManager();
     m_sound       = CApplication::GetInstancePointer()->GetSound();
-    m_object      = nullptr;
-
-    m_progress = 0.0f;
-    m_speed = 0.0f;
-    m_lightRank = -1;
-    m_soundChannel = -1;
-    LightOperFlush();
 }
 
 CPyro::~CPyro()
 {
-    CInstanceManager::GetInstancePointer()->DeleteInstance(CLASS_PYRO, this);
 }
 
 void CPyro::DeleteObject()
@@ -85,18 +82,13 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
 
     Math::Vector min, max;
     m_engine->GetObjectBBox(objRank, min, max);
-    Math::Vector pos = obj->GetPosition(0);
+    Math::Vector pos = obj->GetPosition();
 
     DisplayError(type, obj);  // displays eventual messages
 
+    for (const auto& crashSphere : obj->GetAllCrashSpheres())
     {
-        int i = 0;
-        // Copies all spheres of the object.
-        for (; i < 50; i++)
-        {
-            if ( !obj->GetCrashSphere(i, m_crashSpherePos[i], m_crashSphereRadius[i]) )  break;
-        }
-        m_crashSphereUsed = i;
+        m_crashSpheres.push_back(crashSphere.sphere);
     }
 
     // Calculates the size of the effect.
@@ -127,26 +119,25 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
     m_lastParticleSmoke = 0.0f;
     m_lightRank = -1;
 
-    if ( oType == OBJECT_TEEN28 ||
-         oType == OBJECT_TEEN31 )
-    {
-        m_pos.y = pos.y+1.0f;
-    }
-
     // Seeking the position of the battery.
-    CObject* power = obj->GetPower();
-    if ( power == nullptr )
+
+    CObject* power = nullptr;
+    if (obj->Implements(ObjectInterfaceType::Powered))
+        power = dynamic_cast<CPoweredObject*>(obj)->GetPower();
+
+    if (power == nullptr)
     {
         m_power = false;
     }
     else
     {
         m_power = true;
-        pos = power->GetPosition(0);
+        pos = power->GetPosition();
         pos.y += 1.0f;
         Math::Matrix* mat = obj->GetWorldMatrix(0);
         m_posPower = Math::Transform(*mat, pos);
     }
+
     if ( oType == OBJECT_POWER   ||
          oType == OBJECT_ATOMIC  ||
          oType == OBJECT_URANIUM ||
@@ -182,12 +173,6 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
         m_power = true;
         m_posPower = m_pos;
     }
-    if ( oType == OBJECT_SCRAP4 ||
-         oType == OBJECT_SCRAP5 )  // plastic material?
-    {
-        m_power = true;
-        m_posPower = m_pos;
-    }
 
     // Plays the sound of a pyrotechnic effect.
     if ( type == PT_FRAGT  ||
@@ -195,7 +180,7 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
          type == PT_EXPLOT ||
          type == PT_EXPLOW )
     {
-        Sound sound;
+        SoundType sound;
         if ( m_power )
         {
             sound = SOUND_EXPLOp;
@@ -264,7 +249,8 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
         {
             m_sound->Play(SOUND_DEADw, m_pos);
         }
-        if ( type == PT_SHOTH && m_object->GetSelect() )
+        assert(m_object->Implements(ObjectInterfaceType::Controllable));
+        if ( type == PT_SHOTH && dynamic_cast<CControllableObject*>(m_object)->GetSelect() )
         {
             m_sound->Play(SOUND_AIE, m_pos);
             m_sound->Play(SOUND_AIE, m_engine->GetEyePt());
@@ -275,16 +261,16 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
          m_type == PT_FRAGO ||
          m_type == PT_FRAGW )
     {
-        m_engine->DeleteShadow(m_object->GetObjectRank(0));
+        m_engine->DeleteShadowSpot(m_object->GetObjectRank(0));
     }
 
     if ( m_type == PT_DEADG )
     {
-        m_object->SetDead(true);
+        assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+        dynamic_cast<CDestroyableObject*>(m_object)->SetDying(DeathType::Dead);
 
-        CMotion* motion = m_object->GetMotion();
-        if (motion != nullptr)
-            motion->SetAction(MHS_DEADg, 1.0f);
+        assert(obj->Implements(ObjectInterfaceType::Movable));
+        dynamic_cast<CMovableObject*>(obj)->GetMotion()->SetAction(MHS_DEADg, 1.0f);
 
         m_camera->StartCentering(m_object, Math::PI*0.5f, 99.9f, 0.0f, 1.5f);
         m_camera->StartOver(CAM_OVER_EFFECT_FADEOUT_WHITE, m_pos, 1.0f);
@@ -293,13 +279,12 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
     }
     if ( m_type == PT_DEADW )
     {
-        m_object->SetDead(true);
+        assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+        dynamic_cast<CDestroyableObject*>(m_object)->SetDying(DeathType::Dead);
+
+        assert(obj->Implements(ObjectInterfaceType::Movable));
+        dynamic_cast<CMovableObject*>(obj)->GetMotion()->SetAction(MHS_DEADw, 1.0f);
 
-        CMotion* motion = m_object->GetMotion();
-        if ( motion != nullptr )
-        {
-            motion->SetAction(MHS_DEADw, 4.0f);
-        }
         m_camera->StartCentering(m_object, Math::PI*0.5f, 99.9f, 0.0f, 3.0f);
         m_camera->StartOver(CAM_OVER_EFFECT_FADEOUT_BLACK, m_pos, 1.0f);
         m_speed = 1.0f/10.0f;
@@ -315,7 +300,8 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
     }
     if ( m_type == PT_SHOTH )
     {
-        if ( m_camera->GetBlood() && m_object->GetSelect() )
+        assert(m_object->Implements(ObjectInterfaceType::Controllable));
+        if ( m_camera->GetBlood() && dynamic_cast<CControllableObject*>(m_object)->GetSelect() )
         {
             m_camera->StartOver(CAM_OVER_EFFECT_BLOOD, m_pos, force);
         }
@@ -336,7 +322,7 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
     if ( m_type == PT_WPCHECK )
     {
         m_speed = 1.0f/8.0f;
-        m_object->SetEnable(false);  // object more functional
+        m_object->SetLock(true);  // object more functional
     }
     if ( m_type == PT_FLCREATE )
     {
@@ -345,14 +331,12 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
     if ( m_type == PT_FLDELETE )
     {
         m_speed = 1.0f/2.0f;
-        m_object->SetEnable(false);  // object more functional
+        m_object->SetLock(true);  // object more functional
     }
     if ( m_type == PT_RESET )
     {
         m_speed = 1.0f/2.0f;
-        m_object->SetPosition(0, m_object->GetResetPosition());
-        m_object->SetAngle(0, m_object->GetResetAngle());
-        m_object->SetZoom(0, 0.0f);
+        m_resetAngle = m_object->GetRotationY();
     }
     if ( m_type == PT_FINDING )
     {
@@ -367,7 +351,7 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
          m_type == PT_EXPLOW )
     {
         CreateTriangle(obj, oType, 0);
-        m_engine->DeleteShadow(m_object->GetObjectRank(0));
+        m_engine->DeleteShadowSpot(m_object->GetObjectRank(0));
         ExploStart();
     }
 
@@ -398,7 +382,7 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
         Math::Matrix* mat = obj->GetWorldMatrix(0);
         m_pos = Math::Transform(*mat, pos);
 
-        m_engine->DeleteShadow(m_object->GetObjectRank(0));
+        m_engine->DeleteShadowSpot(m_object->GetObjectRank(0));
     }
 
     if ( m_type != PT_EGG  &&
@@ -472,9 +456,7 @@ bool CPyro::Create(PyroType type, CObject* obj, float force)
          m_type == PT_FRAGW  ||
          m_type == PT_SPIDER ||
          m_type == PT_EGG    ||
-        (m_type == PT_EXPLOT && oType == OBJECT_MOBILEtg) ||
-        (m_type == PT_EXPLOT && oType == OBJECT_TEEN28  ) ||
-        (m_type == PT_EXPLOT && oType == OBJECT_TEEN31  ) )
+        (m_type == PT_EXPLOT && oType == OBJECT_MOBILEtg) )
     {
         for (int part = 0; part < OBJECTMAXPART; part++)
         {
@@ -629,7 +611,7 @@ bool CPyro::EventProcess(const Event &event)
 
     if (m_soundChannel != -1 && m_object != nullptr)
     {
-        Math::Vector pos = m_object->GetPosition(0);
+        Math::Vector pos = m_object->GetPosition();
         m_sound->Position(m_soundChannel, pos);
 
         if (m_lightRank != -1)
@@ -644,18 +626,19 @@ bool CPyro::EventProcess(const Event &event)
     {
         m_lastParticle = m_time;
 
-        if ( m_crashSphereUsed > 0 )
+        if (m_crashSpheres.size() > 0)
         {
-            int i = rand()%m_crashSphereUsed;
-            Math::Vector pos = m_crashSpherePos[i];
-            pos.x += (Math::Rand()-0.5f)*m_crashSphereRadius[i]*2.0f;
-            pos.z += (Math::Rand()-0.5f)*m_crashSphereRadius[i]*2.0f;
+            int i = rand() % m_crashSpheres.size();
+            Math::Vector pos = m_crashSpheres[i].pos;
+            float radius = m_crashSpheres[i].radius;
+            pos.x += (Math::Rand()-0.5f)*radius*2.0f;
+            pos.z += (Math::Rand()-0.5f)*radius*2.0f;
             Math::Vector speed;
-            speed.x = (Math::Rand()-0.5f)*m_crashSphereRadius[i]*0.5f;
-            speed.z = (Math::Rand()-0.5f)*m_crashSphereRadius[i]*0.5f;
-            speed.y = Math::Rand()*m_crashSphereRadius[i]*1.0f;
+            speed.x = (Math::Rand()-0.5f)*radius*0.5f;
+            speed.z = (Math::Rand()-0.5f)*radius*0.5f;
+            speed.y = Math::Rand()*radius*1.0f;
             Math::Point dim;
-            dim.x = Math::Rand()*m_crashSphereRadius[i]*0.5f+m_crashSphereRadius[i]*0.75f*m_force;
+            dim.x = Math::Rand()*radius*0.5f+radius*0.75f*m_force;
             dim.y = dim.x;
             m_particle->CreateParticle(pos, speed, dim, PARTISMOKE1, 3.0f);
         }
@@ -726,16 +709,17 @@ bool CPyro::EventProcess(const Event &event)
     {
         m_lastParticle = m_time;
 
-        if ( m_crashSphereUsed > 0 )
+        if (m_crashSpheres.size() > 0)
         {
-            int i = rand()%m_crashSphereUsed;
-            Math::Vector pos = m_crashSpherePos[i];
-            pos.x += (Math::Rand()-0.5f)*m_crashSphereRadius[i]*2.0f;
-            pos.z += (Math::Rand()-0.5f)*m_crashSphereRadius[i]*2.0f;
+            int i = rand() % m_crashSpheres.size();
+            Math::Vector pos = m_crashSpheres[i].pos;
+            float radius = m_crashSpheres[i].radius;
+            pos.x += (Math::Rand()-0.5f)*radius*2.0f;
+            pos.z += (Math::Rand()-0.5f)*radius*2.0f;
             Math::Vector speed;
-            speed.x = (Math::Rand()-0.5f)*m_crashSphereRadius[i]*0.5f;
-            speed.z = (Math::Rand()-0.5f)*m_crashSphereRadius[i]*0.5f;
-            speed.y = Math::Rand()*m_crashSphereRadius[i]*1.0f;
+            speed.x = (Math::Rand()-0.5f)*radius*0.5f;
+            speed.z = (Math::Rand()-0.5f)*radius*0.5f;
+            speed.y = Math::Rand()*radius*1.0f;
             Math::Point dim;
             dim.x = 1.0f*m_force;
             dim.y = dim.x;
@@ -905,19 +889,19 @@ bool CPyro::EventProcess(const Event &event)
 
         if(m_object != nullptr)
         {
-            Math::Vector angle = m_object->GetAngle(0);
+            Math::Vector angle = m_object->GetRotation();
             angle.y = m_progress*20.0f;
             angle.x = sinf(m_progress*49.0f)*0.3f;
             angle.z = sinf(m_progress*47.0f)*0.2f;
-            m_object->SetAngle(0, angle);
+            m_object->SetRotation(angle);
 
             Math::Vector pos = m_pos;
             pos.y += factor;
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
 
             if ( m_progress > 0.85f )
             {
-                m_object->SetZoom(0, 1.0f-(m_progress-0.85f)/0.15f);
+                m_object->SetScale(1.0f-(m_progress-0.85f)/0.15f);
             }
         }
     }
@@ -944,12 +928,12 @@ bool CPyro::EventProcess(const Event &event)
 
         if(m_object != nullptr)
         {
-            Math::Vector angle = m_object->GetAngle(0);
+            Math::Vector angle = m_object->GetRotation();
             angle.x = sinf(m_progress*49.0f)*0.3f*(1.0f-m_progress);
             angle.z = sinf(m_progress*47.0f)*0.2f*(1.0f-m_progress);
-            m_object->SetAngle(0, angle);
+            m_object->SetRotation(angle);
 
-            m_object->SetZoom(0, m_progress);
+            m_object->SetScale(m_progress);
         }
     }
 
@@ -975,13 +959,13 @@ bool CPyro::EventProcess(const Event &event)
 
         if(m_object != nullptr)
         {
-            Math::Vector angle = m_object->GetAngle(0);
+            Math::Vector angle = m_object->GetRotation();
             angle.y = m_progress*20.0f;
             angle.x = sinf(m_progress*49.0f)*0.3f;
             angle.z = sinf(m_progress*47.0f)*0.2f;
-            m_object->SetAngle(0, angle);
+            m_object->SetRotation(angle);
 
-            m_object->SetZoom(0, 1.0f-m_progress);
+            m_object->SetScale(1.0f-m_progress);
         }
     }
 
@@ -1019,9 +1003,9 @@ bool CPyro::EventProcess(const Event &event)
 
         if(m_object != nullptr)
         {
-            Math::Vector angle = m_object->GetResetAngle();
-            m_object->SetAngleY(0, angle.y-powf((1.0f-m_progress)*5.0f, 2.0f));
-            m_object->SetZoom(0, m_progress);
+            float angle = m_resetAngle;
+            m_object->SetRotationY(angle-powf((1.0f-m_progress)*5.0f, 2.0f));
+            m_object->SetScale(m_progress);
         }
     }
 
@@ -1059,7 +1043,7 @@ bool CPyro::EventProcess(const Event &event)
 
             float factor = m_size/25.0f;  // 1 = standard size
 
-            Math::Vector pos = m_object->GetPosition(0);
+            Math::Vector pos = m_object->GetPosition();
             pos.y -= m_object->GetCharacter()->height;
             pos.x += (Math::Rand()-0.5f)*(4.0f+8.0f*m_progress)*factor;
             pos.z += (Math::Rand()-0.5f)*(4.0f+8.0f*m_progress)*factor;
@@ -1072,7 +1056,7 @@ bool CPyro::EventProcess(const Event &event)
             dim.y = dim.x;
             m_particle->CreateParticle(pos, speed, dim, PARTIFLAME, 2.0f, 0.0f, 0.2f);
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.y -= m_object->GetCharacter()->height;
             pos.x += (Math::Rand()-0.5f)*(2.0f+4.0f*m_progress)*factor;
             pos.z += (Math::Rand()-0.5f)*(2.0f+4.0f*m_progress)*factor;
@@ -1083,7 +1067,7 @@ bool CPyro::EventProcess(const Event &event)
             dim.y = dim.x;
             m_particle->CreateParticle(pos, speed, dim, PARTIFLAME, 2.0f, 0.0f, 0.2f);
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.y -= 2.0f;
             pos.x += (Math::Rand()-0.5f)*5.0f*factor;
             pos.z += (Math::Rand()-0.5f)*5.0f*factor;
@@ -1109,7 +1093,7 @@ bool CPyro::EventProcess(const Event &event)
             {
                 float prog = (m_progress-0.8f)/0.2f;  // 0..1
                 speed.y = -prog*6.0f;  // sinks into the ground
-                m_object->SetZoom(0, 1.0f-prog*0.5f);
+                m_object->SetScale(1.0f-prog*0.5f);
             }
             m_object->SetLinVibration(speed);
         }
@@ -1122,7 +1106,7 @@ bool CPyro::EventProcess(const Event &event)
         {
             m_lastParticle = m_time;
 
-            Math::Vector pos = m_object->GetPosition(0);
+            Math::Vector pos = m_object->GetPosition();
             pos.y += 1.5f;
             Math::Vector speed;
             speed.x = (Math::Rand()-0.5f)*10.0f;
@@ -1144,7 +1128,7 @@ bool CPyro::EventProcess(const Event &event)
         {
             m_lastParticle = m_time;
 
-            Math::Vector pos = m_object->GetPosition(0);
+            Math::Vector pos = m_object->GetPosition();
             pos.y -= 2.0f;
             pos.x += (Math::Rand()-0.5f)*10.0f;
             pos.z += (Math::Rand()-0.5f)*10.0f;
@@ -1218,16 +1202,15 @@ Error CPyro::IsEnded()
 
     if ( m_type == PT_FLCREATE )
     {
-        m_object->SetAngleX(0, 0.0f);
-        m_object->SetAngleZ(0, 0.0f);
-        m_object->SetZoom(0, 1.0f);
+        m_object->SetRotationX(0.0f);
+        m_object->SetRotationZ(0.0f);
+        m_object->SetScale(1.0f);
     }
 
     if ( m_type == PT_RESET )
     {
-        m_object->SetPosition(0, m_object->GetResetPosition());
-        m_object->SetAngle(0, m_object->GetResetAngle());
-        m_object->SetZoom(0, 1.0f);
+        m_object->SetRotationY(m_resetAngle);
+        m_object->SetScale(1.0f);
     }
 
     if ( m_lightRank != -1 )
@@ -1315,7 +1298,7 @@ void CPyro::DisplayError(PyroType type, CObject* obj)
              oType == OBJECT_END      )
         {
             err = ERR_DELETEBUILDING;
-            m_main->DisplayError(err, obj->GetPosition(0), 5.0f);
+            m_main->DisplayError(err, obj->GetPosition(), 5.0f);
             return;
         }
 
@@ -1356,53 +1339,60 @@ void CPyro::DeleteObject(bool primary, bool secondary)
 {
     if (m_object == nullptr) return;
 
-    if (m_object->GetResetCap() == RESET_MOVE)  // resettable object?
-    {
-        m_object->SetEnable(false);  // object cache and inactive
-        Math::Vector pos = m_object->GetPosition(0);
-        pos.y = -100.0f;
-        m_object->SetPosition(0, pos);
-        return;
-    }
-
     ObjectType type = m_object->GetType();
     if ( secondary              &&
          type != OBJECT_FACTORY &&
          type != OBJECT_NUCLEAR &&
          type != OBJECT_ENERGY )
     {
-        CObject* sub = m_object->GetPower();
-        if ( sub != 0 )
+        if (m_object->Implements(ObjectInterfaceType::Powered))
         {
-            sub->DeleteObject();  // removes the battery
-            delete sub;
-            m_object->SetPower(0);
+            CPoweredObject* poweredObject = dynamic_cast<CPoweredObject*>(m_object);
+            CObject* sub = poweredObject->GetPower();
+            if (sub != nullptr)
+            {
+                CObjectManager::GetInstancePointer()->DeleteObject(sub);
+                poweredObject->SetPower(nullptr);
+            }
         }
 
-        sub = m_object->GetFret();
-        if ( sub != 0 )
+        if (m_object->Implements(ObjectInterfaceType::Carrier))
         {
-            sub->DeleteObject();  // removes the object transported
-            delete sub;
-            m_object->SetFret(nullptr);
+            CCarrierObject* carrierObject = dynamic_cast<CCarrierObject*>(m_object);
+            CObject* sub = carrierObject->GetCargo();
+            if (sub != nullptr)
+            {
+                CObjectManager::GetInstancePointer()->DeleteObject(sub);
+                carrierObject->SetCargo(nullptr);
+            }
         }
     }
 
     if (primary)
     {
-        CObject* truck = m_object->GetTruck();
-        if ( truck != 0 )  // object carries?
+        if (m_object->Implements(ObjectInterfaceType::Transportable))
         {
-            if (truck->GetPower() == m_object)
-                truck->SetPower(nullptr);
+            // TODO: this should be handled in the object's destructor
+            CObject* transporter = dynamic_cast<CTransportableObject*>(m_object)->GetTransporter();
+            if (transporter != nullptr)
+            {
+                if (transporter->Implements(ObjectInterfaceType::Powered))
+                {
+                    CPoweredObject* powered = dynamic_cast<CPoweredObject*>(transporter);
+                    if (powered->GetPower() == m_object)
+                        powered->SetPower(nullptr);
+                }
 
-            if (truck->GetFret() == m_object)
-                truck->SetFret(nullptr);
+                if (transporter->Implements(ObjectInterfaceType::Carrier))
+                {
+                    CCarrierObject* carrier = dynamic_cast<CCarrierObject*>(transporter);
+                    if (carrier->GetCargo() == m_object)
+                        carrier->SetCargo(nullptr);
+                }
+            }
         }
 
-        CObject* sub = m_object;
-        sub->DeleteObject();  // removes the object (*)
-        delete sub;
+        CObjectManager::GetInstancePointer()->DeleteObject(m_object);
         m_object = nullptr;
     }
 }
@@ -1424,8 +1414,7 @@ void CPyro::CreateTriangle(CObject* obj, ObjectType oType, int part)
         oType == OBJECT_ATOMIC   ||
         oType == OBJECT_URANIUM  ||
         oType == OBJECT_TNT      ||
-        oType == OBJECT_BOMB     ||
-        oType == OBJECT_TEEN28)
+        oType == OBJECT_BOMB     )
     {
         percent = 0.75f;
     }
@@ -1434,17 +1423,8 @@ void CPyro::CreateTriangle(CObject* obj, ObjectType oType, int part)
         percent = 0.50f;
     }
 
-    LODLevel lodLevel = LOD_High;
-
-    if (oType == OBJECT_MOTHER ||
-        oType == OBJECT_TEEN28 ||
-        oType == OBJECT_TEEN31)
-    {
-        lodLevel = LOD_Constant;
-    }
-
     std::vector<EngineTriangle> buffer;
-    total = m_engine->GetPartialTriangles(objRank, lodLevel, percent, 100, buffer);
+    total = m_engine->GetPartialTriangles(objRank, percent, 100, buffer);
 
     for (int i = 0; i < total; i++)
     {
@@ -1551,21 +1531,22 @@ void CPyro::ExploStart()
 {
     m_burnType = m_object->GetType();
 
-    Math::Vector oPos = m_object->GetPosition(0);
+    Math::Vector oPos = m_object->GetPosition();
     m_burnFall = m_terrain->GetHeightToFloor(oPos, true);
 
     m_object->Simplify();
     m_object->SetLock(true);  // ruin not usable yet
-    m_object->SetExplo(true);  // being destroyed
+    assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+    dynamic_cast<CDestroyableObject*>(m_object)->SetDying(DeathType::Exploding);  // being destroyed
     m_object->FlatParent();
 
-    if ( m_object->GetSelect() )
+    if ( m_object->Implements(ObjectInterfaceType::Controllable) && dynamic_cast<CControllableObject*>(m_object)->GetSelect() )
     {
-        m_object->SetSelect(false);  // deselects the object
+        dynamic_cast<CControllableObject*>(m_object)->SetSelect(false);  // deselects the object
         m_camera->SetType(CAM_TYPE_EXPLO);
         m_main->DeselectAll();
     }
-    m_object->DeleteDeselList(m_object);
+    m_main->RemoveFromSelectionHistory(m_object);
 
     for (int i = 0; i < OBJECTMAXPART; i++)
     {
@@ -1583,7 +1564,9 @@ void CPyro::ExploStart()
             m_engine->ChangeSecondTexture(objRank, "dirty04.png");
         }
 
-        Math::Vector pos = m_object->GetPosition(i);
+        // TODO: temporary hack (hopefully)
+        assert(m_object->Implements(ObjectInterfaceType::Old));
+        Math::Vector pos = dynamic_cast<COldObject*>(m_object)->GetPartPosition(i);
 
         Math::Vector speed;
         float weight;
@@ -1624,19 +1607,19 @@ void CPyro::BurnStart()
 {
     m_burnType = m_object->GetType();
 
-    Math::Vector oPos = m_object->GetPosition(0);
+    Math::Vector oPos = m_object->GetPosition();
     m_burnFall = m_terrain->GetHeightToFloor(oPos, true);
 
     m_object->Simplify();
     m_object->SetLock(true);  // ruin not usable yet
 
-    if ( m_object->GetSelect() )
+    if ( m_object->Implements(ObjectInterfaceType::Controllable) && dynamic_cast<CControllableObject*>(m_object)->GetSelect() )
     {
-        m_object->SetSelect(false);  // deselects the object
+        dynamic_cast<CControllableObject*>(m_object)->SetSelect(false);  // deselects the object
         m_camera->SetType(CAM_TYPE_EXPLO);
         m_main->DeselectAll();
     }
-    m_object->DeleteDeselList(m_object);
+    m_main->RemoveFromSelectionHistory(m_object);
 
     for (int i = 0; i < OBJECTMAXPART; i++)
     {
@@ -1737,15 +1720,6 @@ void CPyro::BurnStart()
         angle.y = 0.0f;
         angle.z = (Math::Rand()-0.5f)*0.4f;
     }
-    else if ( m_burnType == OBJECT_TEEN31 )  // basket?
-    {
-        pos.x =   0.0f;
-        pos.y =   0.0f;
-        pos.z =   0.0f;
-        angle.x = (Math::Rand()-0.5f)*0.8f;
-        angle.y = 0.0f;
-        angle.z = (Math::Rand()-0.5f)*0.2f;
-    }
     else
     {
         pos.x =   0.0f;
@@ -2104,11 +2078,15 @@ void CPyro::BurnStart()
 
 void CPyro::BurnAddPart(int part, Math::Vector pos, Math::Vector angle)
 {
+    // TODO: temporary hack (hopefully)
+    assert(m_object->Implements(ObjectInterfaceType::Old));
+    COldObject* oldObj = dynamic_cast<COldObject*>(m_object);
+
     int i = m_burnPartTotal;
     m_burnPart[i].part = part;
-    m_burnPart[i].initialPos = m_object->GetPosition(part);
+    m_burnPart[i].initialPos = oldObj->GetPartPosition(part);
     m_burnPart[i].finalPos = m_burnPart[i].initialPos+pos;
-    m_burnPart[i].initialAngle = m_object->GetAngle(part);
+    m_burnPart[i].initialAngle = oldObj->GetPartRotation(part);
     m_burnPart[i].finalAngle = m_burnPart[i].initialAngle+angle;
 
     m_burnPartTotal++;
@@ -2116,11 +2094,6 @@ void CPyro::BurnAddPart(int part, Math::Vector pos, Math::Vector angle)
 
 void CPyro::BurnProgress()
 {
-    if ( m_burnType == OBJECT_TEEN31 )  // basket?
-    {
-        m_object->SetZoomY(0, 1.0f-m_progress*0.5f);  // slight flattening
-    }
-
     for (int i = 0; i < m_burnPartTotal; i++)
     {
         Math::Vector pos = m_burnPart[i].initialPos + m_progress*(m_burnPart[i].finalPos-m_burnPart[i].initialPos);
@@ -2130,15 +2103,23 @@ void CPyro::BurnProgress()
             if ( h > m_burnFall )  h = m_burnFall;
             pos.y -= h;
         }
-        m_object->SetPosition(m_burnPart[i].part, pos);
+
+        // TODO: temporary hack (hopefully)
+        assert(m_object->Implements(ObjectInterfaceType::Old));
+        COldObject* oldObj = dynamic_cast<COldObject*>(m_object);
+
+        oldObj->SetPartPosition(m_burnPart[i].part, pos);
 
         pos = m_burnPart[i].initialAngle + m_progress*(m_burnPart[i].finalAngle-m_burnPart[i].initialAngle);
-        m_object->SetAngle(m_burnPart[i].part, pos);
+        oldObj->SetPartRotation(m_burnPart[i].part, pos);
     }
 
-    CObject* sub = m_object->GetPower();
-    if (sub != nullptr)  // is there a battery?
-        sub->SetZoomY(0, 1.0f - m_progress);  // complete flattening
+    if (m_object->Implements(ObjectInterfaceType::Powered))
+    {
+        CObject* sub = dynamic_cast<CPoweredObject*>(m_object)->GetPower();
+        if (sub != nullptr)  // is there a battery?
+            sub->SetScaleY(1.0f - m_progress);  // complete flattening
+    }
 }
 
 bool CPyro::BurnIsKeepPart(int part)
@@ -2189,23 +2170,20 @@ void CPyro::BurnTerminate()
          m_burnType == OBJECT_START    ||
          m_burnType == OBJECT_END      )
     {
-        m_object->SetType(OBJECT_RUINfactory);  // others become a ruin
-        m_object->SetLock(false);
+        m_object->SetType(OBJECT_RUINfactory); // Ruin
     }
     else
     {
-        m_object->SetType(OBJECT_RUINmobilew1);  // others become a ruin
-        m_object->SetLock(false);
+        m_object->SetType(OBJECT_RUINmobilew1); // Wreck (recoverable by Recycler)
     }
-
-    m_object->SetBurn(false);  // ruin usable (c-e-d. recoverable)
+    m_object->SetLock(false);
 }
 
 void CPyro::FallStart()
 {
-    m_object->SetBurn(true);  // usable
+    m_object->SetLock(true);  // usable
 
-    Math::Vector pos = m_object->GetPosition(0);
+    Math::Vector pos = m_object->GetPosition();
     m_fallFloor = m_terrain->GetFloorLevel(pos);
     m_fallSpeed = 0.0f;
     m_fallBulletTime = 0.0f;
@@ -2214,98 +2192,48 @@ void CPyro::FallStart()
 
 CObject* CPyro::FallSearchBeeExplo()
 {
-    Math::Vector iPos;
-    float iRadius;
-    m_object->GetCrashSphere(0, iPos, iRadius);
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    auto bulletCrashSphere = m_object->GetFirstCrashSphere();
+
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        CObject* pObj = it.second;
+        if (obj == m_object) continue;
+        if (obj->GetType() == OBJECT_BEE) continue;
+        if ( !obj->Implements(ObjectInterfaceType::Destroyable) )  continue;
 
-        ObjectType oType = pObj->GetType();
-        if ( oType != OBJECT_HUMAN    &&
-             oType != OBJECT_MOBILEfa &&
-             oType != OBJECT_MOBILEta &&
-             oType != OBJECT_MOBILEwa &&
-             oType != OBJECT_MOBILEia &&
-             oType != OBJECT_MOBILEfc &&
-             oType != OBJECT_MOBILEtc &&
-             oType != OBJECT_MOBILEwc &&
-             oType != OBJECT_MOBILEic &&
-             oType != OBJECT_MOBILEfi &&
-             oType != OBJECT_MOBILEti &&
-             oType != OBJECT_MOBILEwi &&
-             oType != OBJECT_MOBILEii &&
-             oType != OBJECT_MOBILEfs &&
-             oType != OBJECT_MOBILEts &&
-             oType != OBJECT_MOBILEws &&
-             oType != OBJECT_MOBILEis &&
-             oType != OBJECT_MOBILErt &&
-             oType != OBJECT_MOBILErc &&
-             oType != OBJECT_MOBILErr &&
-             oType != OBJECT_MOBILErs &&
-             oType != OBJECT_MOBILEsa &&
-             oType != OBJECT_MOBILEtg &&
-             oType != OBJECT_MOBILEft &&
-             oType != OBJECT_MOBILEtt &&
-             oType != OBJECT_MOBILEwt &&
-             oType != OBJECT_MOBILEit &&
-             oType != OBJECT_MOBILEdr &&
-             oType != OBJECT_BASE     &&
-             oType != OBJECT_DERRICK  &&
-             oType != OBJECT_STATION  &&
-             oType != OBJECT_FACTORY  &&
-             oType != OBJECT_REPAIR   &&
-             oType != OBJECT_DESTROYER&&
-             oType != OBJECT_CONVERT  &&
-             oType != OBJECT_TOWER    &&
-             oType != OBJECT_RESEARCH &&
-             oType != OBJECT_RADAR    &&
-             oType != OBJECT_INFO     &&
-             oType != OBJECT_ENERGY   &&
-             oType != OBJECT_LABO     &&
-             oType != OBJECT_NUCLEAR  &&
-             oType != OBJECT_PARA     &&
-             oType != OBJECT_SAFE     &&
-             oType != OBJECT_HUSTON   &&
-             oType != OBJECT_METAL    &&
-             oType != OBJECT_POWER    &&
-             oType != OBJECT_ATOMIC   )  continue;
+        if (IsObjectBeingTransported(obj)) continue;
 
-        if ( pObj->GetTruck() != 0 )  continue;  // object transported?
+        Math::Vector oPos = obj->GetPosition();
 
-        Math::Vector oPos = pObj->GetPosition(0);
-
-        float distance;
-
-        float shieldRadius = pObj->GetShieldRadius();
-        if ( shieldRadius > 0.0f )
+        if (obj->GetType() == OBJECT_MOBILErs)
         {
-            distance = Math::Distance(oPos, iPos);
-            if (distance <= shieldRadius) return pObj;
+            float shieldRadius = dynamic_cast<CShielder*>(obj)->GetActiveShieldRadius();
+            if ( shieldRadius > 0.0f )
+            {
+                float distance = Math::Distance(oPos, bulletCrashSphere.sphere.pos);
+                if (distance <= shieldRadius)
+                    return obj;
+            }
         }
 
-        if ( oType == OBJECT_BASE )
+        if ( obj->GetType() == OBJECT_BASE )
         {
-            distance = Math::Distance(oPos, iPos);
-            if (distance < 25.0f) return pObj;
+            float distance = Math::Distance(oPos, bulletCrashSphere.sphere.pos);
+            if (distance < 25.0f)
+                return obj;
         }
 
         // Test the center of the object, which is necessary for objects
         // that have no sphere in the center (station).
-        distance = Math::Distance(oPos, iPos)-4.0f;
-        if (distance < 5.0f) return pObj;
+        float distance = Math::Distance(oPos, bulletCrashSphere.sphere.pos)-4.0f;
+        if (distance < 5.0f)
+            return obj;
 
         // Test with all spheres of the object.
-        Math::Vector ooPos;
-        float ooRadius;
-        int j = 0;
-        while (pObj->GetCrashSphere(j++, ooPos, ooRadius))
+        for (const auto& objCrashSphere : obj->GetAllCrashSpheres())
         {
-            distance = Math::Distance(ooPos, iPos);
-            if (distance <= iRadius+ooRadius)
+            if (Math::DistanceBetweenSpheres(objCrashSphere.sphere, bulletCrashSphere.sphere) <= 0.0f)
             {
-                return pObj;
+                return obj;
             }
         }
     }
@@ -2319,7 +2247,7 @@ void CPyro::FallProgress(float rTime)
 
     m_fallSpeed += rTime*50.0f;  // v2 = v1 + a*dt
     Math::Vector pos;
-    pos = m_object->GetPosition(0);
+    pos = m_object->GetPosition();
     pos.y -= m_fallSpeed*rTime;  // dd -= v2*dt
 
     bool floor = false;
@@ -2329,7 +2257,7 @@ void CPyro::FallProgress(float rTime)
         pos.y = m_fallFloor;
         floor = true;
     }
-    m_object->SetPosition(0, pos);
+    m_object->SetPosition(pos);
 
     if (m_object->GetType() == OBJECT_BULLET)
     {
@@ -2344,12 +2272,13 @@ void CPyro::FallProgress(float rTime)
             {
                 if (floor)  // reaches the ground?
                 {
-                    m_object->ExploObject(EXPLO_BOUM, 0.0f);  // start explosion
+                    assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+                    dynamic_cast<CDestroyableObject*>(m_object)->DestroyObject(DestructionType::Explosion);
                 }
             }
             else
             {
-                if (obj->GetShieldRadius() > 0.0f)  // protected by shield?
+                if (obj->GetType() == OBJECT_MOBILErs && dynamic_cast<CShielder*>(obj)->GetActiveShieldRadius() > 0.0f)  // protected by shield?
                 {
                     m_particle->CreateParticle(pos, Math::Vector(0.0f, 0.0f, 0.0f),
                                                 Math::Point(6.0f, 6.0f), PARTIGUNDEL, 2.0f, 0.0f, 0.0f);
@@ -2359,13 +2288,15 @@ void CPyro::FallProgress(float rTime)
                 }
                 else
                 {
-                    if (obj->ExploObject(EXPLO_BOUM, 1.0f))  // start explosion
+                    assert(obj->Implements(ObjectInterfaceType::Damageable));
+                    if (dynamic_cast<CDamageableObject*>(obj)->DamageObject(DamageType::FallingObject))
                     {
                         DeleteObject(true, true);  // removes the ball
                     }
                     else
                     {
-                        m_object->ExploObject(EXPLO_BOUM, 0.0f);  // start explosion
+                        assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+                        dynamic_cast<CDestroyableObject*>(m_object)->DestroyObject(DestructionType::Explosion);
                     }
                 }
             }
@@ -2380,42 +2311,41 @@ void CPyro::FallProgress(float rTime)
 
 Error CPyro::FallIsEnded()
 {
-    if (m_fallEnding || m_object == 0) return ERR_STOP;
+    if (m_fallEnding || m_object == nullptr) return ERR_STOP;
 
-    Math::Vector pos = m_object->GetPosition(0);
+    Math::Vector pos = m_object->GetPosition();
     if (pos.y > m_fallFloor) return ERR_CONTINUE;
 
     m_sound->Play(SOUND_BOUM, pos);
-    m_object->SetBurn(false);  // usable again
+    m_object->SetLock(false);  // usable again
 
     return ERR_STOP;
 }
 
 void CPyro::LightOperFlush()
 {
-    m_lightOperTotal = 0;
+    m_lightOper.clear();
 }
 
 void CPyro::LightOperAdd(float progress, float intensity, float r, float g, float b)
 {
-    int i = m_lightOperTotal;
+    PyroLightOper lightOper;
 
-    m_lightOper[i].progress  = progress;
-    m_lightOper[i].intensity = intensity;
-    m_lightOper[i].color.r   = r;
-    m_lightOper[i].color.g   = g;
-    m_lightOper[i].color.b   = b;
+    lightOper.progress  = progress;
+    lightOper.intensity = intensity;
+    lightOper.color.r   = r;
+    lightOper.color.g   = g;
+    lightOper.color.b   = b;
 
-    m_lightOperTotal++;
+    m_lightOper.push_back(lightOper);
 }
 
 void CPyro::LightOperFrame(float rTime)
 {
-    for (int i = 0; i < m_lightOperTotal; i++)
+    for (std::size_t i = 1; i < m_lightOper.size(); i++)
     {
         if ( m_progress < m_lightOper[i].progress )
         {
-            assert(i > 0); // TODO: if assert fails, fix the code
             float progress = (m_progress-m_lightOper[i-1].progress) / (m_lightOper[i].progress-m_lightOper[i-1].progress);
 
             float intensity = m_lightOper[i-1].intensity + (m_lightOper[i].intensity-m_lightOper[i-1].intensity)*progress;
@@ -2433,4 +2363,3 @@ void CPyro::LightOperFrame(float rTime)
 
 
 } // namespace Gfx
-
