@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,8 @@
 
 
 // Graphics module namespace
-namespace Gfx {
+namespace Gfx
+{
 
 
 CTerrain::CTerrain()
@@ -57,6 +58,11 @@ CTerrain::CTerrain()
     m_defaultHardness = 0.5f;
     m_useMaterials    = false;
 
+    m_flyingMaxHeight = 0.0f;
+    m_maxMaterialID = 0;
+    m_materialAutoID = 0;
+    m_materialPointCount = 0;
+
     FlushBuildingLevel();
     FlushFlyingLimit();
     FlushMaterials();
@@ -123,7 +129,7 @@ bool CTerrain::InitTextures(const std::string& baseName, int* table, int dx, int
     m_useMaterials = false;
 
     m_texBaseName = baseName;
-    size_t pos = baseName.rfind('.');
+    auto pos = baseName.rfind('.');
     if(pos < baseName.find_last_of('/')) pos = std::string::npos; // If last . is not a part of filename (some directory, possibly . or ..)
 
     if (pos == std::string::npos)
@@ -339,50 +345,50 @@ bool CTerrain::RandomizeRelief()
     // Perlin noise
     // Based on Python implementation by Marek Rogalski (mafik)
     // http://amt2014.pl/archiwum/perlin.py
-    
+
     int size = (m_mosaicCount*m_brickCount)+1;
-    const int ilosc_oktaw = 6;
-    
-    float* oktawy[ilosc_oktaw];
-    for(int i=0; i<ilosc_oktaw; i++)
+    const int octaveCount = 6;
+
+    std::unique_ptr<float[]> octaves[octaveCount];
+    for(int i = 0; i < octaveCount; i++)
     {
         int pxCount = static_cast<int>(pow(2, (i+1)*2));
-        oktawy[i] = new float[pxCount];
-        for(int j=0; j<pxCount; j++)
+        octaves[i] = MakeUniqueArray<float>(pxCount);
+        for(int j = 0; j < pxCount; j++)
         {
-            oktawy[i][j] = Math::Rand();
+            octaves[i][j] = Math::Rand();
         }
     }
-    
-    for(int y2=0; y2 < size; y2++)
+
+    for(int y2 = 0; y2 < size; y2++)
     {
         float y = static_cast<float>(y2) / size;
-        for(int x2=0; x2 < size; x2++)
+        for(int x2 = 0; x2 < size; x2++)
         {
             float x = static_cast<float>(x2) / size;
-            
-            float wart = 0;
-            for(int i=0; i<ilosc_oktaw; i++)
+
+            float value = 0;
+            for(int i = 0; i < octaveCount; i++)
             {
-                int rozmiar_oktawy = sqrt(static_cast<int>(pow(2, (i+1)*2)));
+                int octaveSize = sqrt(static_cast<int>(pow(2, (i+1)*2)));
                 double xi, yi, a, b;
-                a = modf(x * (rozmiar_oktawy-1), &xi);
-                b = modf(y * (rozmiar_oktawy-1), &yi);
-                
-                float lg = oktawy[i][static_cast<int>(yi * rozmiar_oktawy + xi)];
-                float pg = oktawy[i][static_cast<int>(yi * rozmiar_oktawy + xi + 1)];
-                float ld = oktawy[i][static_cast<int>((yi+1) * rozmiar_oktawy + xi)];
-                float pd = oktawy[i][static_cast<int>((yi+1) * rozmiar_oktawy + xi + 1)];
-                
+                a = modf(x * (octaveSize-1), &xi);
+                b = modf(y * (octaveSize-1), &yi);
+
+                float lg = octaves[i][static_cast<int>(yi * octaveSize + xi)];
+                float pg = octaves[i][static_cast<int>(yi * octaveSize + xi + 1)];
+                float ld = octaves[i][static_cast<int>((yi+1) * octaveSize + xi)];
+                float pd = octaves[i][static_cast<int>((yi+1) * octaveSize + xi + 1)];
+
                 float g = pg * a + lg * (1-a);
                 float d = pd * a + ld * (1-a);
                 float res = d * b + g * (1-b);
-                wart += res;
+                value += res;
             }
-            
-            wart /= ilosc_oktaw;
 
-            m_relief[x2+y2*size] = wart * 255.0f;
+            value /= octaveCount;
+
+            m_relief[x2+y2*size] = value * 255.0f;
         }
     }
     return true;
@@ -572,7 +578,7 @@ bool CTerrain::CreateMosaic(int ox, int oy, int step, int objRank,
     std::string texName1;
     std::string texName2;
 
-    if ( step == 1 && m_engine->GetGroundSpot() )
+    if ( step == 1 )
     {
         int i = (ox/5) + (oy/5)*(m_mosaicCount/5);
         std::stringstream s;
@@ -716,7 +722,7 @@ bool CTerrain::CreateMosaic(int ox, int oy, int step, int objRank,
                     buffer.vertices.push_back(p2);
                 }
 
-                m_engine->AddBaseObjQuick(baseObjRank, buffer, texName1, texName2, LOD_Constant, true);
+                m_engine->AddBaseObjQuick(baseObjRank, buffer, texName1, texName2, true);
             }
         }
     }
@@ -730,7 +736,7 @@ bool CTerrain::CreateMosaic(int ox, int oy, int step, int objRank,
     return true;
 }
 
-TerrainMaterial* CTerrain::FindMaterial(int id)
+CTerrain::TerrainMaterial* CTerrain::FindMaterial(int id)
 {
     for (int i = 0; i < static_cast<int>( m_materials.size() ); i++)
     {
@@ -1879,4 +1885,3 @@ float CTerrain::GetFlyingLimit(Math::Vector pos, bool noLimit)
 
 
 } // namespace Gfx
-
