@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,9 +22,12 @@
 
 #include "app/app.h"
 
+#include "graphics/engine/engine.h"
+
 #include "common/image.h"
 #include "common/logger.h"
 #include "common/stringutils.h"
+
 #include "common/resources/resourcemanager.h"
 
 #include "math/func.h"
@@ -34,7 +37,21 @@
 
 
 // Graphics module namespace
-namespace Gfx {
+namespace Gfx
+{
+
+/**
+ * \struct MultisizeFont
+ * \brief Font with multiple possible sizes
+ */
+struct MultisizeFont
+{
+    std::string fileName;
+    std::map<int, std::unique_ptr<CachedFont>> fonts;
+
+    explicit MultisizeFont(const std::string &fn)
+        : fileName(fn) {}
+};
 
 
 /**
@@ -43,14 +60,28 @@ namespace Gfx {
  */
 struct CachedFont
 {
-    TTF_Font* font;
+    std::unique_ptr<CSDLFileWrapper> fontFile;
+    TTF_Font* font = nullptr;
     std::map<UTF8Char, CharTexture> cache;
 
-    CachedFont() : font(nullptr) {}
+    CachedFont(std::unique_ptr<CSDLFileWrapper> fontFile, int pointSize)
+        : fontFile(std::move(fontFile))
+    {
+        font = TTF_OpenFontRW(this->fontFile->GetHandler(), 0, pointSize);
+    }
+
+    ~CachedFont()
+    {
+        if (font != nullptr)
+            TTF_CloseFont(font);
+    }
 };
 
 
+namespace
+{
 const Math::IntPoint REFERENCE_SIZE(800, 600);
+} // anonymous namespace
 
 
 CText::CText(CEngine* engine)
@@ -80,12 +111,12 @@ bool CText::Create()
         return false;
     }
 
-    m_fonts[FONT_COLOBOT]        = new MultisizeFont("fonts/dvu_sans.ttf");
-    m_fonts[FONT_COLOBOT_BOLD]   = new MultisizeFont("fonts/dvu_sans_bold.ttf");
-    m_fonts[FONT_COLOBOT_ITALIC] = new MultisizeFont("fonts/dvu_sans_italic.ttf");
+    m_fonts[FONT_COLOBOT]        = MakeUnique<MultisizeFont>("fonts/dvu_sans.ttf");
+    m_fonts[FONT_COLOBOT_BOLD]   = MakeUnique<MultisizeFont>("fonts/dvu_sans_bold.ttf");
+    m_fonts[FONT_COLOBOT_ITALIC] = MakeUnique<MultisizeFont>("fonts/dvu_sans_italic.ttf");
 
-    m_fonts[FONT_COURIER]        = new MultisizeFont("fonts/dvu_sans_mono.ttf");
-    m_fonts[FONT_COURIER_BOLD]   = new MultisizeFont("fonts/dvu_sans_mono_bold.ttf");
+    m_fonts[FONT_COURIER]        = MakeUnique<MultisizeFont>("fonts/dvu_sans_mono.ttf");
+    m_fonts[FONT_COURIER_BOLD]   = MakeUnique<MultisizeFont>("fonts/dvu_sans_mono_bold.ttf");
 
     for (auto it = m_fonts.begin(); it != m_fonts.end(); ++it)
     {
@@ -100,27 +131,11 @@ bool CText::Create()
 
 void CText::Destroy()
 {
-    for (auto it = m_fonts.begin(); it != m_fonts.end(); ++it)
-    {
-        MultisizeFont* mf = (*it).second;
-
-        for (auto jt = mf->fonts.begin(); jt != mf->fonts.end(); ++jt)
-        {
-            CachedFont* cf = (*jt).second;
-
-            TTF_CloseFont(cf->font);
-
-            cf->font = nullptr;
-            delete cf;
-        }
-
-        mf->fonts.clear();
-        delete mf;
-    }
-
     m_fonts.clear();
 
     m_lastCachedFont = nullptr;
+    m_lastFontType = FONT_COLOBOT;
+    m_lastFontSize = 0;
 
     TTF_Quit();
 }
@@ -137,25 +152,23 @@ std::string CText::GetError()
 
 void CText::FlushCache()
 {
-    for (auto it = m_fonts.begin(); it != m_fonts.end(); ++it)
+    for (auto& multisizeFont : m_fonts)
     {
-        MultisizeFont *mf = (*it).second;
-        for (auto jt = mf->fonts.begin(); jt != mf->fonts.end(); ++jt)
+        for (auto& cachedFont : multisizeFont.second->fonts)
         {
-            CachedFont *f = (*jt).second;
-            for (auto ct = f->cache.begin(); ct != f->cache.end(); ++ct)
+            for (auto& charTexture : cachedFont.second->cache)
             {
                 Texture tex;
-                tex.id = (*ct).second.id;
+                tex.id = charTexture.second.id;
                 m_device->DestroyTexture(tex);
             }
-            f->cache.clear();
+            cachedFont.second->cache.clear();
         }
     }
 
+    m_lastCachedFont = nullptr;
     m_lastFontType = FONT_COLOBOT;
     m_lastFontSize = 0;
-    m_lastCachedFont = nullptr;
 }
 
 int CText::GetTabSize()
@@ -350,7 +363,8 @@ float CText::GetStringWidth(std::string text, FontType font, float size)
 
 float CText::GetCharWidth(UTF8Char ch, FontType font, float size, float offset)
 {
-    if (font == FONT_BUTTON) {
+    if (font == FONT_BUTTON)
+    {
         Math::IntPoint windowSize = m_engine->GetWindowSize();
         float height = GetHeight(FONT_COLOBOT, size);
         float width = height*(static_cast<float>(windowSize.y)/windowSize.x);
@@ -639,16 +653,48 @@ void CText::DrawString(const std::string &text, std::vector<FontMetaChar>::itera
             break;
         }
 
+        Color c = color;
         FontHighlight hl = static_cast<FontHighlight>(format[fmtIndex] & FONT_MASK_HIGHLIGHT);
         if (hl != FONT_HIGHLIGHT_NONE)
         {
-            Math::Point charSize;
-            charSize.x = GetCharWidth(ch, font, size, offset);
-            charSize.y = GetHeight(font, size);
-            DrawHighlight(hl, pos, charSize);
+            if (hl == FONT_HIGHLIGHT_TOKEN)
+            {
+                c = Color(0.490f, 0.380f, 0.165f, 1.0f); // #7D612A
+            }
+            else if (hl == FONT_HIGHLIGHT_TYPE)
+            {
+                c = Color(0.31f, 0.443f, 0.196f, 1.0f); // #4F7132
+            }
+            else if (hl == FONT_HIGHLIGHT_CONST)
+            {
+                c = Color(0.882f, 0.176f, 0.176f, 1.0f); // #E12D2D
+            }
+            else if (hl == FONT_HIGHLIGHT_THIS)
+            {
+                c = Color(0.545f, 0.329f, 0.608f, 1.0f); // #8B549B
+            }
+            else if (hl == FONT_HIGHLIGHT_COMMENT)
+            {
+                c = Color(0.251f, 0.271f, 0.306f, 1.0f); // #40454E
+            }
+            else if (hl == FONT_HIGHLIGHT_KEYWORD)
+            {
+                c = Color(0.239f, 0.431f, 0.588f, 1.0f); // #3D6E96
+            }
+            else if (hl == FONT_HIGHLIGHT_STRING)
+            {
+                c = Color(0.239f, 0.384f, 0.341f, 1.0f); // #3D6257
+            }
+            else
+            {
+                Math::Point charSize;
+                charSize.x = GetCharWidth(ch, font, size, offset);
+                charSize.y = GetHeight(font, size);
+                DrawHighlight(hl, pos, charSize);
+            }
         }
 
-        DrawCharAndAdjustPos(ch, font, size, pos, color);
+        DrawCharAndAdjustPos(ch, font, size, pos, c);
 
         // increment fmtIndex for each byte in multibyte character
         if ( ch.c1 != 0 )
@@ -690,7 +736,9 @@ void CText::StringToUTFCharList(const std::string &text, std::vector<UTF8Char> &
     }
 }
 
-void CText::StringToUTFCharList(const std::string &text, std::vector<UTF8Char> &chars, std::vector<FontMetaChar>::iterator format, std::vector<FontMetaChar>::iterator end)
+void CText::StringToUTFCharList(const std::string &text, std::vector<UTF8Char> &chars,
+                                std::vector<FontMetaChar>::iterator format,
+                                std::vector<FontMetaChar>::iterator end)
 {
     unsigned int index = 0;
     unsigned int totalLength = text.length();
@@ -699,7 +747,7 @@ void CText::StringToUTFCharList(const std::string &text, std::vector<UTF8Char> &
         UTF8Char ch;
 
         FontType font = FONT_COLOBOT;
-        if(format + index != end)
+        if (format + index != end)
             font = static_cast<FontType>(*(format + index) & FONT_MASK_FONT);
 
         int len;
@@ -754,26 +802,6 @@ void CText::DrawHighlight(FontHighlight hl, Math::Point pos, Math::Point size)
             grad[0] = grad[1] = grad[2] = grad[3] = Color(0.0f, 0.0f, 1.0f, 0.5f);
             break;
 
-        case FONT_HIGHLIGHT_TOKEN:
-            grad[0] = grad[1] = Color(248.0f / 256.0f, 248.0f / 256.0f, 248.0f / 256.0f, 0.5f);
-            grad[2] = grad[3] = Color(248.0f / 256.0f, 220.0f / 256.0f, 188.0f / 256.0f, 0.5f);
-            break;
-
-        case FONT_HIGHLIGHT_TYPE:
-            grad[0] = grad[1] = Color(248.0f / 256.0f, 248.0f / 256.0f, 248.0f / 256.0f, 0.5f);
-            grad[2] = grad[3] = Color(169.0f / 256.0f, 234.0f / 256.0f, 169.0f / 256.0f, 0.5f);
-            break;
-
-        case FONT_HIGHLIGHT_CONST:
-            grad[0] = grad[1] = Color(248.0f / 256.0f, 248.0f / 256.0f, 248.0f / 256.0f, 0.5f);
-            grad[2] = grad[3] = Color(248.0f / 256.0f, 176.0f / 256.0f, 169.0f / 256.0f, 0.5f);
-            break;
-
-        case FONT_HIGHLIGHT_REM:
-            grad[0] = grad[1] = Color(248.0f / 256.0f, 248.0f / 256.0f, 248.0f / 256.0f, 0.5f);
-            grad[2] = grad[3] = Color(248.0f / 256.0f, 169.0f / 256.0f, 248.0f / 256.0f, 0.5f);
-            break;
-
         case FONT_HIGHLIGHT_KEY:
             grad[0] = grad[1] = grad[2] = grad[3] =
                 Color(192.0f / 256.0f, 192.0f / 256.0f, 192.0f / 256.0f, 0.5f);
@@ -836,13 +864,7 @@ void CText::DrawCharAndAdjustPos(UTF8Char ch, FontType font, float size, Math::P
 
         // For whatever reason ch.c1 is a SIGNED char, we need to fix that
         unsigned char icon = static_cast<unsigned char>(ch.c1);
-        if ( icon >= 192 )
-        {
-            icon -= 192;
-            m_engine->SetTexture("textures/interface/text.png");
-            m_engine->SetState(ENG_RSTATE_TTEXTURE_WHITE);
-        }
-        else if ( icon >= 128 )
+        if ( icon >= 128 )
         {
             icon -= 128;
             m_engine->SetTexture("textures/interface/button3.png");
@@ -890,35 +912,19 @@ void CText::DrawCharAndAdjustPos(UTF8Char ch, FontType font, float size, Math::P
     }
     else
     {
-        CachedFont* cf = GetOrOpenFont(font, size);
-
-        if (cf == nullptr)
-            return;
-
         int width = 1;
         if (ch.c1 > 0 && ch.c1 < 32)
         {
             if (ch.c1 == '\t')
+            {
+                color = Color(1.0f, 0.0f, 0.0f, 1.0f);
                 width = m_tabSize;
+            }
 
             ch = TranslateSpecialChar(ch.c1);
         }
 
-        auto it = cf->cache.find(ch);
-        CharTexture tex;
-        if (it != cf->cache.end())
-        {
-            tex = (*it).second;
-        }
-        else
-        {
-            tex = CreateCharTexture(ch, cf);
-
-            if (tex.id == 0) // invalid
-                return;
-
-            cf->cache[ch] = tex;
-        }
+        CharTexture tex = GetCharTexture(ch, font, size);
 
         Math::Point p1(pos.x, pos.y + tex.charSize.y - tex.texSize.y);
         Math::Point p2(pos.x + tex.texSize.x, pos.y + tex.charSize.y);
@@ -946,10 +952,11 @@ CachedFont* CText::GetOrOpenFont(FontType font, float size)
     Math::IntPoint windowSize = m_engine->GetWindowSize();
     int pointSize = static_cast<int>(size * (windowSize.Length() / REFERENCE_SIZE.Length()));
 
-    if (m_lastCachedFont != nullptr)
+    if (m_lastCachedFont != nullptr &&
+        m_lastFontType == font &&
+        m_lastFontSize == pointSize)
     {
-        if (m_lastFontType == font && m_lastFontSize == pointSize)
-            return m_lastCachedFont;
+        return m_lastCachedFont;
     }
 
     auto it = m_fonts.find(font);
@@ -959,30 +966,33 @@ CachedFont* CText::GetOrOpenFont(FontType font, float size)
         return nullptr;
     }
 
-    MultisizeFont* mf = (*it).second;
+    MultisizeFont* mf = it->second.get();
 
     auto jt = mf->fonts.find(pointSize);
     if (jt != mf->fonts.end())
     {
-        m_lastCachedFont = (*jt).second;
+        m_lastCachedFont = jt->second.get();
         m_lastFontType = font;
         m_lastFontSize = pointSize;
         return m_lastCachedFont;
     }
 
-    m_lastCachedFont = new CachedFont();
-    SDL_RWops* file = CResourceManager::GetSDLFileHandler(mf->fileName);
-    if(file == nullptr)
+    auto file = CResourceManager::GetSDLFileHandler(mf->fileName);
+    if (!file->IsOpen())
     {
         m_error = std::string("Unable to open file");
         return nullptr;
     }
-    m_lastCachedFont->font = TTF_OpenFontRW(file, 1, pointSize);
-    if (m_lastCachedFont->font == nullptr)
+
+    auto newFont = MakeUnique<CachedFont>(std::move(file), pointSize);
+    if (newFont->font == nullptr)
+    {
         m_error = std::string("TTF_OpenFont error ") + std::string(TTF_GetError());
+        return nullptr;
+    }
 
-    mf->fonts[pointSize] = m_lastCachedFont;
-
+    m_lastCachedFont = newFont.get();
+    mf->fonts[pointSize] = std::move(newFont);
     return m_lastCachedFont;
 }
 
@@ -1007,7 +1017,7 @@ CharTexture CText::CreateCharTexture(UTF8Char ch, CachedFont* font)
     textSurface->flags = textSurface->flags & (~SDL_SRCALPHA);
     SDL_Surface* textureSurface = SDL_CreateRGBSurface(0, w, h, 32, 0x00ff0000, 0x0000ff00,
                                                        0x000000ff, 0xff000000);
-    SDL_BlitSurface(textSurface, NULL, textureSurface, NULL);
+    SDL_BlitSurface(textSurface, nullptr, textureSurface, nullptr);
 
     ImageData data;
     data.surface = textureSurface;
@@ -1038,6 +1048,30 @@ CharTexture CText::CreateCharTexture(UTF8Char ch, CachedFont* font)
     return texture;
 }
 
+CharTexture CText::GetCharTexture(UTF8Char ch, FontType font, float size)
+{
+    CachedFont* cf = GetOrOpenFont(font, size);
+
+    if (cf == nullptr)
+        return CharTexture();
+
+    auto it = cf->cache.find(ch);
+    CharTexture tex;
+    if (it != cf->cache.end())
+    {
+        tex = (*it).second;
+    }
+    else
+    {
+        tex = CreateCharTexture(ch, cf);
+
+        if (tex.id == 0) // invalid
+            return CharTexture();
+
+        cf->cache[ch] = tex;
+    }
+    return tex;
+}
+
 
 } // namespace Gfx
-
