@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -19,12 +19,16 @@
 
 
 #include "graphics/opengl/gl21device.h"
-#include "graphics/engine/engine.h"
 
 #include "common/config.h"
+
+#include "common/config_file.h"
 #include "common/image.h"
 #include "common/logger.h"
-#include "common/profile.h"
+
+#include "graphics/engine/engine.h"
+
+#include "graphics/opengl/glframebuffer.h"
 
 #include "math/geometry.h"
 
@@ -36,25 +40,12 @@
 
 
 // Graphics module namespace
-namespace Gfx {
-
-CGL21Device::CGL21Device(const GLDeviceConfig &config)
+namespace Gfx
 {
-    m_config = config;
-    m_lighting = false;
-    m_lastVboId = 0;
-    m_anisotropyAvailable = false;
-    m_maxAnisotropy = 1;
-    m_glMajor = 1;
-    m_glMinor = 1;
 
-    m_framebuffer = 0;
-    m_colorBuffer = 0;
-    m_depthBuffer = 0;
-    m_offscreenRenderingEnabled = false;
-
-    m_perPixelLighting = false;
-}
+CGL21Device::CGL21Device(const DeviceConfig &config)
+    : m_config(config)
+{}
 
 
 CGL21Device::~CGL21Device()
@@ -199,15 +190,6 @@ bool CGL21Device::Create()
 
         GetLogger()->Info("OpenGL %d.%d\n", m_glMajor, m_glMinor);
 
-        m_framebufferObject = glewIsSupported("GL_EXT_framebuffer_object");
-        if (m_framebufferObject)
-        {
-            glGetIntegerv(GL_MAX_RENDERBUFFER_SIZE_EXT, &m_maxRenderbufferSize);
-
-            GetLogger()->Info("Offscreen rendering available\n");
-            GetLogger()->Info("Maximum renderbuffer size: %d\n", m_maxRenderbufferSize);
-        }
-
         // Detect support of anisotropic filtering
         m_anisotropyAvailable = glewIsSupported("GL_EXT_texture_filter_anisotropic");
         if(m_anisotropyAvailable)
@@ -216,7 +198,7 @@ bool CGL21Device::Create()
             float level;
             glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &level);
             m_maxAnisotropy = static_cast<int>(level);
-            
+
             GetLogger()->Info("Anisotropic filtering available\n");
             GetLogger()->Info("Maximum anisotropy: %d\n", m_maxAnisotropy);
         }
@@ -224,6 +206,17 @@ bool CGL21Device::Create()
         {
             GetLogger()->Info("Anisotropic filtering not available\n");
         }
+
+        // Read maximum sample count for MSAA
+        if(glewIsSupported("GL_ARB_multisample"))
+        {
+            glGetIntegerv(GL_MAX_SAMPLES_EXT, &m_maxSamples);
+            GetLogger()->Info("Multisampling supported, max samples: %d\n", m_maxSamples);
+        }
+        else
+        {
+            GetLogger()->Info("Multisampling not supported\n");
+        }
     }
 
     // This is mostly done in all modern hardware by default
@@ -251,7 +244,7 @@ bool CGL21Device::Create()
     m_textureStageParams = std::vector<TextureStageParams>(maxTextures, TextureStageParams());
 
     int value;
-    if (CProfile::GetInstance().GetIntProperty("Setup", "PerPixelLighting", value))
+    if (CConfigFile::GetInstance().GetIntProperty("Setup", "PerPixelLighting", value))
     {
         m_perPixelLighting = value > 0;
     }
@@ -356,6 +349,19 @@ bool CGL21Device::Create()
     for (int i = 0; i < 8; i++)
         glUniform1i(uni_LightEnabled[i], 0);
 
+    // create default framebuffer object
+    FramebufferParams framebufferParams;
+
+    framebufferParams.width = m_config.size.x;
+    framebufferParams.height = m_config.size.y;
+    framebufferParams.depth = m_config.depthSize;
+
+    m_framebuffers["default"] = MakeUnique<CDefaultFramebuffer>(framebufferParams);
+
+    m_framebufferSupport = DetectFramebufferSupport();
+    if (m_framebufferSupport != FBS_NONE)
+        GetLogger()->Debug("Framebuffer supported\n");
+
     GetLogger()->Info("CDevice created successfully\n");
 
     return true;
@@ -368,6 +374,12 @@ void CGL21Device::Destroy()
     glUseProgram(0);
     glDeleteProgram(m_program);
 
+    // delete framebuffers
+    for (auto& framebuffer : m_framebuffers)
+        framebuffer.second->Destroy();
+
+    m_framebuffers.clear();
+
     DestroyAllTextures();
 
     m_lights.clear();
@@ -378,7 +390,7 @@ void CGL21Device::Destroy()
     m_textureStageParams.clear();
 }
 
-void CGL21Device::ConfigChanged(const GLDeviceConfig& newConfig)
+void CGL21Device::ConfigChanged(const DeviceConfig& newConfig)
 {
     m_config = newConfig;
 
@@ -413,7 +425,9 @@ void CGL21Device::SetTransform(TransformType type, const Math::Matrix &matrix)
 
         // normal transform
         Math::Matrix normalMat = matrix;
-        normalMat = normalMat.Inverse();
+
+        if (fabs(normalMat.Det()) > 1e-6)
+            normalMat = normalMat.Inverse();
 
         glUniformMatrix4fv(uni_NormalMatrix, 1, GL_TRUE, normalMat.Array());
     }
@@ -560,7 +574,7 @@ Texture CGL21Device::CreateTexture(ImageData *data, const TextureCreateParams &p
 
     if (!Math::IsPowerOfTwo(result.size.x) || !Math::IsPowerOfTwo(result.size.y))
         GetLogger()->Warn("Creating non-power-of-2 texture (%dx%d)!\n", result.size.x, result.size.y);
-    
+
     result.originalSize = result.size;
 
     glActiveTexture(GL_TEXTURE0);
@@ -596,24 +610,24 @@ Texture CGL21Device::CreateTexture(ImageData *data, const TextureCreateParams &p
     // Set mipmap level and automatic mipmap generation if neccesary
     if (params.mipmap)
     {
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, mipmapLevel - 1);
-	    glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, mipmapLevel - 1);
+        glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
     }
     else
     {
-	    // Has to be set to 0 because no mipmaps are generated
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_FALSE);
+        // Has to be set to 0 because no mipmaps are generated
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_FALSE);
     }
 
     // Set anisotropy level if available
     if (m_anisotropyAvailable)
     {
-	    float level = Math::Min(m_maxAnisotropy, CEngine::GetInstance().GetTextureAnisotropyLevel());
+        float level = Math::Min(m_maxAnisotropy, CEngine::GetInstance().GetTextureAnisotropyLevel());
 
-	    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, level);
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, level);
     }
 
     bool convert = false;
@@ -687,7 +701,8 @@ Texture CGL21Device::CreateTexture(ImageData *data, const TextureCreateParams &p
                 convert = true;
             }
         }
-        else {
+        else
+        {
             GetLogger()->Error("Unknown data surface format");
             assert(false);
         }
@@ -725,6 +740,8 @@ Texture CGL21Device::CreateTexture(ImageData *data, const TextureCreateParams &p
 
     SDL_FreeSurface(convertedSurface);
 
+    m_allTextures.insert(result);
+
     // Restore the previous state of 1st stage
     glBindTexture(GL_TEXTURE_2D, m_currentTextures[0].id);
 
@@ -737,7 +754,7 @@ Texture CGL21Device::CreateTexture(ImageData *data, const TextureCreateParams &p
 Texture CGL21Device::CreateDepthTexture(int width, int height, int depth)
 {
     Texture result;
-    
+
     result.alpha = false;
     result.size.x = width;
     result.size.y = height;
@@ -789,11 +806,12 @@ void CGL21Device::DestroyTexture(const Texture &texture)
             SetTexture(index, Texture()); // set to invalid texture
     }
 
-    glDeleteTextures(1, &texture.id);
-
     auto it = m_allTextures.find(texture);
     if (it != m_allTextures.end())
+    {
+        glDeleteTextures(1, &texture.id);
         m_allTextures.erase(it);
+    }
 }
 
 void CGL21Device::DestroyAllTextures()
@@ -833,9 +851,8 @@ void CGL21Device::SetTexture(int index, const Texture &texture)
     glBindTexture(GL_TEXTURE_2D, texture.id);
 
     // Params need to be updated for the new bound texture
-    UpdateTextureParams(index);
-
     UpdateTextureStatus();
+    UpdateTextureParams(index);
 }
 
 void CGL21Device::SetTexture(int index, unsigned int textureId)
@@ -852,9 +869,8 @@ void CGL21Device::SetTexture(int index, unsigned int textureId)
     glBindTexture(GL_TEXTURE_2D, textureId);
 
     // Params need to be updated for the new bound texture
-    UpdateTextureParams(index);
-
     UpdateTextureStatus();
+    UpdateTextureParams(index);
 }
 
 void CGL21Device::SetTextureEnabled(int index, bool enabled)
@@ -868,25 +884,16 @@ void CGL21Device::SetTextureEnabled(int index, bool enabled)
     if (same)
         return; // nothing to do
 
-    glUniform1i(uni_TextureEnabled[index], enabled ? 1 : 0);
-
-    //UpdateTextureStatus();
+    UpdateTextureStatus();
 }
 
 void CGL21Device::UpdateTextureStatus()
 {
-    glUniform1i(uni_TextureEnabled[0], m_currentTextures[0].id != 0 ? 1 : 0);
-
-    /*
-    bool enabled = m_texturesEnabled[0] && m_currentTextures[0].id != 0;
-    glUniform1i(uni_PrimaryTextureEnabled, enabled ? 1 : 0);
-
-    enabled = m_texturesEnabled[1] && m_currentTextures[1].id != 0;
-    glUniform1i(uni_SecondaryTextureEnabled, enabled ? 1 : 0);
-
-    enabled = m_texturesEnabled[2] && m_currentTextures[2].id != 0;
-    glUniform1i(uni_ShadowTextureEnabled, enabled ? 1 : 0);
-    */
+    for (int i = 0; i < 3; i++)
+    {
+        bool enabled = m_texturesEnabled[i] && (m_currentTextures[i].id != 0);
+        glUniform1i(uni_TextureEnabled[i], enabled ? 1 : 0);
+    }
 }
 
 inline void CGL21Device::BindVBO(GLuint vbo)
@@ -1131,7 +1138,7 @@ void CGL21Device::SetTextureStageWrap(int index, TexWrapMode wrapS, TexWrapMode
         return;
 
     glActiveTexture(GL_TEXTURE0 + index);
-    
+
     if      (wrapS == TEX_WRAP_CLAMP)
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
     else if (wrapS == TEX_WRAP_CLAMP_TO_BORDER)
@@ -1197,7 +1204,7 @@ void CGL21Device::DrawPrimitive(PrimitiveType type, const VertexTex2 *vertices,
     glClientActiveTexture(GL_TEXTURE1);
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
     glTexCoordPointer(2, GL_FLOAT, sizeof(VertexTex2), reinterpret_cast<GLfloat*>(&vs[0].texCoord2));
-    
+
     glColor4fv(color.Array());
 
     glDrawArrays(TranslateGfxPrimitive(type), 0, vertexCount);
@@ -1561,25 +1568,6 @@ void CGL21Device::SetRenderState(RenderState state, bool enabled)
 
         return;
     }
-    else if (state == RENDER_STATE_OFFSCREEN_RENDERING)
-    {
-        if (!m_framebufferObject)
-        {
-            GetLogger()->Error("Cannot enable offscreen rendering without framebuffer object!\n");
-            return;
-        }
-
-        m_offscreenRenderingEnabled = enabled;
-
-        if (m_framebuffer == 0)
-            InitOffscreenBuffer(2048, 2048);
-
-        GLuint toBind = (enabled ? m_framebuffer : 0);
-
-        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, toBind);
-
-        return;
-    }
     else if (state == RENDER_STATE_ALPHA_TEST)
     {
         glUniform1i(uni_AlphaTestEnabled, enabled ? 1 : 0);
@@ -1693,79 +1681,6 @@ void CGL21Device::SetFillMode(FillMode mode)
     else assert(false);
 }
 
-void CGL21Device::InitOffscreenBuffer(int width, int height)
-{
-    if (!m_framebufferObject) return;
-
-    width = Math::Min(width, m_maxRenderbufferSize);
-    height = Math::Min(height, m_maxRenderbufferSize);
-
-    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
-
-    if (m_colorBuffer != 0)
-        glDeleteRenderbuffersEXT(1, &m_colorBuffer);
-
-    glGenRenderbuffersEXT(1, &m_colorBuffer);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_colorBuffer);
-    glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_RGBA8, width, height);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0);
-
-    if (m_depthBuffer != 0)
-        glDeleteRenderbuffersEXT(1, &m_depthBuffer);
-
-    glGenRenderbuffersEXT(1, &m_depthBuffer);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_depthBuffer);
-    glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, width, height);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0);
-
-    if (m_framebuffer == 0)
-        glGenFramebuffersEXT(1, &m_framebuffer);
-
-    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_framebuffer);
-    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_RENDERBUFFER_EXT, m_colorBuffer);
-    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, m_depthBuffer);
-    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
-
-    GetLogger()->Info("Initialized offscreen buffer %dx%d\n", width, height);
-}
-
-void CGL21Device::SetRenderTexture(RenderTarget target, int texture)
-{
-    if (!m_framebufferObject) return;
-    if (!m_offscreenRenderingEnabled) return;
-
-    GLenum attachment;
-    GLuint defaultBuffer;
-
-    switch (target)
-    {
-    case RENDER_TARGET_COLOR:
-        attachment = GL_COLOR_ATTACHMENT0_EXT;
-        defaultBuffer = m_colorBuffer;
-        break;
-    case RENDER_TARGET_DEPTH:
-        attachment = GL_DEPTH_ATTACHMENT_EXT;
-        defaultBuffer = m_depthBuffer;
-        break;
-    case RENDER_TARGET_STENCIL:
-        attachment = GL_STENCIL_ATTACHMENT_EXT;
-        defaultBuffer = 0;
-        break;
-    default: assert(false); break;
-    }
-
-    if (texture == 0)       // unbind texture and bind default buffer
-    {
-        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment, GL_TEXTURE_2D, 0, 0);
-        glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, attachment, GL_RENDERBUFFER_EXT, defaultBuffer);
-    }
-    else            // unbind default buffer and bind texture
-    {
-        glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, attachment, GL_RENDERBUFFER_EXT, 0);
-        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment, GL_TEXTURE_2D, texture, 0);
-    }
-}
-
 void CGL21Device::CopyFramebufferToTexture(Texture& texture, int xOffset, int yOffset, int x, int y, int width, int height)
 {
     if (texture.id == 0) return;
@@ -1780,18 +1695,87 @@ void CGL21Device::CopyFramebufferToTexture(Texture& texture, int xOffset, int yO
     glBindTexture(GL_TEXTURE_2D, m_currentTextures[0].id);
 }
 
-void* CGL21Device::GetFrameBufferPixels()const{
+std::unique_ptr<CFrameBufferPixels> CGL21Device::GetFrameBufferPixels() const
+{
+    return GetGLFrameBufferPixels(m_config.size);
+}
 
-    GLubyte* pixels = new GLubyte[4 * m_config.size.x * m_config.size.y];
+CFramebuffer* CGL21Device::GetFramebuffer(std::string name)
+{
+    auto it = m_framebuffers.find(name);
+    if (it == m_framebuffers.end())
+        return nullptr;
 
-    glReadPixels(0, 0, m_config.size.x, m_config.size.y, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+    return it->second.get();
+}
 
-    unsigned int* p = static_cast<unsigned int*> ( static_cast<void*>(pixels) );
+CFramebuffer* CGL21Device::CreateFramebuffer(std::string name, const FramebufferParams& params)
+{
+    // existing framebuffer was found
+    if (m_framebuffers.find(name) != m_framebuffers.end())
+    {
+        return nullptr;
+    }
 
-    for (int i = 0; i < m_config.size.x * m_config.size.y; ++i)
-        p[i] |= 0xFF000000;
+    std::unique_ptr<CFramebuffer> framebuffer;
 
-    return static_cast<void*>(p);
+    if (m_framebufferSupport == FBS_ARB)
+        framebuffer = MakeUnique<CGLFramebuffer>(params);
+    else if (m_framebufferSupport == FBS_EXT)
+        framebuffer = MakeUnique<CGLFramebufferEXT>(params);
+    else
+        return nullptr;
+
+    framebuffer->Create();
+
+    CFramebuffer* framebufferPtr = framebuffer.get();
+    m_framebuffers[name] = std::move(framebuffer);
+    return framebufferPtr;
+}
+
+void CGL21Device::DeleteFramebuffer(std::string name)
+{
+    // can't delete default framebuffer
+    if (name == "default") return;
+
+    auto it = m_framebuffers.find(name);
+    if (it != m_framebuffers.end())
+    {
+        it->second->Destroy();
+        m_framebuffers.erase(it);
+    }
+}
+
+bool CGL21Device::IsAnisotropySupported()
+{
+    return m_anisotropyAvailable;
+}
+
+int CGL21Device::GetMaxAnisotropyLevel()
+{
+    return m_maxAnisotropy;
+}
+
+int CGL21Device::GetMaxSamples()
+{
+    return m_maxSamples;
+}
+
+bool CGL21Device::IsShadowMappingSupported()
+{
+    return true;
+}
+
+int CGL21Device::GetMaxTextureSize()
+{
+    int value;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &value);
+    return value;
+}
+
+bool CGL21Device::IsFramebufferSupported()
+{
+    return m_framebufferSupport != FBS_NONE;
 }
 
 } // namespace Gfx
