@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,12 +18,17 @@
  */
 
 #include "graphics/opengl/gl33device.h"
-#include "graphics/engine/engine.h"
 
 #include "common/config.h"
+
+#include "common/config_file.h"
 #include "common/image.h"
 #include "common/logger.h"
-#include "common/profile.h"
+#include "common/make_unique.h"
+
+#include "graphics/engine/engine.h"
+
+#include "graphics/opengl/glframebuffer.h"
 
 #include "math/geometry.h"
 
@@ -35,25 +40,12 @@
 
 
 // Graphics module namespace
-namespace Gfx {
-
-CGL33Device::CGL33Device(const GLDeviceConfig &config)
+namespace Gfx
 {
-    m_config = config;
-    m_lighting = false;
-    m_lastVboId = 0;
-    m_anisotropyAvailable = false;
-    m_maxAnisotropy = 1;
-    m_glMajor = 1;
-    m_glMinor = 1;
 
-    m_framebuffer = 0;
-    m_colorBuffer = 0;
-    m_depthBuffer = 0;
-    m_offscreenRenderingEnabled = false;
-
-    m_perPixelLighting = false;
-}
+CGL33Device::CGL33Device(const DeviceConfig &config)
+    : m_config(config)
+{}
 
 
 CGL33Device::~CGL33Device()
@@ -205,9 +197,6 @@ bool CGL33Device::Create()
             GetLogger()->Info("OpenGL %d.%d\n", m_glMajor, m_glMinor);
         }
 
-        glGetIntegerv(GL_MAX_RENDERBUFFER_SIZE, &m_maxRenderbufferSize);
-        GetLogger()->Info("Maximum renderbuffer size: %d\n", m_maxRenderbufferSize);
-
         // Detect support of anisotropic filtering
         m_anisotropyAvailable = glewIsSupported("GL_EXT_texture_filter_anisotropic");
         if(m_anisotropyAvailable)
@@ -216,7 +205,7 @@ bool CGL33Device::Create()
             float level;
             glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &level);
             m_maxAnisotropy = static_cast<int>(level);
-            
+
             GetLogger()->Info("Anisotropic filtering available\n");
             GetLogger()->Info("Maximum anisotropy: %d\n", m_maxAnisotropy);
         }
@@ -224,6 +213,17 @@ bool CGL33Device::Create()
         {
             GetLogger()->Info("Anisotropic filtering not available\n");
         }
+
+        // Read maximum sample count for MSAA
+        if(glewIsSupported("GL_ARB_multisample"))
+        {
+            glGetIntegerv(GL_MAX_SAMPLES_EXT, &m_maxSamples);
+            GetLogger()->Info("Multisampling supported, max samples: %d\n", m_maxSamples);
+        }
+        else
+        {
+            GetLogger()->Info("Multisampling not supported\n");
+        }
     }
 
     // Set just to be sure
@@ -249,10 +249,8 @@ bool CGL33Device::Create()
     m_vertexTex2 = CreateStaticBuffer(PRIMITIVE_POINTS, static_cast<VertexTex2*>(nullptr), 1);
     m_vertexCol = CreateStaticBuffer(PRIMITIVE_POINTS, static_cast<VertexCol*>(nullptr), 1);
 
-    GetLogger()->Info("CDevice created successfully\n");
-
     int value;
-    if (CProfile::GetInstance().GetIntProperty("Setup", "PerPixelLighting", value))
+    if (CConfigFile::GetInstance().GetIntProperty("Setup", "PerPixelLighting", value))
     {
         m_perPixelLighting = value > 0;
     }
@@ -304,7 +302,7 @@ bool CGL33Device::Create()
     uni_PrimaryTextureEnabled = glGetUniformLocation(m_shaderProgram, "uni_PrimaryTextureEnabled");
     uni_SecondaryTextureEnabled = glGetUniformLocation(m_shaderProgram, "uni_SecondaryTextureEnabled");
     uni_ShadowTextureEnabled = glGetUniformLocation(m_shaderProgram, "uni_ShadowTextureEnabled");
-    
+
     uni_FogEnabled = glGetUniformLocation(m_shaderProgram, "uni_FogEnabled");
     uni_FogRange = glGetUniformLocation(m_shaderProgram, "uni_FogRange");
     uni_FogColor = glGetUniformLocation(m_shaderProgram, "uni_FogColor");
@@ -379,22 +377,31 @@ bool CGL33Device::Create()
     for (int i = 0; i < 8; i++)
         glUniform1i(uni_Light[i].Enabled, 0);
 
+    // create default framebuffer object
+    FramebufferParams framebufferParams;
+
+    framebufferParams.width = m_config.size.x;
+    framebufferParams.height = m_config.size.y;
+    framebufferParams.depth = m_config.depthSize;
+
+    m_framebuffers["default"] = MakeUnique<CDefaultFramebuffer>(framebufferParams);
+
+    GetLogger()->Info("CDevice created successfully\n");
+
     return true;
 }
 
 void CGL33Device::Destroy()
 {
+    // delete shader program
     glUseProgram(0);
     glDeleteProgram(m_shaderProgram);
 
-    if (m_framebuffer != 0)
-    {
-        glDeleteFramebuffers(1, &m_framebuffer);
-        glDeleteRenderbuffers(1, &m_colorBuffer);
-        glDeleteRenderbuffers(1, &m_depthBuffer);
+    // delete framebuffers
+    for (auto& framebuffer : m_framebuffers)
+        framebuffer.second->Destroy();
 
-        m_framebuffer = 0;
-    }
+    m_framebuffers.clear();
 
     // Delete the remaining textures
     // Should not be strictly necessary, but just in case
@@ -408,7 +415,7 @@ void CGL33Device::Destroy()
     m_textureStageParams.clear();
 }
 
-void CGL33Device::ConfigChanged(const GLDeviceConfig& newConfig)
+void CGL33Device::ConfigChanged(const DeviceConfig& newConfig)
 {
     m_config = newConfig;
 
@@ -446,7 +453,9 @@ void CGL33Device::SetTransform(TransformType type, const Math::Matrix &matrix)
 
         // normal transform
         Math::Matrix normalMat = matrix;
-        normalMat = normalMat.Inverse();
+
+        if (fabs(normalMat.Det()) > 1e-6)
+            normalMat = normalMat.Inverse();
 
         glUniformMatrix4fv(uni_NormalMatrix, 1, GL_TRUE, normalMat.Array());
     }
@@ -595,7 +604,7 @@ Texture CGL33Device::CreateTexture(ImageData *data, const TextureCreateParams &p
 
     result.size.x = data->surface->w;
     result.size.y = data->surface->h;
-    
+
     result.originalSize = result.size;
 
     glActiveTexture(GL_TEXTURE0);
@@ -630,22 +639,22 @@ Texture CGL33Device::CreateTexture(ImageData *data, const TextureCreateParams &p
     // Set mipmap level and automatic mipmap generation if neccesary
     if (params.mipmap)
     {
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, mipmapLevel - 1);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, mipmapLevel - 1);
     }
     else
     {
-	    // Has to be set to 0 because no mipmaps are generated
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+        // Has to be set to 0 because no mipmaps are generated
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
     }
 
     // Set anisotropy level if available
     if (m_anisotropyAvailable)
     {
-	    float level = Math::Min(m_maxAnisotropy, CEngine::GetInstance().GetTextureAnisotropyLevel());
+        float level = Math::Min(m_maxAnisotropy, CEngine::GetInstance().GetTextureAnisotropyLevel());
 
-	    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, level);
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, level);
     }
 
     bool convert = false;
@@ -719,7 +728,8 @@ Texture CGL33Device::CreateTexture(ImageData *data, const TextureCreateParams &p
                 convert = true;
             }
         }
-        else {
+        else
+        {
             GetLogger()->Error("Unknown data surface format");
             assert(false);
         }
@@ -760,6 +770,8 @@ Texture CGL33Device::CreateTexture(ImageData *data, const TextureCreateParams &p
 
     SDL_FreeSurface(convertedSurface);
 
+    m_allTextures.insert(result);
+
     // Restore the previous state of 1st stage
     glBindTexture(GL_TEXTURE_2D, m_currentTextures[0].id);
 
@@ -820,11 +832,12 @@ void CGL33Device::DestroyTexture(const Texture &texture)
             SetTexture(index, Texture()); // set to invalid texture
     }
 
-    glDeleteTextures(1, &texture.id);
-
     auto it = m_allTextures.find(texture);
     if (it != m_allTextures.end())
+    {
+        glDeleteTextures(1, &texture.id);
         m_allTextures.erase(it);
+    }
 }
 
 void CGL33Device::DestroyAllTextures()
@@ -1135,7 +1148,7 @@ void CGL33Device::SetTextureStageWrap(int index, TexWrapMode wrapS, TexWrapMode
         return;
 
     glActiveTexture(GL_TEXTURE0 + index);
-    
+
     if      (wrapS == TEX_WRAP_CLAMP)
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
     else if (wrapS == TEX_WRAP_CLAMP_TO_BORDER)
@@ -1301,7 +1314,7 @@ void CGL33Device::DrawPrimitive(PrimitiveType type, const VertexCol *vertices, i
 unsigned int CGL33Device::CreateStaticBuffer(PrimitiveType primitiveType, const Vertex* vertices, int vertexCount)
 {
     unsigned int id = 0;
-    
+
     id = ++m_lastVboId;
 
     VertexBufferInfo info;
@@ -1443,6 +1456,8 @@ void CGL33Device::UpdateStaticBuffer(unsigned int bufferId, PrimitiveType primit
 
     bool changed = (info.vertexType != VERTEX_TYPE_NORMAL) || (size > info.size);
 
+    if (info.vertexType != VERTEX_TYPE_NORMAL) CLogger::GetInstance().Debug("Changing static buffer type\n");
+
     info.primitiveType = primitiveType;
     info.vertexType = VERTEX_TYPE_NORMAL;
     info.vertexCount = vertexCount;
@@ -1498,6 +1513,8 @@ void CGL33Device::UpdateStaticBuffer(unsigned int bufferId, PrimitiveType primit
 
     bool changed = (info.vertexType != VERTEX_TYPE_TEX2) || (size > info.size);
 
+    if (info.vertexType != VERTEX_TYPE_TEX2) CLogger::GetInstance().Debug("Changing static buffer type\n");
+
     info.primitiveType = primitiveType;
     info.vertexType = VERTEX_TYPE_TEX2;
     info.vertexCount = vertexCount;
@@ -1551,7 +1568,9 @@ void CGL33Device::UpdateStaticBuffer(unsigned int bufferId, PrimitiveType primit
 
     unsigned int size = vertexCount * sizeof(VertexCol);
 
-    bool changed = (info.vertexType != VERTEX_TYPE_TEX2) || (size > info.size);
+    bool changed = (info.vertexType != VERTEX_TYPE_COL) || (size > info.size);
+
+    if (info.vertexType != VERTEX_TYPE_NORMAL) CLogger::GetInstance().Debug("Changing static buffer type\n");
 
     info.primitiveType = primitiveType;
     info.vertexType = VERTEX_TYPE_COL;
@@ -1735,19 +1754,6 @@ void CGL33Device::SetRenderState(RenderState state, bool enabled)
 
         return;
     }
-    else if (state == RENDER_STATE_OFFSCREEN_RENDERING)
-    {
-        if (m_framebuffer == 0)
-            InitOffscreenBuffer(1024, 1024);
-
-        m_offscreenRenderingEnabled = enabled;
-
-        GLuint toBind = (enabled ? m_framebuffer : 0);
-
-        glBindFramebuffer(GL_FRAMEBUFFER, toBind);
-
-        return;
-    }
     else if (state == RENDER_STATE_FOG)
     {
         glUniform1i(uni_FogEnabled, enabled ? 1 : 0);
@@ -1860,76 +1866,6 @@ void CGL33Device::SetFillMode(FillMode mode)
     else assert(false);
 }
 
-void CGL33Device::InitOffscreenBuffer(int width, int height)
-{
-    width = Math::Min(width, m_maxRenderbufferSize);
-    height = Math::Min(height, m_maxRenderbufferSize);
-
-    glBindFramebuffer(GL_FRAMEBUFFER, 0);
-
-    if (m_colorBuffer != 0)
-        glDeleteRenderbuffers(1, &m_colorBuffer);
-
-    glGenRenderbuffers(1, &m_colorBuffer);
-    glBindRenderbuffer(GL_RENDERBUFFER, m_colorBuffer);
-    glRenderbufferStorage(GL_RENDERBUFFER, GL_RGBA8, width, height);
-    glBindRenderbuffer(GL_RENDERBUFFER, 0);
-
-    if (m_depthBuffer != 0)
-        glDeleteRenderbuffers(1, &m_depthBuffer);
-
-    glGenRenderbuffers(1, &m_depthBuffer);
-    glBindRenderbuffer(GL_RENDERBUFFER, m_depthBuffer);
-    glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT24, width, height);
-    glBindRenderbuffer(GL_RENDERBUFFER, 0);
-
-    if (m_framebuffer == 0)
-        glGenFramebuffers(1, &m_framebuffer);
-
-    glBindFramebuffer(GL_FRAMEBUFFER, m_framebuffer);
-    glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, m_colorBuffer);
-    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, m_depthBuffer);
-    glBindFramebufferEXT(GL_FRAMEBUFFER, 0);
-
-    GetLogger()->Info("Initialized offscreen buffer %dx%d\n", width, height);
-}
-
-void CGL33Device::SetRenderTexture(RenderTarget target, int texture)
-{
-    if (!m_offscreenRenderingEnabled) return;
-
-    GLenum attachment;
-    GLuint defaultBuffer;
-
-    switch (target)
-    {
-    case RENDER_TARGET_COLOR:
-        attachment = GL_COLOR_ATTACHMENT0;
-        defaultBuffer = m_colorBuffer;
-        break;
-    case RENDER_TARGET_DEPTH:
-        attachment = GL_DEPTH_ATTACHMENT;
-        defaultBuffer = m_depthBuffer;
-        break;
-    case RENDER_TARGET_STENCIL:
-        attachment = GL_STENCIL_ATTACHMENT;
-        defaultBuffer = 0;
-        break;
-    default: assert(false); break;
-    }
-
-    if (texture == 0)       // unbind texture and bind default buffer
-    {
-        glFramebufferTexture2DEXT(GL_FRAMEBUFFER, attachment, GL_TEXTURE_2D, 0, 0);
-        glFramebufferRenderbufferEXT(GL_FRAMEBUFFER, attachment, GL_RENDERBUFFER, defaultBuffer);
-    }
-    else            // unbind default buffer and bind texture
-    {
-        glFramebufferRenderbuffer(GL_FRAMEBUFFER, attachment, GL_RENDERBUFFER, 0);
-        glFramebufferTexture2D(GL_FRAMEBUFFER, attachment, GL_TEXTURE_2D, texture, 0);
-    }
-}
-
 void CGL33Device::CopyFramebufferToTexture(Texture& texture, int xOffset, int yOffset, int x, int y, int width, int height)
 {
     if (texture.id == 0) return;
@@ -1943,18 +1879,47 @@ void CGL33Device::CopyFramebufferToTexture(Texture& texture, int xOffset, int yO
     glBindTexture(GL_TEXTURE_2D, m_currentTextures[0].id);
 }
 
-void* CGL33Device::GetFrameBufferPixels() const
+std::unique_ptr<CFrameBufferPixels> CGL33Device::GetFrameBufferPixels() const
 {
-    GLubyte* pixels = new GLubyte[4 * m_config.size.x * m_config.size.y];
+    return GetGLFrameBufferPixels(m_config.size);
+}
 
-    glReadPixels(0, 0, m_config.size.x, m_config.size.y, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+CFramebuffer* CGL33Device::GetFramebuffer(std::string name)
+{
+    auto it = m_framebuffers.find(name);
+    if (it == m_framebuffers.end())
+        return nullptr;
 
-    unsigned int* p = static_cast<unsigned int*> ( static_cast<void*>(pixels) );
+    return it->second.get();
+}
 
-    for (int i = 0; i < m_config.size.x * m_config.size.y; ++i)
-        p[i] |= 0xFF000000;
+CFramebuffer* CGL33Device::CreateFramebuffer(std::string name, const FramebufferParams& params)
+{
+    // existing framebuffer was found
+    if (m_framebuffers.find(name) != m_framebuffers.end())
+    {
+        return nullptr;
+    }
 
-    return static_cast<void*>(p);
+    auto framebuffer = MakeUnique<CGLFramebuffer>(params);
+    framebuffer->Create();
+
+    CFramebuffer* framebufferPtr = framebuffer.get();
+    m_framebuffers[name] = std::move(framebuffer);
+    return framebufferPtr;
+}
+
+void CGL33Device::DeleteFramebuffer(std::string name)
+{
+    // can't delete default framebuffer
+    if (name == "default") return;
+
+    auto it = m_framebuffers.find(name);
+    if (it != m_framebuffers.end())
+    {
+        it->second->Destroy();
+        m_framebuffers.erase(it);
+    }
 }
 
 void CGL33Device::UpdateRenderingMode()
@@ -1985,4 +1950,36 @@ inline void CGL33Device::BindVAO(GLuint vao)
     m_currentVAO = vao;
 }
 
+bool CGL33Device::IsAnisotropySupported()
+{
+    return m_anisotropyAvailable;
+}
+
+int CGL33Device::GetMaxAnisotropyLevel()
+{
+    return m_maxAnisotropy;
+}
+
+int CGL33Device::GetMaxSamples()
+{
+    return m_maxSamples;
+}
+
+bool CGL33Device::IsShadowMappingSupported()
+{
+    return true;
+}
+
+int CGL33Device::GetMaxTextureSize()
+{
+    int value;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &value);
+    return value;
+}
+
+bool CGL33Device::IsFramebufferSupported()
+{
+    return true;
+}
+
 } // namespace Gfx
