@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,12 +19,16 @@
 
 
 #include "graphics/opengl/gldevice.h"
-#include "graphics/engine/engine.h"
 
 #include "common/config.h"
+
 #include "common/image.h"
 #include "common/logger.h"
 
+#include "graphics/engine/engine.h"
+
+#include "graphics/opengl/glframebuffer.h"
+
 #include "math/geometry.h"
 
 
@@ -34,29 +38,12 @@
 
 
 // Graphics module namespace
-namespace Gfx {
-
-CGLDevice::CGLDevice(const GLDeviceConfig &config)
+namespace Gfx
 {
-    m_config = config;
-    m_lighting = false;
-    m_lastVboId = 0;
-    m_multitextureAvailable = false;
-    m_vboAvailable = false;
-    m_vertexBufferType = VBT_DISPLAY_LIST;
-    m_anisotropyAvailable = false;
-    m_maxAnisotropy = 1;
-    m_glMajor = 1;
-    m_glMinor = 1;
-    m_shadowMappingSupport = SMS_NONE;
-    m_shadowAmbientSupported = false;
-
-    m_framebuffer = 0;
-    m_colorBuffer = 0;
-    m_depthBuffer = 0;
-    m_offscreenRenderingEnabled = false;
-}
 
+CGLDevice::CGLDevice(const DeviceConfig &config)
+    : m_config(config)
+{}
 
 CGLDevice::~CGLDevice()
 {
@@ -197,17 +184,8 @@ bool CGLDevice::Create()
         if (!m_multitextureAvailable)
             GetLogger()->Warn("GLEW reports multitexturing not supported - graphics quality will be degraded!\n");
 
-        m_framebufferObject = glewIsSupported("GL_EXT_framebuffer_object");
-        if (m_framebufferObject)
-        {
-            glGetIntegerv(GL_MAX_RENDERBUFFER_SIZE_EXT, &m_maxRenderbufferSize);
-
-            GetLogger()->Info("Offscreen rendering available\n");
-            GetLogger()->Info("Maximum renderbuffer size: %d\n", m_maxRenderbufferSize);
-        }
-
         // Detect Shadow mapping support
-        if (m_glMajor >= 2 || m_glMinor >= 4)     // Core depth texture+shadow, OpenGL 1.4+
+        if (m_glMajor > 1 || m_glMinor >= 4)     // Core depth texture+shadow, OpenGL 1.4+
         {
             m_shadowMappingSupport = SMS_CORE;
             GetLogger()->Info("Shadow mapping available (core)\n");
@@ -235,7 +213,7 @@ bool CGLDevice::Create()
             float level;
             glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &level);
             m_maxAnisotropy = static_cast<int>(level);
-            
+
             GetLogger()->Info("Anisotropic filtering available\n");
             GetLogger()->Info("Maximum anisotropy: %d\n", m_maxAnisotropy);
         }
@@ -243,41 +221,39 @@ bool CGLDevice::Create()
         {
             GetLogger()->Info("Anisotropic filtering not available\n");
         }
-        
-        if (m_config.vboMode == VBO_MODE_ENABLE)
+
+        // Read maximum sample count for MSAA
+        if(glewIsSupported("GL_ARB_multisample"))
         {
-            GetLogger()->Info("VBO enabled by override - using VBOs\n");
-            SetVertexBufferType(VBT_VBO_CORE);
-        }
-        else if (m_config.vboMode == VBO_MODE_DISABLE)
-        {
-            GetLogger()->Info("VBO disabled by override - using display lists\n");
-            SetVertexBufferType(VBT_DISPLAY_LIST);
+            glGetIntegerv(GL_MAX_SAMPLES_EXT, &m_maxSamples);
+            GetLogger()->Info("Multisampling supported, max samples: %d\n", m_maxSamples);
         }
         else
         {
-            GetLogger()->Info("Auto-detecting VBO support\n");
-            
-            // detecting VBO ARB extension
-            bool vboARB = glewIsSupported("GL_ARB_vertex_buffer_object");
+            GetLogger()->Info("Multisampling not supported\n");
+        }
 
-            // VBO is core OpenGL feature since 1.5
-            // everything below 1.5 means no VBO support
-            if (m_glMajor > 1 || m_glMinor > 4)
-            {
-                GetLogger()->Info("Core VBO supported\n", m_glMajor, m_glMinor);
-                SetVertexBufferType(VBT_VBO_CORE);
-            }
-            else if(vboARB)     // VBO ARB extension available
-            {
-                GetLogger()->Info("ARB VBO supported\n");
-                SetVertexBufferType(VBT_VBO_ARB);
-            }
-            else                // no VBO support
-            {
-                GetLogger()->Info("VBO not supported\n");
-                SetVertexBufferType(VBT_DISPLAY_LIST);
-            }
+        GetLogger()->Info("Auto-detecting VBO support\n");
+
+        // detecting VBO ARB extension
+        bool vboARB = glewIsSupported("GL_ARB_vertex_buffer_object");
+
+        // VBO is core OpenGL feature since 1.5
+        // everything below 1.5 means no VBO support
+        if (m_glMajor > 1 || m_glMinor > 4)
+        {
+            GetLogger()->Info("Core VBO supported\n", m_glMajor, m_glMinor);
+            m_vertexBufferType = VBT_VBO_CORE;
+        }
+        else if(vboARB)     // VBO ARB extension available
+        {
+            GetLogger()->Info("ARB VBO supported\n");
+            m_vertexBufferType = VBT_VBO_ARB;
+        }
+        else                // no VBO support
+        {
+            GetLogger()->Info("VBO not supported\n");
+            m_vertexBufferType = VBT_DISPLAY_LIST;
         }
     }
 
@@ -317,6 +293,19 @@ bool CGLDevice::Create()
     m_texturesEnabled    = std::vector<bool>              (maxTextures, false);
     m_textureStageParams = std::vector<TextureStageParams>(maxTextures, TextureStageParams());
 
+    // create default framebuffer object
+    FramebufferParams framebufferParams;
+
+    framebufferParams.width = m_config.size.x;
+    framebufferParams.height = m_config.size.y;
+    framebufferParams.depth = m_config.depthSize;
+
+    m_framebuffers["default"] = MakeUnique<CDefaultFramebuffer>(framebufferParams);
+
+    m_framebufferSupport = DetectFramebufferSupport();
+    if (m_framebufferSupport != FBS_NONE)
+        GetLogger()->Debug("Framebuffer supported\n");
+
     GetLogger()->Info("CDevice created successfully\n");
 
     return true;
@@ -324,6 +313,12 @@ bool CGLDevice::Create()
 
 void CGLDevice::Destroy()
 {
+    // delete framebuffers
+    for (auto& framebuffer : m_framebuffers)
+        framebuffer.second->Destroy();
+
+    m_framebuffers.clear();
+
     // Delete the remaining textures
     // Should not be strictly necessary, but just in case
     DestroyAllTextures();
@@ -336,7 +331,7 @@ void CGLDevice::Destroy()
     m_textureStageParams.clear();
 }
 
-void CGLDevice::ConfigChanged(const GLDeviceConfig& newConfig)
+void CGLDevice::ConfigChanged(const DeviceConfig& newConfig)
 {
     m_config = newConfig;
 
@@ -346,18 +341,6 @@ void CGLDevice::ConfigChanged(const GLDeviceConfig& newConfig)
     Create();
 }
 
-void CGLDevice::SetUseVbo(bool vboAvailable)
-{
-    m_vboAvailable = vboAvailable;
-    m_vertexBufferType = vboAvailable ? VBT_VBO_CORE : VBT_DISPLAY_LIST;
-}
-
-void CGLDevice::SetVertexBufferType(VertexBufferType type)
-{
-    m_vertexBufferType = type;
-    m_vboAvailable = (type != VBT_DISPLAY_LIST);
-}
-
 void CGLDevice::BeginScene()
 {
     Clear();
@@ -557,7 +540,7 @@ Texture CGLDevice::CreateTexture(ImageData *data, const TextureCreateParams &par
 
     if (!Math::IsPowerOfTwo(result.size.x) || !Math::IsPowerOfTwo(result.size.y))
         GetLogger()->Warn("Creating non-power-of-2 texture (%dx%d)!\n", result.size.x, result.size.y);
-    
+
     result.originalSize = result.size;
 
     // Use & enable 1st texture stage
@@ -596,24 +579,24 @@ Texture CGLDevice::CreateTexture(ImageData *data, const TextureCreateParams &par
     // Set mipmap level and automatic mipmap generation if neccesary
     if (params.mipmap)
     {
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, mipmapLevel - 1);
-	    glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, mipmapLevel - 1);
+        glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
     }
     else
     {
-	    // Has to be set to 0 because no mipmaps are generated
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
-	    glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_FALSE);
+        // Has to be set to 0 because no mipmaps are generated
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+        glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_FALSE);
     }
 
     // Set anisotropy level if available
     if (m_anisotropyAvailable)
     {
-	    float level = Math::Min(m_maxAnisotropy, CEngine::GetInstance().GetTextureAnisotropyLevel());
+        float level = Math::Min(m_maxAnisotropy, CEngine::GetInstance().GetTextureAnisotropyLevel());
 
-	    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, level);
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, level);
     }
 
     bool convert = false;
@@ -687,7 +670,8 @@ Texture CGLDevice::CreateTexture(ImageData *data, const TextureCreateParams &par
                 convert = true;
             }
         }
-        else {
+        else
+        {
             GetLogger()->Error("Unknown data surface format");
             assert(false);
         }
@@ -725,6 +709,7 @@ Texture CGLDevice::CreateTexture(ImageData *data, const TextureCreateParams &par
 
     SDL_FreeSurface(convertedSurface);
 
+    m_allTextures.insert(result);
 
     // Restore the previous state of 1st stage
     glBindTexture(GL_TEXTURE_2D, m_currentTextures[0].id);
@@ -744,7 +729,7 @@ Texture CGLDevice::CreateDepthTexture(int width, int height, int depth)
         result.id = 0;
         return result;
     }
-    
+
     result.alpha = false;
     result.size.x = width;
     result.size.y = height;
@@ -799,7 +784,7 @@ Texture CGLDevice::CreateDepthTexture(int width, int height, int depth)
 
     if (m_shadowAmbientSupported)
     {
-        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FAIL_VALUE_ARB, 0.35f);
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FAIL_VALUE_ARB, 0.5f);
     }
 
     float color[] = { 1.0f, 1.0f, 1.0f, 1.0f };
@@ -824,11 +809,12 @@ void CGLDevice::DestroyTexture(const Texture &texture)
             SetTexture(index, Texture()); // set to invalid texture
     }
 
-    glDeleteTextures(1, &texture.id);
-
     auto it = m_allTextures.find(texture);
     if (it != m_allTextures.end())
+    {
+        glDeleteTextures(1, &texture.id);
         m_allTextures.erase(it);
+    }
 }
 
 void CGLDevice::DestroyAllTextures()
@@ -1167,7 +1153,7 @@ void CGLDevice::SetTextureStageWrap(int index, TexWrapMode wrapS, TexWrapMode wr
 
     if (m_multitextureAvailable)
         glActiveTexture(GL_TEXTURE0 + index);
-    
+
     if      (wrapS == TEX_WRAP_CLAMP)
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
     else if (wrapS == TEX_WRAP_CLAMP_TO_BORDER)
@@ -1365,7 +1351,7 @@ unsigned int CGLDevice::CreateStaticBuffer(PrimitiveType primitiveType, const Ve
         info.vertexType = VERTEX_TYPE_COL;
         info.vertexCount = vertexCount;
         info.bufferId = 0;
-        
+
         if(m_vertexBufferType == VBT_VBO_CORE)
         {
             glGenBuffers(1, &info.bufferId);
@@ -1514,7 +1500,7 @@ void CGLDevice::DrawStaticBuffer(unsigned int bufferId)
             return;
 
         glEnable(GL_VERTEX_ARRAY);
-        
+
         if(m_vertexBufferType == VBT_VBO_CORE)
             glBindBuffer(GL_ARRAY_BUFFER, (*it).second.bufferId);
         else
@@ -1594,7 +1580,7 @@ void CGLDevice::DrawStaticBuffer(unsigned int bufferId)
             glBindBuffer(GL_ARRAY_BUFFER, 0);
         else
             glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
-        
+
         glDisable(GL_VERTEX_ARRAY);
     }
     else
@@ -1734,25 +1720,6 @@ void CGLDevice::SetRenderState(RenderState state, bool enabled)
 
         return;
     }
-    else if (state == RENDER_STATE_OFFSCREEN_RENDERING)
-    {
-        if (!m_framebufferObject)
-        {
-            GetLogger()->Error("Cannot enable offscreen rendering without framebuffer object!\n");
-            return;
-        }
-
-        m_offscreenRenderingEnabled = enabled;
-
-        if (m_framebuffer == 0)
-            InitOffscreenBuffer(2048, 2048);
-
-        GLuint toBind = (enabled ? m_framebuffer : 0);
-
-        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, toBind);
-
-        return;
-    }
 
     GLenum flag = 0;
 
@@ -1850,79 +1817,6 @@ void CGLDevice::SetFillMode(FillMode mode)
     else assert(false);
 }
 
-void CGLDevice::InitOffscreenBuffer(int width, int height)
-{
-    if (!m_framebufferObject) return;
-
-    width = Math::Min(width, m_maxRenderbufferSize);
-    height = Math::Min(height, m_maxRenderbufferSize);
-
-    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
-
-    if (m_colorBuffer != 0)
-        glDeleteRenderbuffersEXT(1, &m_colorBuffer);
-
-    glGenRenderbuffersEXT(1, &m_colorBuffer);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_colorBuffer);
-    glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_RGBA8, width, height);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0);
-
-    if (m_depthBuffer != 0)
-        glDeleteRenderbuffersEXT(1, &m_depthBuffer);
-
-    glGenRenderbuffersEXT(1, &m_depthBuffer);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_depthBuffer);
-    glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, width, height);
-    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0);
-
-    if (m_framebuffer == 0)
-        glGenFramebuffersEXT(1, &m_framebuffer);
-
-    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_framebuffer);
-    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_RENDERBUFFER_EXT, m_colorBuffer);
-    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, m_depthBuffer);
-    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
-
-    GetLogger()->Info("Initialized offscreen buffer %dx%d\n", width, height);
-}
-
-void CGLDevice::SetRenderTexture(RenderTarget target, int texture)
-{
-    if (!m_framebufferObject) return;
-    if (!m_offscreenRenderingEnabled) return;
-
-    GLenum attachment;
-    GLuint defaultBuffer;
-
-    switch (target)
-    {
-    case RENDER_TARGET_COLOR:
-        attachment = GL_COLOR_ATTACHMENT0_EXT;
-        defaultBuffer = m_colorBuffer;
-        break;
-    case RENDER_TARGET_DEPTH:
-        attachment = GL_DEPTH_ATTACHMENT_EXT;
-        defaultBuffer = m_depthBuffer;
-        break;
-    case RENDER_TARGET_STENCIL:
-        attachment = GL_STENCIL_ATTACHMENT_EXT;
-        defaultBuffer = 0;
-        break;
-    default: assert(false); break;
-    }
-
-    if (texture == 0)       // unbind texture and bind default buffer
-    {
-        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment, GL_TEXTURE_2D, 0, 0);
-        glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, attachment, GL_RENDERBUFFER_EXT, defaultBuffer);
-    }
-    else            // unbind default buffer and bind texture
-    {
-        glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, attachment, GL_RENDERBUFFER_EXT, 0);
-        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment, GL_TEXTURE_2D, texture, 0);
-    }
-}
-
 void CGLDevice::CopyFramebufferToTexture(Texture& texture, int xOffset, int yOffset, int x, int y, int width, int height)
 {
     if (texture.id == 0) return;
@@ -1938,19 +1832,87 @@ void CGLDevice::CopyFramebufferToTexture(Texture& texture, int xOffset, int yOff
     glBindTexture(GL_TEXTURE_2D, m_currentTextures[0].id);
 }
 
-void* CGLDevice::GetFrameBufferPixels()const{
+std::unique_ptr<CFrameBufferPixels> CGLDevice::GetFrameBufferPixels() const
+{
+    return GetGLFrameBufferPixels(m_config.size);
+}
 
-    GLubyte* pixels = new GLubyte[4 * m_config.size.x * m_config.size.y];
+CFramebuffer* CGLDevice::GetFramebuffer(std::string name)
+{
+    auto it = m_framebuffers.find(name);
+    if (it == m_framebuffers.end())
+        return nullptr;
 
-    glReadPixels(0, 0, m_config.size.x, m_config.size.y, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+    return it->second.get();
+}
 
-    unsigned int* p = static_cast<unsigned int*> ( static_cast<void*>(pixels) );
+CFramebuffer* CGLDevice::CreateFramebuffer(std::string name, const FramebufferParams& params)
+{
+    // existing framebuffer was found
+    if (m_framebuffers.find(name) != m_framebuffers.end())
+    {
+        return nullptr;
+    }
 
-    for (int i = 0; i < m_config.size.x * m_config.size.y; ++i)
-        p[i] |= 0xFF000000;
+    std::unique_ptr<CFramebuffer> framebuffer;
 
-    return static_cast<void*>(p);
+    if (m_framebufferSupport == FBS_ARB)
+        framebuffer = MakeUnique<CGLFramebuffer>(params);
+    else if (m_framebufferSupport == FBS_EXT)
+        framebuffer = MakeUnique<CGLFramebufferEXT>(params);
+    else
+        return nullptr;
+
+    framebuffer->Create();
+
+    CFramebuffer* framebufferPtr = framebuffer.get();
+    m_framebuffers[name] = std::move(framebuffer);
+    return framebufferPtr;
+}
+
+void CGLDevice::DeleteFramebuffer(std::string name)
+{
+    // can't delete default framebuffer
+    if (name == "default") return;
+
+    auto it = m_framebuffers.find(name);
+    if (it != m_framebuffers.end())
+    {
+        it->second->Destroy();
+        m_framebuffers.erase(it);
+    }
+}
+
+bool CGLDevice::IsAnisotropySupported()
+{
+    return m_anisotropyAvailable;
+}
+
+int CGLDevice::GetMaxAnisotropyLevel()
+{
+    return m_maxAnisotropy;
+}
+
+int CGLDevice::GetMaxSamples()
+{
+    return m_maxSamples;
+}
+
+bool CGLDevice::IsShadowMappingSupported()
+{
+    return m_shadowMappingSupport != SMS_NONE;
+}
+
+int CGLDevice::GetMaxTextureSize()
+{
+    int value;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &value);
+    return value;
+}
+
+bool CGLDevice::IsFramebufferSupported()
+{
+    return m_framebufferSupport != FBS_NONE;
 }
 
 } // namespace Gfx
-
