@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@
 
 #include "math/const.h"
 #include "math/func.h"
-#include "math/point.h"
 #include "math/matrix.h"
+#include "math/point.h"
 #include "math/vector.h"
 
 
@@ -37,7 +37,8 @@
 
 
 // Math module namespace
-namespace Math {
+namespace Math
+{
 
 
 //! Returns py up on the line \a a - \a b
@@ -518,7 +519,7 @@ inline Math::Vector NormalToPlane(const Math::Vector &p1, const Math::Vector &p2
 inline Math::Vector SegmentPoint(const Math::Vector &p1, const Math::Vector &p2, float dist)
 {
     Math::Vector direction = p2 - p1;
-    
+
     direction.Normalize();
 
     return p1 + direction * dist;
