@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,24 +18,30 @@
  */
 
 
-#include <stdio.h>
-
 #include "object/auto/autobase.h"
 
-#include "graphics/engine/terrain.h"
 #include "graphics/engine/cloud.h"
-#include "graphics/engine/planet.h"
 #include "graphics/engine/lightning.h"
+#include "graphics/engine/planet.h"
+#include "graphics/engine/terrain.h"
+
+#include "level/robotmain.h"
 
 #include "math/geometry.h"
 
-#include "object/objman.h"
-#include "object/robotmain.h"
+#include "object/object_manager.h"
+#include "object/old_object.h"
+
+#include "object/interface/transportable_object.h"
 
 #include "physics/physics.h"
 
-#include "ui/interface.h"
-#include "ui/window.h"
+#include "sound/sound.h"
+
+#include "ui/controls/interface.h"
+#include "ui/controls/window.h"
+
+#include "sound/sound.h"
 
 
 
@@ -53,7 +59,7 @@ const float BASE_TRANSIT_TIME       = 15.0f;    // transit duration
 
 // Object's constructor.
 
-CAutoBase::CAutoBase(CObject* object) : CAuto(object)
+CAutoBase::CAutoBase(COldObject* object) : CAuto(object)
 {
     m_fogStart = m_engine->GetFogStart();
     m_deepView = m_engine->GetDeepView();
@@ -93,7 +99,7 @@ void CAutoBase::Init()
     m_lastParticle = 0.0f;
     m_lastMotorParticle = 0.0f;
 
-    m_pos = m_object->GetPosition(0);
+    m_pos = m_object->GetPosition();
     m_lastPos = m_pos;
 
     m_phase    = ABP_WAIT;
@@ -119,7 +125,6 @@ void CAutoBase::Start(int param)
 bool CAutoBase::EventProcess(const Event &event)
 {
     Math::Matrix*   mat;
-    Event       newEvent;
     CObject*    pObj;
     Math::Vector    pos, speed, vibCir, iPos;
     Math::Point     dim, p;
@@ -131,7 +136,7 @@ bool CAutoBase::EventProcess(const Event &event)
     if ( m_engine->GetPause() )  return true;
 
 begin:
-    iPos = m_object->GetPosition(0);
+    iPos = m_object->GetPosition();
 
     if ( m_phase == ABP_START )
     {
@@ -149,24 +154,25 @@ begin:
 
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetAngleZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
-                m_object->SetAngleX(10+i, -10.0f*Math::PI/180.0f);
-                m_object->SetAngleX(18+i,  10.0f*Math::PI/180.0f);
-                m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
-                m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
+                m_object->SetPartRotationZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
+                m_object->SetPartRotationX(10+i, -10.0f*Math::PI/180.0f);
+                m_object->SetPartRotationX(18+i,  10.0f*Math::PI/180.0f);
+                m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
+                m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
             }
 
             pObj = m_main->GetSelectObject();
             m_main->SelectObject(pObj);
             m_camera->SetControllingObject(pObj);
-            if ( pObj == 0 )
+            if ( pObj == nullptr )
             {
                 m_camera->SetType(Gfx::CAM_TYPE_BACK);
             }
             else
             {
-                m_camera->SetType(pObj->GetCameraType());
-                m_camera->SetDist(pObj->GetCameraDist());
+                assert(pObj->Implements(ObjectInterfaceType::Controllable));
+                m_camera->SetType(dynamic_cast<CControllableObject*>(pObj)->GetCameraType());
+                m_camera->SetDist(dynamic_cast<CControllableObject*>(pObj)->GetCameraDist());
             }
 
             m_main->StartMusic();
@@ -180,11 +186,11 @@ begin:
 
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetAngleZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
-                m_object->SetAngleX(10+i, -10.0f*Math::PI/180.0f);
-                m_object->SetAngleX(18+i,  10.0f*Math::PI/180.0f);
-                m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
-                m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
+                m_object->SetPartRotationZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
+                m_object->SetPartRotationX(10+i, -10.0f*Math::PI/180.0f);
+                m_object->SetPartRotationX(18+i,  10.0f*Math::PI/180.0f);
+                m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
+                m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
             }
         }
 
@@ -206,7 +212,7 @@ begin:
             m_camera->SetScriptEye(pos);
             m_posSound = pos;
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.y += 300.0f+50.0f;
             m_camera->SetScriptLookat(pos);
 
@@ -227,11 +233,11 @@ begin:
 
         if ( m_param == PARAM_PORTICO )  // gate on the porch?
         {
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             m_finalPos = pos;
             pos.z += BASE_PORTICO_TIME_MOVE*5.0f;  // back
             pos.y += 10.0f;  // rises (the gate)
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
             MoveCargo();  // all cargo moves
 
             m_phase    = ABP_PORTICO_MOVE;
@@ -249,11 +255,11 @@ begin:
             m_progress = 0.0f;
             m_speed    = 1.0f/BASE_TRANSIT_TIME;
 
-            m_object->SetAngleZ(0, -Math::PI/2.0f);
-            pos = m_object->GetPosition(0);
+            m_object->SetRotationZ(-Math::PI/2.0f);
+            pos = m_object->GetPosition();
             pos.y += 10000.0f;  // in space
             m_finalPos = pos;
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
 
             m_main->SetMovieLock(true);  // blocks everything until the end of the landing
             m_bMotor = true;  // lights the jet engine
@@ -328,7 +334,7 @@ begin:
         {
             pos = m_pos;
             pos.y += powf(1.0f-m_progress, 2.0f)*300.0f;
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
             MoveCargo();  // all cargo moves
 
             vibCir.z = sinf(m_time*Math::PI* 2.01f)*(Math::PI/150.0f)+
@@ -347,7 +353,7 @@ begin:
             pos.y += 10.0f;
             m_camera->SetScriptEye(pos);
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.y += 50.0f;
             m_camera->SetScriptLookat(pos);
 
@@ -375,7 +381,7 @@ begin:
                 }
 
                 // Particles are ejected from the jet engine.
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 pos.y += 6.0f;
                 h = m_terrain->GetHeightToFloor(pos)/300.0f;
                 speed.x = (Math::Rand()-0.5f)*(80.0f-50.0f*h);
@@ -405,7 +411,7 @@ begin:
         {
             m_bMotor = false;  // put out the reactor
 
-            m_object->SetPosition(0, m_pos);  // setting down
+            m_object->SetPosition(m_pos);  // setting down
             m_object->SetCirVibration(Math::Vector(0.0f, 0.0f, 0.0f));
             MoveCargo();  // all cargo moves
 
@@ -477,7 +483,7 @@ begin:
             angle = -m_progress*124.0f*Math::PI/180.0f;
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetAngleZ(1+i, Math::PI/2.0f+angle);
+                m_object->SetPartRotationZ(1+i, Math::PI/2.0f+angle);
             }
 
             if ( m_param != PARAM_PORTICO )
@@ -499,7 +505,7 @@ begin:
         {
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetAngleZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
+                m_object->SetPartRotationZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
             }
 
             // Clash the doors with the ground.
@@ -536,10 +542,10 @@ begin:
             len = 7.0f-m_progress*(7.0f+11.5f);
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f,  len));
-                m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f, -len));
-                m_object->SetAngleX(10+i, -10.0f*Math::PI/180.0f*m_progress);
-                m_object->SetAngleX(18+i,  10.0f*Math::PI/180.0f*m_progress);
+                m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f,  len));
+                m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f, -len));
+                m_object->SetPartRotationX(10+i, -10.0f*Math::PI/180.0f*m_progress);
+                m_object->SetPartRotationX(18+i,  10.0f*Math::PI/180.0f*m_progress);
             }
 
             if ( m_param != PARAM_PORTICO )
@@ -561,10 +567,10 @@ begin:
         {
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
-                m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
-                m_object->SetAngleX(10+i, -10.0f*Math::PI/180.0f);
-                m_object->SetAngleX(18+i,  10.0f*Math::PI/180.0f);
+                m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
+                m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
+                m_object->SetPartRotationX(10+i, -10.0f*Math::PI/180.0f);
+                m_object->SetPartRotationX(18+i,  10.0f*Math::PI/180.0f);
             }
 
             m_phase    = ABP_LDWAIT;
@@ -586,16 +592,17 @@ begin:
                 pObj = m_main->GetSelectObject();
                 m_main->SelectObject(pObj);
                 m_camera->SetControllingObject(pObj);
-                if ( pObj == 0 )
+                if ( pObj == nullptr )
                 {
                     m_camera->SetType(Gfx::CAM_TYPE_BACK);
                 }
                 else
                 {
-                    m_camera->SetType(pObj->GetCameraType());
-                    m_camera->SetDist(pObj->GetCameraDist());
+                    assert(pObj->Implements(ObjectInterfaceType::Controllable));
+                    m_camera->SetType(dynamic_cast<CControllableObject*>(pObj)->GetCameraType());
+                    m_camera->SetDist(dynamic_cast<CControllableObject*>(pObj)->GetCameraDist());
                 }
-                m_sound->Play(SOUND_BOUM, m_object->GetPosition(0));
+                m_sound->Play(SOUND_BOUM, m_object->GetPosition());
                 m_soundChannel = -1;
 
                 m_engine->SetFogStart(m_fogStart);
@@ -615,20 +622,20 @@ begin:
             len = 7.0f-(1.0f-m_progress)*(7.0f+11.5f);
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f,  len));
-                m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f, -len));
-                m_object->SetAngleX(10+i, -10.0f*Math::PI/180.0f*(1.0f-m_progress));
-                m_object->SetAngleX(18+i,  10.0f*Math::PI/180.0f*(1.0f-m_progress));
+                m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f,  len));
+                m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f, -len));
+                m_object->SetPartRotationX(10+i, -10.0f*Math::PI/180.0f*(1.0f-m_progress));
+                m_object->SetPartRotationX(18+i,  10.0f*Math::PI/180.0f*(1.0f-m_progress));
             }
         }
         else
         {
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f,  7.0f));
-                m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f, -7.0f));
-                m_object->SetAngleX(10+i, 0.0f);
-                m_object->SetAngleX(18+i, 0.0f);
+                m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f,  7.0f));
+                m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f, -7.0f));
+                m_object->SetPartRotationX(10+i, 0.0f);
+                m_object->SetPartRotationX(18+i, 0.0f);
             }
 
             m_soundChannel = m_sound->Play(SOUND_MANIP, m_posSound, 0.0f, 0.3f, true);
@@ -649,14 +656,14 @@ begin:
             angle = -(1.0f-m_progress)*124.0f*Math::PI/180.0f;
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetAngleZ(1+i, Math::PI/2.0f+angle);
+                m_object->SetPartRotationZ(1+i, Math::PI/2.0f+angle);
             }
         }
         else
         {
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetAngleZ(1+i, Math::PI/2.0f);
+                m_object->SetPartRotationZ(1+i, Math::PI/2.0f);
             }
             m_bMotor = true;  // lights the jet engine
 
@@ -676,7 +683,7 @@ begin:
                 time = Math::Rand()*1.0f+1.0f;
                 m_particle->CreateParticle(pos, speed, dim, Gfx::PARTICRASH, time);
             }
-            m_sound->Play(SOUND_BOUM, m_object->GetPosition(0));
+            m_sound->Play(SOUND_BOUM, m_object->GetPosition());
 
             m_soundChannel = -1;
             m_bOpen    = false;
@@ -708,7 +715,7 @@ begin:
                 m_lastParticle = m_time;
 
                 // Particles are ejected from the reactor.
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 pos.y += 6.0f;
                 speed.x = (Math::Rand()-0.5f)*160.0f;
                 speed.z = (Math::Rand()-0.5f)*160.0f;
@@ -736,7 +743,7 @@ begin:
         {
             pos = m_pos;
             pos.y += powf(m_progress, 2.0f)*600.0f;
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
             MoveCargo();  // all cargo moves
 
             vibCir.z = sinf(m_time*Math::PI*19.01f)*(Math::PI/400.0f);
@@ -750,7 +757,7 @@ begin:
             pos.y += 10.0f;
             m_camera->SetScriptEye(pos);
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.y += 50.0f;
             m_camera->SetScriptLookat(pos);
 
@@ -778,7 +785,7 @@ begin:
                 }
 
                 // Particles are ejected from the reactor.
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 pos.y += 6.0f;
                 speed.x = (Math::Rand()-0.5f)*40.0f;
                 speed.z = (Math::Rand()-0.5f)*40.0f;
@@ -790,7 +797,7 @@ begin:
                 m_particle->CreateParticle(pos, speed, dim, Gfx::PARTIGAS, 2.0f, 10.0f, 2.0f);
 
                 // Black smoke from the reactor.
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 pos.y += 3.0f;
                 speed.x = (Math::Rand()-0.5f)*10.0f*(4.0f-m_progress*3.0f);
                 speed.z = (Math::Rand()-0.5f)*10.0f*(4.0f-m_progress*3.0f);
@@ -803,8 +810,7 @@ begin:
         else
         {
             m_soundChannel = -1;
-            newEvent.type = EVENT_WIN;
-            m_eventQueue->AddEvent(newEvent);
+            m_eventQueue->AddEvent(Event(EVENT_WIN));
 
             m_phase    = ABP_WAIT;
             m_progress = 0.0f;
@@ -816,9 +822,9 @@ begin:
     {
         if ( m_progress < 1.0f )
         {
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.z -= event.rTime*5.0f;
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
             MoveCargo();  // all cargo moves
         }
         else
@@ -843,9 +849,9 @@ begin:
     {
         if ( m_progress < 1.0f )
         {
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.y -= event.rTime*(10.0f/BASE_PORTICO_TIME_DOWN);
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
             MoveCargo();  // all cargo moves
         }
         else
@@ -899,15 +905,15 @@ begin:
     {
         if ( m_progress < 1.0f )
         {
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             pos.x += event.rTime*(2000.0f/BASE_TRANSIT_TIME);
-            m_object->SetPosition(0, pos);
+            m_object->SetPosition(pos);
             pos.x += 60.0f;
             m_camera->SetScriptLookat(pos);
         }
         else
         {
-            m_object->SetAngleZ(0, 0.0f);
+            m_object->SetRotationZ(0.0f);
 
             m_param = PARAM_LANDING;
             m_phase    = ABP_START;
@@ -940,7 +946,7 @@ begin:
             }
             else
             {
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 if ( m_phase == ABP_TRANSIT_MOVE )
                 {
                     vSpeed = (pos.x-iPos.x)/event.rTime;
@@ -1053,13 +1059,12 @@ begin:
 
 bool CAutoBase::Abort()
 {
-    Event       newEvent;
     CObject*    pObj;
     int         i;
 
     if ( m_phase == ABP_TRANSIT_MOVE )  // transit ?
     {
-        m_object->SetAngleZ(0, 0.0f);
+        m_object->SetRotationZ(0.0f);
 
         m_param = PARAM_LANDING;
         m_phase    = ABP_START;
@@ -1079,16 +1084,16 @@ bool CAutoBase::Abort()
 
     if ( m_param == PARAM_PORTICO )  // gate on the porch?
     {
-        m_object->SetPosition(0, m_finalPos);
+        m_object->SetPosition(m_finalPos);
         MoveCargo();  // all cargo moves
 
         for ( i=0 ; i<8 ; i++ )
         {
-            m_object->SetAngleZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
-            m_object->SetAngleX(10+i, -10.0f*Math::PI/180.0f);
-            m_object->SetAngleX(18+i,  10.0f*Math::PI/180.0f);
-            m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
-            m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
+            m_object->SetPartRotationZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
+            m_object->SetPartRotationX(10+i, -10.0f*Math::PI/180.0f);
+            m_object->SetPartRotationX(18+i,  10.0f*Math::PI/180.0f);
+            m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
+            m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
         }
     }
     else
@@ -1101,16 +1106,16 @@ bool CAutoBase::Abort()
             m_bMotor = false;  // put out the jet engine
             m_bOpen = true;
 
-            m_object->SetPosition(0, m_pos);  // setting down
+            m_object->SetPosition(m_pos);  // setting down
             m_object->SetCirVibration(Math::Vector(0.0f, 0.0f, 0.0f));
             MoveCargo();  // all cargo moves
             for ( i=0 ; i<8 ; i++ )
             {
-                m_object->SetAngleZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
-                m_object->SetAngleX(10+i, -10.0f*Math::PI/180.0f);
-                m_object->SetAngleX(18+i,  10.0f*Math::PI/180.0f);
-                m_object->SetPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
-                m_object->SetPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
+                m_object->SetPartRotationZ(1+i, Math::PI/2.0f-124.0f*Math::PI/180.0f);
+                m_object->SetPartRotationX(10+i, -10.0f*Math::PI/180.0f);
+                m_object->SetPartRotationX(18+i,  10.0f*Math::PI/180.0f);
+                m_object->SetPartPosition(10+i, Math::Vector(23.5f, 0.0f, -11.5f));
+                m_object->SetPartPosition(18+i, Math::Vector(23.5f, 0.0f,  11.5f));
             }
 
             m_main->SetMovieLock(false);  // you can play!
@@ -1118,14 +1123,15 @@ bool CAutoBase::Abort()
             pObj = m_main->GetSelectObject();
             m_main->SelectObject(pObj);
             m_camera->SetControllingObject(pObj);
-            if ( pObj == 0 )
+            if ( pObj == nullptr )
             {
                 m_camera->SetType(Gfx::CAM_TYPE_BACK);
             }
             else
             {
-                m_camera->SetType(pObj->GetCameraType());
-                m_camera->SetDist(pObj->GetCameraDist());
+                assert(pObj->Implements(ObjectInterfaceType::Controllable));
+                m_camera->SetType(dynamic_cast<CControllableObject*>(pObj)->GetCameraType());
+                m_camera->SetDist(dynamic_cast<CControllableObject*>(pObj)->GetCameraDist());
             }
 
             m_engine->SetFogStart(m_fogStart);
@@ -1136,12 +1142,11 @@ bool CAutoBase::Abort()
              m_phase == ABP_TOWAIT  ||
              m_phase == ABP_TAKEOFF )  // off?
         {
-            newEvent.type = EVENT_WIN;
-            m_eventQueue->AddEvent(newEvent);
+            m_eventQueue->AddEvent(Event(EVENT_WIN));
         }
     }
 
-    m_object->SetAngleZ(0, 0.0f);
+    m_object->SetRotationZ(0.0f);
     FreezeCargo(false);  // frees all cargo
 
     if ( m_soundChannel != -1 )
@@ -1225,47 +1230,31 @@ bool CAutoBase::CreateInterface(bool bSelect)
 
 void CAutoBase::UpdateInterface()
 {
-//    Ui::CWindow*    pw;
-
     if ( !m_object->GetSelect() )  return;
 
     CAuto::UpdateInterface();
-
-//    pw = static_cast< Ui::CWindow* >( m_interface->SearchControl(EVENT_WINDOW0));
 }
 
 
 // Freeze or frees all cargo.
 
-void CAutoBase::FreezeCargo(bool bFreeze)
+void CAutoBase::FreezeCargo(bool freeze)
 {
-    CObject*    pObj;
-    CPhysics*   physics;
-    Math::Vector    oPos;
-    float       dist;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    m_cargoObjects.clear();
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
+        if ( obj == m_object )  continue;  // yourself?
+        if (IsObjectBeingTransported(obj)) continue;
 
-        pObj->SetCargo(false);
-
-        if ( pObj == m_object )  continue;  // yourself?
-        if ( pObj->GetTruck() != 0 )  continue;  // transport object?
-
-        oPos = pObj->GetPosition(0);
-        dist = Math::DistanceProjected(m_pos, oPos);
+        Math::Vector oPos = obj->GetPosition();
+        float dist = Math::DistanceProjected(m_pos, oPos);
         if ( dist < 32.0f )
         {
-            if ( bFreeze )
+            m_cargoObjects.insert(obj);
+            if ( obj->Implements(ObjectInterfaceType::Movable) )
             {
-                pObj->SetCargo(true);
-            }
-
-            physics = pObj->GetPhysics();
-            if ( physics != 0 )
-            {
-                physics->SetFreeze(bFreeze);
+                CPhysics* physics = dynamic_cast<CMovableObject*>(obj)->GetPhysics();
+                physics->SetFreeze(freeze);
             }
         }
     }
@@ -1275,23 +1264,16 @@ void CAutoBase::FreezeCargo(bool bFreeze)
 
 void CAutoBase::MoveCargo()
 {
-    CObject*    pObj;
-    Math::Vector    oPos, sPos;
+    Math::Vector sPos = m_object->GetPosition();
 
-    sPos = m_object->GetPosition(0);
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* obj : m_cargoObjects)
     {
-        pObj = it.second;
-
-        if ( !pObj->GetCargo() )  continue;
-
-        oPos = pObj->GetPosition(0);
+        Math::Vector oPos = obj->GetPosition();
         oPos.y = sPos.y+30.0f;
-        oPos.y += pObj->GetCharacter()->height;
+        oPos.y += obj->GetCharacter()->height;
         oPos.x += sPos.x-m_lastPos.x;
         oPos.z += sPos.z-m_lastPos.z;
-        pObj->SetPosition(0, oPos);
+        obj->SetPosition(oPos);
     }
 
     m_lastPos = sPos;
@@ -1302,26 +1284,19 @@ void CAutoBase::MoveCargo()
 
 Error CAutoBase::CheckCloseDoor()
 {
-    CObject*    pObj;
-    Math::Vector    oPos;
-    ObjectType  type;
-    float       oRad, dist;
-    int         j;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
+        if ( obj == m_object )  continue;  // yourself?
+        if ( !obj->GetActive() )  continue;  // inactive?
 
-        if ( pObj == m_object )  continue;  // yourself?
-        if ( !pObj->GetActif() )  continue;  // inactive?
-
-        type = pObj->GetType();
+        ObjectType type = obj->GetType();
         if ( type == OBJECT_PORTICO )  continue;
 
-        j = 0;
-        while ( pObj->GetCrashSphere(j++, oPos, oRad) )
+        for (const auto& crashSphere : obj->GetAllCrashSpheres())
         {
-            dist = Math::DistanceProjected(m_pos, oPos);
+            Math::Vector oPos = crashSphere.sphere.pos;
+            float oRad = crashSphere.sphere.radius;
+            float dist = Math::DistanceProjected(m_pos, oPos);
             if ( dist+oRad > 32.0f &&
                  dist-oRad < 72.0f )
             {
@@ -1345,33 +1320,31 @@ void CAutoBase::BeginTransit()
 {
     if ( m_param == PARAM_TRANSIT2 )
     {
-        m_bgBack = "back01.png";  // clouds orange / blue
+        m_bgBack = "textures/back01.png";  // clouds orange / blue
     }
     else if ( m_param == PARAM_TRANSIT3 )
     {
-        m_bgBack = "back22.png";  // blueberries clouds
+        m_bgBack = "textures/back22.png";  // blueberries clouds
     }
     else
     {
-        m_bgBack = "back46.png";  // paintings
+        m_bgBack = "textures/back46.png";  // paintings
     }
 
     m_engine->SetFogStart(0.9f);  // hardly any fog
     m_engine->SetDeepView(2000.0f);  // we see very far
     m_engine->ApplyChange();
 
-    bool full;
-    m_engine->GetBackground(m_bgName, m_bgUp, m_bgDown, m_bgCloudUp, m_bgCloudDown, full);
-    m_engine->DeleteTexture(m_bgName);
+    bool full, scale;
+    m_engine->GetBackground(m_bgName, m_bgUp, m_bgDown, m_bgCloudUp, m_bgCloudDown, full, scale);
 
     m_engine->SetBackground(m_bgBack, Gfx::Color(0.0f, 0.0f, 0.0f, 0.0f),
             Gfx::Color(0.0f, 0.0f, 0.0f, 0.0f),
             Gfx::Color(0.0f, 0.0f, 0.0f, 0.0f),
             Gfx::Color(0.0f, 0.0f, 0.0f, 0.0f));
-    m_engine->LoadTexture("textures/"+m_bgBack);
 
     m_cloud->SetEnabled(false);  // cache clouds
-    m_planet->SetMode(1);
+    m_planet->SetVisiblePlanetType(Gfx::PlanetType::OuterSpace);
 }
 
 // End of a transit.
@@ -1382,35 +1355,29 @@ void CAutoBase::EndTransit()
     m_engine->SetDeepView(m_deepView);  // gives initial depth
     m_engine->ApplyChange();
 
-    m_engine->DeleteTexture(m_bgBack);
-
     m_engine->SetBackground(m_bgName, m_bgUp, m_bgDown, m_bgCloudUp, m_bgCloudDown);
-    m_engine->LoadTexture("textures/"+m_bgName);
 
     m_cloud->SetEnabled(true);  // gives the clouds
-    m_planet->SetMode(0);
+    m_planet->SetVisiblePlanetType(Gfx::PlanetType::Sky);
 
     m_main->StartMusic();
 }
 
 Error CAutoBase::TakeOff(bool printMsg)
 {
-
-    Event           newEvent;
-    Math::Vector    pos;
-    Error           err;
-	
-    err = CheckCloseDoor();
-    if ( err != ERR_OK )
+    Error err = CheckCloseDoor();
+    if (err != ERR_OK)
     {
-        if(printMsg) m_main->DisplayError(err, m_object);
+        if (printMsg)
+            m_main->DisplayError(err, m_object);
         return err;
     }
 
     err = m_main->CheckEndMission(false);
-    if ( err != ERR_OK )
+    if (err != ERR_OK)
     {
-        if(printMsg) m_main->DisplayError(err, m_object);
+        if (printMsg)
+            m_main->DisplayError(err, m_object);
         return err;
     }
 
@@ -1418,19 +1385,18 @@ Error CAutoBase::TakeOff(bool printMsg)
     m_main->SetMovieLock(true);  // blocks everything until the end
     m_main->DeselectAll();
 
-    newEvent.type = EVENT_UPDINTERFACE;
-    m_eventQueue->AddEvent(newEvent);
+    m_eventQueue->AddEvent(Event(EVENT_UPDINTERFACE));
 
     m_camera->SetType(Gfx::CAM_TYPE_SCRIPT);
 
-    pos = m_pos;
+    Math::Vector pos = m_pos;
     pos.x -= 110.0f;
     m_terrain->AdjustToFloor(pos);
     pos.y += 10.0f;
     m_camera->SetScriptEye(pos);
     m_posSound = pos;
 
-    pos = m_object->GetPosition(0);
+    pos = m_object->GetPosition();
     pos.y += 50.0f;
     m_camera->SetScriptLookat(pos);
 
@@ -1445,4 +1411,3 @@ Error CAutoBase::TakeOff(bool printMsg)
     m_speed    = 1.0f/BASE_DOOR_TIME2;
     return ERR_OK;
 }
-
