@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,32 +20,41 @@
 
 #include "object/auto/autofactory.h"
 
-#include "common/global.h"
+#include "common/make_unique.h"
 
 #include "math/geometry.h"
 
-#include "object/brain.h"
-#include "object/objman.h"
-#include "object/robotmain.h"
-#include "object/level/parserline.h"
-#include "object/level/parserparam.h"
+#include "level/robotmain.h"
+
+#include "level/parser/parser.h"
+#include "level/parser/parserline.h"
+#include "level/parser/parserparam.h"
+
+#include "object/object_create_params.h"
+#include "object/object_manager.h"
+#include "object/old_object.h"
+
+#include "object/interface/programmable_object.h"
+#include "object/interface/program_storage_object.h"
+#include "object/interface/transportable_object.h"
 
 #include "physics/physics.h"
 
-#include "script/cmdtoken.h"
 #include "script/script.h"
 
-#include "ui/interface.h"
-#include "ui/window.h"
+#include "sound/sound.h"
 
-#include <stdio.h>
-#include <string.h>
+#include "ui/controls/interface.h"
+#include "ui/controls/window.h"
+
+
+#include <boost/regex.hpp>
 
 
 
 // Object's constructor.
 
-CAutoFactory::CAutoFactory(CObject* object) : CAuto(object)
+CAutoFactory::CAutoFactory(COldObject* object) : CAuto(object)
 {
     Init();
     m_type  = OBJECT_MOBILEws;
@@ -62,25 +71,20 @@ CAutoFactory::~CAutoFactory()
 
 // Destroys the object.
 
-void CAutoFactory::DeleteObject(bool bAll)
+void CAutoFactory::DeleteObject(bool all)
 {
-    CObject*    fret;
-    CObject*    vehicle;
-
-    if ( !bAll )
+    if ( !all )
     {
-        fret = SearchFret();  // transform metal?
-        if ( fret != 0 )
+        CObject* cargo = SearchCargo();  // transform metal?
+        if ( cargo != nullptr )
         {
-            fret->DeleteObject();  // destroys the metal
-            delete fret;
+            CObjectManager::GetInstancePointer()->DeleteObject(cargo);
         }
 
-        vehicle = SearchVehicle();
-        if ( vehicle != 0 )
+        CObject* vehicle = SearchVehicle();
+        if ( vehicle != nullptr )
         {
-            vehicle->DeleteObject();  // destroys the vehicle
-            delete vehicle;
+            CObjectManager::GetInstancePointer()->DeleteObject(vehicle);
         }
     }
 
@@ -91,7 +95,7 @@ void CAutoFactory::DeleteObject(bool bAll)
         m_channelSound = -1;
     }
 
-    CAuto::DeleteObject(bAll);
+    CAuto::DeleteObject(all);
 }
 
 
@@ -106,9 +110,9 @@ void CAutoFactory::Init()
     m_time = 0.0f;
     m_lastParticle = 0.0f;
 
-    m_fretPos = m_object->GetPosition(0);
+    m_cargoPos = m_object->GetPosition();
 
-    m_program = nullptr;
+    m_program = "";
 
     CAuto::Init();
 }
@@ -118,7 +122,7 @@ void CAutoFactory::Init()
 
 Error CAutoFactory::StartAction(int param)
 {
-    CObject*   fret;
+    CObject*   cargo;
     ObjectType type = static_cast<ObjectType>(param);
 
     if ( type != OBJECT_NULL )
@@ -130,8 +134,8 @@ Error CAutoFactory::StartAction(int param)
 
         m_type = type;
 
-        fret = SearchFret();  // transform metal?
-        if ( fret == 0 )
+        cargo = SearchCargo();  // transform metal?
+        if ( cargo == nullptr )
         {
             return ERR_FACTORY_NULL;
         }
@@ -140,11 +144,12 @@ Error CAutoFactory::StartAction(int param)
             return ERR_FACTORY_NEAR;
         }
 
+        m_program = "";
         SetBusy(true);
         InitProgressTotal(3.0f+2.0f+15.0f+2.0f+3.0f);
         UpdateInterface();
 
-        fret->SetLock(true);  // usable metal
+        cargo->SetLock(true);  // usable metal
         SoundManip(3.0f, 1.0f, 0.5f);
 
         m_phase    = AFP_CLOSE_S;
@@ -152,25 +157,50 @@ Error CAutoFactory::StartAction(int param)
         m_speed    = 1.0f/3.0f;
         return ERR_OK;
     }
-    return ERR_GENERIC;
+    return ERR_UNKNOWN;
 }
 
 
 // Sets program for created robot
 
-void CAutoFactory::SetProgram(const char* program)
+void CAutoFactory::SetProgram(const std::string& program)
 {
-    m_program = new char[strlen(program)+1];
-    strcpy(m_program, program);
+    m_program = program;
 }
 
+ObjectType ObjectTypeFromFactoryButton(EventType eventType)
+{
+    if ( eventType == EVENT_OBJECT_FACTORYwa )  return OBJECT_MOBILEwa;
+    if ( eventType == EVENT_OBJECT_FACTORYta )  return OBJECT_MOBILEta;
+    if ( eventType == EVENT_OBJECT_FACTORYfa )  return OBJECT_MOBILEfa;
+    if ( eventType == EVENT_OBJECT_FACTORYia )  return OBJECT_MOBILEia;
+    if ( eventType == EVENT_OBJECT_FACTORYws )  return OBJECT_MOBILEws;
+    if ( eventType == EVENT_OBJECT_FACTORYts )  return OBJECT_MOBILEts;
+    if ( eventType == EVENT_OBJECT_FACTORYfs )  return OBJECT_MOBILEfs;
+    if ( eventType == EVENT_OBJECT_FACTORYis )  return OBJECT_MOBILEis;
+    if ( eventType == EVENT_OBJECT_FACTORYwc )  return OBJECT_MOBILEwc;
+    if ( eventType == EVENT_OBJECT_FACTORYtc )  return OBJECT_MOBILEtc;
+    if ( eventType == EVENT_OBJECT_FACTORYfc )  return OBJECT_MOBILEfc;
+    if ( eventType == EVENT_OBJECT_FACTORYic )  return OBJECT_MOBILEic;
+    if ( eventType == EVENT_OBJECT_FACTORYwi )  return OBJECT_MOBILEwi;
+    if ( eventType == EVENT_OBJECT_FACTORYti )  return OBJECT_MOBILEti;
+    if ( eventType == EVENT_OBJECT_FACTORYfi )  return OBJECT_MOBILEfi;
+    if ( eventType == EVENT_OBJECT_FACTORYii )  return OBJECT_MOBILEii;
+    if ( eventType == EVENT_OBJECT_FACTORYrt )  return OBJECT_MOBILErt;
+    if ( eventType == EVENT_OBJECT_FACTORYrc )  return OBJECT_MOBILErc;
+    if ( eventType == EVENT_OBJECT_FACTORYrr )  return OBJECT_MOBILErr;
+    if ( eventType == EVENT_OBJECT_FACTORYrs )  return OBJECT_MOBILErs;
+    if ( eventType == EVENT_OBJECT_FACTORYsa )  return OBJECT_MOBILEsa;
+
+    return OBJECT_NULL;
+}
 
 // Management of an event.
 
 bool CAutoFactory::EventProcess(const Event &event)
 {
     ObjectType  type;
-    CObject*    fret;
+    CObject*    cargo;
     CObject*    vehicle;
     Math::Matrix*   mat;
     CPhysics*   physics;
@@ -190,34 +220,13 @@ bool CAutoFactory::EventProcess(const Event &event)
             CreateInterface(true);
         }
 
-        type = OBJECT_NULL;
-        if ( event.type == EVENT_OBJECT_FACTORYwa )  type = OBJECT_MOBILEwa;
-        if ( event.type == EVENT_OBJECT_FACTORYta )  type = OBJECT_MOBILEta;
-        if ( event.type == EVENT_OBJECT_FACTORYfa )  type = OBJECT_MOBILEfa;
-        if ( event.type == EVENT_OBJECT_FACTORYia )  type = OBJECT_MOBILEia;
-        if ( event.type == EVENT_OBJECT_FACTORYws )  type = OBJECT_MOBILEws;
-        if ( event.type == EVENT_OBJECT_FACTORYts )  type = OBJECT_MOBILEts;
-        if ( event.type == EVENT_OBJECT_FACTORYfs )  type = OBJECT_MOBILEfs;
-        if ( event.type == EVENT_OBJECT_FACTORYis )  type = OBJECT_MOBILEis;
-        if ( event.type == EVENT_OBJECT_FACTORYwc )  type = OBJECT_MOBILEwc;
-        if ( event.type == EVENT_OBJECT_FACTORYtc )  type = OBJECT_MOBILEtc;
-        if ( event.type == EVENT_OBJECT_FACTORYfc )  type = OBJECT_MOBILEfc;
-        if ( event.type == EVENT_OBJECT_FACTORYic )  type = OBJECT_MOBILEic;
-        if ( event.type == EVENT_OBJECT_FACTORYwi )  type = OBJECT_MOBILEwi;
-        if ( event.type == EVENT_OBJECT_FACTORYti )  type = OBJECT_MOBILEti;
-        if ( event.type == EVENT_OBJECT_FACTORYfi )  type = OBJECT_MOBILEfi;
-        if ( event.type == EVENT_OBJECT_FACTORYii )  type = OBJECT_MOBILEii;
-        if ( event.type == EVENT_OBJECT_FACTORYrt )  type = OBJECT_MOBILErt;
-        if ( event.type == EVENT_OBJECT_FACTORYrc )  type = OBJECT_MOBILErc;
-        if ( event.type == EVENT_OBJECT_FACTORYrr )  type = OBJECT_MOBILErr;
-        if ( event.type == EVENT_OBJECT_FACTORYrs )  type = OBJECT_MOBILErs;
-        if ( event.type == EVENT_OBJECT_FACTORYsa )  type = OBJECT_MOBILEsa;
+        type = ObjectTypeFromFactoryButton(event.type);
 
         Error err = StartAction(type);
-        if( err != ERR_OK && err != ERR_GENERIC )
+        if( err != ERR_OK && err != ERR_UNKNOWN )
             m_main->DisplayError(err, m_object);
 
-        if( err != ERR_GENERIC )
+        if( err != ERR_UNKNOWN )
             return false;
     }
 
@@ -245,16 +254,16 @@ bool CAutoFactory::EventProcess(const Event &event)
                 zoom = 0.30f+(m_progress-0.5f+i/16.0f)*2.0f*0.70f;
                 if ( zoom < 0.30f )  zoom = 0.30f;
                 if ( zoom > 1.00f )  zoom = 1.00f;
-                m_object->SetZoomZ( 1+i, zoom);
-                m_object->SetZoomZ(10+i, zoom);
+                m_object->SetPartScaleZ( 1+i, zoom);
+                m_object->SetPartScaleZ(10+i, zoom);
             }
         }
         else
         {
             for ( i=0 ; i<9 ; i++ )
             {
-                m_object->SetZoomZ( 1+i, 1.0f);
-                m_object->SetZoomZ(10+i, 1.0f);
+                m_object->SetPartScaleZ( 1+i, 1.0f);
+                m_object->SetPartScaleZ(10+i, 1.0f);
             }
 
             SoundManip(2.0f, 1.0f, 1.2f);
@@ -272,19 +281,19 @@ bool CAutoFactory::EventProcess(const Event &event)
             for ( i=0 ; i<9 ; i++ )
             {
                 angle = -m_progress*(Math::PI/2.0f)+Math::PI/2.0f;
-                m_object->SetAngleZ( 1+i,  angle);
-                m_object->SetAngleZ(10+i, -angle);
+                m_object->SetPartRotationZ( 1+i,  angle);
+                m_object->SetPartRotationZ(10+i, -angle);
             }
         }
         else
         {
             for ( i=0 ; i<9 ; i++ )
             {
-                m_object->SetAngleZ( 1+i, 0.0f);
-                m_object->SetAngleZ(10+i, 0.0f);
+                m_object->SetPartRotationZ( 1+i, 0.0f);
+                m_object->SetPartRotationZ(10+i, 0.0f);
             }
 
-            m_channelSound = m_sound->Play(SOUND_FACTORY, m_object->GetPosition(0), 0.0f, 1.0f, true);
+            m_channelSound = m_sound->Play(SOUND_FACTORY, m_object->GetPosition(), 0.0f, 1.0f, true);
             m_sound->AddEnvelope(m_channelSound, 1.0f, 1.0f,  2.0f, SOPER_CONTINUE);
             m_sound->AddEnvelope(m_channelSound, 1.0f, 1.0f, 11.0f, SOPER_CONTINUE);
             m_sound->AddEnvelope(m_channelSound, 0.0f, 1.0f,  2.0f, SOPER_STOP);
@@ -301,10 +310,10 @@ bool CAutoFactory::EventProcess(const Event &event)
         {
             if ( !CreateVehicle() )
             {
-                fret = SearchFret();  // transform metal?
-                if ( fret != 0 )
+                cargo = SearchCargo();  // transform metal?
+                if ( cargo != nullptr )
                 {
-                    fret->SetLock(false);  // metal usable again
+                    cargo->SetLock(false);  // metal usable again
                 }
 
                 if ( m_channelSound != -1 )
@@ -335,37 +344,25 @@ bool CAutoFactory::EventProcess(const Event &event)
             {
                 prog = 1.0f-m_progress;
             }
-            angle = powf(prog*10.0f, 2.0f)+m_object->GetAngleY(0);
+            angle = powf(prog*10.0f, 2.0f)+m_object->GetRotationY();
 
             vehicle = SearchVehicle();
-            if ( vehicle != 0 )
+            if ( vehicle != nullptr )
             {
-                vehicle->SetAngleY(0, angle+Math::PI);
-                vehicle->SetZoom(0, m_progress);
+                vehicle->SetRotationY(angle+Math::PI);
+                vehicle->SetScale(m_progress);
             }
 
-            fret = SearchFret();  // transform metal?
-            if ( fret != 0 )
+            cargo = SearchCargo();  // transform metal?
+            if ( cargo != nullptr )
             {
-                fret->SetZoom(0, 1.0f-m_progress);
+                cargo->SetScale(1.0f-m_progress);
             }
 
             if ( m_lastParticle+m_engine->ParticleAdapt(0.05f) <= m_time )
             {
                 m_lastParticle = m_time;
 
-#if 0
-                pos = m_fretPos;
-                pos.x += (Math::Rand()-0.5f)*20.0f;
-                pos.z += (Math::Rand()-0.5f)*20.0f;
-                pos.y += 1.0f;
-                speed.x = (Math::Rand()-0.5f)*12.0f;
-                speed.z = (Math::Rand()-0.5f)*12.0f;
-                speed.y = Math::Rand()*12.0f;
-                dim.x = Math::Rand()*12.0f+10.0f;
-                dim.y = dim.x;
-                m_particle->CreateParticle(pos, speed, dim, PARTIBLUE, 1.0f, 0.0f, 0.0f);
-#else
                 mat = m_object->GetWorldMatrix(0);
                 pos = Math::Vector(-12.0f, 20.0f, -4.0f);  // position of chimney
                 pos = Math::Transform(*mat, pos);
@@ -378,7 +375,6 @@ bool CAutoFactory::EventProcess(const Event &event)
                 dim.x = Math::Rand()*1.5f+1.0f;
                 dim.y = dim.x;
                 m_particle->CreateParticle(pos, speed, dim, Gfx::PARTISMOKE3, 4.0f);
-#endif
             }
         }
         else
@@ -386,26 +382,46 @@ bool CAutoFactory::EventProcess(const Event &event)
             m_main->DisplayError(INFO_FACTORY, m_object);
             SoundManip(2.0f, 1.0f, 1.2f);
 
-            fret = SearchFret();  // transform metal?
-            if ( fret != 0 )
+            cargo = SearchCargo();  // transform metal?
+            if ( cargo != nullptr )
             {
-                fret->DeleteObject();  // removes the metal
-                delete fret;
+                CObjectManager::GetInstancePointer()->DeleteObject(cargo);
             }
 
-            m_vehicle = vehicle = SearchVehicle();
-            if ( vehicle != 0 )
+            vehicle = SearchVehicle();
+            if ( vehicle != nullptr )
             {
-                physics = vehicle->GetPhysics();
-                if ( physics != 0 )
-                {
-                    physics->SetFreeze(false);  // can move
-                }
+                assert(vehicle->Implements(ObjectInterfaceType::Movable));
+                physics = dynamic_cast<CMovableObject*>(vehicle)->GetPhysics();
+                physics->SetFreeze(false);  // can move
 
                 vehicle->SetLock(false);  // vehicle useable
-//?             vehicle->GetPhysics()->GetBrain()->StartTaskAdvance(16.0f);
-                vehicle->SetAngleY(0, m_object->GetAngleY(0)+Math::PI);
-                vehicle->SetZoom(0, 1.0f);
+                vehicle->SetRotationY(m_object->GetRotationY()+Math::PI);
+                vehicle->SetScale(1.0f);
+
+                if ( !m_program.empty() )
+                {
+                    if (vehicle->Implements(ObjectInterfaceType::Programmable) && vehicle->Implements(ObjectInterfaceType::ProgramStorage))
+                    {
+                        Program* program = dynamic_cast<CProgramStorageObject*>(vehicle)->AddProgram();
+
+                        if (boost::regex_search(m_program, boost::regex("^[A-Za-z0-9_]+$"))) // Public function name?
+                        {
+                            std::string code = "extern void object::Start_"+m_program+"()\n{\n\t\n\t//Automatically generated by object.factory()\n\t"+m_program+"();\n\t\n}\n";
+                            program->script->SendScript(code.c_str());
+                        }
+                        else if (boost::regex_search(m_program, boost::regex("\\.txt$"))) // File name (with .txt extension)?
+                        {
+                            program->script->ReadScript(m_program.c_str());
+                        }
+                        else // Program code?
+                        {
+                            program->script->SendScript(m_program.c_str());
+                        }
+
+                        dynamic_cast<CProgrammableObject*>(vehicle)->RunProgram(program);
+                    }
+                }
             }
 
             m_main->CreateShortcuts();
@@ -423,15 +439,15 @@ bool CAutoFactory::EventProcess(const Event &event)
             for ( i=0 ; i<9 ; i++ )
             {
                 angle = -(1.0f-m_progress)*(Math::PI/2.0f)+Math::PI/2.0f;
-                m_object->SetAngleZ( 1+i,  angle);
-                m_object->SetAngleZ(10+i, -angle);
+                m_object->SetPartRotationZ( 1+i,  angle);
+                m_object->SetPartRotationZ(10+i, -angle);
             }
 
             if ( m_lastParticle+m_engine->ParticleAdapt(0.1f) <= m_time )
             {
                 m_lastParticle = m_time;
 
-                pos = m_fretPos;
+                pos = m_cargoPos;
                 pos.x += (Math::Rand()-0.5f)*10.0f;
                 pos.z += (Math::Rand()-0.5f)*10.0f;
                 pos.y += Math::Rand()*10.0f;
@@ -445,8 +461,8 @@ bool CAutoFactory::EventProcess(const Event &event)
         {
             for ( i=0 ; i<9 ; i++ )
             {
-                m_object->SetAngleZ( 1+i,  Math::PI/2.0f);
-                m_object->SetAngleZ(10+i, -Math::PI/2.0f);
+                m_object->SetPartRotationZ( 1+i,  Math::PI/2.0f);
+                m_object->SetPartRotationZ(10+i, -Math::PI/2.0f);
             }
 
             SoundManip(3.0f, 1.0f, 0.5f);
@@ -466,15 +482,15 @@ bool CAutoFactory::EventProcess(const Event &event)
                 zoom = 0.30f+((1.0f-m_progress)-0.5f+i/16.0f)*2.0f*0.70f;
                 if ( zoom < 0.30f )  zoom = 0.30f;
                 if ( zoom > 1.00f )  zoom = 1.00f;
-                m_object->SetZoomZ( 1+i, zoom);
-                m_object->SetZoomZ(10+i, zoom);
+                m_object->SetPartScaleZ( 1+i, zoom);
+                m_object->SetPartScaleZ(10+i, zoom);
             }
 
             if ( m_lastParticle+m_engine->ParticleAdapt(0.1f) <= m_time )
             {
                 m_lastParticle = m_time;
 
-                pos = m_fretPos;
+                pos = m_cargoPos;
                 pos.x += (Math::Rand()-0.5f)*10.0f;
                 pos.z += (Math::Rand()-0.5f)*10.0f;
                 pos.y += Math::Rand()*10.0f;
@@ -488,19 +504,8 @@ bool CAutoFactory::EventProcess(const Event &event)
         {
             for ( i=0 ; i<9 ; i++ )
             {
-                m_object->SetZoomZ( 1+i, 0.30f);
-                m_object->SetZoomZ(10+i, 0.30f);
-            }
-
-            if ( m_program != nullptr )
-            {
-                CBrain* brain = m_vehicle->GetBrain();
-                if ( brain != nullptr )
-                {
-                    Program* program = brain->AddProgram();
-                    program->script->SendScript(const_cast<const char*>(m_program));
-                    brain->RunProgram(program);
-                }
+                m_object->SetPartScaleZ( 1+i, 0.30f);
+                m_object->SetPartScaleZ(10+i, 0.30f);
             }
 
             SetBusy(false);
@@ -521,12 +526,12 @@ bool CAutoFactory::EventProcess(const Event &event)
 bool CAutoFactory::Write(CLevelParserLine* line)
 {
     if ( m_phase == AFP_WAIT )  return false;
-    
-    line->AddParam("aExist", new CLevelParserParam(true));
+
+    line->AddParam("aExist", MakeUnique<CLevelParserParam>(true));
     CAuto::Write(line);
-    line->AddParam("aPhase", new CLevelParserParam(static_cast<int>(m_phase)));
-    line->AddParam("aProgress", new CLevelParserParam(m_progress));
-    line->AddParam("aSpeed", new CLevelParserParam(m_speed));
+    line->AddParam("aPhase", MakeUnique<CLevelParserParam>(static_cast<int>(m_phase)));
+    line->AddParam("aProgress", MakeUnique<CLevelParserParam>(m_progress));
+    line->AddParam("aSpeed", MakeUnique<CLevelParserParam>(m_speed));
 
     return true;
 }
@@ -543,7 +548,7 @@ bool CAutoFactory::Read(CLevelParserLine* line)
     m_speed = line->GetParam("aSpeed")->AsFloat(1.0f);
 
     m_lastParticle = 0.0f;
-    m_fretPos = m_object->GetPosition(0);
+    m_cargoPos = m_object->GetPosition();
 
     return true;
 }
@@ -551,46 +556,32 @@ bool CAutoFactory::Read(CLevelParserLine* line)
 
 //Seeks the cargo.
 
-CObject* CAutoFactory::SearchFret()
+CObject* CAutoFactory::SearchCargo()
 {
-    CObject*    pObj;
-    Math::Vector    oPos;
-    ObjectType  type;
-    float       dist;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-        
-        type = pObj->GetType();
+        ObjectType type = obj->GetType();
         if ( type != OBJECT_METAL )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;
-        
-        oPos = pObj->GetPosition(0);
-        dist = Math::Distance(oPos, m_fretPos);
-        
-        if ( dist < 8.0f )  return pObj;
+        if (IsObjectBeingTransported(obj))  continue;
+
+        Math::Vector oPos = obj->GetPosition();
+        float dist = Math::Distance(oPos, m_cargoPos);
+
+        if ( dist < 8.0f )  return obj;
     }
-    
-    return 0;
+
+    return nullptr;
 }
 
 // Search if a vehicle is too close.
 
 bool CAutoFactory::NearestVehicle()
 {
-    CObject*    pObj;
-    Math::Vector    cPos, oPos;
-    ObjectType  type;
-    float       oRadius, dist;
+    Math::Vector cPos = m_object->GetPosition();
 
-    cPos = m_object->GetPosition(0);
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-
-        type = pObj->GetType();
+        ObjectType type = obj->GetType();
         if ( type != OBJECT_HUMAN    &&
              type != OBJECT_MOBILEfa &&
              type != OBJECT_MOBILEta &&
@@ -625,10 +616,11 @@ bool CAutoFactory::NearestVehicle()
              type != OBJECT_BEE      &&
              type != OBJECT_WORM     )  continue;
 
-        if ( !pObj->GetCrashSphere(0, oPos, oRadius) )  continue;
-        dist = Math::Distance(oPos, cPos)-oRadius;
+        if (obj->GetCrashSphereCount() == 0) continue;
 
-        if ( dist < 10.0f )  return true;
+        auto crashSphere = obj->GetFirstCrashSphere();
+        if (Math::DistanceToSphere(cPos, crashSphere.sphere) < 10.0f)
+            return true;
     }
 
     return false;
@@ -639,16 +631,9 @@ bool CAutoFactory::NearestVehicle()
 
 bool CAutoFactory::CreateVehicle()
 {
-    CObject*    vehicle;
-    Math::Matrix*   mat;
-    CPhysics*   physics;
-    Math::Vector    pos;
-    float       angle;
-    char*       name;
+    float angle = m_object->GetRotationY();
 
-    angle = m_object->GetAngleY(0);
-
-    mat = m_object->GetWorldMatrix(0);
+    Math::Vector pos;
     if ( m_type == OBJECT_MOBILErt ||
          m_type == OBJECT_MOBILErc ||
          m_type == OBJECT_MOBILErr ||
@@ -660,35 +645,33 @@ bool CAutoFactory::CreateVehicle()
     {
         pos = Math::Vector(4.0f, 0.0f, 0.0f);
     }
+    Math::Matrix* mat = m_object->GetWorldMatrix(0);
     pos = Transform(*mat, pos);
 
-    vehicle = new CObject();
-    if ( !vehicle->CreateVehicle(pos, angle, m_type, -1.0f, false, false) )
-    {
-        delete vehicle;
-        m_main->DisplayError(ERR_TOOMANY, m_object);
-        return false;
-    }
-    vehicle->UpdateMapping();
+    ObjectCreateParams params;
+    params.pos = pos;
+    params.angle = angle;
+    params.type = m_type;
+    params.team = m_object->GetTeam();
+    CObject* vehicle = CObjectManager::GetInstancePointer()->CreateObject(params);
+
     vehicle->SetLock(true);  // not usable
-    vehicle->SetRange(30.0f);
 
-    physics = vehicle->GetPhysics();
-    if ( physics != 0 )
-    {
-        physics->SetFreeze(true);  // it doesn't move
-    }
+    assert(vehicle->Implements(ObjectInterfaceType::Movable));
+    CPhysics* physics = dynamic_cast<CMovableObject*>(vehicle)->GetPhysics();
+    physics->SetFreeze(true);  // it doesn't move
 
-    CBrain* brain = vehicle->GetBrain();
-    if(brain != nullptr)
+    if (vehicle->Implements(ObjectInterfaceType::ProgramStorage))
     {
+        CProgramStorageObject* programStorage = dynamic_cast<CProgramStorageObject*>(vehicle);
         for ( int i=0 ; ; i++ )
         {
-            name = m_main->GetNewScriptName(m_type, i);
-            if ( name == nullptr )  break;
-            Program* prog = brain->GetOrAddProgram(i);
-            vehicle->ReadProgram(prog, name);
+            std::string name = m_main->GetNewScriptName(m_type, i);
+            if (name.empty()) break;
+            Program* prog = programStorage->GetOrAddProgram(i);
+            programStorage->ReadProgram(prog, InjectLevelPathsForCurrentLevel(name));
             prog->readOnly = true;
+            prog->filename = name;
         }
     }
 
@@ -698,29 +681,22 @@ bool CAutoFactory::CreateVehicle()
 // Seeking the vehicle during manufacture.
 
 CObject* CAutoFactory::SearchVehicle()
-{    
-    CObject*    pObj;
-    Math::Vector    oPos;
-    ObjectType  type;
-    float       dist;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+{
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-        
-        if ( !pObj->GetLock() )  continue;
-        
-        type = pObj->GetType();
+        if ( !obj->GetLock() )  continue;
+
+        ObjectType  type = obj->GetType();
         if ( type != m_type )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;
-        
-        oPos = pObj->GetPosition(0);
-        dist = Math::Distance(oPos, m_fretPos);
-        
-        if ( dist < 8.0f )  return pObj;
+        if (IsObjectBeingTransported(obj))  continue;
+
+        Math::Vector oPos = obj->GetPosition();
+        float dist = Math::Distance(oPos, m_cargoPos);
+
+        if ( dist < 8.0f )  return obj;
     }
-    
-    return 0;
+
+    return nullptr;
 }
 
 // Creates all the interface when the object is selected.
@@ -854,110 +830,8 @@ void CAutoFactory::UpdateInterface()
 
 void CAutoFactory::UpdateButton(Ui::CWindow *pw, EventType event, bool bBusy)
 {
-    bool        bEnable = true;
-
     EnableInterface(pw, event, !bBusy);
-
-    if ( event == EVENT_OBJECT_FACTORYta )
-    {
-        bEnable = g_researchDone&RESEARCH_TANK;
-    }
-    if ( event == EVENT_OBJECT_FACTORYfa )
-    {
-        bEnable = g_researchDone&RESEARCH_FLY;
-    }
-    if ( event == EVENT_OBJECT_FACTORYia )
-    {
-        bEnable = g_researchDone&RESEARCH_iPAW;
-    }
-
-    if ( event == EVENT_OBJECT_FACTORYws )
-    {
-        bEnable = g_researchDone&RESEARCH_SNIFFER;
-    }
-    if ( event == EVENT_OBJECT_FACTORYts )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_SNIFFER) &&
-                    (g_researchDone&RESEARCH_TANK)    );
-    }
-    if ( event == EVENT_OBJECT_FACTORYfs )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_SNIFFER) &&
-                    (g_researchDone&RESEARCH_FLY)     );
-    }
-    if ( event == EVENT_OBJECT_FACTORYis )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_SNIFFER) &&
-                    (g_researchDone&RESEARCH_iPAW)    );
-    }
-
-    if ( event == EVENT_OBJECT_FACTORYwc )
-    {
-        bEnable = g_researchDone&RESEARCH_CANON;
-    }
-    if ( event == EVENT_OBJECT_FACTORYtc )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_CANON) &&
-                    (g_researchDone&RESEARCH_TANK)  );
-    }
-    if ( event == EVENT_OBJECT_FACTORYfc )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_CANON) &&
-                    (g_researchDone&RESEARCH_FLY)   );
-    }
-    if ( event == EVENT_OBJECT_FACTORYic )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_CANON) &&
-                    (g_researchDone&RESEARCH_iPAW)  );
-    }
-
-    if ( event == EVENT_OBJECT_FACTORYwi )
-    {
-        bEnable = g_researchDone&RESEARCH_iGUN;
-    }
-    if ( event == EVENT_OBJECT_FACTORYti )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_iGUN) &&
-                    (g_researchDone&RESEARCH_TANK) );
-    }
-    if ( event == EVENT_OBJECT_FACTORYfi )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_iGUN) &&
-                    (g_researchDone&RESEARCH_FLY)  );
-    }
-    if ( event == EVENT_OBJECT_FACTORYii )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_iGUN) &&
-                    (g_researchDone&RESEARCH_iPAW) );
-    }
-
-    if ( event == EVENT_OBJECT_FACTORYrt )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_THUMP) &&
-                    (g_researchDone&RESEARCH_TANK)  );
-    }
-    if ( event == EVENT_OBJECT_FACTORYrc )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_PHAZER) &&
-                    (g_researchDone&RESEARCH_TANK)   );
-    }
-    if ( event == EVENT_OBJECT_FACTORYrr )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_RECYCLER) &&
-                    (g_researchDone&RESEARCH_TANK)     );
-    }
-    if ( event == EVENT_OBJECT_FACTORYrs )
-    {
-        bEnable = ( (g_researchDone&RESEARCH_SHIELD) &&
-                    (g_researchDone&RESEARCH_TANK)   );
-    }
-
-    if ( event == EVENT_OBJECT_FACTORYsa )
-    {
-        bEnable = g_researchDone&RESEARCH_SUBM;
-    }
-
-    DeadInterface(pw, event, bEnable);
+    DeadInterface(pw, event, m_main->CanFactory(ObjectTypeFromFactoryButton(event), m_object->GetTeam()));
 }
 
 // Plays the sound of the manipulator arm.
@@ -966,9 +840,8 @@ void CAutoFactory::SoundManip(float time, float amplitude, float frequency)
 {
     int     i;
 
-    i = m_sound->Play(SOUND_MANIP, m_object->GetPosition(0), 0.0f, 0.3f*frequency, true);
+    i = m_sound->Play(SOUND_MANIP, m_object->GetPosition(), 0.0f, 0.3f*frequency, true);
     m_sound->AddEnvelope(i, 0.5f*amplitude, 1.0f*frequency, 0.1f, SOPER_CONTINUE);
     m_sound->AddEnvelope(i, 0.5f*amplitude, 1.0f*frequency, time-0.1f, SOPER_CONTINUE);
     m_sound->AddEnvelope(i, 0.0f, 0.3f*frequency, 0.1f, SOPER_STOP);
 }
-
