@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// autoflag.h
-
 #pragma once
 
 
@@ -29,21 +27,18 @@
 class CAutoFlag : public CAuto
 {
 public:
-    CAutoFlag(CObject* object);
+    CAutoFlag(COldObject* object);
     ~CAutoFlag();
 
-    void        DeleteObject(bool bAll=false);
+    void        DeleteObject(bool bAll=false) override;
 
-    void        Init();
-    void        Start(int param);
-    bool        EventProcess(const Event &event);
-    Error       GetError();
+    void        Init() override;
+    void        Start(int param) override;
+    bool        EventProcess(const Event &event) override;
+    Error       GetError() override;
 
 protected:
-
-protected:
-    float       m_strong;
-    int         m_param;
-    float       m_progress;
+    float       m_strong = 0.0f;
+    int         m_param = 0;
+    float       m_progress = 0.0f;
 };
-
