@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// autojostle.h
-
 #pragma once
 
 
@@ -29,25 +27,24 @@
 class CAutoJostle : public CAuto
 {
 public:
-    CAutoJostle(CObject* object);
+    CAutoJostle(COldObject* object);
     ~CAutoJostle();
 
-    void        DeleteObject(bool bAll=false);
+    void        DeleteObject(bool bAll=false) override;
 
-    void        Init();
+    void        Init() override;
     void        Start(int param, float force);
-    bool        EventProcess(const Event &event);
-    Error       IsEnded();
+    bool        EventProcess(const Event &event) override;
+    Error       IsEnded() override;
 
 private:
     // Overriden to avoid warning about hiding virtual function
-    virtual void Start(int param) OVERRIDE;
+    void Start(int param) override;
 
 protected:
-    float           m_force;
-    float           m_progress;
-    float           m_speed;
-    float           m_lastParticle;
-    Error           m_error;
+    float           m_force = 0.0f;
+    float           m_progress = 0.0f;
+    float           m_speed = 0.0f;
+    float           m_lastParticle = 0.0f;
+    Error           m_error = ERR_OK;
 };
-
