@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// autoroot.h
-
 #pragma once
 
 
@@ -29,19 +27,18 @@
 class CAutoRoot : public CAuto
 {
 public:
-    CAutoRoot(CObject* object);
+    CAutoRoot(COldObject* object);
     ~CAutoRoot();
 
-    void        DeleteObject(bool bAll=false);
+    void        DeleteObject(bool bAll=false) override;
 
-    void        Init();
-    bool        EventProcess(const Event &event);
-    Error       GetError();
+    void        Init() override;
+    bool        EventProcess(const Event &event) override;
+    Error       GetError() override;
 
 protected:
 
 protected:
-    float           m_lastParticle;
+    float           m_lastParticle = 0.0f;
     Math::Vector        m_center;
 };
-
