@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,33 +22,33 @@
 
 #include "app/app.h"
 
-#include "graphics/engine/modelmanager.h"
+#include "graphics/engine/oldmodelmanager.h"
 #include "graphics/engine/particle.h"
 
+#include "object/old_object.h"
+
+#include "object/subclass/base_alien.h"
+
 #include "physics/physics.h"
 
 
 #include <stdio.h>
 
 
-#define ADJUST_ANGLE 0                  // 1 -> adjusts the angles of the members
 const float START_TIME = 1000.0f;       // beginning of the relative time
 
 
 
 // Object's constructor.
 
-CMotionAnt::CMotionAnt(CObject* object) : CMotion(object)
+CMotionAnt::CMotionAnt(COldObject* object)
+    : CMotion(object),
+      m_armAngles()
 {
     m_armMember      = START_TIME;
     m_armTimeAbs     = START_TIME;
     m_armTimeMarch   = START_TIME;
     m_armTimeAction  = START_TIME;
-    m_armTimeIndex   = 0;
-    m_armPartIndex   = 0;
-    m_armMemberIndex = 0;
-    m_armLastAction  = -1;
-    m_bArmStop = false;
     m_lastParticle = 0.0f;
 }
 
@@ -68,10 +68,9 @@ void CMotionAnt::DeleteObject(bool bAll)
 
 // Creates a vehicle poses some rolling on the floor.
 
-bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
-                        float power)
+void CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
+                        float power, Gfx::COldModelManager* modelManager)
 {
-    Gfx::CModelManager* modelManager = Gfx::CModelManager::GetInstancePointer();
     int rank;
 
     m_object->SetType(type);
@@ -81,13 +80,13 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_VEHICLE);  // this is a moving object
     m_object->SetObjectRank(0, rank);
     modelManager->AddModelReference("ant1.mod", false, rank);
-    m_object->SetPosition(0, pos);
-    m_object->SetAngleY(0, angle);
+    m_object->SetPosition(pos);
+    m_object->SetRotationY(angle);
 
     // A vehicle must have necessarily a collision
     //with a sphere of center (0, y, 0) (see GetCrashSphere).
-    m_object->CreateCrashSphere(Math::Vector(0.0f, -2.0f, 0.0f), 4.0f, SOUND_BOUM, 0.20f);
-    m_object->SetGlobalSphere(Math::Vector(-0.5f, 1.0f, 0.0f), 4.0f);
+    m_object->AddCrashSphere(CrashSphere(Math::Vector(0.0f, -2.0f, 0.0f), 4.0f, SOUND_BOUM, 0.20f));
+    m_object->SetCameraCollisionSphere(Math::Sphere(Math::Vector(-0.5f, 1.0f, 0.0f), 4.0f));
 
     // Creates the head.
     rank = m_engine->CreateObject();
@@ -95,7 +94,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(1, rank);
     m_object->SetObjectParent(1, 0);
     modelManager->AddModelReference("ant2.mod", false, rank);
-    m_object->SetPosition(1, Math::Vector(2.0f, 0.0f, 0.0f));
+    m_object->SetPartPosition(1, Math::Vector(2.0f, 0.0f, 0.0f));
 
     // Creates the tail.
     rank = m_engine->CreateObject();
@@ -103,7 +102,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(2, rank);
     m_object->SetObjectParent(2, 0);
     modelManager->AddModelReference("ant3.mod", false, rank);
-    m_object->SetPosition(2, Math::Vector(-1.0f, 0.0f, 0.0f));
+    m_object->SetPartPosition(2, Math::Vector(-1.0f, 0.0f, 0.0f));
 
     // Creates a right-back thigh.
     rank = m_engine->CreateObject();
@@ -111,7 +110,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(3, rank);
     m_object->SetObjectParent(3, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(3, Math::Vector(-0.4f, -0.1f, -0.3f));
+    m_object->SetPartPosition(3, Math::Vector(-0.4f, -0.1f, -0.3f));
 
     // Creates a right-back leg.
     rank = m_engine->CreateObject();
@@ -119,7 +118,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(4, rank);
     m_object->SetObjectParent(4, 3);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(4, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(4, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates a right-back foot.
     rank = m_engine->CreateObject();
@@ -127,7 +126,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(5, rank);
     m_object->SetObjectParent(5, 4);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(5, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(5, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates two middle-right thighs.
     rank = m_engine->CreateObject();
@@ -135,7 +134,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(6, rank);
     m_object->SetObjectParent(6, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(6, Math::Vector(0.1f, -0.1f, -0.4f));
+    m_object->SetPartPosition(6, Math::Vector(0.1f, -0.1f, -0.4f));
 
     // Creates two middle-right legs.
     rank = m_engine->CreateObject();
@@ -143,7 +142,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(7, rank);
     m_object->SetObjectParent(7, 6);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(7, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(7, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates two middle-right foots.
     rank = m_engine->CreateObject();
@@ -151,7 +150,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(8, rank);
     m_object->SetObjectParent(8, 7);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(8, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(8, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates the right front thigh.
     rank = m_engine->CreateObject();
@@ -159,7 +158,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(9, rank);
     m_object->SetObjectParent(9, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(9, Math::Vector(1.4f, -0.1f, -0.6f));
+    m_object->SetPartPosition(9, Math::Vector(1.4f, -0.1f, -0.6f));
 
     // Creates the right front leg.
     rank = m_engine->CreateObject();
@@ -167,7 +166,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(10, rank);
     m_object->SetObjectParent(10, 9);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(10, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(10, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates the right front foot.
     rank = m_engine->CreateObject();
@@ -175,7 +174,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(11, rank);
     m_object->SetObjectParent(11, 10);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(11, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(11, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates a left-back thigh.
     rank = m_engine->CreateObject();
@@ -183,7 +182,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(12, rank);
     m_object->SetObjectParent(12, 0);
     modelManager->AddModelReference("ant4.mod", true, rank);
-    m_object->SetPosition(12, Math::Vector(-0.4f, -0.1f, 0.3f));
+    m_object->SetPartPosition(12, Math::Vector(-0.4f, -0.1f, 0.3f));
 
     // Creates a left-back leg.
     rank = m_engine->CreateObject();
@@ -191,7 +190,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(13, rank);
     m_object->SetObjectParent(13, 12);
     modelManager->AddModelReference("ant5.mod", true, rank);
-    m_object->SetPosition(13, Math::Vector(0.0f, 0.0f, 1.0f));
+    m_object->SetPartPosition(13, Math::Vector(0.0f, 0.0f, 1.0f));
 
     // Creates a left-back foot.
     rank = m_engine->CreateObject();
@@ -199,7 +198,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(14, rank);
     m_object->SetObjectParent(14, 13);
     modelManager->AddModelReference("ant6.mod", true, rank);
-    m_object->SetPosition(14, Math::Vector(0.0f, 0.0f, 2.0f));
+    m_object->SetPartPosition(14, Math::Vector(0.0f, 0.0f, 2.0f));
 
     // Creates two middle-left thighs.
     rank = m_engine->CreateObject();
@@ -207,7 +206,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(15, rank);
     m_object->SetObjectParent(15, 0);
     modelManager->AddModelReference("ant4.mod", true, rank);
-    m_object->SetPosition(15, Math::Vector(0.1f, -0.1f, 0.4f));
+    m_object->SetPartPosition(15, Math::Vector(0.1f, -0.1f, 0.4f));
 
     // Creates two middle-left legs.
     rank = m_engine->CreateObject();
@@ -215,7 +214,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(16, rank);
     m_object->SetObjectParent(16, 15);
     modelManager->AddModelReference("ant5.mod", true, rank);
-    m_object->SetPosition(16, Math::Vector(0.0f, 0.0f, 1.0f));
+    m_object->SetPartPosition(16, Math::Vector(0.0f, 0.0f, 1.0f));
 
     // Creates two middle-left foot.
     rank = m_engine->CreateObject();
@@ -223,7 +222,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(17, rank);
     m_object->SetObjectParent(17, 16);
     modelManager->AddModelReference("ant6.mod", true, rank);
-    m_object->SetPosition(17, Math::Vector(0.0f, 0.0f, 2.0f));
+    m_object->SetPartPosition(17, Math::Vector(0.0f, 0.0f, 2.0f));
 
     // Creates the left front thigh.
     rank = m_engine->CreateObject();
@@ -231,7 +230,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(18, rank);
     m_object->SetObjectParent(18, 0);
     modelManager->AddModelReference("ant4.mod", true, rank);
-    m_object->SetPosition(18, Math::Vector(1.4f, -0.1f, 0.6f));
+    m_object->SetPartPosition(18, Math::Vector(1.4f, -0.1f, 0.6f));
 
     // Creates the left front leg.
     rank = m_engine->CreateObject();
@@ -239,7 +238,7 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(19, rank);
     m_object->SetObjectParent(19, 18);
     modelManager->AddModelReference("ant5.mod", true, rank);
-    m_object->SetPosition(19, Math::Vector(0.0f, 0.0f, 1.0f));
+    m_object->SetPartPosition(19, Math::Vector(0.0f, 0.0f, 1.0f));
 
     // Creates the left front foot.
     rank = m_engine->CreateObject();
@@ -247,19 +246,17 @@ bool CMotionAnt::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(20, rank);
     m_object->SetObjectParent(20, 19);
     modelManager->AddModelReference("ant6.mod", true, rank);
-    m_object->SetPosition(20, Math::Vector(0.0f, 0.0f, 2.0f));
+    m_object->SetPartPosition(20, Math::Vector(0.0f, 0.0f, 2.0f));
 
     m_object->CreateShadowCircle(4.0f, 0.5f);
 
     CreatePhysics();
     m_object->SetFloorHeight(0.0f);
 
-    pos = m_object->GetPosition(0);
-    m_object->SetPosition(0, pos);  // to display the shadows immediately
+    pos = m_object->GetPosition();
+    m_object->SetPosition(pos);  // to display the shadows immediately
 
     m_engine->LoadAllTextures();
-
-    return true;
 }
 
 // Creates the physics of the object.
@@ -337,8 +334,6 @@ void CMotionAnt::CreatePhysics()
         -35,35,0,   -25,40,0,   -40,65,0,   // s7: feet 1..3
     };
 
-    m_physics->SetType(TYPE_ROLLING);
-
     character = m_object->GetCharacter();
     character->wheelFront = 3.0f;
     character->wheelBack  = 3.0f;
@@ -389,36 +384,6 @@ bool CMotionAnt::EventProcess(const Event &event)
         return EventFrame(event);
     }
 
-    if ( event.type == EVENT_KEY_DOWN )
-    {
-#if ADJUST_ANGLE
-        int     i;
-
-        if ( event.param == 'A' )  m_armTimeIndex++;
-        if ( m_armTimeIndex >= 3 )  m_armTimeIndex = 0;
-
-        if ( event.param == 'Q' )  m_armPartIndex++;
-        if ( m_armPartIndex >= 3 )  m_armPartIndex = 0;
-
-        if ( event.param == 'W' )  m_armMemberIndex++;
-        if ( m_armMemberIndex >= 3 )  m_armMemberIndex = 0;
-
-        i  = m_armMemberIndex*3;
-        i += m_armPartIndex*3*3;
-        i += m_armTimeIndex*3*3*3;
-//?     i += 3*3*3*3;
-
-        if ( event.param == 'E' )  m_armAngles[i+0] += 5;
-        if ( event.param == 'D' )  m_armAngles[i+0] -= 5;
-        if ( event.param == 'R' )  m_armAngles[i+1] += 5;
-        if ( event.param == 'F' )  m_armAngles[i+1] -= 5;
-        if ( event.param == 'T' )  m_armAngles[i+2] += 5;
-        if ( event.param == 'G' )  m_armAngles[i+2] -= 5;
-
-        if ( event.param == 'Y' )  m_bArmStop = !m_bArmStop;
-#endif
-    }
-
     return true;
 }
 
@@ -434,7 +399,7 @@ bool CMotionAnt::EventFrame(const Event &event)
     bool        bStop;
 
     if ( m_engine->GetPause() )  return true;
-    if ( !m_engine->IsVisiblePoint(m_object->GetPosition(0)) )  return true;
+    if ( !m_engine->IsVisiblePoint(m_object->GetPosition()) )  return true;
 
     s =     m_physics->GetLinMotionX(MO_MOTSPEED)*1.5f;
     a = fabs(m_physics->GetCirMotionY(MO_MOTSPEED)*2.0f);
@@ -461,13 +426,10 @@ bool CMotionAnt::EventFrame(const Event &event)
         m_armMember += a;
     }
 
-    if ( m_object->GetRuin() )  // destroyed?
+    assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+    if ( dynamic_cast<CDestroyableObject*>(m_object)->GetDying() == DeathType::Burning )  // burning?
     {
-        m_actionType = MAS_RUIN;
-    }
-    if ( m_object->GetBurn() )  // burning?
-    {
-        if ( m_object->GetFixed() )
+        if ( dynamic_cast<CBaseAlien*>(m_object)->GetFixed() )
         {
             m_actionType = MAS_BURN;
         }
@@ -476,6 +438,10 @@ bool CMotionAnt::EventFrame(const Event &event)
             m_actionType = -1;
         }
     }
+    else if ( dynamic_cast<CDestroyableObject*>(m_object)->IsDying() )  // destroyed?
+    {
+        m_actionType = MAS_RUIN;
+    }
 
     for ( i=0 ; i<6 ; i++ )  // the six legs
     {
@@ -490,10 +456,6 @@ bool CMotionAnt::EventFrame(const Event &event)
         {
             if ( i < 3 )  prog = Math::Mod(m_armMember+(2.0f-(i%3))*0.33f+0.0f, 1.0f);
             else          prog = Math::Mod(m_armMember+(2.0f-(i%3))*0.33f+0.3f, 1.0f);
-            if ( m_bArmStop )
-            {
-                prog = static_cast< float >(m_armTimeIndex/3.0f);
-            }
             if ( prog < 0.33f )  // t0..t1 ?
             {
                 prog = prog/0.33f;  // 0..1
@@ -552,39 +514,30 @@ bool CMotionAnt::EventFrame(const Event &event)
 
         if ( i < 3 )  // right leg (1..3) ?
         {
-            m_object->SetAngleX(3+3*i+0, Math::Smooth(m_object->GetAngleX(3+3*i+0), Math::PropAngle(tSt[0], tNd[0], prog), time));
-            m_object->SetAngleY(3+3*i+0, Math::Smooth(m_object->GetAngleY(3+3*i+0), Math::PropAngle(tSt[1], tNd[1], prog), time));
-            m_object->SetAngleZ(3+3*i+0, Math::Smooth(m_object->GetAngleZ(3+3*i+0), Math::PropAngle(tSt[2], tNd[2], prog), time));
-            m_object->SetAngleX(3+3*i+1, Math::Smooth(m_object->GetAngleX(3+3*i+1), Math::PropAngle(tSt[3], tNd[3], prog), time));
-            m_object->SetAngleY(3+3*i+1, Math::Smooth(m_object->GetAngleY(3+3*i+1), Math::PropAngle(tSt[4], tNd[4], prog), time));
-            m_object->SetAngleZ(3+3*i+1, Math::Smooth(m_object->GetAngleZ(3+3*i+1), Math::PropAngle(tSt[5], tNd[5], prog), time));
-            m_object->SetAngleX(3+3*i+2, Math::Smooth(m_object->GetAngleX(3+3*i+2), Math::PropAngle(tSt[6], tNd[6], prog), time));
-            m_object->SetAngleY(3+3*i+2, Math::Smooth(m_object->GetAngleY(3+3*i+2), Math::PropAngle(tSt[7], tNd[7], prog), time));
-            m_object->SetAngleZ(3+3*i+2, Math::Smooth(m_object->GetAngleZ(3+3*i+2), Math::PropAngle(tSt[8], tNd[8], prog), time));
+            m_object->SetPartRotationX(3+3*i+0, Math::Smooth(m_object->GetPartRotationX(3+3*i+0), Math::PropAngle(tSt[0], tNd[0], prog), time));
+            m_object->SetPartRotationY(3+3*i+0, Math::Smooth(m_object->GetPartRotationY(3+3*i+0), Math::PropAngle(tSt[1], tNd[1], prog), time));
+            m_object->SetPartRotationZ(3+3*i+0, Math::Smooth(m_object->GetPartRotationZ(3+3*i+0), Math::PropAngle(tSt[2], tNd[2], prog), time));
+            m_object->SetPartRotationX(3+3*i+1, Math::Smooth(m_object->GetPartRotationX(3+3*i+1), Math::PropAngle(tSt[3], tNd[3], prog), time));
+            m_object->SetPartRotationY(3+3*i+1, Math::Smooth(m_object->GetPartRotationY(3+3*i+1), Math::PropAngle(tSt[4], tNd[4], prog), time));
+            m_object->SetPartRotationZ(3+3*i+1, Math::Smooth(m_object->GetPartRotationZ(3+3*i+1), Math::PropAngle(tSt[5], tNd[5], prog), time));
+            m_object->SetPartRotationX(3+3*i+2, Math::Smooth(m_object->GetPartRotationX(3+3*i+2), Math::PropAngle(tSt[6], tNd[6], prog), time));
+            m_object->SetPartRotationY(3+3*i+2, Math::Smooth(m_object->GetPartRotationY(3+3*i+2), Math::PropAngle(tSt[7], tNd[7], prog), time));
+            m_object->SetPartRotationZ(3+3*i+2, Math::Smooth(m_object->GetPartRotationZ(3+3*i+2), Math::PropAngle(tSt[8], tNd[8], prog), time));
         }
         else    // left leg (4..6) ?
         {
-            m_object->SetAngleX(3+3*i+0, Math::Smooth(m_object->GetAngleX(3+3*i+0), Math::PropAngle(-tSt[0], -tNd[0], prog), time));
-            m_object->SetAngleY(3+3*i+0, Math::Smooth(m_object->GetAngleY(3+3*i+0), Math::PropAngle(-tSt[1], -tNd[1], prog), time));
-            m_object->SetAngleZ(3+3*i+0, Math::Smooth(m_object->GetAngleZ(3+3*i+0), Math::PropAngle( tSt[2],  tNd[2], prog), time));
-            m_object->SetAngleX(3+3*i+1, Math::Smooth(m_object->GetAngleX(3+3*i+1), Math::PropAngle(-tSt[3], -tNd[3], prog), time));
-            m_object->SetAngleY(3+3*i+1, Math::Smooth(m_object->GetAngleY(3+3*i+1), Math::PropAngle(-tSt[4], -tNd[4], prog), time));
-            m_object->SetAngleZ(3+3*i+1, Math::Smooth(m_object->GetAngleZ(3+3*i+1), Math::PropAngle( tSt[5],  tNd[5], prog), time));
-            m_object->SetAngleX(3+3*i+2, Math::Smooth(m_object->GetAngleX(3+3*i+2), Math::PropAngle(-tSt[6], -tNd[6], prog), time));
-            m_object->SetAngleY(3+3*i+2, Math::Smooth(m_object->GetAngleY(3+3*i+2), Math::PropAngle(-tSt[7], -tNd[7], prog), time));
-            m_object->SetAngleZ(3+3*i+2, Math::Smooth(m_object->GetAngleZ(3+3*i+2), Math::PropAngle( tSt[8],  tNd[8], prog), time));
+            m_object->SetPartRotationX(3+3*i+0, Math::Smooth(m_object->GetPartRotationX(3+3*i+0), Math::PropAngle(-tSt[0], -tNd[0], prog), time));
+            m_object->SetPartRotationY(3+3*i+0, Math::Smooth(m_object->GetPartRotationY(3+3*i+0), Math::PropAngle(-tSt[1], -tNd[1], prog), time));
+            m_object->SetPartRotationZ(3+3*i+0, Math::Smooth(m_object->GetPartRotationZ(3+3*i+0), Math::PropAngle( tSt[2],  tNd[2], prog), time));
+            m_object->SetPartRotationX(3+3*i+1, Math::Smooth(m_object->GetPartRotationX(3+3*i+1), Math::PropAngle(-tSt[3], -tNd[3], prog), time));
+            m_object->SetPartRotationY(3+3*i+1, Math::Smooth(m_object->GetPartRotationY(3+3*i+1), Math::PropAngle(-tSt[4], -tNd[4], prog), time));
+            m_object->SetPartRotationZ(3+3*i+1, Math::Smooth(m_object->GetPartRotationZ(3+3*i+1), Math::PropAngle( tSt[5],  tNd[5], prog), time));
+            m_object->SetPartRotationX(3+3*i+2, Math::Smooth(m_object->GetPartRotationX(3+3*i+2), Math::PropAngle(-tSt[6], -tNd[6], prog), time));
+            m_object->SetPartRotationY(3+3*i+2, Math::Smooth(m_object->GetPartRotationY(3+3*i+2), Math::PropAngle(-tSt[7], -tNd[7], prog), time));
+            m_object->SetPartRotationZ(3+3*i+2, Math::Smooth(m_object->GetPartRotationZ(3+3*i+2), Math::PropAngle( tSt[8],  tNd[8], prog), time));
         }
     }
 
-#if ADJUST_ANGLE
-    if ( m_object->GetSelect() )
-    {
-        char s[100];
-        sprintf(s, "A:time=%d Q:part=%d W:member=%d", m_armTimeIndex, m_armPartIndex, m_armMemberIndex);
-        m_engine->SetInfoText(4, s);
-    }
-#endif
-
     if ( m_actionType == MAS_PREPARE )  // prepares the shooting?
     {
         prog = m_progress;
@@ -592,17 +545,17 @@ bool CMotionAnt::EventFrame(const Event &event)
         dir.x = 0.0f;
         dir.y = 0.0f;
         dir.z = Math::PropAngle(0, -50, prog);
-        SetInclinaison(dir);
-        m_object->SetAngleZ(1, Math::PropAngle(0,  65, prog));  // head
-        m_object->SetAngleZ(2, Math::PropAngle(0, -95, prog));  // tail
+        SetTilt(dir);
+        m_object->SetPartRotationZ(1, Math::PropAngle(0,  65, prog));  // head
+        m_object->SetPartRotationZ(2, Math::PropAngle(0, -95, prog));  // tail
     }
     else if ( m_actionType == MAS_FIRE )  // shooting?
     {
         if ( m_progress < 0.75f )  a = m_progress/0.75f;
         else                       a = (1.0f-m_progress)/0.25f;
-        m_object->SetZoom(2, (a*0.5f)+1.0f);  // tail
-        m_object->SetAngleX(2, (Math::Rand()-0.5f)*0.3f*a);
-        m_object->SetAngleY(2, (Math::Rand()-0.5f)*0.3f*a);
+        m_object->SetPartScale(2, (a*0.5f)+1.0f);  // tail
+        m_object->SetPartRotationX(2, (Math::Rand()-0.5f)*0.3f*a);
+        m_object->SetPartRotationY(2, (Math::Rand()-0.5f)*0.3f*a);
 
         dir.x = (Math::Rand()-0.5f)*0.02f*a;
         dir.y = (Math::Rand()-0.5f)*0.05f*a;
@@ -616,9 +569,9 @@ bool CMotionAnt::EventFrame(const Event &event)
         dir.x = 0.0f;
         dir.y = 0.0f;
         dir.z = Math::PropAngle(0, -50, prog);
-        SetInclinaison(dir);
-        m_object->SetAngleZ(1, Math::PropAngle(0,  65, prog));  // head
-        m_object->SetAngleZ(2, Math::PropAngle(0, -95, prog));  // tail
+        SetTilt(dir);
+        m_object->SetPartRotationZ(1, Math::PropAngle(0,  65, prog));  // head
+        m_object->SetPartRotationZ(2, Math::PropAngle(0, -95, prog));  // tail
     }
     else if ( m_actionType == MAS_BURN )  // burning?
     {
@@ -627,18 +580,18 @@ bool CMotionAnt::EventFrame(const Event &event)
         dir = Math::Vector(0.0f, -1.5f, 0.0f);
         SetLinVibration(dir);
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
         time = event.rTime*1.0f;
-        m_object->SetAngleZ(1, Math::Smooth(m_object->GetAngleZ(1), 0.0f, time));  // head
-        m_object->SetAngleZ(2, Math::Smooth(m_object->GetAngleZ(2), 0.0f, time));  // tail
+        m_object->SetPartRotationZ(1, Math::Smooth(m_object->GetPartRotationZ(1), 0.0f, time));  // head
+        m_object->SetPartRotationZ(2, Math::Smooth(m_object->GetPartRotationZ(2), 0.0f, time));  // tail
     }
     else if ( m_actionType == MAS_RUIN )  // destroyed?
     {
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
         SetLinVibration(dir);
         SetCirVibration(dir);
-        SetInclinaison(dir);
+        SetTilt(dir);
     }
     else if ( m_actionType == MAS_BACK1 )  // starts on the back?
     {
@@ -646,7 +599,7 @@ bool CMotionAnt::EventFrame(const Event &event)
         {
             m_lastParticle = m_armTimeAbs;
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             speed.x = (Math::Rand()-0.5f)*10.0f;
             speed.z = (Math::Rand()-0.5f)*10.0f;
             speed.y = Math::Rand()*5.0f;
@@ -675,7 +628,7 @@ bool CMotionAnt::EventFrame(const Event &event)
         SetCirVibration(dir);
 
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
         if ( m_progress >= 1.0f )
         {
@@ -690,7 +643,7 @@ bool CMotionAnt::EventFrame(const Event &event)
 
             if ( rand()%10 == 0 )
             {
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 pos.x += (Math::Rand()-0.5f)*5.0f;
                 pos.z += (Math::Rand()-0.5f)*5.0f;
                 pos.y -= 1.0f;
@@ -719,12 +672,12 @@ bool CMotionAnt::EventFrame(const Event &event)
                 sinf(m_armTimeAbs*23.0f)*0.03f;
         SetCirVibration(dir);
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
-        m_object->SetAngleY(1, sinf(m_armTimeAbs*8.0f)*0.7f);  // head
-        m_object->SetAngleY(2, cosf(m_armTimeAbs*8.0f)*0.7f);  // tail
-        m_object->SetAngleZ(1, 0.0f);  // head
-        m_object->SetAngleZ(2, 0.0f);  // tail
+        m_object->SetPartRotationY(1, sinf(m_armTimeAbs*8.0f)*0.7f);  // head
+        m_object->SetPartRotationY(2, cosf(m_armTimeAbs*8.0f)*0.7f);  // tail
+        m_object->SetPartRotationZ(1, 0.0f);  // head
+        m_object->SetPartRotationZ(2, 0.0f);  // tail
 
         if ( m_progress >= 1.0f )
         {
@@ -737,7 +690,7 @@ bool CMotionAnt::EventFrame(const Event &event)
         {
             m_lastParticle = m_armTimeAbs;
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             speed.x = (Math::Rand()-0.5f)*10.0f;
             speed.z = (Math::Rand()-0.5f)*10.0f;
             speed.y = Math::Rand()*5.0f;
@@ -766,27 +719,27 @@ bool CMotionAnt::EventFrame(const Event &event)
         SetCirVibration(dir);
 
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
         if ( m_progress >= 1.0f )
         {
             SetAction(-1);
-            m_object->SetFixed(false);  // moving again
+            dynamic_cast<CBaseAlien*>(m_object)->SetFixed(false);  // moving again
         }
     }
     else
     {
-        m_object->SetZoom(2, 1.0f);  // tail
-        m_object->SetAngleX(2, 0.0f);
-        m_object->SetAngleY(2, 0.0f);
+        m_object->SetPartScale(2, 1.0f);  // tail
+        m_object->SetPartRotationX(2, 0.0f);
+        m_object->SetPartRotationY(2, 0.0f);
 
         if ( bStop )
         {
-            m_object->SetAngleZ(2, sinf(m_armTimeAbs*1.7f)*0.15f);  // tail
+            m_object->SetPartRotationZ(2, sinf(m_armTimeAbs*1.7f)*0.15f);  // tail
 
             dir = Math::Vector(0.0f, 0.0f, 0.0f);
             SetLinVibration(dir);
-            SetInclinaison(dir);
+            SetTilt(dir);
         }
         else
         {
@@ -801,9 +754,9 @@ bool CMotionAnt::EventFrame(const Event &event)
             dir.z = sinf(s)*0.1f;
 
             dir.y = 0.0f;
-            SetInclinaison(dir);
+            SetTilt(dir);
 
-            m_object->SetAngleZ(2, -sinf(a)*0.3f);  // tail
+            m_object->SetPartRotationZ(2, -sinf(a)*0.3f);  // tail
 
             a = Math::Mod(m_armMember-0.1f, 1.0f);
             if ( a < 0.33f )
@@ -826,11 +779,10 @@ bool CMotionAnt::EventFrame(const Event &event)
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
         SetCirVibration(dir);
 
-        m_object->SetAngleZ(1, sinf(m_armTimeAbs*1.4f)*0.20f);  // head
-        m_object->SetAngleX(1, sinf(m_armTimeAbs*1.9f)*0.10f);  // head
-        m_object->SetAngleY(1, sinf(m_armTimeAbs*2.1f)*0.50f);  // head
+        m_object->SetPartRotationZ(1, sinf(m_armTimeAbs*1.4f)*0.20f);  // head
+        m_object->SetPartRotationX(1, sinf(m_armTimeAbs*1.9f)*0.10f);  // head
+        m_object->SetPartRotationY(1, sinf(m_armTimeAbs*2.1f)*0.50f);  // head
     }
 
     return true;
 }
-
