@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,9 @@
 
 #include "app/app.h"
 
-#include "graphics/engine/modelmanager.h"
+#include "graphics/engine/oldmodelmanager.h"
+
+#include "object/old_object.h"
 
 #include "physics/physics.h"
 
@@ -30,24 +32,19 @@
 #include <stdio.h>
 
 
-#define ADJUST_ANGLE 0	                // 1 -> adjusts the angles of the members
 const float START_TIME = 1000.0f;       // beginning of the relative time
 
 
 
 // Object's constructor.
 
-CMotionBee::CMotionBee(CObject* object) : CMotion(object)
+CMotionBee::CMotionBee(COldObject* object)
+    : CMotion(object),
+      m_armAngles()
 {
     m_armMember      = START_TIME;
     m_armTimeAbs     = START_TIME;
     m_armTimeMarch   = START_TIME;
-    m_armTimeAction  = START_TIME;
-    m_armTimeIndex   = 0;
-    m_armPartIndex   = 0;
-    m_armMemberIndex = 0;
-    m_armLastAction  = -1;
-    m_bArmStop = false;
 }
 
 // Object's destructor.
@@ -66,10 +63,9 @@ void CMotionBee::DeleteObject(bool bAll)
 
 // Creates a vehicle traveling any lands on the ground.
 
-bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
-                        float power)
+void CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
+                        float power, Gfx::COldModelManager* modelManager)
 {
-    Gfx::CModelManager* modelManager = Gfx::CModelManager::GetInstancePointer();
     int rank;
 
     m_object->SetType(type);
@@ -79,13 +75,13 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_VEHICLE);  // this is a moving object
     m_object->SetObjectRank(0, rank);
     modelManager->AddModelReference("bee1.mod", false, rank);
-    m_object->SetPosition(0, pos);
-    m_object->SetAngleY(0, angle);
+    m_object->SetPosition(pos);
+    m_object->SetRotationY(angle);
 
     // A vehicle must have an obligatory collision
     // with a sphere of center (0, y, 0) (see GetCrashSphere).
-    m_object->CreateCrashSphere(Math::Vector(0.0f, 0.0f, 0.0f), 4.0f, SOUND_BOUM, 0.20f);
-    m_object->SetGlobalSphere(Math::Vector(-1.0f, 1.0f, 0.0f), 5.0f);
+    m_object->AddCrashSphere(CrashSphere(Math::Vector(0.0f, 0.0f, 0.0f), 4.0f, SOUND_BOUM, 0.20f));
+    m_object->SetCameraCollisionSphere(Math::Sphere(Math::Vector(-1.0f, 1.0f, 0.0f), 5.0f));
 
     // Creates the head.
     rank = m_engine->CreateObject();
@@ -93,7 +89,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(1, rank);
     m_object->SetObjectParent(1, 0);
     modelManager->AddModelReference("bee2.mod", false, rank);
-    m_object->SetPosition(1, Math::Vector(1.6f, 0.3f, 0.0f));
+    m_object->SetPartPosition(1, Math::Vector(1.6f, 0.3f, 0.0f));
 
     // Creates the tail.
     rank = m_engine->CreateObject();
@@ -101,7 +97,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(2, rank);
     m_object->SetObjectParent(2, 0);
     modelManager->AddModelReference("bee3.mod", false, rank);
-    m_object->SetPosition(2, Math::Vector(-0.8f, 0.0f, 0.0f));
+    m_object->SetPartPosition(2, Math::Vector(-0.8f, 0.0f, 0.0f));
 
     // Creates a right-back thigh.
     rank = m_engine->CreateObject();
@@ -109,7 +105,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(3, rank);
     m_object->SetObjectParent(3, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(3, Math::Vector(-0.3f, -0.1f, -0.2f));
+    m_object->SetPartPosition(3, Math::Vector(-0.3f, -0.1f, -0.2f));
 
     // Creates a right-back leg.
     rank = m_engine->CreateObject();
@@ -117,7 +113,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(4, rank);
     m_object->SetObjectParent(4, 3);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(4, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(4, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates a right-back foot.
     rank = m_engine->CreateObject();
@@ -125,7 +121,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(5, rank);
     m_object->SetObjectParent(5, 4);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(5, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(5, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates two middle-right thighs.
     rank = m_engine->CreateObject();
@@ -133,7 +129,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(6, rank);
     m_object->SetObjectParent(6, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(6, Math::Vector(0.3f, -0.1f, -0.4f));
+    m_object->SetPartPosition(6, Math::Vector(0.3f, -0.1f, -0.4f));
 
     // Creates two middle-right legs.
     rank = m_engine->CreateObject();
@@ -141,7 +137,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(7, rank);
     m_object->SetObjectParent(7, 6);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(7, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(7, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates two middle-right feet.
     rank = m_engine->CreateObject();
@@ -149,7 +145,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(8, rank);
     m_object->SetObjectParent(8, 7);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(8, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(8, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates the right front thigh.
     rank = m_engine->CreateObject();
@@ -157,7 +153,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(9, rank);
     m_object->SetObjectParent(9, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(9, Math::Vector(1.0f, -0.1f, -0.7f));
+    m_object->SetPartPosition(9, Math::Vector(1.0f, -0.1f, -0.7f));
 
     // Creates the right front leg.
     rank = m_engine->CreateObject();
@@ -165,7 +161,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(10, rank);
     m_object->SetObjectParent(10, 9);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(10, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(10, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates the right front foot.
     rank = m_engine->CreateObject();
@@ -173,7 +169,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(11, rank);
     m_object->SetObjectParent(11, 10);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(11, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(11, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates a left-back thigh.
     rank = m_engine->CreateObject();
@@ -181,8 +177,8 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(12, rank);
     m_object->SetObjectParent(12, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(12, Math::Vector(-0.3f, -0.1f, 0.2f));
-    m_object->SetAngleY(12, Math::PI);
+    m_object->SetPartPosition(12, Math::Vector(-0.3f, -0.1f, 0.2f));
+    m_object->SetPartRotationY(12, Math::PI);
 
     // Creates a left-back leg.
     rank = m_engine->CreateObject();
@@ -190,7 +186,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(13, rank);
     m_object->SetObjectParent(13, 12);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(13, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(13, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates a left-back foot.
     rank = m_engine->CreateObject();
@@ -198,7 +194,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(14, rank);
     m_object->SetObjectParent(14, 13);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(14, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(14, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates two middle-left thigh.
     rank = m_engine->CreateObject();
@@ -206,8 +202,8 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(15, rank);
     m_object->SetObjectParent(15, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(15, Math::Vector(0.3f, -0.1f, 0.4f));
-    m_object->SetAngleY(15, Math::PI);
+    m_object->SetPartPosition(15, Math::Vector(0.3f, -0.1f, 0.4f));
+    m_object->SetPartRotationY(15, Math::PI);
 
     // Creates two middle-left legs.
     rank = m_engine->CreateObject();
@@ -215,7 +211,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(16, rank);
     m_object->SetObjectParent(16, 15);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(16, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(16, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates two middle-left feet.
     rank = m_engine->CreateObject();
@@ -223,7 +219,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(17, rank);
     m_object->SetObjectParent(17, 16);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(17, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(17, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates front-left thigh.
     rank = m_engine->CreateObject();
@@ -231,8 +227,8 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(18, rank);
     m_object->SetObjectParent(18, 0);
     modelManager->AddModelReference("ant4.mod", false, rank);
-    m_object->SetPosition(18, Math::Vector(1.0f, -0.1f, 0.7f));
-    m_object->SetAngleY(18, Math::PI);
+    m_object->SetPartPosition(18, Math::Vector(1.0f, -0.1f, 0.7f));
+    m_object->SetPartRotationY(18, Math::PI);
 
     // Creates front-left leg.
     rank = m_engine->CreateObject();
@@ -240,7 +236,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(19, rank);
     m_object->SetObjectParent(19, 18);
     modelManager->AddModelReference("ant5.mod", false, rank);
-    m_object->SetPosition(19, Math::Vector(0.0f, 0.0f, -1.0f));
+    m_object->SetPartPosition(19, Math::Vector(0.0f, 0.0f, -1.0f));
 
     // Creates front-left foot.
     rank = m_engine->CreateObject();
@@ -248,7 +244,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(20, rank);
     m_object->SetObjectParent(20, 19);
     modelManager->AddModelReference("ant6.mod", false, rank);
-    m_object->SetPosition(20, Math::Vector(0.0f, 0.0f, -2.0f));
+    m_object->SetPartPosition(20, Math::Vector(0.0f, 0.0f, -2.0f));
 
     // Creates the right wing.
     rank = m_engine->CreateObject();
@@ -256,7 +252,7 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(21, rank);
     m_object->SetObjectParent(21, 0);
     modelManager->AddModelReference("bee7.mod", false, rank);
-    m_object->SetPosition(21, Math::Vector(0.8f, 0.4f, -0.5f));
+    m_object->SetPartPosition(21, Math::Vector(0.8f, 0.4f, -0.5f));
 
     // Creates the left wing.
     rank = m_engine->CreateObject();
@@ -264,19 +260,17 @@ bool CMotionBee::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(22, rank);
     m_object->SetObjectParent(22, 0);
     modelManager->AddModelReference("bee7.mod", true, rank);
-    m_object->SetPosition(22, Math::Vector(0.8f, 0.4f, 0.5f));
+    m_object->SetPartPosition(22, Math::Vector(0.8f, 0.4f, 0.5f));
 
     m_object->CreateShadowCircle(6.0f, 0.5f);
 
     CreatePhysics();
     m_object->SetFloorHeight(0.0f);
 
-    pos = m_object->GetPosition(0);
-    m_object->SetPosition(0, pos);  // to display the shadows immediately
+    pos = m_object->GetPosition();
+    m_object->SetPosition(pos);  // to display the shadows immediately
 
     m_engine->LoadAllTextures();
-
-    return true;
 }
 
 // Creates the physical object.
@@ -318,8 +312,6 @@ void CMotionBee::CreatePhysics()
         -110,75,-15,    -130,80,-25,    -125,40,0,  // s2: feet 1..3
     };
 
-    m_physics->SetType(TYPE_FLYING);
-
     character = m_object->GetCharacter();
     character->wheelFront = 3.0f;
     character->wheelBack  = 3.0f;
@@ -371,36 +363,6 @@ bool CMotionBee::EventProcess(const Event &event)
         return EventFrame(event);
     }
 
-    if ( event.type == EVENT_KEY_DOWN )
-    {
-#if ADJUST_ANGLE
-        int     i;
-
-        if ( event.param == 'A' )  m_armTimeIndex++;
-        if ( m_armTimeIndex >= 3 )  m_armTimeIndex = 0;
-
-        if ( event.param == 'Q' )  m_armPartIndex++;
-        if ( m_armPartIndex >= 3 )  m_armPartIndex = 0;
-
-        if ( event.param == 'W' )  m_armMemberIndex++;
-        if ( m_armMemberIndex >= 3 )  m_armMemberIndex = 0;
-
-        i  = m_armMemberIndex*3;
-        i += m_armPartIndex*3*3;
-        i += m_armTimeIndex*3*3*3;
-//?     i += 3*3*3*3;
-
-        if ( event.param == 'E' )  m_armAngles[i+0] += 5;
-        if ( event.param == 'D' )  m_armAngles[i+0] -= 5;
-        if ( event.param == 'R' )  m_armAngles[i+1] += 5;
-        if ( event.param == 'F' )  m_armAngles[i+1] -= 5;
-        if ( event.param == 'T' )  m_armAngles[i+2] += 5;
-        if ( event.param == 'G' )  m_armAngles[i+2] -= 5;
-
-        if ( event.param == 'Y' )  m_bArmStop = !m_bArmStop;
-#endif
-    }
-
     return true;
 }
 
@@ -414,7 +376,7 @@ bool CMotionBee::EventFrame(const Event &event)
     bool            bStop;
 
     if ( m_engine->GetPause() )  return true;
-    if ( !m_engine->IsVisiblePoint(m_object->GetPosition(0)) )  return true;
+    if ( !m_engine->IsVisiblePoint(m_object->GetPosition()) )  return true;
 
     s =     m_physics->GetLinMotionX(MO_MOTSPEED)*0.30f;
     a = fabs(m_physics->GetCirMotionY(MO_MOTSPEED)*2.00f);
@@ -439,16 +401,17 @@ bool CMotionBee::EventFrame(const Event &event)
     action = MB_MARCH;  // flying
 
     m_actionType = -1;
-    if ( m_object->GetFret() != 0 )  m_actionType = MBS_HOLD;  // carries the ball
+    if (IsObjectCarryingCargo(m_object))  m_actionType = MBS_HOLD;  // carries the ball
 
-    if ( m_object->GetRuin() )  // destroyed?
-    {
-        m_actionType = MBS_RUIN;
-    }
-    if ( m_object->GetBurn() )  // burning?
+    assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+    if ( dynamic_cast<CDestroyableObject*>(m_object)->GetDying() == DeathType::Burning )  // burning?
     {
         m_actionType = MBS_BURN;
     }
+    else if ( dynamic_cast<CDestroyableObject*>(m_object)->IsDying() )  // destroyed?
+    {
+        m_actionType = MBS_RUIN;
+    }
 
     for ( i=0 ; i<6 ; i++ )  // the six legs
     {
@@ -461,10 +424,6 @@ bool CMotionBee::EventFrame(const Event &event)
         {
             if ( i < 3 )  prog = Math::Mod(m_armMember+(2.0f-(i%3))*0.33f+0.0f, 1.0f);
             else          prog = Math::Mod(m_armMember+(2.0f-(i%3))*0.33f+0.3f, 1.0f);
-            if ( m_bArmStop )
-            {
-                prog = static_cast< float >(m_armTimeIndex/3.0f);
-            }
             if ( prog < 0.33f )  // t0..t1 ?
             {
                 prog = prog/0.33f;  // 0..1
@@ -489,49 +448,38 @@ bool CMotionBee::EventFrame(const Event &event)
 
         if ( i < 3 )  // right leg (1..3) ?
         {
-            m_object->SetAngleX(3+3*i+0, Math::PropAngle(m_armAngles[st+ 0], m_armAngles[nd+ 0], prog));
-            m_object->SetAngleY(3+3*i+0, Math::PropAngle(m_armAngles[st+ 1], m_armAngles[nd+ 1], prog));
-            m_object->SetAngleZ(3+3*i+0, Math::PropAngle(m_armAngles[st+ 2], m_armAngles[nd+ 2], prog));
-            m_object->SetAngleX(3+3*i+1, Math::PropAngle(m_armAngles[st+ 9], m_armAngles[nd+ 9], prog));
-            m_object->SetAngleY(3+3*i+1, Math::PropAngle(m_armAngles[st+10], m_armAngles[nd+10], prog));
-            m_object->SetAngleZ(3+3*i+1, Math::PropAngle(m_armAngles[st+11], m_armAngles[nd+11], prog));
-            m_object->SetAngleX(3+3*i+2, Math::PropAngle(m_armAngles[st+18], m_armAngles[nd+18], prog));
-            m_object->SetAngleY(3+3*i+2, Math::PropAngle(m_armAngles[st+19], m_armAngles[nd+19], prog));
-            m_object->SetAngleZ(3+3*i+2, Math::PropAngle(m_armAngles[st+20], m_armAngles[nd+20], prog));
+            m_object->SetPartRotationX(3+3*i+0, Math::PropAngle(m_armAngles[st+ 0], m_armAngles[nd+ 0], prog));
+            m_object->SetPartRotationY(3+3*i+0, Math::PropAngle(m_armAngles[st+ 1], m_armAngles[nd+ 1], prog));
+            m_object->SetPartRotationZ(3+3*i+0, Math::PropAngle(m_armAngles[st+ 2], m_armAngles[nd+ 2], prog));
+            m_object->SetPartRotationX(3+3*i+1, Math::PropAngle(m_armAngles[st+ 9], m_armAngles[nd+ 9], prog));
+            m_object->SetPartRotationY(3+3*i+1, Math::PropAngle(m_armAngles[st+10], m_armAngles[nd+10], prog));
+            m_object->SetPartRotationZ(3+3*i+1, Math::PropAngle(m_armAngles[st+11], m_armAngles[nd+11], prog));
+            m_object->SetPartRotationX(3+3*i+2, Math::PropAngle(m_armAngles[st+18], m_armAngles[nd+18], prog));
+            m_object->SetPartRotationY(3+3*i+2, Math::PropAngle(m_armAngles[st+19], m_armAngles[nd+19], prog));
+            m_object->SetPartRotationZ(3+3*i+2, Math::PropAngle(m_armAngles[st+20], m_armAngles[nd+20], prog));
         }
         else    // left leg(4..6) ?
         {
-            m_object->SetAngleX(3+3*i+0, Math::PropAngle(   -m_armAngles[st+ 0],    -m_armAngles[nd+ 0], prog));
-            m_object->SetAngleY(3+3*i+0, Math::PropAngle(180-m_armAngles[st+ 1], 180-m_armAngles[nd+ 1], prog));
-            m_object->SetAngleZ(3+3*i+0, Math::PropAngle(   -m_armAngles[st+ 2],    -m_armAngles[nd+ 2], prog));
-            m_object->SetAngleX(3+3*i+1, Math::PropAngle(    m_armAngles[st+ 9],     m_armAngles[nd+ 9], prog));
-            m_object->SetAngleY(3+3*i+1, Math::PropAngle(   -m_armAngles[st+10],    -m_armAngles[nd+10], prog));
-            m_object->SetAngleZ(3+3*i+1, Math::PropAngle(   -m_armAngles[st+11],    -m_armAngles[nd+11], prog));
-            m_object->SetAngleX(3+3*i+2, Math::PropAngle(    m_armAngles[st+18],     m_armAngles[nd+18], prog));
-            m_object->SetAngleY(3+3*i+2, Math::PropAngle(   -m_armAngles[st+19],    -m_armAngles[nd+19], prog));
-            m_object->SetAngleZ(3+3*i+2, Math::PropAngle(   -m_armAngles[st+20],    -m_armAngles[nd+20], prog));
+            m_object->SetPartRotationX(3+3*i+0, Math::PropAngle(   -m_armAngles[st+ 0],    -m_armAngles[nd+ 0], prog));
+            m_object->SetPartRotationY(3+3*i+0, Math::PropAngle(180-m_armAngles[st+ 1], 180-m_armAngles[nd+ 1], prog));
+            m_object->SetPartRotationZ(3+3*i+0, Math::PropAngle(   -m_armAngles[st+ 2],    -m_armAngles[nd+ 2], prog));
+            m_object->SetPartRotationX(3+3*i+1, Math::PropAngle(    m_armAngles[st+ 9],     m_armAngles[nd+ 9], prog));
+            m_object->SetPartRotationY(3+3*i+1, Math::PropAngle(   -m_armAngles[st+10],    -m_armAngles[nd+10], prog));
+            m_object->SetPartRotationZ(3+3*i+1, Math::PropAngle(   -m_armAngles[st+11],    -m_armAngles[nd+11], prog));
+            m_object->SetPartRotationX(3+3*i+2, Math::PropAngle(    m_armAngles[st+18],     m_armAngles[nd+18], prog));
+            m_object->SetPartRotationY(3+3*i+2, Math::PropAngle(   -m_armAngles[st+19],    -m_armAngles[nd+19], prog));
+            m_object->SetPartRotationZ(3+3*i+2, Math::PropAngle(   -m_armAngles[st+20],    -m_armAngles[nd+20], prog));
         }
     }
 
-#if ADJUST_ANGLE
-    if ( m_object->GetSelect() )
-    {
-        char s[100];
-        sprintf(s, "A:time=%d Q:part=%d W:member=%d", m_armTimeIndex, m_armPartIndex, m_armMemberIndex);
-        m_engine->SetInfoText(4, s);
-    }
-#endif
-
     if ( m_physics->GetLand() )  // on the ground?
     {
-        if ( m_object->GetRuin() )
+        assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+        if ( bStop || dynamic_cast<CDestroyableObject*>(m_object)->GetDying() == DeathType::Burning )
         {
+            m_object->SetPartRotationZ(2, sinf(m_armTimeAbs*1.7f)*0.15f+0.35f);  // tail
         }
-        else if ( bStop || m_object->GetBurn() )
-        {
-            m_object->SetAngleZ(2, sinf(m_armTimeAbs*1.7f)*0.15f+0.35f);  // tail
-        }
-        else
+        if ( !dynamic_cast<CDestroyableObject*>(m_object)->IsDying() )
         {
             a = Math::Mod(m_armTimeMarch, 1.0f);
             if ( a < 0.5f )  a = -1.0f+4.0f*a;  // -1..1
@@ -544,9 +492,9 @@ bool CMotionBee::EventFrame(const Event &event)
             dir.z = sinf(s)*0.1f;
 
             dir.y = 0.0f;
-            m_object->SetInclinaison(dir);
+            m_object->SetTilt(dir);
 
-            m_object->SetAngleZ(2, -sinf(a)*0.3f);  // tail
+            m_object->SetPartRotationZ(2, -sinf(a)*0.3f);  // tail
 
             a = Math::Mod(m_armMember-0.1f, 1.0f);
             if ( a < 0.33f )
@@ -577,39 +525,15 @@ bool CMotionBee::EventFrame(const Event &event)
         prog = 1.00f;
     }
 
-#if 0
-    a = Math::Rand()*Math::PI/2.0f*prog;
-    m_object->SetAngleX(21, a);  // right wing
-    a = -Math::Rand()*Math::PI/4.0f*prog;
-    m_object->SetAngleY(21, a);
+    m_object->SetPartRotationX(21, (sinf(m_armTimeAbs*30.0f)+1.0f)*(Math::PI/4.0f)*prog);
+    m_object->SetPartRotationY(21, -Math::Rand()*Math::PI/6.0f*prog);
 
-    a = -Math::Rand()*Math::PI/2.0f*prog;
-    m_object->SetAngleX(22, a);  // left wing
-    a = Math::Rand()*Math::PI/4.0f*prog;
-    m_object->SetAngleY(22, a);
-#else
-    m_object->SetAngleX(21, (sinf(m_armTimeAbs*30.0f)+1.0f)*(Math::PI/4.0f)*prog);
-    m_object->SetAngleY(21, -Math::Rand()*Math::PI/6.0f*prog);
+    m_object->SetPartRotationX(22, -(sinf(m_armTimeAbs*30.0f)+1.0f)*(Math::PI/4.0f)*prog);
+    m_object->SetPartRotationY(22, Math::Rand()*Math::PI/6.0f*prog);
 
-    m_object->SetAngleX(22, -(sinf(m_armTimeAbs*30.0f)+1.0f)*(Math::PI/4.0f)*prog);
-    m_object->SetAngleY(22, Math::Rand()*Math::PI/6.0f*prog);
-#endif
-
-    m_object->SetAngleZ(1, sinf(m_armTimeAbs*1.4f)*0.20f);  // head
-    m_object->SetAngleX(1, sinf(m_armTimeAbs*1.9f)*0.10f);  // head
-    m_object->SetAngleY(1, sinf(m_armTimeAbs*2.1f)*0.50f);  // head
-
-#if 0
-    h = m_terrain->GetFloorHeight(GetPosition(0));
-    radius = 4.0f+h/4.0f;
-    color.r = 0.3f+h/80.0f;
-    color.g = color.r;
-    color.b = color.r;
-    color.a = color.r;
-    m_engine->SetObjectShadowRadius(m_objectPart[0].object, radius);
-    m_engine->SetObjectShadowColor(m_objectPart[0].object, color);
-#endif
+    m_object->SetPartRotationZ(1, sinf(m_armTimeAbs*1.4f)*0.20f);  // head
+    m_object->SetPartRotationX(1, sinf(m_armTimeAbs*1.9f)*0.10f);  // head
+    m_object->SetPartRotationY(1, sinf(m_armTimeAbs*2.1f)*0.50f);  // head
 
     return true;
 }
-
