@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,33 +22,32 @@
 
 #include "app/app.h"
 
-#include "graphics/engine/modelmanager.h"
+#include "graphics/engine/oldmodelmanager.h"
 #include "graphics/engine/particle.h"
 
+#include "object/old_object.h"
+
+#include "object/subclass/base_alien.h"
+
 #include "physics/physics.h"
 
 #include <stdio.h>
 
 
 
-#define ADJUST_ANGLE 0                  // 1 -> adjusts the angles of the members
 const float START_TIME = 1000.0f;       // beginning of the relative time
 
 
 
 // Object's constructor.
 
-CMotionSpider::CMotionSpider(CObject* object) : CMotion(object)
+CMotionSpider::CMotionSpider(COldObject* object)
+    : CMotion(object),
+      m_armAngles()
 {
     m_armMember      = START_TIME;
     m_armTimeAbs     = START_TIME;
-    m_armTimeMarch   = START_TIME;
     m_armTimeAction  = START_TIME;
-    m_armTimeIndex   = 0;
-    m_armPartIndex   = 0;
-    m_armMemberIndex = 0;
-    m_armLastAction  = -1;
-    m_bArmStop = false;
     m_lastParticle = 0.0f;
 }
 
@@ -68,8 +67,8 @@ void CMotionSpider::DeleteObject(bool bAll)
 
 // Creates a vehicle traveling any lands on the ground.
 
-bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
-                           float power)
+void CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
+                           float power, Gfx::COldModelManager* modelManager)
 {
     int         rank, i, j, parent;
     char        name[50];
@@ -98,8 +97,6 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
          0.0f,   0.0f,  -2.0f,
     };
 
-    Gfx::CModelManager* modelManager = Gfx::CModelManager::GetInstancePointer();
-
     m_object->SetType(type);
 
     // Creates the main base.
@@ -107,13 +104,13 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_VEHICLE);  // this is a moving object
     m_object->SetObjectRank(0, rank);
     // This is an "empty" object, without triangles
-    m_object->SetPosition(0, pos);
-    m_object->SetAngleY(0, angle);
+    m_object->SetPosition(pos);
+    m_object->SetRotationY(angle);
 
     // A vehicle must have a obligatory collision
     // with a sphere of center (0, y, 0) (see GetCrashSphere).
-    m_object->CreateCrashSphere(Math::Vector(0.0f, -2.0f, 0.0f), 4.0f, SOUND_BOUM, 0.20f);
-    m_object->SetGlobalSphere(Math::Vector(-0.5f, 1.0f, 0.0f), 4.0f);
+    m_object->AddCrashSphere(CrashSphere(Math::Vector(0.0f, -2.0f, 0.0f), 4.0f, SOUND_BOUM, 0.20f));
+    m_object->SetCameraCollisionSphere(Math::Sphere(Math::Vector(-0.5f, 1.0f, 0.0f), 4.0f));
 
     // Creates the abdomen.
     rank = m_engine->CreateObject();
@@ -121,7 +118,7 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(1, rank);
     m_object->SetObjectParent(1, 0);
     modelManager->AddModelReference("spider1.mod", false, rank);
-    m_object->SetPosition(1, Math::Vector(1.0f, 0.0f, 0.0f));
+    m_object->SetPartPosition(1, Math::Vector(1.0f, 0.0f, 0.0f));
 
     // Creates the head.
     rank = m_engine->CreateObject();
@@ -129,7 +126,7 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(2, rank);
     m_object->SetObjectParent(2, 0);
     modelManager->AddModelReference("spider2.mod", false, rank);
-    m_object->SetPosition(2, Math::Vector(1.0f, 0.0f, 0.0f));
+    m_object->SetPartPosition(2, Math::Vector(1.0f, 0.0f, 0.0f));
 
     // Creates legs.
     for ( i=0 ; i<4 ; i++ )
@@ -149,7 +146,7 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
             pos.x = table[i*12+j*3+0];
             pos.y = table[i*12+j*3+1];
             pos.z = table[i*12+j*3+2];
-            m_object->SetPosition(3+i*4+j, pos);
+            m_object->SetPartPosition(3+i*4+j, pos);
 
             // Creates the left leg.
             rank = m_engine->CreateObject();
@@ -162,7 +159,7 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
             pos.x =  table[i*12+j*3+0];
             pos.y =  table[i*12+j*3+1];
             pos.z = -table[i*12+j*3+2];
-            m_object->SetPosition(19+i*4+j, pos);
+            m_object->SetPartPosition(19+i*4+j, pos);
         }
     }
 
@@ -172,7 +169,7 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(35, rank);
     m_object->SetObjectParent(35, 1);
     modelManager->AddModelReference("spider7.mod", false, rank);
-    m_object->SetPosition(35, Math::Vector(0.0f, 0.0f, -0.3f));
+    m_object->SetPartPosition(35, Math::Vector(0.0f, 0.0f, -0.3f));
 
     // Creates the left mandible.
     rank = m_engine->CreateObject();
@@ -180,19 +177,17 @@ bool CMotionSpider::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(36, rank);
     m_object->SetObjectParent(36, 1);
     modelManager->AddModelReference("spider7.mod", true, rank);
-    m_object->SetPosition(36, Math::Vector(0.0f, 0.0f, 0.3f));
+    m_object->SetPartPosition(36, Math::Vector(0.0f, 0.0f, 0.3f));
 
     m_object->CreateShadowCircle(4.0f, 0.5f);
 
     CreatePhysics();
     m_object->SetFloorHeight(0.0f);
 
-    pos = m_object->GetPosition(0);
-    m_object->SetPosition(0, pos);  // to display the shadows immediately
+    pos = m_object->GetPosition();
+    m_object->SetPosition(pos);  // to display the shadows immediately
 
     m_engine->LoadAllTextures();
-
-    return true;
 }
 
 // Creates the physics of the object.
@@ -274,8 +269,6 @@ void CMotionSpider::CreatePhysics()
         -5,0,0,     -5,0,0,     -5,0,0,     -5,0,0,     // s5: fingers 1..4
     };
 
-    m_physics->SetType(TYPE_ROLLING);
-
     character = m_object->GetCharacter();
     character->wheelFront = 4.0f;
     character->wheelBack  = 4.0f;
@@ -326,37 +319,6 @@ bool CMotionSpider::EventProcess(const Event &event)
         return EventFrame(event);
     }
 
-    if ( event.type == EVENT_KEY_DOWN )
-    {
-#if ADJUST_ANGLE
-        int     i;
-
-        if ( event.param == 'A' )  m_armTimeIndex++;
-        if ( m_armTimeIndex >= 3 )  m_armTimeIndex = 0;
-
-        if ( event.param == 'Q' )  m_armPartIndex++;
-        if ( m_armPartIndex >= 4 )  m_armPartIndex = 0;
-
-        if ( event.param == 'W' )  m_armMemberIndex++;
-        if ( m_armMemberIndex >= 4 )  m_armMemberIndex = 0;
-
-        i  = m_armMemberIndex*3;
-        i += m_armPartIndex*3*4;
-        i += m_armTimeIndex*3*4*4;
-
-        if ( event.param == 'E' )  m_armAngles[i+0] += 5;
-        if ( event.param == 'D' )  m_armAngles[i+0] -= 5;
-        if ( event.param == 'R' )  m_armAngles[i+1] += 5;
-        if ( event.param == 'F' )  m_armAngles[i+1] -= 5;
-        if ( event.param == 'T' )  m_armAngles[i+2] += 5;
-        if ( event.param == 'G' )  m_armAngles[i+2] -= 5;
-        if ( event.param == 'Z' )  m_armAngles[i+3] += 5;
-        if ( event.param == 'H' )  m_armAngles[i+3] -= 5;
-
-        if ( event.param == 'Y' )  m_bArmStop = !m_bArmStop;
-#endif
-    }
-
     return true;
 }
 
@@ -372,7 +334,7 @@ bool CMotionSpider::EventFrame(const Event &event)
     bool        bStop;
 
     if ( m_engine->GetPause() )  return true;
-    if ( !m_engine->IsVisiblePoint(m_object->GetPosition(0)) )  return true;
+    if ( !m_engine->IsVisiblePoint(m_object->GetPosition()) )  return true;
 
     s =     m_physics->GetLinMotionX(MO_MOTSPEED)*1.5f;
     a = fabs(m_physics->GetCirMotionY(MO_MOTSPEED)*2.0f);
@@ -381,7 +343,6 @@ bool CMotionSpider::EventFrame(const Event &event)
 
     m_armTimeAbs += event.rTime;
     m_armTimeAction += event.rTime;
-    m_armTimeMarch += (s)*event.rTime*0.15f;
     m_armMember += (s+a)*event.rTime*0.15f;
 
     bStop = ( a == 0.0f && s == 0.0f );  // stop?
@@ -400,13 +361,10 @@ bool CMotionSpider::EventFrame(const Event &event)
         m_armMember += a;
     }
 
-    if ( m_object->GetRuin() )  // destroyed?
+    assert(m_object->Implements(ObjectInterfaceType::Destroyable));
+    if (dynamic_cast<CDestroyableObject*>(m_object)->GetDying() == DeathType::Burning )  // burning?
     {
-        m_actionType = MSS_RUIN;
-    }
-    if ( m_object->GetBurn() )  // burning?
-    {
-        if ( m_object->GetFixed() )
+        if ( dynamic_cast<CBaseAlien*>(m_object)->GetFixed() )
         {
             m_actionType = MSS_BURN;
         }
@@ -415,6 +373,10 @@ bool CMotionSpider::EventFrame(const Event &event)
             m_actionType = -1;
         }
     }
+    else if ( dynamic_cast<CDestroyableObject*>(m_object)->IsDying() )  // destroyed?
+    {
+        m_actionType = MSS_RUIN;
+    }
 
     for ( i=0 ; i<8 ; i++ )  // the 8 legs
     {
@@ -431,11 +393,6 @@ bool CMotionSpider::EventFrame(const Event &event)
 //?         else          prog = Math::Mod(m_armMember+(2.0f-(i%4))*0.25f+0.3f, 1.0f);
             if ( i < 4 )  prog = Math::Mod(m_armMember+(2.0f-(i%4))*0.25f+0.0f, 1.0f);
             else          prog = Math::Mod(m_armMember+(2.0f-(i%4))*0.25f+0.5f, 1.0f);
-            if ( m_bArmStop )
-            {
-                prog = static_cast< float >(m_armTimeIndex/3.0f);
-                action = MS_MARCH;
-            }
             if ( prog < 0.33f )  // t0..t1 ?
             {
                 prog = prog/0.33f;  // 0..1
@@ -501,68 +458,59 @@ bool CMotionSpider::EventFrame(const Event &event)
 
         if ( i < 4 )  // right leg (1..4) ?
         {
-            m_object->SetAngleX(3+4*i+0, Math::Smooth(m_object->GetAngleX(3+4*i+0), Math::PropAngle(tSt[ 0], tNd[ 0], prog), time));
-            m_object->SetAngleY(3+4*i+0, Math::Smooth(m_object->GetAngleY(3+4*i+0), Math::PropAngle(tSt[ 1], tNd[ 1], prog), time));
-            m_object->SetAngleZ(3+4*i+0, Math::Smooth(m_object->GetAngleZ(3+4*i+0), Math::PropAngle(tSt[ 2], tNd[ 2], prog), time));
-            m_object->SetAngleX(3+4*i+1, Math::Smooth(m_object->GetAngleX(3+4*i+1), Math::PropAngle(tSt[ 3], tNd[ 3], prog), time));
-            m_object->SetAngleY(3+4*i+1, Math::Smooth(m_object->GetAngleY(3+4*i+1), Math::PropAngle(tSt[ 4], tNd[ 4], prog), time));
-            m_object->SetAngleZ(3+4*i+1, Math::Smooth(m_object->GetAngleZ(3+4*i+1), Math::PropAngle(tSt[ 5], tNd[ 5], prog), time));
-            m_object->SetAngleX(3+4*i+2, Math::Smooth(m_object->GetAngleX(3+4*i+2), Math::PropAngle(tSt[ 6], tNd[ 6], prog), time));
-            m_object->SetAngleY(3+4*i+2, Math::Smooth(m_object->GetAngleY(3+4*i+2), Math::PropAngle(tSt[ 7], tNd[ 7], prog), time));
-            m_object->SetAngleZ(3+4*i+2, Math::Smooth(m_object->GetAngleZ(3+4*i+2), Math::PropAngle(tSt[ 8], tNd[ 8], prog), time));
-            m_object->SetAngleX(3+4*i+3, Math::Smooth(m_object->GetAngleX(3+4*i+3), Math::PropAngle(tSt[ 9], tNd[ 9], prog), time));
-            m_object->SetAngleY(3+4*i+3, Math::Smooth(m_object->GetAngleY(3+4*i+3), Math::PropAngle(tSt[10], tNd[10], prog), time));
-            m_object->SetAngleZ(3+4*i+3, Math::Smooth(m_object->GetAngleZ(3+4*i+3), Math::PropAngle(tSt[11], tNd[11], prog), time));
+            m_object->SetPartRotationX(3+4*i+0, Math::Smooth(m_object->GetPartRotationX(3+4*i+0), Math::PropAngle(tSt[ 0], tNd[ 0], prog), time));
+            m_object->SetPartRotationY(3+4*i+0, Math::Smooth(m_object->GetPartRotationY(3+4*i+0), Math::PropAngle(tSt[ 1], tNd[ 1], prog), time));
+            m_object->SetPartRotationZ(3+4*i+0, Math::Smooth(m_object->GetPartRotationZ(3+4*i+0), Math::PropAngle(tSt[ 2], tNd[ 2], prog), time));
+            m_object->SetPartRotationX(3+4*i+1, Math::Smooth(m_object->GetPartRotationX(3+4*i+1), Math::PropAngle(tSt[ 3], tNd[ 3], prog), time));
+            m_object->SetPartRotationY(3+4*i+1, Math::Smooth(m_object->GetPartRotationY(3+4*i+1), Math::PropAngle(tSt[ 4], tNd[ 4], prog), time));
+            m_object->SetPartRotationZ(3+4*i+1, Math::Smooth(m_object->GetPartRotationZ(3+4*i+1), Math::PropAngle(tSt[ 5], tNd[ 5], prog), time));
+            m_object->SetPartRotationX(3+4*i+2, Math::Smooth(m_object->GetPartRotationX(3+4*i+2), Math::PropAngle(tSt[ 6], tNd[ 6], prog), time));
+            m_object->SetPartRotationY(3+4*i+2, Math::Smooth(m_object->GetPartRotationY(3+4*i+2), Math::PropAngle(tSt[ 7], tNd[ 7], prog), time));
+            m_object->SetPartRotationZ(3+4*i+2, Math::Smooth(m_object->GetPartRotationZ(3+4*i+2), Math::PropAngle(tSt[ 8], tNd[ 8], prog), time));
+            m_object->SetPartRotationX(3+4*i+3, Math::Smooth(m_object->GetPartRotationX(3+4*i+3), Math::PropAngle(tSt[ 9], tNd[ 9], prog), time));
+            m_object->SetPartRotationY(3+4*i+3, Math::Smooth(m_object->GetPartRotationY(3+4*i+3), Math::PropAngle(tSt[10], tNd[10], prog), time));
+            m_object->SetPartRotationZ(3+4*i+3, Math::Smooth(m_object->GetPartRotationZ(3+4*i+3), Math::PropAngle(tSt[11], tNd[11], prog), time));
         }
         else    // left leg (5..8) ?
         {
-            m_object->SetAngleX(3+4*i+0, Math::Smooth(m_object->GetAngleX(3+4*i+0), Math::PropAngle(-tSt[ 0], -tNd[ 0], prog), time));
-            m_object->SetAngleY(3+4*i+0, Math::Smooth(m_object->GetAngleY(3+4*i+0), Math::PropAngle(-tSt[ 1], -tNd[ 1], prog), time));
-            m_object->SetAngleZ(3+4*i+0, Math::Smooth(m_object->GetAngleZ(3+4*i+0), Math::PropAngle( tSt[ 2],  tNd[ 2], prog), time));
-            m_object->SetAngleX(3+4*i+1, Math::Smooth(m_object->GetAngleX(3+4*i+1), Math::PropAngle(-tSt[ 3], -tNd[ 3], prog), time));
-            m_object->SetAngleY(3+4*i+1, Math::Smooth(m_object->GetAngleY(3+4*i+1), Math::PropAngle(-tSt[ 4], -tNd[ 4], prog), time));
-            m_object->SetAngleZ(3+4*i+1, Math::Smooth(m_object->GetAngleZ(3+4*i+1), Math::PropAngle( tSt[ 5],  tNd[ 5], prog), time));
-            m_object->SetAngleX(3+4*i+2, Math::Smooth(m_object->GetAngleX(3+4*i+2), Math::PropAngle(-tSt[ 6], -tNd[ 6], prog), time));
-            m_object->SetAngleY(3+4*i+2, Math::Smooth(m_object->GetAngleY(3+4*i+2), Math::PropAngle(-tSt[ 7], -tNd[ 7], prog), time));
-            m_object->SetAngleZ(3+4*i+2, Math::Smooth(m_object->GetAngleZ(3+4*i+2), Math::PropAngle( tSt[ 8],  tNd[ 8], prog), time));
-            m_object->SetAngleX(3+4*i+3, Math::Smooth(m_object->GetAngleX(3+4*i+3), Math::PropAngle(-tSt[ 9], -tNd[ 9], prog), time));
-            m_object->SetAngleY(3+4*i+3, Math::Smooth(m_object->GetAngleY(3+4*i+3), Math::PropAngle(-tSt[10], -tNd[10], prog), time));
-            m_object->SetAngleZ(3+4*i+3, Math::Smooth(m_object->GetAngleZ(3+4*i+3), Math::PropAngle( tSt[11],  tNd[11], prog), time));
+            m_object->SetPartRotationX(3+4*i+0, Math::Smooth(m_object->GetPartRotationX(3+4*i+0), Math::PropAngle(-tSt[ 0], -tNd[ 0], prog), time));
+            m_object->SetPartRotationY(3+4*i+0, Math::Smooth(m_object->GetPartRotationY(3+4*i+0), Math::PropAngle(-tSt[ 1], -tNd[ 1], prog), time));
+            m_object->SetPartRotationZ(3+4*i+0, Math::Smooth(m_object->GetPartRotationZ(3+4*i+0), Math::PropAngle( tSt[ 2],  tNd[ 2], prog), time));
+            m_object->SetPartRotationX(3+4*i+1, Math::Smooth(m_object->GetPartRotationX(3+4*i+1), Math::PropAngle(-tSt[ 3], -tNd[ 3], prog), time));
+            m_object->SetPartRotationY(3+4*i+1, Math::Smooth(m_object->GetPartRotationY(3+4*i+1), Math::PropAngle(-tSt[ 4], -tNd[ 4], prog), time));
+            m_object->SetPartRotationZ(3+4*i+1, Math::Smooth(m_object->GetPartRotationZ(3+4*i+1), Math::PropAngle( tSt[ 5],  tNd[ 5], prog), time));
+            m_object->SetPartRotationX(3+4*i+2, Math::Smooth(m_object->GetPartRotationX(3+4*i+2), Math::PropAngle(-tSt[ 6], -tNd[ 6], prog), time));
+            m_object->SetPartRotationY(3+4*i+2, Math::Smooth(m_object->GetPartRotationY(3+4*i+2), Math::PropAngle(-tSt[ 7], -tNd[ 7], prog), time));
+            m_object->SetPartRotationZ(3+4*i+2, Math::Smooth(m_object->GetPartRotationZ(3+4*i+2), Math::PropAngle( tSt[ 8],  tNd[ 8], prog), time));
+            m_object->SetPartRotationX(3+4*i+3, Math::Smooth(m_object->GetPartRotationX(3+4*i+3), Math::PropAngle(-tSt[ 9], -tNd[ 9], prog), time));
+            m_object->SetPartRotationY(3+4*i+3, Math::Smooth(m_object->GetPartRotationY(3+4*i+3), Math::PropAngle(-tSt[10], -tNd[10], prog), time));
+            m_object->SetPartRotationZ(3+4*i+3, Math::Smooth(m_object->GetPartRotationZ(3+4*i+3), Math::PropAngle( tSt[11],  tNd[11], prog), time));
         }
     }
 
-#if ADJUST_ANGLE
-    if ( m_object->GetSelect() )
-    {
-        char s[100];
-        sprintf(s, "A:time=%d Q:part=%d W:member=%d", m_armTimeIndex, m_armPartIndex, m_armMemberIndex);
-        m_engine->SetInfoText(4, s);
-    }
-#endif
-
     if ( m_actionType == MSS_BURN )  // burning?
     {
         dir = Math::Vector(Math::PI, 0.0f, 0.0f);
         SetCirVibration(dir);
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
         SetLinVibration(dir);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
         time = event.rTime*1.0f;
-        m_object->SetAngleZ(1, Math::Smooth(m_object->GetAngleZ(1), 0.0f, time));  // head
+        m_object->SetPartRotationZ(1, Math::Smooth(m_object->GetPartRotationZ(1), 0.0f, time));  // head
     }
     else if ( m_actionType == MSS_RUIN )  // destroyed?
     {
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
         SetLinVibration(dir);
         SetCirVibration(dir);
-        SetInclinaison(dir);
+        SetTilt(dir);
     }
     else if ( m_actionType == MSS_EXPLO )  // exploded?
     {
-        m_object->SetZoomY(1, 1.0f+m_progress);
-        m_object->SetZoomZ(1, 1.0f+m_progress);
-        m_object->SetZoomX(1, 1.0f+m_progress/2.0f);
+        m_object->SetPartScaleY(1, 1.0f+m_progress);
+        m_object->SetPartScaleZ(1, 1.0f+m_progress);
+        m_object->SetPartScaleX(1, 1.0f+m_progress/2.0f);
 
         dir.x = (Math::Rand()-0.5f)*0.1f*m_progress;
         dir.y = (Math::Rand()-0.5f)*0.1f*m_progress;
@@ -575,7 +523,7 @@ bool CMotionSpider::EventFrame(const Event &event)
         {
             m_lastParticle = m_armTimeAbs;
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             speed.x = (Math::Rand()-0.5f)*10.0f;
             speed.z = (Math::Rand()-0.5f)*10.0f;
             speed.y = Math::Rand()*5.0f;
@@ -604,7 +552,7 @@ bool CMotionSpider::EventFrame(const Event &event)
         SetCirVibration(dir);
 
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
         if ( m_progress >= 1.0f )
         {
@@ -619,7 +567,7 @@ bool CMotionSpider::EventFrame(const Event &event)
 
             if ( rand()%10 == 0 )
             {
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 pos.x += (Math::Rand()-0.5f)*8.0f;
                 pos.z += (Math::Rand()-0.5f)*8.0f;
                 pos.y -= 1.0f;
@@ -648,12 +596,12 @@ bool CMotionSpider::EventFrame(const Event &event)
                 sinf(m_armTimeAbs*15.0f)*0.03f;
         SetCirVibration(dir);
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
-        m_object->SetAngleY(1, sinf(m_armTimeAbs*5.0f)*0.05f);  // tail
-        m_object->SetAngleY(2, cosf(m_armTimeAbs*5.0f)*0.20f);  // head
-        m_object->SetAngleZ(1, 0.4f);  // tail
-        m_object->SetAngleZ(2, 0.0f);  // head
+        m_object->SetPartRotationY(1, sinf(m_armTimeAbs*5.0f)*0.05f);  // tail
+        m_object->SetPartRotationY(2, cosf(m_armTimeAbs*5.0f)*0.20f);  // head
+        m_object->SetPartRotationZ(1, 0.4f);  // tail
+        m_object->SetPartRotationZ(2, 0.0f);  // head
 
         if ( m_progress >= 1.0f )
         {
@@ -666,7 +614,7 @@ bool CMotionSpider::EventFrame(const Event &event)
         {
             m_lastParticle = m_armTimeAbs;
 
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             speed.x = (Math::Rand()-0.5f)*10.0f;
             speed.z = (Math::Rand()-0.5f)*10.0f;
             speed.y = Math::Rand()*5.0f;
@@ -695,12 +643,12 @@ bool CMotionSpider::EventFrame(const Event &event)
         SetCirVibration(dir);
 
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
-        SetInclinaison(dir);
+        SetTilt(dir);
 
         if ( m_progress >= 1.0f )
         {
             SetAction(-1);
-            m_object->SetFixed(false);  // moving again
+            dynamic_cast<CBaseAlien*>(m_object)->SetFixed(false);  // moving again
         }
     }
     else
@@ -708,7 +656,7 @@ bool CMotionSpider::EventFrame(const Event &event)
         if ( bStop )
         {
             dir = Math::Vector(0.0f, 0.0f, 0.0f);
-            SetInclinaison(dir);
+            SetTilt(dir);
         }
         else
         {
@@ -723,26 +671,25 @@ bool CMotionSpider::EventFrame(const Event &event)
             dir.z = sinf(s)*0.1f;
 
             dir.y = 0.0f;
-            SetInclinaison(dir);
+            SetTilt(dir);
         }
 
         dir = Math::Vector(0.0f, 0.0f, 0.0f);
         SetLinVibration(dir);
         SetCirVibration(dir);
 
-        m_object->SetAngleZ(1, sinf(m_armTimeAbs*1.7f)*0.02f);  // tail
-        m_object->SetAngleX(1, sinf(m_armTimeAbs*1.3f)*0.05f);
-        m_object->SetAngleY(1, sinf(m_armTimeAbs*2.4f)*0.10f);
-        m_object->SetZoom(1, 1.0f+sinf(m_armTimeAbs*3.3f)*0.05f);
+        m_object->SetPartRotationZ(1, sinf(m_armTimeAbs*1.7f)*0.02f);  // tail
+        m_object->SetPartRotationX(1, sinf(m_armTimeAbs*1.3f)*0.05f);
+        m_object->SetPartRotationY(1, sinf(m_armTimeAbs*2.4f)*0.10f);
+        m_object->SetPartScale(1, 1.0f+sinf(m_armTimeAbs*3.3f)*0.05f);
 
-        m_object->SetAngleZ(2, sinf(m_armTimeAbs*1.4f)*0.20f);  // head
-        m_object->SetAngleX(2, sinf(m_armTimeAbs*1.9f)*0.10f);
-        m_object->SetAngleY(2, sinf(m_armTimeAbs*2.1f)*0.10f);
+        m_object->SetPartRotationZ(2, sinf(m_armTimeAbs*1.4f)*0.20f);  // head
+        m_object->SetPartRotationX(2, sinf(m_armTimeAbs*1.9f)*0.10f);
+        m_object->SetPartRotationY(2, sinf(m_armTimeAbs*2.1f)*0.10f);
 
-        m_object->SetAngleY(35,  sinf(m_armTimeAbs*3.1f)*0.20f);  // mandible
-        m_object->SetAngleY(36, -sinf(m_armTimeAbs*3.1f)*0.20f);  // mandible
+        m_object->SetPartRotationY(35,  sinf(m_armTimeAbs*3.1f)*0.20f);  // mandible
+        m_object->SetPartRotationY(36, -sinf(m_armTimeAbs*3.1f)*0.20f);  // mandible
     }
 
     return true;
 }
-
