@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,26 +22,28 @@
 
 #include "app/app.h"
 
-#include "math/geometry.h"
-
-#include "graphics/engine/modelmanager.h"
+#include "graphics/engine/oldmodelmanager.h"
 #include "graphics/engine/terrain.h"
 #include "graphics/engine/water.h"
 
-#include "object/robotmain.h"
+#include "level/robotmain.h"
+
+#include "math/geometry.h"
+
+#include "object/old_object.h"
+
+#include "sound/sound.h"
 
 
 #include <stdio.h>
 
 
 
-const float START_TIME = 1000.0f;   // beginning of the relative time
-
 
 
 // Object's constructor.
 
-CMotionToto::CMotionToto(CObject* object) : CMotion(object)
+CMotionToto::CMotionToto(COldObject* object) : CMotion(object)
 {
     m_time = 0.0f;
     m_bDisplayInfo = false;
@@ -80,10 +82,9 @@ void CMotionToto::DeleteObject(bool bAll)
 
 // Creates a vehicle traveling any lands on the ground.
 
-bool CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
-                         float power)
+void CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
+                         float power, Gfx::COldModelManager* modelManager)
 {
-    Gfx::CModelManager* modelManager = Gfx::CModelManager::GetInstancePointer();
     int             rank;
 
 
@@ -94,8 +95,8 @@ bool CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_VEHICLE);  // this is a moving object
     m_object->SetObjectRank(0, rank);
     modelManager->AddModelReference("toto1.mod", false, rank);
-    m_object->SetPosition(0, pos);
-    m_object->SetAngleY(0, angle);
+    m_object->SetPosition(pos);
+    m_object->SetRotationY(angle);
 
     // Creates mouth.
     rank = m_engine->CreateObject();
@@ -103,7 +104,7 @@ bool CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(1, rank);
     m_object->SetObjectParent(1, 0);
     modelManager->AddModelReference("toto2.mod", false, rank);
-    m_object->SetPosition(1, Math::Vector(1.00f, 0.17f, 0.00f));
+    m_object->SetPartPosition(1, Math::Vector(1.00f, 0.17f, 0.00f));
 
     // Creates the left eye.
     rank = m_engine->CreateObject();
@@ -111,8 +112,8 @@ bool CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(2, rank);
     m_object->SetObjectParent(2, 0);
     modelManager->AddModelReference("toto3.mod", true, rank);
-    m_object->SetPosition(2, Math::Vector(0.85f, 1.04f, 0.25f));
-    m_object->SetAngleY(2, -20.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(2, Math::Vector(0.85f, 1.04f, 0.25f));
+    m_object->SetPartRotationY(2, -20.0f*Math::PI/180.0f);
 
     // Creates the right eye.
     rank = m_engine->CreateObject();
@@ -120,8 +121,8 @@ bool CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(3, rank);
     m_object->SetObjectParent(3, 0);
     modelManager->AddModelReference("toto3.mod", false, rank);
-    m_object->SetPosition(3, Math::Vector(0.85f, 1.04f, -0.25f));
-    m_object->SetAngleY(3, 20.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(3, Math::Vector(0.85f, 1.04f, -0.25f));
+    m_object->SetPartRotationY(3, 20.0f*Math::PI/180.0f);
 
     // Creates left antenna.
     rank = m_engine->CreateObject();
@@ -129,24 +130,24 @@ bool CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(4, rank);
     m_object->SetObjectParent(4, 0);
     modelManager->AddModelReference("toto4.mod", false, rank);
-    m_object->SetPosition(4, Math::Vector(0.0f, 1.9f, 0.3f));
-    m_object->SetAngleX(4, 30.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(4, Math::Vector(0.0f, 1.9f, 0.3f));
+    m_object->SetPartRotationX(4, 30.0f*Math::PI/180.0f);
 
     rank = m_engine->CreateObject();
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_DESCENDANT);
     m_object->SetObjectRank(5, rank);
     m_object->SetObjectParent(5, 4);
     modelManager->AddModelReference("toto4.mod", false, rank);
-    m_object->SetPosition(5, Math::Vector(0.0f, 0.67f, 0.0f));
-    m_object->SetAngleX(5, 30.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(5, Math::Vector(0.0f, 0.67f, 0.0f));
+    m_object->SetPartRotationX(5, 30.0f*Math::PI/180.0f);
 
     rank = m_engine->CreateObject();
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_DESCENDANT);
     m_object->SetObjectRank(6, rank);
     m_object->SetObjectParent(6, 5);
     modelManager->AddModelReference("toto5.mod", false, rank);
-    m_object->SetPosition(6, Math::Vector(0.0f, 0.70f, 0.0f));
-    m_object->SetAngleX(6, 30.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(6, Math::Vector(0.0f, 0.70f, 0.0f));
+    m_object->SetPartRotationX(6, 30.0f*Math::PI/180.0f);
 
     // Creates right antenna.
     rank = m_engine->CreateObject();
@@ -154,34 +155,32 @@ bool CMotionToto::Create(Math::Vector pos, float angle, ObjectType type,
     m_object->SetObjectRank(7, rank);
     m_object->SetObjectParent(7, 0);
     modelManager->AddModelReference("toto4.mod", false, rank);
-    m_object->SetPosition(7, Math::Vector(0.0f, 1.9f, -0.3f));
-    m_object->SetAngleX(7, -30.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(7, Math::Vector(0.0f, 1.9f, -0.3f));
+    m_object->SetPartRotationX(7, -30.0f*Math::PI/180.0f);
 
     rank = m_engine->CreateObject();
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_DESCENDANT);
     m_object->SetObjectRank(8, rank);
     m_object->SetObjectParent(8, 7);
     modelManager->AddModelReference("toto4.mod", false, rank);
-    m_object->SetPosition(8, Math::Vector(0.0f, 0.67f, 0.0f));
-    m_object->SetAngleX(8, -30.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(8, Math::Vector(0.0f, 0.67f, 0.0f));
+    m_object->SetPartRotationX(8, -30.0f*Math::PI/180.0f);
 
     rank = m_engine->CreateObject();
     m_engine->SetObjectType(rank, Gfx::ENG_OBJTYPE_DESCENDANT);
     m_object->SetObjectRank(9, rank);
     m_object->SetObjectParent(9, 8);
     modelManager->AddModelReference("toto5.mod", false, rank);
-    m_object->SetPosition(9, Math::Vector(0.0f, 0.70f, 0.0f));
-    m_object->SetAngleX(9, -30.0f*Math::PI/180.0f);
+    m_object->SetPartPosition(9, Math::Vector(0.0f, 0.70f, 0.0f));
+    m_object->SetPartRotationX(9, -30.0f*Math::PI/180.0f);
 
-    m_object->SetZoom(0, 0.5f);  // is little
+    m_object->SetScale(0.5f);  // is little
     m_object->SetFloorHeight(0.0f);
 
-    pos = m_object->GetPosition(0);
-    m_object->SetPosition(0, pos);  // to display the shadows immediately
+    pos = m_object->GetPosition();
+    m_object->SetPosition(pos);  // to display the shadows immediately
 
     m_engine->LoadAllTextures();
-
-    return true;
 }
 
 
@@ -197,7 +196,7 @@ return;
     m_actionTime = 0.0f;
     m_progress   = 0.0f;
 
-    m_object->SetAngleY(0, 0.0f);
+    m_object->SetRotationY(0.0f);
     m_mousePos = Math::Point(0.5f, 0.5f);
 }
 
@@ -261,21 +260,14 @@ bool CMotionToto::EventFrame(const Event &event)
         {
             bHidden = true;
         }
-        if ( !m_engine->GetTotoMode() )
-        {
-            if ( !m_main->GetEditLock() )  // current edition?
-            {
-                bHidden = true;
-            }
-        }
     }
 
     if ( bHidden )
     {
-        nPos = m_object->GetPosition(0);
+        nPos = m_object->GetPosition();
         m_terrain->AdjustToFloor(nPos, true);
         nPos.y -= 100.0f;  // hidden under the ground!
-        m_object->SetPosition(0, nPos);
+        m_object->SetPosition(nPos);
         return true;
     }
 
@@ -330,15 +322,9 @@ bool CMotionToto::EventFrame(const Event &event)
     }
     else
     {
-#if 0
-        distance = 30.0f-progress*24.5f;  // remoteness
-        shift    = 18.0f-progress*15.4f;  // shift is left
-        verti    = 10.0f-progress* 9.6f;  // shift at the top
-#else
         distance = 30.0f-progress*18.0f;  // remoteness
         shift    = 18.0f-progress*11.0f;  // shift is left
         verti    = 10.0f-progress* 8.0f;  // shift at the top
-#endif
 
         if ( m_actionType == -1 &&
              (m_type == OBJECT_HUMAN    ||
@@ -384,7 +370,7 @@ bool CMotionToto::EventFrame(const Event &event)
                     m_clownRadius = 0.0f;
                     m_clownDelay  = 2.0f+Math::Rand()*2.0f;
                 }
-                pos = m_object->GetPosition(0);
+                pos = m_object->GetPosition();
                 if ( pos.y < m_water->GetLevel() )  // underwater?
                 {
                     m_clownRadius /= 1.5f;
@@ -422,7 +408,7 @@ bool CMotionToto::EventFrame(const Event &event)
         }
         else
         {
-            aPos = m_object->GetPosition(0);
+            aPos = m_object->GetPosition();
             if ( m_actionType == -1 )
             {
                 level = 4.0f;
@@ -458,7 +444,7 @@ bool CMotionToto::EventFrame(const Event &event)
     }
     level = Math::Min(linSpeed*0.1f, 1.0f);
     nAngle = nAngle*(1.0f-level) + mAngle*level;
-    aAngle = Math::NormAngle(m_object->GetAngleY(0));
+    aAngle = Math::NormAngle(m_object->GetRotationY());
 
     if ( nAngle < aAngle )
     {
@@ -485,7 +471,6 @@ bool CMotionToto::EventFrame(const Event &event)
     aAntenna.z -= angle;  // leans forward
 
     // Calculates the residual motion.
-#if 1
     vibLin.y += (sinf(m_time*2.00f)*0.5f+
                  sinf(m_time*2.11f)*0.2f)*(1.0f-progress);
 
@@ -500,7 +485,6 @@ bool CMotionToto::EventFrame(const Event &event)
     vibCir.y += (sinf(m_time*Math::PI* 1.07f)*(Math::PI/ 10.0f)+
                  sinf(m_time*Math::PI* 1.19f)*(Math::PI/ 17.0f)+
                  sinf(m_time*Math::PI* 1.57f)*(Math::PI/ 31.0f))*(1.0f-progress);
-#endif
 
     // Calculates the animations in action.
     if ( m_actionType == MT_ERROR )  // no-no?
@@ -576,8 +560,8 @@ bool CMotionToto::EventFrame(const Event &event)
         }
         nPos.y -= vibLin.y;
     }
-    m_object->SetPosition(0, nPos);
-    m_object->SetAngleY(0, nAngle);
+    m_object->SetPosition(nPos);
+    m_object->SetRotationY(nAngle);
 
     SetLinVibration(vibLin);
     SetCirVibration(vibCir);
@@ -596,9 +580,9 @@ bool CMotionToto::EventFrame(const Event &event)
              sinf(m_time*Math::PI*2.83f)*(Math::PI/70.0f)+
              sinf(m_time*Math::PI*3.09f)*(Math::PI/90.0f);
 
-    m_object->SetAngle(4, pos);  // left antenna
-    m_object->SetAngle(5, pos);  // left antenna
-    m_object->SetAngle(6, pos);  // left antenna
+    m_object->SetPartRotation(4, pos);  // left antenna
+    m_object->SetPartRotation(5, pos);  // left antenna
+    m_object->SetPartRotation(6, pos);  // left antenna
 
     pos = aAntenna*0.40f;
     pos.x = -pos.x;
@@ -614,45 +598,45 @@ bool CMotionToto::EventFrame(const Event &event)
              sinf(m_time*Math::PI*2.38f)*(Math::PI/70.0f)+
              sinf(m_time*Math::PI*2.79f)*(Math::PI/90.0f);
 
-    m_object->SetAngle(7, pos);  // right antenna
-    m_object->SetAngle(8, pos);  // right antenna
-    m_object->SetAngle(9, pos);  // right antenna
+    m_object->SetPartRotation(7, pos);  // right antenna
+    m_object->SetPartRotation(8, pos);  // right antenna
+    m_object->SetPartRotation(9, pos);  // right antenna
 
     // Movement of the mouth.
     if ( m_actionType == MT_ERROR )  // no-no?
     {
-        m_object->SetAngleX(1, 0.0f);
-        m_object->SetAngleZ(1, 0.2f+sinf(m_time*10.0f)*0.2f);
-        m_object->SetZoomY(1, 2.0f+sinf(m_time*10.0f));
-        m_object->SetZoomZ(1, 1.0f);
+        m_object->SetPartRotationX(1, 0.0f);
+        m_object->SetPartRotationZ(1, 0.2f+sinf(m_time*10.0f)*0.2f);
+        m_object->SetPartScaleY(1, 2.0f+sinf(m_time*10.0f));
+        m_object->SetPartScaleZ(1, 1.0f);
     }
     else if ( m_actionType == MT_WARNING )  // warning?
     {
-        m_object->SetAngleX(1, 15.0f*Math::PI/180.0f);
-        m_object->SetAngleZ(1, 0.0f);
-        m_object->SetZoomY(1, 1.0f);
-        m_object->SetZoomZ(1, 1.0f);
+        m_object->SetPartRotationX(1, 15.0f*Math::PI/180.0f);
+        m_object->SetPartRotationZ(1, 0.0f);
+        m_object->SetPartScaleY(1, 1.0f);
+        m_object->SetPartScaleZ(1, 1.0f);
     }
     else if ( m_actionType == MT_INFO )  // yes-yes?
     {
-        m_object->SetAngleX(1, 0.0f);
-        m_object->SetAngleZ(1, 0.0f);
-        m_object->SetZoomY(1, 1.0f);
-        m_object->SetZoomZ(1, 0.7f+sinf(m_time*10.0f)*0.3f);
+        m_object->SetPartRotationX(1, 0.0f);
+        m_object->SetPartRotationZ(1, 0.0f);
+        m_object->SetPartScaleY(1, 1.0f);
+        m_object->SetPartScaleZ(1, 0.7f+sinf(m_time*10.0f)*0.3f);
     }
     else if ( m_actionType == MT_MESSAGE )  // message?
     {
-        m_object->SetAngleX(1, 0.0f);
-        m_object->SetAngleZ(1, 0.0f);
-        m_object->SetZoomY(1, 1.0f);
-        m_object->SetZoomZ(1, 0.8f+sinf(m_time*7.0f)*0.2f);
+        m_object->SetPartRotationX(1, 0.0f);
+        m_object->SetPartRotationZ(1, 0.0f);
+        m_object->SetPartScaleY(1, 1.0f);
+        m_object->SetPartScaleZ(1, 0.8f+sinf(m_time*7.0f)*0.2f);
     }
     else
     {
-        m_object->SetAngleX(1, 0.0f);
-        m_object->SetAngleZ(1, 0.0f);
-        m_object->SetZoomY(1, 1.0f);
-        m_object->SetZoomZ(1, 1.0f);
+        m_object->SetPartRotationX(1, 0.0f);
+        m_object->SetPartRotationZ(1, 0.0f);
+        m_object->SetPartScaleY(1, 1.0f);
+        m_object->SetPartScaleZ(1, 1.0f);
     }
 
     // Eye blinking management.
@@ -669,42 +653,42 @@ bool CMotionToto::EventFrame(const Event &event)
         {
             if ( m_blinkProgress < 0.5f )  zoom = m_blinkProgress/0.5f;
             else                           zoom = 2.0f-m_blinkProgress/0.5f;
-            m_object->SetZoomY(2, 1.0f-zoom*0.9f);
-            m_object->SetZoomY(3, 1.0f-zoom*0.9f);
+            m_object->SetPartScaleY(2, 1.0f-zoom*0.9f);
+            m_object->SetPartScaleY(3, 1.0f-zoom*0.9f);
         }
         else
         {
             m_blinkProgress = -1.0f;
             m_blinkTime = 0.1f+Math::Rand()*4.0f;
-            m_object->SetZoomY(2, 1.0f);
-            m_object->SetZoomY(3, 1.0f);
+            m_object->SetPartScaleY(2, 1.0f);
+            m_object->SetPartScaleY(3, 1.0f);
         }
     }
 
     if ( m_actionType == MT_ERROR )  // no-no?
     {
-        m_object->SetAngleX(2, -30.0f*Math::PI/180.0f);
-        m_object->SetAngleX(3,  30.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(2, -30.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(3,  30.0f*Math::PI/180.0f);
     }
     else if ( m_actionType == MT_WARNING )  // warning?
     {
-        m_object->SetAngleX(2, -15.0f*Math::PI/180.0f);
-        m_object->SetAngleX(3,  15.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(2, -15.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(3,  15.0f*Math::PI/180.0f);
     }
     else if ( m_actionType == MT_INFO )  // yes-yes?
     {
-        m_object->SetAngleX(2,  40.0f*Math::PI/180.0f);
-        m_object->SetAngleX(3, -40.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(2,  40.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(3, -40.0f*Math::PI/180.0f);
     }
     else if ( m_actionType == MT_MESSAGE )  // message?
     {
-        m_object->SetAngleX(2,  20.0f*Math::PI/180.0f);
-        m_object->SetAngleX(3, -20.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(2,  20.0f*Math::PI/180.0f);
+        m_object->SetPartRotationX(3, -20.0f*Math::PI/180.0f);
     }
     else
     {
-        m_object->SetAngleX(2, 0.0f);
-        m_object->SetAngleX(3, 0.0f);
+        m_object->SetPartRotationX(2, 0.0f);
+        m_object->SetPartRotationX(3, 0.0f);
     }
 
     mat = m_object->GetWorldMatrix(0);  // must be done every time!
@@ -717,7 +701,7 @@ bool CMotionToto::EventFrame(const Event &event)
         if ( m_bDisplayInfo )  sheet = Gfx::SH_FRONT;
         else                   sheet = Gfx::SH_WORLD;
 
-        pos = m_object->GetPosition(0);
+        pos = m_object->GetPosition();
         if ( !m_bDisplayInfo             &&
              pos.y < m_water->GetLevel() )  // underwater?
         {
@@ -783,7 +767,7 @@ bool CMotionToto::EventFrame(const Event &event)
             m_particle->CreateParticle(pos, speed, dim, type, 0.5f+Math::Rand()*0.5f, 0.0f, 1.0f, Gfx::SH_INTERFACE);
         }
 
-//?     if ( m_bDisplayInfo && m_main->GetGlint() )
+//?     if ( m_bDisplayInfo && m_main->GetInterfaceGlint() )
         if ( false )
         {
             pos.x = (Math::Rand()-0.5f)*1.4f;
@@ -815,7 +799,7 @@ bool CMotionToto::EventFrame(const Event &event)
     // Move the sound.
     if ( m_soundChannel != -1 )
     {
-        if ( !m_sound->Position(m_soundChannel, m_object->GetPosition(0)) )
+        if ( !m_sound->Position(m_soundChannel, m_object->GetPosition()) )
         {
             m_soundChannel = -1;
         }
@@ -829,7 +813,7 @@ bool CMotionToto::EventFrame(const Event &event)
 
 Error CMotionToto::SetAction(int action, float time)
 {
-    Sound   sound;
+    SoundType   sound;
 
     CMotion::SetAction(action, time);
 
@@ -843,7 +827,7 @@ Error CMotionToto::SetAction(int action, float time)
 
     if ( sound != SOUND_CLICK )
     {
-        m_soundChannel = m_sound->Play(sound, m_object->GetPosition(0));
+        m_soundChannel = m_sound->Play(sound, m_object->GetPosition());
     }
 
     return ERR_OK;
@@ -855,4 +839,3 @@ void CMotionToto::SetLinkType(ObjectType type)
 {
     m_type = type;
 }
-
