@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// taskadvance.h
-
 #pragma once
 
 
@@ -28,27 +26,24 @@
 
 
 
-class CTaskAdvance : public CTask
+class CTaskAdvance : public CForegroundTask
 {
 public:
-    CTaskAdvance(CObject* object);
+    CTaskAdvance(COldObject* object);
     ~CTaskAdvance();
 
-    bool    EventProcess(const Event &event);
+    bool    EventProcess(const Event &event) override;
 
     Error   Start(float length);
-    Error   IsEnded();
+    Error   IsEnded() override;
 
 protected:
-
-protected:
-    float       m_totalLength;
-    float       m_advanceLength;
-    float       m_direction;
-    float       m_timeLimit;
+    float       m_totalLength = 0.0f;
+    float       m_advanceLength = 0.0f;
+    float       m_direction = 0.0f;
+    float       m_timeLimit = 0.0f;
     Math::Vector    m_startPos;
-    float       m_lastDist;
-    float       m_fixTime;
-    bool        m_bError;
+    float       m_lastDist = 0.0f;
+    float       m_fixTime = 0.0f;
+    bool        m_bError = false;
 };
-
