@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,24 +22,34 @@
 
 #include "graphics/core/color.h"
 #include "graphics/core/light.h"
+
 #include "graphics/engine/lightman.h"
 #include "graphics/engine/terrain.h"
 #include "graphics/engine/water.h"
 
+#include "level/robotmain.h"
+
 #include "math/geometry.h"
 
+#include "object/object_manager.h"
+#include "object/old_object.h"
+
 #include "object/auto/auto.h"
+
+#include "object/interface/carrier_object.h"
+#include "object/interface/transportable_object.h"
+
 #include "object/motion/motionhuman.h"
-#include "object/objman.h"
-#include "object/robotmain.h"
 
 #include "physics/physics.h"
 
+#include "sound/sound.h"
+
 #include <string.h>
 
 // Object's constructor.
 
-CTaskBuild::CTaskBuild(CObject* object) : CTask(object)
+CTaskBuild::CTaskBuild(COldObject* object) : CForegroundTask(object)
 {
     int     i;
 
@@ -78,14 +88,13 @@ CTaskBuild::~CTaskBuild()
 
 bool CTaskBuild::CreateBuilding(Math::Vector pos, float angle)
 {
-    m_building = new CObject();
-    if ( !m_building->CreateBuilding(pos, angle, 0.0f, m_type, 0.0f) )
-    {
-        delete m_building;
-        m_building = 0;
-        return false;
-    }
-    m_building->UpdateMapping();
+    ObjectCreateParams params;
+    params.pos = pos;
+    params.angle = angle;
+    params.type = m_type;
+    params.power = 0.0f;
+    params.team = m_object->GetTeam();
+    m_building = CObjectManager::GetInstancePointer()->CreateObject(params);
     m_building->SetLock(true);  // not yet usable
 
     if ( m_type == OBJECT_DERRICK  )  m_buildingHeight = 35.0f;
@@ -105,9 +114,9 @@ bool CTaskBuild::CreateBuilding(Math::Vector pos, float angle)
     if ( m_type == OBJECT_HUSTON   )  m_buildingHeight = 45.0f;
     m_buildingHeight *= 0.25f;
 
-    m_buildingPos = m_building->GetPosition(0);
+    m_buildingPos = m_building->GetPosition();
     m_buildingPos.y -= m_buildingHeight;
-    m_building->SetPosition(0, m_buildingPos);
+    m_building->SetPosition(m_buildingPos);
     return true;
 }
 
@@ -123,7 +132,7 @@ void CTaskBuild::CreateLight()
 
     if ( !m_engine->GetLightMode() )  return;
 
-    center = m_metal->GetPosition(0);
+    center = m_metal->GetPosition();
 
     angle = 0;
     for ( i=0 ; i<TBMAXLIGHT ; i++ )
@@ -206,9 +215,25 @@ bool CTaskBuild::EventProcess(const Event &event)
 
     m_progress += event.rTime*m_speed;  // other advance
 
+    // Cancel if the player tries to move
+    float axeX = event.motionInput.x;
+    float axeY = event.motionInput.y;
+    float axeZ = event.motionInput.z;
+    if ( m_object->GetType() == OBJECT_HUMAN && m_object->GetSelect() &&
+         (axeX != 0.0f || axeY != 0.0f || axeZ != 0.0f) &&
+         (m_phase == TBP_TURN || m_phase == TBP_MOVE || m_phase == TBP_TAKE || m_phase == TBP_PREP) )
+    {
+        m_phase = TBP_STOP;
+    }
+
+    if ( m_phase == TBP_STOP )  // stops?
+    {
+        return true;
+    }
+
     if ( m_phase == TBP_TURN )  // preliminary rotation?
     {
-        a = m_object->GetAngleY(0);
+        a = m_object->GetRotationY();
         g = m_angleY;
         cirSpeed = Math::Direction(a, g)*1.0f;
         if ( cirSpeed >  1.0f )  cirSpeed =  1.0f;
@@ -220,7 +245,7 @@ bool CTaskBuild::EventProcess(const Event &event)
 
     if ( m_phase == TBP_MOVE )  // preliminary forward/backward?
     {
-        dist = Math::Distance(m_object->GetPosition(0), m_metal->GetPosition(0));
+        dist = Math::Distance(m_object->GetPosition(), m_metal->GetPosition());
         linSpeed = 0.0f;
         if ( dist > 30.0f )  linSpeed =  1.0f;
         if ( dist < 30.0f )  linSpeed = -1.0f;
@@ -253,19 +278,22 @@ bool CTaskBuild::EventProcess(const Event &event)
     {
         m_bBuild = true;
 
-        pos = m_metal->GetPosition(0);
-        a   = m_object->GetAngleY(0);
+        pos = m_metal->GetPosition();
+        a   = m_object->GetRotationY();
         if ( !CreateBuilding(pos, a+Math::PI) )
         {
             m_metal->SetLock(false);  // usable again
             m_motion->SetAction(-1);
-            m_object->SetObjectParent(14, 0);
-            m_object->SetPosition(14, Math::Vector(-1.5f, 0.3f, -1.35f));
-            m_object->SetAngleZ(14, Math::PI);
+            if (m_object->GetType() == OBJECT_HUMAN)
+            {
+                m_object->SetObjectParent(14, 0);
+                m_object->SetPartPosition(14, Math::Vector(-1.5f, 0.3f, -1.35f));
+                m_object->SetPartRotationZ(14, Math::PI);
+            }
             m_camera->FlushEffect();
             Abort();
             m_bError = true;
-            m_main->DisplayError(ERR_TOOMANY, m_object->GetPosition(0));
+            m_main->DisplayError(ERR_TOOMANY, m_object->GetPosition());
             return false;
         }
         CreateLight();
@@ -273,10 +301,10 @@ bool CTaskBuild::EventProcess(const Event &event)
 
     pos = m_buildingPos;
     pos.y += m_buildingHeight*m_progress;
-    m_building->SetPosition(0, pos);  // the building rises
+    m_building->SetPosition(pos);  // the building rises
 
-    m_building->SetZoom(0, m_progress*0.75f+0.25f);
-    m_metal->SetZoom(0, 1.0f-m_progress);
+    m_building->SetScale(m_progress*0.75f+0.25f);
+    m_metal->SetScale(1.0f-m_progress);
 
     a = (2.0f-2.0f*m_progress);
     if ( a > 1.0f )  a = 1.0f;
@@ -294,7 +322,7 @@ bool CTaskBuild::EventProcess(const Event &event)
     {
         m_lastParticle = m_time;
 
-        pos = m_metal->GetPosition(0);
+        pos = m_metal->GetPosition();
         speed.x = (Math::Rand()-0.5f)*20.0f;
         speed.z = (Math::Rand()-0.5f)*20.0f;
         speed.y = Math::Rand()*10.0f;
@@ -303,9 +331,25 @@ bool CTaskBuild::EventProcess(const Event &event)
         m_particle->CreateParticle(pos, speed, dim, Gfx::PARTIFIRE);
 
         pos = Math::Vector(0.0f, 0.5f, 0.0f);
-        mat = m_object->GetWorldMatrix(14);
+        switch(m_object->GetType())
+        {
+            case OBJECT_HUMAN:
+                mat = m_object->GetWorldMatrix(14);
+                break;
+
+            case OBJECT_MOBILEfa:
+            case OBJECT_MOBILEta:
+            case OBJECT_MOBILEwa:
+            case OBJECT_MOBILEia:
+                mat = m_object->GetWorldMatrix(3);
+                break;
+
+            default:
+                mat = m_object->GetWorldMatrix(0);
+                break;
+        }
         pos = Transform(*mat, pos);
-        speed = m_metal->GetPosition(0);
+        speed = m_metal->GetPosition();
         speed.x += (Math::Rand()-0.5f)*5.0f;
         speed.z += (Math::Rand()-0.5f)*5.0f;
         speed -= pos;
@@ -315,7 +359,7 @@ bool CTaskBuild::EventProcess(const Event &event)
 
         if ( Math::Rand() < 0.3f )
         {
-            m_sound->Play(SOUND_BUILD, m_object->GetPosition(0), 0.5f, 1.0f*Math::Rand()*1.5f);
+            m_sound->Play(SOUND_BUILD, m_object->GetPosition(), 0.5f, 1.0f*Math::Rand()*1.5f);
         }
     }
 
@@ -335,13 +379,13 @@ Error CTaskBuild::Start(ObjectType type)
     m_lastParticle = 0.0f;
     m_progress = 0.0f;
 
-    iAngle = m_object->GetAngleY(0);
+    iAngle = m_object->GetRotationY();
     iAngle = Math::NormAngle(iAngle);  // 0..2*Math::PI
     oAngle = iAngle;
 
     m_bError = true;  // operation impossible
 
-    pos = m_object->GetPosition(0);
+    pos = m_object->GetPosition();
     if ( pos.y < m_water->GetLevel() )  return ERR_BUILD_WATER;
 
     if ( !m_physics->GetLand() )  return ERR_BUILD_FLY;
@@ -350,10 +394,10 @@ Error CTaskBuild::Start(ObjectType type)
     if ( speed.x != 0.0f ||
          speed.z != 0.0f )  return ERR_BUILD_MOTOR;
 
-    if ( m_object->GetFret() != 0 )  return ERR_MANIP_BUSY;
+    if (IsObjectCarryingCargo(m_object))  return ERR_MANIP_BUSY;
 
     m_metal = SearchMetalObject(oAngle, 2.0f, 100.0f, Math::PI*0.25f, err);
-    if ( err == ERR_BUILD_METALNEAR && m_metal != 0 )
+    if ( err == ERR_BUILD_METALNEAR && m_metal != nullptr )
     {
         err = FlatFloor();
         if ( err != ERR_OK )  return err;
@@ -370,9 +414,9 @@ Error CTaskBuild::Start(ObjectType type)
     m_phase = TBP_TURN;  // rotation necessary preliminary
     m_angleY = oAngle;  // angle was reached
 
-    pv = m_object->GetPosition(0);
+    pv = m_object->GetPosition();
     pv.y += 8.3f;
-    pm = m_metal->GetPosition(0);
+    pm = m_metal->GetPosition();
     m_angleZ = Math::RotateAngle(Math::DistanceProjected(pv, pm), fabs(pv.y-pm.y));
 
     m_physics->SetFreeze(true);  // it does not move
@@ -394,14 +438,14 @@ Error CTaskBuild::IsEnded()
 
     if ( m_phase == TBP_TURN )  // preliminary rotation?
     {
-        angle = m_object->GetAngleY(0);
+        angle = m_object->GetRotationY();
         angle = Math::NormAngle(angle);  // 0..2*Math::PI
 
         if ( Math::TestAngle(angle, m_angleY-Math::PI*0.01f, m_angleY+Math::PI*0.01f) )
         {
             m_physics->SetMotorSpeedZ(0.0f);
 
-            dist = Math::Distance(m_object->GetPosition(0), m_metal->GetPosition(0));
+            dist = Math::Distance(m_object->GetPosition(), m_metal->GetPosition());
             if ( dist > 30.0f )
             {
                 time = m_physics->GetLinTimeLength(dist-30.0f, 1.0f);
@@ -420,7 +464,7 @@ Error CTaskBuild::IsEnded()
 
     if ( m_phase == TBP_MOVE )  // preliminary forward/backward?
     {
-        dist = Math::Distance(m_object->GetPosition(0), m_metal->GetPosition(0));
+        dist = Math::Distance(m_object->GetPosition(), m_metal->GetPosition());
 
         if ( dist >= 25.0f && dist <= 35.0f )
         {
@@ -448,9 +492,12 @@ Error CTaskBuild::IsEnded()
         if ( m_progress < 1.0f )  return ERR_CONTINUE;
 
         m_motion->SetAction(MHS_FIRE);  // shooting position
-        m_object->SetObjectParent(14, 4);
-        m_object->SetPosition(14, Math::Vector(0.6f, 0.1f, 0.3f));
-        m_object->SetAngleZ(14, 0.0f);
+        if (m_object->GetType() == OBJECT_HUMAN)
+        {
+            m_object->SetObjectParent(14, 4);
+            m_object->SetPartPosition(14, Math::Vector(0.6f, 0.1f, 0.3f));
+            m_object->SetPartRotationZ(14, 0.0f);
+        }
 
         m_phase = TBP_PREP;
         m_speed = 1.0f/1.0f;
@@ -461,12 +508,12 @@ Error CTaskBuild::IsEnded()
     {
         if ( m_progress < 1.0f )  return ERR_CONTINUE;
 
-        m_soundChannel = m_sound->Play(SOUND_TREMBLE, m_object->GetPosition(0), 0.0f, 1.0f, true);
+        m_soundChannel = m_sound->Play(SOUND_TREMBLE, m_object->GetPosition(), 0.0f, 1.0f, true);
         m_sound->AddEnvelope(m_soundChannel, 0.7f, 1.0f, 1.0f, SOPER_CONTINUE);
         m_sound->AddEnvelope(m_soundChannel, 0.7f, 1.5f, 7.0f, SOPER_CONTINUE);
         m_sound->AddEnvelope(m_soundChannel, 0.0f, 1.5f, 2.0f, SOPER_STOP);
 
-        m_camera->StartEffect(Gfx::CAM_EFFECT_VIBRATION, m_metal->GetPosition(0), 1.0f);
+        m_camera->StartEffect(Gfx::CAM_EFFECT_VIBRATION, m_metal->GetPosition(), 1.0f);
 
         m_phase = TBP_BUILD;
         m_speed = 1.0f/10.f;  // duration of 10s
@@ -477,20 +524,19 @@ Error CTaskBuild::IsEnded()
     {
         if ( m_progress < 1.0f )  return ERR_CONTINUE;
 
-        DeleteMark(m_metal->GetPosition(0), 20.0f);
+        DeleteMark(m_metal->GetPosition(), 20.0f);
 
-        m_metal->DeleteObject();  // removes the metal
-        delete m_metal;
-        m_metal = 0;
+        CObjectManager::GetInstancePointer()->DeleteObject(m_metal);
+        m_metal = nullptr;
 
-        m_building->SetZoom(0, 1.0f);
+        m_building->SetScale(1.0f);
         m_building->SetCirVibration(Math::Vector(0.0f, 0.0f, 0.0f));
         m_building->SetLock(false);  // building usable
         m_main->CreateShortcuts();
         m_main->DisplayError(INFO_BUILD, m_buildingPos, 10.0f, 50.0f);
 
         automat = m_building->GetAuto();
-        if ( automat != 0 )
+        if ( automat != nullptr )
         {
             automat->Init();
         }
@@ -506,9 +552,12 @@ Error CTaskBuild::IsEnded()
         if ( m_progress < 1.0f )  return ERR_CONTINUE;
 
         m_motion->SetAction(-1);
-        m_object->SetObjectParent(14, 0);
-        m_object->SetPosition(14, Math::Vector(-1.5f, 0.3f, -1.35f));
-        m_object->SetAngleZ(14, Math::PI);
+        if (m_object->GetType() == OBJECT_HUMAN)
+        {
+            m_object->SetObjectParent(14, 0);
+            m_object->SetPartPosition(14, Math::Vector(-1.5f, 0.3f, -1.35f));
+            m_object->SetPartRotationZ(14, Math::PI);
+        }
 
         if ( m_type == OBJECT_FACTORY  ||
              m_type == OBJECT_RESEARCH ||
@@ -528,6 +577,31 @@ Error CTaskBuild::IsEnded()
         m_physics->SetMotorSpeedX(0.0f);
     }
 
+    if ( m_phase == TBP_STOP ) // canceled?
+    {
+        if ( m_progress < 1.0f && m_motion->GetAction() == MHS_GUN )  return ERR_CONTINUE;
+
+        if ( m_motion->GetAction() == MHS_FIRE )
+        {
+            m_motion->SetAction(MHS_GUN);
+            m_speed = 1.0f/1.0f;
+            m_progress = 0.0f;
+            return ERR_CONTINUE;
+        }
+
+        m_motion->SetAction(-1);
+
+        // Place gun back
+        m_object->SetObjectParent(14, 0);
+        m_object->SetPartPosition(14, Math::Vector(-1.5f, 0.3f, -1.35f));
+        m_object->SetPartRotationZ(14, Math::PI);
+
+        m_physics->SetMotorSpeedX(0.0f);
+        m_physics->SetMotorSpeedZ(0.0f);
+
+        m_metal->SetLock(false); // make titanium usable
+    }
+
     Abort();
     return ERR_STOP;
 }
@@ -554,12 +628,10 @@ bool CTaskBuild::Abort()
 
 Error CTaskBuild::FlatFloor()
 {
-    CObject     *pObj;
     ObjectType  type;
-    Math::Vector    center, pos, oPos, bPos;
+    Math::Vector    center, pos, bPos;
     Math::Point     c, p;
-    float       radius, max, oRadius, bRadius = 0.0f, angle, dist;
-    int         j;
+    float       radius, max, bRadius = 0.0f, angle, dist;
     bool        bLittleFlat, bBase;
 
     radius = 0.0f;
@@ -577,9 +649,9 @@ Error CTaskBuild::FlatFloor()
     if ( m_type == OBJECT_PARA     )  radius = 20.0f;
     if ( m_type == OBJECT_INFO     )  radius =  5.0f;
     if ( m_type == OBJECT_DESTROYER)  radius = 20.0f;
-    //if ( radius == 0.0f )  return ERR_GENERIC;
+    //if ( radius == 0.0f )  return ERR_UNKNOWN;
 
-    center = m_metal->GetPosition(0);
+    center = m_metal->GetPosition();
     angle = m_terrain->GetFineSlope(center);
     bLittleFlat = ( angle < Gfx::TERRAIN_FLATLIMIT);
 
@@ -595,33 +667,33 @@ Error CTaskBuild::FlatFloor()
 
     max = 100000.0f;
     bBase = false;
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* pObj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-
-        if ( !pObj->GetActif() )  continue;  // inactive?
-        if ( pObj->GetTruck() != 0 )  continue;  // object transported?
+        if ( !pObj->GetActive() )  continue;  // inactive?
+        if (IsObjectBeingTransported(pObj))  continue;
         if ( pObj == m_metal )  continue;
         if ( pObj == m_object )  continue;
 
         type = pObj->GetType();
         if ( type == OBJECT_BASE )
         {
-            oPos = pObj->GetPosition(0);
+            Math::Vector oPos = pObj->GetPosition();
             dist = Math::Distance(center, oPos)-80.0f;
             if ( dist < max )
             {
                 max = dist;
                 bPos = oPos;
-                bRadius = oRadius;
+                bRadius = 0.0f;
                 bBase = true;
             }
         }
         else
         {
-            j = 0;
-            while ( pObj->GetCrashSphere(j++, oPos, oRadius) )
+            for (const auto& crashSphere : pObj->GetAllCrashSpheres())
             {
+                Math::Vector oPos = crashSphere.sphere.pos;
+                float oRadius = crashSphere.sphere.radius;
+
                 dist = Math::Distance(center, oPos)-oRadius;
                 if ( dist < max )
                 {
@@ -642,12 +714,10 @@ Error CTaskBuild::FlatFloor()
     }
 
     max = 100000.0f;
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* pObj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-
-        if ( !pObj->GetActif() )  continue;  // inactive?
-        if ( pObj->GetTruck() != 0 )  continue;  // object transported?
+        if ( !pObj->GetActive() )  continue;  // inactive?
+        if (IsObjectBeingTransported(pObj))  continue;
         if ( pObj == m_metal )  continue;
         if ( pObj == m_object )  continue;
 
@@ -671,9 +741,11 @@ Error CTaskBuild::FlatFloor()
              type == OBJECT_SAFE     ||
              type == OBJECT_HUSTON   )  // building?
         {
-            j = 0;
-            while ( pObj->GetCrashSphere(j++, oPos, oRadius) )
+            for (const auto& crashSphere : pObj->GetAllCrashSpheres())
             {
+                Math::Vector oPos = crashSphere.sphere.pos;
+                float oRadius = crashSphere.sphere.radius;
+
                 dist = Math::Distance(center, oPos)-oRadius;
                 if ( dist < max )
                 {
@@ -699,32 +771,30 @@ Error CTaskBuild::FlatFloor()
 CObject* CTaskBuild::SearchMetalObject(float &angle, float dMin, float dMax,
                                        float aLimit, Error &err)
 {
-    CObject     *pObj, *pBest;
+    CObject     *pBest;
     Math::Vector    iPos, oPos;
     ObjectType  type;
     float       min, iAngle, a, aa, aBest, distance, magic;
     bool        bMetal;
 
-    iPos   = m_object->GetPosition(0);
-    iAngle = m_object->GetAngleY(0);
+    iPos   = m_object->GetPosition();
+    iAngle = m_object->GetRotationY();
     iAngle = Math::NormAngle(iAngle);  // 0..2*Math::PI
 
     min = 1000000.0f;
-    pBest = 0;
+    pBest = nullptr;
     bMetal = false;
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* pObj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-
-        if ( !pObj->GetActif() )  continue;  // objet inactive?
-        if ( pObj->GetTruck() != 0 )  continue;  // object transported?
+        if ( !pObj->GetActive() )  continue;  // objet inactive?
+        if (IsObjectBeingTransported(pObj))  continue;
 
         type = pObj->GetType();
         if ( type != OBJECT_METAL )  continue;
 
         bMetal = true;  // metal exists
 
-        oPos = pObj->GetPosition(0);
+        oPos = pObj->GetPosition();
         distance = Math::Distance(oPos, iPos);
         a = Math::RotateAngle(oPos.x-iPos.x, iPos.z-oPos.z);  // CW!
 
@@ -749,7 +819,7 @@ CObject* CTaskBuild::SearchMetalObject(float &angle, float dMin, float dMax,
         }
     }
 
-    if ( pBest == 0 )
+    if ( pBest == nullptr )
     {
         if ( bMetal )  err = ERR_BUILD_METALAWAY;  // too far
         else           err = ERR_BUILD_METALINEX;  // non-existent
@@ -766,16 +836,11 @@ CObject* CTaskBuild::SearchMetalObject(float &angle, float dMin, float dMax,
 
 void CTaskBuild::DeleteMark(Math::Vector pos, float radius)
 {
-    CObject*    pObj;
-    Math::Vector    oPos;
-    ObjectType  type;
-    float       distance;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
-    {
-        pObj = it.second;
+    std::vector<CObject*> objectsToDelete;
 
-        type = pObj->GetType();
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
+    {
+        ObjectType type = obj->GetType();
         if ( type != OBJECT_MARKSTONE   &&
              type != OBJECT_MARKURANIUM &&
              type != OBJECT_MARKKEYa    &&
@@ -784,13 +849,16 @@ void CTaskBuild::DeleteMark(Math::Vector pos, float radius)
              type != OBJECT_MARKKEYd    &&
              type != OBJECT_MARKPOWER   )  continue;
 
-        oPos = pObj->GetPosition(0);
-        distance = Math::Distance(oPos, pos);
+        Math::Vector oPos = obj->GetPosition();
+        float distance = Math::Distance(oPos, pos);
         if ( distance <= radius )
         {
-            pObj->DeleteObject();  // removes the mark
-            delete pObj;
+            objectsToDelete.push_back(obj);
         }
     }
-}
 
+    for (CObject* obj : objectsToDelete)
+    {
+        CObjectManager::GetInstancePointer()->DeleteObject(obj);
+    }
+}
