@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,29 +17,27 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// tasksearch.h
-
 #pragma once
 
 
 #include "object/task/task.h"
-#include "object/object.h"
 
 
-class CTaskDeleteMark : public CTask
+class CTaskDeleteMark : public CForegroundTask
 {
 public:
-    CTaskDeleteMark(CObject* object);
+    CTaskDeleteMark(COldObject* object);
     ~CTaskDeleteMark();
 
-    bool        EventProcess(const Event &event);
+    bool        EventProcess(const Event &event) override;
 
     Error       Start();
-    Error       IsEnded();
-    bool        Abort();
+    Error       IsEnded() override;
+    bool        Abort() override;
 
 protected:
     void    DeleteMark();
+
 protected:
-    bool    m_bExecuted;
-};
\ No newline at end of file
+    bool    m_bExecuted = false;
+};
