@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// taskfire.h
-
 #pragma once
 
 
@@ -26,30 +24,28 @@
 
 
 
-class CTaskFire : public CTask
+class CTaskFire : public CForegroundTask
 {
 public:
-    CTaskFire(CObject* object);
+    CTaskFire(COldObject* object);
     ~CTaskFire();
 
-    bool        EventProcess(const Event &event);
+    bool        EventProcess(const Event &event) override;
 
     Error       Start(float delay);
-    Error       IsEnded();
-    bool        Abort();
+    Error       IsEnded() override;
+    bool        Abort() override;
+    bool        IsPilot() override;
 
 protected:
-
-protected:
-    float       m_delay;
-    float       m_progress;
-    bool        m_bError;
-    bool        m_bRay;
-    bool        m_bOrganic;
-    float       m_time;
-    float       m_speed;
-    float       m_lastParticle;
-    float       m_lastSound;
-    int     m_soundChannel;
+    float       m_delay = 0.0f;
+    float       m_progress = 0.0f;
+    bool        m_bError = false;
+    bool        m_bRay = false;
+    bool        m_bOrganic = false;
+    float       m_time = 0.0f;
+    float       m_speed = 0.0f;
+    float       m_lastParticle = 0.0f;
+    float       m_lastSound = 0.0f;
+    int     m_soundChannel = 0;
 };
-
