@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,18 +18,23 @@
  */
 
 
-#include <stdio.h>
-
 #include "object/task/taskgoto.h"
 
+#include "common/global.h"
 #include "common/event.h"
+#include "common/make_unique.h"
 
 #include "graphics/engine/terrain.h"
 #include "graphics/engine/water.h"
 
 #include "math/geometry.h"
 
-#include "object/objman.h"
+#include "object/object_manager.h"
+#include "object/old_object.h"
+
+#include "object/interface/transportable_object.h"
+
+#include "object/subclass/base_alien.h"
 
 #include "physics/physics.h"
 
@@ -50,9 +55,9 @@ const float SAFETY_MARGIN   = 0.5f;     // Smallest distance between two objects
 
 // Object's constructor.
 
-CTaskGoto::CTaskGoto(CObject* object) : CTask(object)
+CTaskGoto::CTaskGoto(COldObject* object) : CForegroundTask(object)
 {
-    m_bmArray = 0;
+    m_bmArray = nullptr;
 }
 
 // Object's destructor.
@@ -76,7 +81,8 @@ bool CTaskGoto::EventProcess(const Event &event)
     if ( event.type != EVENT_FRAME )  return true;
 
     // Momentarily stationary object (ant on the back)?
-    if ( m_object->GetFixed() )
+    CBaseAlien* alien = dynamic_cast<CBaseAlien*>(m_object);
+    if ( alien != nullptr && alien->GetFixed() )
     {
         m_physics->SetMotorSpeedX(0.0f);  // stops the advance
         m_physics->SetMotorSpeedZ(0.0f);  // stops the rotation
@@ -94,7 +100,7 @@ bool CTaskGoto::EventProcess(const Event &event)
     {
         m_leakTime += event.rTime;
 
-        pos = m_object->GetPosition(0);
+        pos = m_object->GetPosition();
 
         rot.x = m_leakPos.x-pos.x;
         rot.y = m_leakPos.z-pos.z;
@@ -105,7 +111,7 @@ bool CTaskGoto::EventProcess(const Event &event)
             rot.y /= dist;
         }
 
-        a = m_object->GetAngleY(0);
+        a = m_object->GetRotationY();
         g = Math::RotateAngle(rot.x, -rot.y);  // CW !
         a = Math::Direction(a, g)*1.0f;
         cirSpeed = a;
@@ -139,12 +145,12 @@ bool CTaskGoto::EventProcess(const Event &event)
         if ( m_bmStep == 0 )
         {
             // Frees the area around the departure.
-            BitmapClearCircle(m_object->GetPosition(0), BM_DIM_STEP*1.8f);
+            BitmapClearCircle(m_object->GetPosition(), BM_DIM_STEP*1.8f);
         }
 
-        pos = m_object->GetPosition(0);
+        pos = m_object->GetPosition();
 
-        if ( m_bmFretObject == 0 )
+        if ( m_bmCargoObject == nullptr )
         {
             goal = m_goal;
             dist = 0.0f;
@@ -153,7 +159,7 @@ bool CTaskGoto::EventProcess(const Event &event)
         {
             goal = m_goalObject;
             dist = TAKE_DIST+2.0f;
-            if ( m_bmFretObject->GetType() == OBJECT_BASE )  dist = 12.0f;
+            if ( m_bmCargoObject->GetType() == OBJECT_BASE )  dist = 12.0f;
         }
 
         ret = BeamSearch(pos, goal, dist);
@@ -162,7 +168,7 @@ bool CTaskGoto::EventProcess(const Event &event)
             if ( m_physics->GetLand() )  m_phase = TGP_BEAMWCOLD;
             else                         m_phase = TGP_BEAMGOTO;
             m_bmIndex = 0;
-            m_bmWatchDogPos = m_object->GetPosition(0);
+            m_bmWatchDogPos = m_object->GetPosition();
             m_bmWatchDogTime = 0.0f;
         }
         if ( ret == ERR_GOTO_IMPOSSIBLE || ret == ERR_GOTO_ITER )
@@ -191,9 +197,9 @@ bool CTaskGoto::EventProcess(const Event &event)
             m_physics->SetCollision(false);  // there's more
         }
 
-        pos = m_object->GetPosition(0);
+        pos = m_object->GetPosition();
 
-        if ( m_physics->GetType() == TYPE_FLYING && m_altitude == 0.0f )
+        if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude == 0.0f )
         {
             if ( m_physics->GetLand() )
             {
@@ -205,7 +211,7 @@ bool CTaskGoto::EventProcess(const Event &event)
             }
         }
 
-        if ( m_physics->GetType() == TYPE_FLYING && m_altitude > 0.0f )
+        if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude > 0.0f )
         {
             goal = m_bmPoints[m_bmIndex];
             goal.y = pos.y;
@@ -240,7 +246,7 @@ bool CTaskGoto::EventProcess(const Event &event)
         rot.x /= dist;
         rot.y /= dist;
 
-        a = m_object->GetAngleY(0);
+        a = m_object->GetRotationY();
         g = Math::RotateAngle(rot.x, -rot.y);  // CW !
         cirSpeed = Math::Direction(a, g)*2.0f;
         if ( cirSpeed >  1.0f )  cirSpeed =  1.0f;
@@ -313,7 +319,7 @@ bool CTaskGoto::EventProcess(const Event &event)
             }
         }
 
-        pos = m_object->GetPosition(0);
+        pos = m_object->GetPosition();
 
         if ( m_altitude > 0.0f )
         {
@@ -332,7 +338,7 @@ bool CTaskGoto::EventProcess(const Event &event)
 
         rot.x = m_goal.x-pos.x;
         rot.y = m_goal.z-pos.z;
-        a = m_object->GetAngleY(0);
+        a = m_object->GetRotationY();
         g = Math::RotateAngle(rot.x, -rot.y);  // CW !
         cirSpeed = Math::Direction(a, g)*1.0f;
         if ( cirSpeed >  1.0f )  cirSpeed =  1.0f;
@@ -344,16 +350,16 @@ bool CTaskGoto::EventProcess(const Event &event)
     }
 
     if ( m_phase != TGP_TURN                 &&
-         m_physics->GetType() == TYPE_FLYING &&
+         m_object->Implements(ObjectInterfaceType::Flying) &&
          m_altitude > 0.0f                   )
     {
-        pos = m_object->GetPosition(0);
+        pos = m_object->GetPosition();
         dist = Math::DistanceProjected(m_goal, pos);
         factor = (dist-20.0f)/20.0f;
         if ( factor < 0.0f )  factor = 0.0f;
         if ( factor > 1.0f )  factor = 1.0f;
 
-        h = m_terrain->GetHeightToFloor(m_object->GetPosition(0), true, true);
+        h = m_terrain->GetHeightToFloor(m_object->GetPosition(), true, true);
         linSpeed = 0.0f;
         if ( h < (m_altitude-0.5f)*factor && factor == 1.0f )
         {
@@ -379,24 +385,7 @@ bool CTaskGoto::EventProcess(const Event &event)
             return true;
         }
 
-#if 0
-        pos = m_object->GetPosition(0);
-        a = m_object->GetAngleY(0);
-        g = Math::RotateAngle(m_goal.x-pos.x, pos.z-m_goal.z);  // CW !
-        cirSpeed = Math::Direction(a, g)*1.0f;
-        if ( cirSpeed >  1.0f )  cirSpeed =  1.0f;
-        if ( cirSpeed < -1.0f )  cirSpeed = -1.0f;
-
-        dist = Math::DistanceProjected(m_goal, pos);
-        linSpeed = dist/(m_physics->GetLinStopLength()*1.5f);
-        if ( linSpeed >  1.0f )  linSpeed =  1.0f;
-
-        if ( dist < 20.0f && fabs(cirSpeed) >= 0.5f )
-        {
-            linSpeed = 0.0f;  // turns first, then advance
-        }
-#else
-        pos = m_object->GetPosition(0);
+        pos = m_object->GetPosition();
 
         rot.x = m_goal.x-pos.x;
         rot.y = m_goal.z-pos.z;
@@ -408,10 +397,10 @@ bool CTaskGoto::EventProcess(const Event &event)
         rot.x += repulse.x*2.0f;
         rot.y += repulse.y*2.0f;
 
-        a = m_object->GetAngleY(0);
+        a = m_object->GetRotationY();
         g = Math::RotateAngle(rot.x, -rot.y);  // CW !
         cirSpeed = Math::Direction(a, g)*1.0f;
-//?     if ( m_physics->GetType() == TYPE_FLYING &&
+//?     if ( m_object->Implements(ObjectInterfaceType::Flying) &&
 //?          m_physics->GetLand()                )  // flying on the ground?
 //?     {
 //?         cirSpeed *= 4.0f;  // more fishing
@@ -421,7 +410,7 @@ bool CTaskGoto::EventProcess(const Event &event)
 
         dist = Math::DistanceProjected(m_goal, pos);
         linSpeed = dist/(m_physics->GetLinStopLength()*1.5f);
-//?     if ( m_physics->GetType() == TYPE_FLYING &&
+//?     if ( m_object->Implements(ObjectInterfaceType::Flying) &&
 //?          m_physics->GetLand()                )  // flying on the ground?
 //?     {
 //?         linSpeed *= 8.0f;  // more fishing
@@ -434,7 +423,6 @@ bool CTaskGoto::EventProcess(const Event &event)
         {
             linSpeed = 0.0f;  // turns first, then advance
         }
-#endif
 
         m_physics->SetMotorSpeedZ(cirSpeed);  // turns left / right
         m_physics->SetMotorSpeedX(linSpeed);  // advance
@@ -444,7 +432,7 @@ bool CTaskGoto::EventProcess(const Event &event)
          m_phase == TGP_CRTURN ||  // turns after collision?
          m_phase == TGP_CLTURN )   // turns after collision?
     {
-        a = m_object->GetAngleY(0);
+        a = m_object->GetRotationY();
         g = m_angle;
         cirSpeed = Math::Direction(a, g)*1.0f;
         if ( cirSpeed >  1.0f )  cirSpeed =  1.0f;
@@ -499,20 +487,13 @@ bool CTaskGoto::EventProcess(const Event &event)
 
 CObject* CTaskGoto::WormSearch(Math::Vector &impact)
 {
-    CObject*    pObj;
-    CObject*    pBest = 0;
-    Math::Vector    iPos, oPos;
-    ObjectType  oType;
-    float       distance, min, radius;
+    Math::Vector iPos = m_object->GetPosition();
+    float min = 1000000.0f;
 
-    iPos = m_object->GetPosition(0);
-    min = 1000000.0f;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    CObject* best = nullptr;
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-
-        oType = pObj->GetType();
+        ObjectType oType = obj->GetType();
         if ( oType != OBJECT_MOBILEfa &&
              oType != OBJECT_MOBILEta &&
              oType != OBJECT_MOBILEwa &&
@@ -557,20 +538,22 @@ CObject* CTaskGoto::WormSearch(Math::Vector &impact)
              oType != OBJECT_SAFE     &&
              oType != OBJECT_HUSTON   )  continue;
 
-        if ( pObj->GetVirusMode() )  continue;  // object infected?
+        if ( obj->GetVirusMode() )  continue;  // object infected?
 
-        if ( !pObj->GetCrashSphere(0, oPos, radius) )  continue;
-        distance = Math::DistanceProjected(oPos, iPos);
-        if ( distance < min )
+        if (obj->GetCrashSphereCount() == 0) continue;
+
+        Math::Vector oPos = obj->GetFirstCrashSphere().sphere.pos;
+        float distance = Math::DistanceProjected(oPos, iPos);
+        if (distance < min)
         {
             min = distance;
-            pBest = pObj;
+            best = obj;
         }
     }
-    if ( pBest == 0 )  return 0;
+    if ( best == nullptr )  return nullptr;
 
-    impact = pBest->GetPosition(0);
-    return pBest;
+    impact = best->GetPosition();
+    return best;
 }
 
 // Contaminate objects near the worm.
@@ -588,9 +571,9 @@ void CTaskGoto::WormFrame(float rTime)
         m_wormLastTime = 0.0f;
 
         pObj = WormSearch(impact);
-        if ( pObj != 0 )
+        if ( pObj != nullptr )
         {
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             dist = Math::Distance(pos, impact);
             if ( dist <= 15.0f )
             {
@@ -652,10 +635,10 @@ Error CTaskGoto::Start(Math::Vector goal, float altitude,
     m_phase = TGP_ADVANCE;
     m_error = ERR_OK;
     m_try = 0;
-    m_bmFretObject = 0;
+    m_bmCargoObject = nullptr;
     m_bmFinalMove = 0.0f;
 
-    pos = m_object->GetPosition(0);
+    pos = m_object->GetPosition();
     dist = Math::DistanceProjected(pos, m_goal);
     if ( dist < 10.0f && m_crashMode == TGC_BEAM )
     {
@@ -688,9 +671,9 @@ Error CTaskGoto::Start(Math::Vector goal, float altitude,
     if ( !m_bApprox && m_crashMode != TGC_BEAM )
     {
         target = SearchTarget(goal, 1.0f);
-        if ( target != 0 )
+        if ( target != nullptr )
         {
-            m_goal = target->GetPosition(0);
+            m_goal = target->GetPosition();
             dist = 0.0f;
             if ( !AdjustBuilding(m_goal, 1.0f, dist) )
             {
@@ -707,9 +690,9 @@ Error CTaskGoto::Start(Math::Vector goal, float altitude,
     if ( m_crashMode == TGC_BEAM )  // with the algorithm of rays?
     {
         target = SearchTarget(goal, 1.0f);
-        if ( target != 0 )
+        if ( target != nullptr )
         {
-            m_goal = target->GetPosition(0);
+            m_goal = target->GetPosition();
             dist = 4.0f;
             if ( AdjustBuilding(m_goal, 1.0f, dist) )
             {
@@ -720,7 +703,7 @@ Error CTaskGoto::Start(Math::Vector goal, float altitude,
                 dist = 4.0f;
                 if ( AdjustTarget(target, m_goal, dist) )
                 {
-                    m_bmFretObject = target;  // cargo on the ground
+                    m_bmCargoObject = target;  // cargo on the ground
                 }
                 else
                 {
@@ -730,9 +713,9 @@ Error CTaskGoto::Start(Math::Vector goal, float altitude,
             m_bTake = true;  // object was taken on arrival (final rotation)
         }
 
-        if ( m_physics->GetType() == TYPE_FLYING && m_altitude == 0.0f )
+        if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude == 0.0f )
         {
-            pos = m_object->GetPosition(0);
+            pos = m_object->GetPosition();
             dist = Math::DistanceProjected(pos, m_goal);
             if ( dist > FLY_DIST_GROUND )  // over 20 meters?
             {
@@ -742,7 +725,7 @@ Error CTaskGoto::Start(Math::Vector goal, float altitude,
 
         BeamStart();
 
-        if ( m_bmFretObject == 0 )
+        if ( m_bmCargoObject == nullptr )
         {
             x = static_cast<int>((m_goal.x+1600.0f)/BM_DIM_STEP);
             y = static_cast<int>((m_goal.z+1600.0f)/BM_DIM_STEP);
@@ -767,7 +750,7 @@ Error CTaskGoto::IsEnded()
     if ( m_engine->GetPause() )  return ERR_CONTINUE;
     if ( m_error != ERR_OK )  return m_error;
 
-    pos = m_object->GetPosition(0);
+    pos = m_object->GetPosition();
 
     if ( m_phase == TGP_BEAMLEAK )  // leak?
     {
@@ -789,13 +772,13 @@ Error CTaskGoto::IsEnded()
     if ( m_phase == TGP_BEAMWCOLD )  // expects cool reactor?
     {
         if ( m_altitude != 0.0f &&
-             m_physics->GetReactorRange() < 1.0f )  return ERR_CONTINUE;
+             (m_object->Implements(ObjectInterfaceType::JetFlying) && dynamic_cast<CJetFlyingObject*>(m_object)->GetReactorRange() < 1.0f) )  return ERR_CONTINUE;
         m_phase = TGP_BEAMUP;
     }
 
     if ( m_phase == TGP_BEAMUP )  // off?
     {
-        if ( m_physics->GetType() == TYPE_FLYING && m_altitude > 0.0f )
+        if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude > 0.0f )
         {
             level = m_terrain->GetFloorLevel(pos, true, true);
             h = level+m_altitude-20.0f;
@@ -811,7 +794,7 @@ Error CTaskGoto::IsEnded()
     if ( m_phase == TGP_BEAMGOTO )  // goto dot list ?
     {
         if ( m_altitude != 0.0f &&
-             m_physics->GetReactorRange() < 0.1f )  // overheating?
+             (m_object->Implements(ObjectInterfaceType::JetFlying) && dynamic_cast<CJetFlyingObject*>(m_object)->GetReactorRange() < 0.1f) )  // overheating?
         {
             m_physics->SetMotorSpeedX(0.0f);  // stops the advance
             m_physics->SetMotorSpeedZ(0.0f);  // stops the rotation
@@ -848,7 +831,7 @@ Error CTaskGoto::IsEnded()
 
     if ( m_phase == TGP_BEAMDOWN )  // landed?
     {
-        if ( m_physics->GetType() == TYPE_FLYING && m_altitude > 0.0f )
+        if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude > 0.0f )
         {
             if ( !m_physics->GetLand() )  return ERR_CONTINUE;
             m_physics->SetMotorSpeedY(0.0f);  // stops the descent
@@ -899,7 +882,7 @@ Error CTaskGoto::IsEnded()
 
     if ( m_phase == TGP_LAND )  // landed?
     {
-        if ( m_physics->GetType() == TYPE_FLYING && m_altitude > 0.0f )
+        if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude > 0.0f )
         {
             if ( !m_physics->GetLand() )  return ERR_CONTINUE;
             m_physics->SetMotorSpeedY(0.0f);
@@ -918,7 +901,7 @@ Error CTaskGoto::IsEnded()
 
     if ( m_phase == TGP_TURN )  // turns to the object?
     {
-        angle = Math::NormAngle(m_object->GetAngleY(0));
+        angle = Math::NormAngle(m_object->GetRotationY());
         limit = 0.02f;
         if ( m_bApprox )  limit = 0.10f;
         if ( fabs(angle-m_angle) < limit )
@@ -926,7 +909,7 @@ Error CTaskGoto::IsEnded()
             m_physics->SetMotorSpeedZ(0.0f);  // stops the rotation
             if ( m_bmFinalMove == 0.0f )  return ERR_STOP;
 
-            m_bmFinalPos = m_object->GetPosition(0);
+            m_bmFinalPos = m_object->GetPosition();
             m_bmFinalDist = m_physics->GetLinLength(m_bmFinalMove);
             m_bmTimeLimit = m_physics->GetLinTimeLength(fabs(m_bmFinalMove))*1.5f;
             if ( m_bmTimeLimit < 0.5f )  m_bmTimeLimit = 0.5f;
@@ -940,7 +923,7 @@ Error CTaskGoto::IsEnded()
         {
             m_physics->SetMotorSpeedX(0.0f);  // stops the advance
             m_physics->SetMotorSpeedZ(0.0f);  // stops the rotation
-            m_error = ERR_GENERIC;
+            m_error = ERR_UNKNOWN;
             return m_error;
         }
         if ( m_time >= 1.0f )
@@ -948,7 +931,7 @@ Error CTaskGoto::IsEnded()
             if ( m_crashMode == TGC_RIGHTLEFT ||
                  m_crashMode == TGC_RIGHT     )  angle =  Math::PI/2.0f;  // 90 deegres to the right
             else                            angle = -Math::PI/2.0f;  // 90 deegres to the left
-            m_angle = Math::NormAngle(m_object->GetAngleY(0)+angle);
+            m_angle = Math::NormAngle(m_object->GetRotationY()+angle);
             m_phase = TGP_CRTURN;
 //?         m_phase = TGP_ADVANCE;
         }
@@ -956,7 +939,7 @@ Error CTaskGoto::IsEnded()
 
     if ( m_phase == TGP_CRTURN )  // turns after collision?
     {
-        angle = Math::NormAngle(m_object->GetAngleY(0));
+        angle = Math::NormAngle(m_object->GetRotationY());
         limit = 0.1f;
         if ( fabs(angle-m_angle) < limit )
         {
@@ -982,14 +965,14 @@ Error CTaskGoto::IsEnded()
             if ( m_crashMode == TGC_LEFTRIGHT )  angle =  Math::PI;
             if ( m_crashMode == TGC_RIGHT     )  angle =  Math::PI/2.0f;
             if ( m_crashMode == TGC_LEFT      )  angle = -Math::PI/2.0f;
-            m_angle = Math::NormAngle(m_object->GetAngleY(0)+angle);
+            m_angle = Math::NormAngle(m_object->GetRotationY()+angle);
             m_phase = TGP_CLTURN;
         }
     }
 
     if ( m_phase == TGP_CLTURN )  // turns after collision?
     {
-        angle = Math::NormAngle(m_object->GetAngleY(0));
+        angle = Math::NormAngle(m_object->GetRotationY());
         limit = 0.1f;
         if ( fabs(angle-m_angle) < limit )
         {
@@ -1017,7 +1000,7 @@ Error CTaskGoto::IsEnded()
             return ERR_STOP;
         }
 
-        dist = Math::Distance(m_bmFinalPos, m_object->GetPosition(0));
+        dist = Math::Distance(m_bmFinalPos, m_object->GetPosition());
         if ( dist < m_bmFinalDist )  return ERR_CONTINUE;
         m_physics->SetMotorSpeedX(0.0f);  // stops the advance
         return ERR_STOP;
@@ -1040,7 +1023,6 @@ CObject* CTaskGoto::SearchTarget(Math::Vector pos, float margin)
 bool CTaskGoto::AdjustTarget(CObject* pObj, Math::Vector &pos, float &distance)
 {
     ObjectType  type;
-    Character*  character;
     Math::Matrix*   mat;
     Math::Vector    goal;
     float       dist, suppl;
@@ -1049,40 +1031,22 @@ bool CTaskGoto::AdjustTarget(CObject* pObj, Math::Vector &pos, float &distance)
     if ( type == OBJECT_BEE  ||
          type == OBJECT_WORM )
     {
-        pos = pObj->GetPosition(0);
+        pos = pObj->GetPosition();
         return false;  // single approach
     }
 
     type = pObj->GetType();
 
-    if ( type == OBJECT_FRET         ||
-         type == OBJECT_STONE        ||
-         type == OBJECT_URANIUM      ||
-         type == OBJECT_METAL        ||
-         type == OBJECT_POWER        ||
-         type == OBJECT_ATOMIC       ||
-         type == OBJECT_BULLET       ||
-         type == OBJECT_BBOX         ||
-         type == OBJECT_KEYa         ||
-         type == OBJECT_KEYb         ||
-         type == OBJECT_KEYc         ||
-         type == OBJECT_KEYd         ||
-         type == OBJECT_TNT          ||
-         type == OBJECT_SCRAP1       ||
-         type == OBJECT_SCRAP2       ||
-         type == OBJECT_SCRAP3       ||
-         type == OBJECT_SCRAP4       ||
-         type == OBJECT_SCRAP5       ||
-         type == OBJECT_BOMB         ||
-         type == OBJECT_RUINmobilew1 ||
+    if ( pObj->Implements(ObjectInterfaceType::Transportable) ||
+         type == OBJECT_RUINmobilew1 || // TODO: CRecoverableObject?
          type == OBJECT_RUINmobilew2 ||
          type == OBJECT_RUINmobilet1 ||
          type == OBJECT_RUINmobilet2 ||
          type == OBJECT_RUINmobiler1 ||
          type == OBJECT_RUINmobiler2 )
     {
-        pos = m_object->GetPosition(0);
-        goal = pObj->GetPosition(0);
+        pos = m_object->GetPosition();
+        goal = pObj->GetPosition();
         dist = Math::Distance(goal, pos);
         pos = (pos-goal)*(TAKE_DIST+distance)/dist + goal;
         return true;  // approach from all sites
@@ -1090,8 +1054,8 @@ bool CTaskGoto::AdjustTarget(CObject* pObj, Math::Vector &pos, float &distance)
 
     if ( type == OBJECT_BASE )
     {
-        pos = m_object->GetPosition(0);
-        goal = pObj->GetPosition(0);
+        pos = m_object->GetPosition();
+        goal = pObj->GetPosition();
         dist = Math::Distance(goal, pos);
         pos = (pos-goal)*(TAKE_DIST+distance)/dist + goal;
         return true;  // approach from all sites
@@ -1125,8 +1089,8 @@ bool CTaskGoto::AdjustTarget(CObject* pObj, Math::Vector &pos, float &distance)
          type == OBJECT_MOBILEit ||
          type == OBJECT_MOBILEdr )
     {
-        character = pObj->GetCharacter();
-        pos = character->posPower;
+        assert(pObj->Implements(ObjectInterfaceType::Powered));
+        pos = dynamic_cast<CPoweredObject*>(pObj)->GetPowerPosition();
         pos.x -= TAKE_DIST+TAKE_DIST_OTHER+distance;
         mat = pObj->GetWorldMatrix(0);
         pos = Transform(*mat, pos);
@@ -1140,7 +1104,7 @@ bool CTaskGoto::AdjustTarget(CObject* pObj, Math::Vector &pos, float &distance)
         return false;  // single approach
     }
 
-    pos = pObj->GetPosition(0);
+    pos = pObj->GetPosition();
     distance = 0.0f;
     return false;  // single approach
 }
@@ -1150,22 +1114,18 @@ bool CTaskGoto::AdjustTarget(CObject* pObj, Math::Vector &pos, float &distance)
 
 bool CTaskGoto::AdjustBuilding(Math::Vector &pos, float margin, float &distance)
 {
-    CObject*    pObj;
-    Math::Vector    oPos;
-    float       dist, suppl;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
+        if ( !obj->GetActive() )  continue;
+        if (IsObjectBeingTransported(obj))  continue;
 
-        if ( !pObj->GetActif() )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;  // object transported?
-
-        if ( !GetHotPoint(pObj, oPos, false, 0.0f, suppl) )  continue;
-        dist = Math::DistanceProjected(pos, oPos);
+        Math::Vector oPos;
+        float suppl = 0.0f;
+        if ( !GetHotPoint(obj, oPos, false, 0.0f, suppl) )  continue;
+        float dist = Math::DistanceProjected(pos, oPos);
         if ( dist <= margin )
         {
-            GetHotPoint(pObj, pos, true, distance, suppl);
+            GetHotPoint(obj, pos, true, distance, suppl);
             distance += suppl;
             return true;
         }
@@ -1285,7 +1245,7 @@ bool CTaskGoto::GetHotPoint(CObject *pObj, Math::Vector &pos,
         return true;
     }
 
-    if ( type == OBJECT_PARA && m_physics->GetType() == TYPE_FLYING )
+    if ( type == OBJECT_PARA && m_object->Implements(ObjectInterfaceType::Flying) )
     {
         mat = pObj->GetWorldMatrix(0);
         if ( bTake && distance != 0.0f )  suppl = 20.0f;
@@ -1303,52 +1263,44 @@ bool CTaskGoto::GetHotPoint(CObject *pObj, Math::Vector &pos,
 
 bool CTaskGoto::LeakSearch(Math::Vector &pos, float &delay)
 {
-    CObject     *pObj, *pObstacle = nullptr;
-    Math::Vector    iPos, oPos, bPos;
-    float       iRadius, oRadius, bRadius, dist, min, dir;
-    int         j;
+    if (!m_physics->GetLand())  return false;  // in flight?
 
-    if ( !m_physics->GetLand() )  return false;  // in flight?
+    Math::Sphere crashSphere = m_object->GetFirstCrashSphere().sphere;
 
-    m_object->GetCrashSphere(0, iPos, iRadius);
-
-    min = 100000.0f;
-    bRadius = 0.0f;
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    float min = 100000.0f;
+    CObject* obstacle = nullptr;
+    Math::Sphere obstacleCrashSphere;
+    for (CObject* obj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
+        if ( obj == m_object )  continue;
+        if ( !obj->GetDetectable() )  continue;
+        if (IsObjectBeingTransported(obj))  continue;
 
-        if ( pObj == m_object )  continue;
-        if ( !pObj->GetActif() )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;  // object transported?
-
-        j = 0;
-        while ( pObj->GetCrashSphere(j++, oPos, oRadius) )
+        for (const auto& objCrashSphere : obj->GetAllCrashSpheres())
         {
-            dist = Math::DistanceProjected(oPos, iPos);
-            if ( dist < min )
+            float dist = Math::DistanceProjected(crashSphere.pos, objCrashSphere.sphere.pos);
+            if (dist < min)
             {
                 min = dist;
-                bPos = oPos;
-                bRadius = oRadius;
-                pObstacle = pObj;
+                obstacleCrashSphere = objCrashSphere.sphere;
+                obstacle = obj;
             }
         }
     }
-    if ( min > iRadius+bRadius+4.0f )  return false;
+    if (min > crashSphere.radius + obstacleCrashSphere.radius + 4.0f)  return false;
 
     m_bLeakRecede = false;
 
-    dist = 4.0f;
-    dir  = 1.0f;
-    if ( pObstacle->GetType() == OBJECT_FACTORY )
+    float dist = 4.0f;
+    float dir  = 1.0f;
+    if (obstacle->GetType() == OBJECT_FACTORY)
     {
         dist = 16.0f;
         dir  = -1.0f;
         m_bLeakRecede = true;  // simply recoils
     }
 
-    pos = bPos;
+    pos = obstacleCrashSphere.pos;
     delay = m_physics->GetLinTimeLength(dist, dir);
     return true;
 }
@@ -1359,57 +1311,9 @@ bool CTaskGoto::LeakSearch(Math::Vector &pos, float &delay)
 
 void CTaskGoto::ComputeRepulse(Math::Point &dir)
 {
-#if 0
-    Math::Vector    iPos, oPos;
-    Math::Point     repulse;
-    CObject     *pObj;
-    float       dist, iRadius, oRadius;
-    int         i;
-
-    dir.x = 0.0f;
-    dir.y = 0.0f;
-
-    m_object->GetCrashSphere(0, iPos, iRadius);
-
-    for ( i=0 ; i<1000000 ; i++ )
-    {
-        pObj = static_cast<CObject*>(iMan->SearchInstance(CLASS_OBJECT, i));
-        if ( pObj == 0 )  break;
-
-        if ( pObj == m_object )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;
-
-        oPos = pObj->GetPosition(0);
-        dist = Math::Distance(oPos, m_goalObject);
-        if ( dist <= 1.0f )  continue;
-
-        pObj->GetGlobalSphere(oPos, oRadius);
-        oRadius += iRadius+m_physics->GetLinStopLength()*1.1f;
-        dist = Math::DistanceProjected(oPos, iPos);
-        if ( dist <= oRadius )
-        {
-            repulse.x = iPos.x-oPos.x;
-            repulse.y = iPos.z-oPos.z;
-
-//?         dist = 0.2f-(0.2f*dist/oRadius);
-            dist = powf(dist/oRadius, 2.0f);
-            dist = 0.2f-0.2f*dist;
-            repulse.x *= dist;
-            repulse.y *= dist;
-//?         repulse.x /= dist;
-//?         repulse.y /= dist;
-
-            dir.x += repulse.x;
-            dir.y += repulse.y;
-        }
-    }
-#else
     ObjectType  iType, oType;
-    Math::Vector    iPos, oPos;
     Math::Point     repulse;
-    CObject     *pObj;
-    float       gDist, add, addi, fac, dist, iRadius, oRadius;
-    int         j;
+    float       gDist, add, addi, fac, dist;
     bool        bAlien;
 
     dir.x = 0.0f;
@@ -1419,7 +1323,10 @@ void CTaskGoto::ComputeRepulse(Math::Point &dir)
     iType = m_object->GetType();
     if ( iType == OBJECT_WORM || iType == OBJECT_CONTROLLER )  return;
 
-    m_object->GetCrashSphere(0, iPos, iRadius);
+    auto firstCrashSphere = m_object->GetFirstCrashSphere();
+    Math::Vector iPos = firstCrashSphere.sphere.pos;
+    float iRadius = firstCrashSphere.sphere.radius;
+
     gDist = Math::Distance(iPos, m_goal);
 
     add = m_physics->GetLinStopLength()*1.1f;  // braking distance
@@ -1493,13 +1400,11 @@ void CTaskGoto::ComputeRepulse(Math::Point &dir)
     {
         bAlien = true;
     }
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
-    {
-        pObj = it.second;
 
+    for (CObject* pObj : CObjectManager::GetInstancePointer()->GetAllObjects())
+    {
         if ( pObj == m_object )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;
+        if (IsObjectBeingTransported(pObj))  continue;
 
         oType = pObj->GetType();
 
@@ -1507,23 +1412,7 @@ void CTaskGoto::ComputeRepulse(Math::Point &dir)
 
         if ( bAlien )
         {
-            if ( oType == OBJECT_STONE   ||
-                 oType == OBJECT_URANIUM ||
-                 oType == OBJECT_METAL   ||
-                 oType == OBJECT_POWER   ||
-                 oType == OBJECT_ATOMIC  ||
-                 oType == OBJECT_BULLET  ||
-                 oType == OBJECT_BBOX    ||
-                 oType == OBJECT_KEYa    ||
-                 oType == OBJECT_KEYb    ||
-                 oType == OBJECT_KEYc    ||
-                 oType == OBJECT_KEYd    ||
-                 oType == OBJECT_TNT     ||
-                 oType == OBJECT_SCRAP1  ||
-                 oType == OBJECT_SCRAP2  ||
-                 oType == OBJECT_SCRAP3  ||
-                 oType == OBJECT_SCRAP4  ||
-                 oType == OBJECT_SCRAP5  ||
+            if ( pObj->Implements(ObjectInterfaceType::Transportable) ||
                  oType == OBJECT_BOMB    ||
                 (oType >= OBJECT_PLANT0    &&
                  oType <= OBJECT_PLANT19   ) ||
@@ -1538,9 +1427,11 @@ void CTaskGoto::ComputeRepulse(Math::Point &dir)
             addi = 2.0f;  // between wasps, do not annoy too much
         }
 
-        j = 0;
-        while ( pObj->GetCrashSphere(j++, oPos, oRadius) )
+        for (const auto& crashSphere : pObj->GetAllCrashSpheres())
         {
+            Math::Vector oPos = crashSphere.sphere.pos;
+            float oRadius = crashSphere.sphere.radius;
+
             if ( oPos.y-oRadius > iPos.y+iRadius )  continue;
             if ( oPos.y+oRadius < iPos.y-iRadius )  continue;
 
@@ -1565,7 +1456,6 @@ void CTaskGoto::ComputeRepulse(Math::Point &dir)
             }
         }
     }
-#endif
 }
 
 // Calculates the force of vertical repulsion according to barriers.
@@ -1573,37 +1463,33 @@ void CTaskGoto::ComputeRepulse(Math::Point &dir)
 
 void CTaskGoto::ComputeFlyingRepulse(float &dir)
 {
-    ObjectType  oType;
-    Math::Vector    iPos, oPos;
-    CObject     *pObj;
-    float       add, fac, dist, iRadius, oRadius, repulse;
-    int         j;
+    auto firstCrashSphere = m_object->GetFirstCrashSphere();
+    Math::Vector iPos = firstCrashSphere.sphere.pos;
+    float iRadius = firstCrashSphere.sphere.radius;
 
-    m_object->GetCrashSphere(0, iPos, iRadius);
-
-    add = 0.0f;
-    fac = 1.5f;
+    float add = 0.0f;
+    float fac = 1.5f;
     dir = 0.0f;
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+
+    for (CObject* pObj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-
         if ( pObj == m_object )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;
+        if (IsObjectBeingTransported(pObj))  continue;
 
-        oType = pObj->GetType();
+        ObjectType oType = pObj->GetType();
 
         if ( oType == OBJECT_WORM )  continue;
 
-        j = 0;
-        while ( pObj->GetCrashSphere(j++, oPos, oRadius) )
+        for (const auto& crashSphere : pObj->GetAllCrashSpheres())
         {
+            Math::Vector oPos = crashSphere.sphere.pos;
+            float oRadius = crashSphere.sphere.radius;
+
             oRadius += iRadius+add;
-            dist = Math::DistanceProjected(oPos, iPos);
+            float dist = Math::DistanceProjected(oPos, iPos);
             if ( dist <= oRadius )
             {
-                repulse = iPos.y-oPos.y;
+                float repulse = iPos.y-oPos.y;
 
                 dist = powf(dist/oRadius, fac);
                 dist = 0.2f-0.2f*dist;
@@ -1647,7 +1533,7 @@ void CTaskGoto::BeamStart()
     BitmapOpen();
     BitmapObject();
 
-    min = m_object->GetPosition(0);
+    min = m_object->GetPosition();
     max = m_goal;
     if ( min.x > max.x )  Math::Swap(min.x, max.x);
     if ( min.z > max.z )  Math::Swap(min.z, max.z);
@@ -1835,7 +1721,7 @@ bool CTaskGoto::BitmapTestLine(const Math::Vector &start, const Math::Vector &go
     float       distNoB2;
     int         i, max, x, y;
 
-    if ( m_bmArray == 0 )  return true;
+    if ( m_bmArray == nullptr )  return true;
 
     dist = Math::DistanceProjected(start, goal);
     if ( dist == 0.0f )  return true;
@@ -1890,34 +1776,29 @@ bool CTaskGoto::BitmapTestLine(const Math::Vector &start, const Math::Vector &go
 
 void CTaskGoto::BitmapObject()
 {
-    CObject     *pObj;
-    ObjectType  type;
-    Math::Vector    iPos, oPos;
-    float       iRadius, oRadius, h;
-    int         j;
+    auto firstCrashSphere = m_object->GetFirstCrashSphere();
+    float iRadius = firstCrashSphere.sphere.radius;
 
-    m_object->GetCrashSphere(0, iPos, iRadius);
-    
-    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
+    for (CObject* pObj : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = it.second;
-
-        type = pObj->GetType();
+        ObjectType type = pObj->GetType();
 
         if ( pObj == m_object )  continue;
-        if ( pObj == m_bmFretObject )  continue;
-        if ( pObj->GetTruck() != 0 )  continue;
+        if ( pObj == m_bmCargoObject )  continue;
+        if (IsObjectBeingTransported(pObj))  continue;
 
-        h = m_terrain->GetFloorLevel(pObj->GetPosition(0), false);
-        if ( m_physics->GetType() == TYPE_FLYING && m_altitude > 0.0f )
+        float h = m_terrain->GetFloorLevel(pObj->GetPosition(), false);
+        if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude > 0.0f )
         {
             h += m_altitude;
         }
 
-        j = 0;
-        while ( pObj->GetCrashSphere(j++, oPos, oRadius) )
+        for (const auto& crashSphere : pObj->GetAllCrashSpheres())
         {
-            if ( m_physics->GetType() == TYPE_FLYING && m_altitude > 0.0f )  // flying?
+            Math::Vector oPos = crashSphere.sphere.pos;
+            float oRadius = crashSphere.sphere.radius;
+
+            if ( m_object->Implements(ObjectInterfaceType::Flying) && m_altitude > 0.0f )  // flying?
             {
                 if ( oPos.y-oRadius > h+8.0f ||
                      oPos.y+oRadius < h-8.0f )  continue;
@@ -2088,8 +1969,7 @@ bool CTaskGoto::BitmapOpen()
     BitmapClose();
 
     m_bmSize = static_cast<int>(3200.0f/BM_DIM_STEP);
-    m_bmArray = new unsigned char[m_bmSize*m_bmSize/8*2];
-    memset(m_bmArray, 0, m_bmSize*m_bmSize/8*2);
+    m_bmArray = MakeUniqueArray<unsigned char>(m_bmSize*m_bmSize/8*2);
 
     m_bmOffset = m_bmSize/2;
     m_bmLine = m_bmSize/8;
@@ -2106,8 +1986,7 @@ bool CTaskGoto::BitmapOpen()
 
 bool CTaskGoto::BitmapClose()
 {
-    delete[] m_bmArray;
-    m_bmArray = 0;
+    m_bmArray.reset();
     return true;
 }
 
@@ -2193,4 +2072,3 @@ bool CTaskGoto::BitmapTestDot(int rank, int x, int y)
 
     return m_bmArray[rank*m_bmLine*m_bmSize + m_bmLine*y + x/8] & (1<<x%8);
 }
-
