@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,34 +17,30 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// taskinfo.h
-
 #pragma once
 
 
 #include "object/task/task.h"
 
+class CExchangePost;
 
-
-class CTaskInfo : public CTask
+class CTaskInfo : public CForegroundTask
 {
 public:
-    CTaskInfo(CObject* object);
+    CTaskInfo(COldObject* object);
     ~CTaskInfo();
 
-    bool        EventProcess(const Event &event);
+    bool        EventProcess(const Event &event) override;
 
-    Error       Start(const char *name, float value, float power, bool bSend);
-    Error       IsEnded();
-    bool        Abort();
+    Error       Start(const char *name, float value, float power, bool send);
+    Error       IsEnded() override;
+    bool        Abort() override;
+
+    CExchangePost* FindExchangePost(float power);
 
 protected:
-    CObject*    SearchInfo(float power);
-
-protected:
-    float           m_progress;
-    float           m_speed;
-    float           m_time;
-    bool            m_bError;
+    float           m_progress = 0.0f;
+    float           m_speed = 0.0f;
+    float           m_time = 0.0f;
+    bool            m_error = false;
 };
-
