@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// taskspiderexplo.h
-
 #pragma once
 
 
@@ -26,22 +24,19 @@
 
 
 
-class CTaskSpiderExplo : public CTask
+class CTaskSpiderExplo : public CForegroundTask
 {
 public:
-    CTaskSpiderExplo(CObject* object);
+    CTaskSpiderExplo(COldObject* object);
     ~CTaskSpiderExplo();
 
-    bool        EventProcess(const Event &event);
+    bool        EventProcess(const Event &event) override;
 
     Error       Start();
-    Error       IsEnded();
-    bool        Abort();
+    Error       IsEnded() override;
+    bool        Abort() override;
 
 protected:
-
-protected:
-    float       m_time;
-    bool        m_bError;
+    float       m_time = 0.0f;
+    bool        m_bError = false;
 };
-
