@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// taskturn.h
-
 #pragma once
 
 
@@ -26,24 +24,21 @@
 
 
 
-class CTaskTurn : public CTask
+class CTaskTurn : public CForegroundTask
 {
 public:
-    CTaskTurn(CObject* object);
+    CTaskTurn(COldObject* object);
     ~CTaskTurn();
 
-    bool    EventProcess(const Event &event);
+    bool    EventProcess(const Event &event) override;
 
     Error   Start(float angle);
-    Error   IsEnded();
+    Error   IsEnded() override;
 
 protected:
-
-protected:
-    float       m_angle;
-    float       m_startAngle;
-    float       m_finalAngle;
-    bool        m_bLeft;
-    bool        m_bError;
+    float       m_angle = 0.0f;
+    float       m_startAngle = 0.0f;
+    float       m_finalAngle = 0.0f;
+    bool        m_bLeft = false;
+    bool        m_bError = false;
 };
-
