@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,28 +20,28 @@
 
 #include "script/script.h"
 
-
-#include "app/pausemanager.h"
-
-#include "common/resources/resourcemanager.h"
-#include "common/resources/inputstream.h"
-#include "common/resources/outputstream.h"
 #include "common/restext.h"
 #include "common/stringutils.h"
 
+#include "common/resources/inputstream.h"
+#include "common/resources/outputstream.h"
+#include "common/resources/resourcemanager.h"
+
 #include "graphics/engine/engine.h"
 #include "graphics/engine/text.h"
 
-#include "object/task/taskmanager.h"
-#include "object/robotmain.h"
+#include "level/robotmain.h"
+
+#include "object/old_object.h"
 
 #include "script/cbottoken.h"
 
-#include "ui/interface.h"
-#include "ui/edit.h"
-#include "ui/list.h"
 #include "ui/displaytext.h"
 
+#include "ui/controls/edit.h"
+#include "ui/controls/interface.h"
+#include "ui/controls/list.h"
+
 
 
 const int CBOT_IPF = 100;       // CBOT: default number of instructions / frame
@@ -49,24 +49,23 @@ const int CBOT_IPF = 100;       // CBOT: default number of instructions / frame
 
 // Object's constructor.
 
-CScript::CScript(CObject* object, CTaskManager** secondaryTask)
+CScript::CScript(COldObject* object)
 {
+    assert(object->Implements(ObjectInterfaceType::Programmable));
+
+    m_object        = object;
+    assert(m_object->Implements(ObjectInterfaceType::TaskExecutor));
+    m_taskExecutor  = dynamic_cast<CTaskExecutorObject*>(m_object);
+
     m_engine        = Gfx::CEngine::GetInstancePointer();
     m_main          = CRobotMain::GetInstancePointer();
     m_terrain       = m_main->GetTerrain();
     m_water         = m_engine->GetWater();
-    m_botProg       = nullptr;
-    m_object        = object;
-    m_primaryTask   = nullptr;
-    m_secondaryTask = secondaryTask;
-    
-    m_interface = m_main->GetInterface();
-    m_pause = CPauseManager::GetInstancePointer();
-    
+    m_interface     = m_main->GetInterface();
+
     m_ipf = CBOT_IPF;
     m_errMode = ERM_STOP;
     m_len = 0;
-    m_script = nullptr;
     m_bRun = false;
     m_bStepMode = false;
     m_bCompile = false;
@@ -81,15 +80,6 @@ CScript::CScript(CObject* object, CTaskManager** secondaryTask)
 
 CScript::~CScript()
 {
-    delete m_botProg;
-    m_botProg = nullptr;
-
-    delete m_primaryTask;
-    m_primaryTask = nullptr;
-
-    delete[] m_script;
-    m_script = nullptr;
-
     m_len = 0;
 }
 
@@ -104,34 +94,29 @@ void CScript::PutScript(Ui::CEdit* edit, const char* name)
     }
     else
     {
-        edit->SetText(m_script);
+        edit->SetText(m_script.get());
         edit->SetCursor(m_cursor2, m_cursor1);
         edit->ShowSelect();
     }
-    edit->SetFocus(true);
+    m_interface->SetFocus(edit);
 }
 
 // The script takes a paved text.
 
 bool CScript::GetScript(Ui::CEdit* edit)
 {
-    int     len;
+    int len = edit->GetTextLength();
+    m_script = MakeUniqueArray<char>(len+1);
 
-    delete[] m_script;
-    m_script = nullptr;
-
-    len = edit->GetTextLength();
-    m_script = new char[len+1];
-
-    edit->GetText(m_script, len+1);
+    edit->GetText(m_script.get(), len+1);
     edit->GetCursor(m_cursor2, m_cursor1);
-    m_len = strlen(m_script);
+    m_len = strlen(m_script.get());
 
     if ( !CheckToken() )
     {
         edit->SetCursor(m_cursor2, m_cursor1);
         edit->ShowSelect();
-        edit->SetFocus(true);
+        m_interface->SetFocus(edit);
         return false;
     }
 
@@ -139,7 +124,7 @@ bool CScript::GetScript(Ui::CEdit* edit)
     {
         edit->SetCursor(m_cursor2, m_cursor1);
         edit->ShowSelect();
-        edit->SetFocus(true);
+        m_interface->SetFocus(edit);
         return false;
     }
 
@@ -157,9 +142,7 @@ bool CScript::GetCompile()
 
 bool CScript::IsEmpty()
 {
-    int     i;
-
-    for ( i=0 ; i<m_len ; i++ )
+    for (int i = 0; i < m_len; i++)
     {
         if ( m_script[i] != ' '  &&
              m_script[i] != '\n' )  return false;
@@ -192,9 +175,9 @@ bool CScript::CheckToken()
         used[i] = 0;  // token not used
     }
 
-    allBt = CBotToken::CompileTokens(m_script, error);
+    allBt = CBotToken::CompileTokens(m_script.get(), error);
     bt = allBt;
-    while ( bt != 0 )
+    while ( bt != nullptr )
     {
         bs = bt->GetString();
         token = bs;
@@ -257,17 +240,16 @@ bool CScript::Compile()
 
     if ( IsEmpty() )  // program exist?
     {
-        delete m_botProg;
-        m_botProg = 0;
+        m_botProg.reset();
         return true;
     }
 
-    if ( m_botProg == 0 )
+    if (m_botProg == nullptr)
     {
-        m_botProg = new CBotProgram(m_object->GetBotVar());
+        m_botProg = MakeUnique<CBotProgram>(m_object->GetBotVar());
     }
 
-    if ( m_botProg->Compile(m_script, liste, this) )
+    if ( m_botProg->Compile(m_script.get(), liste, this) )
     {
         if ( liste.GetSize() == 0 )
         {
@@ -337,18 +319,22 @@ void CScript::SetStepMode(bool bStep)
     m_bStepMode = bStep;
 }
 
+bool CScript::GetStepMode()
+{
+    return m_bStepMode;
+}
+
 
 // Runs the program from the beginning.
 
 bool CScript::Run()
 {
-    if( m_botProg == 0 )  return false;
+    if (m_botProg == nullptr)  return false;
     if ( m_script == nullptr || m_len == 0 )  return false;
     if ( m_mainFunction[0] == 0 ) return false;
-    
+
     if ( !m_botProg->Start(m_mainFunction) )  return false;
 
-    m_object->SetRunScript(this);
     m_bRun = true;
     m_bContinue = false;
     m_ipf = CBOT_IPF;
@@ -356,9 +342,7 @@ bool CScript::Run()
 
     if ( m_bStepMode )  // step by step mode?
     {
-        Event   newEvent;
-        memset(&newEvent, 0, sizeof(Event));
-        Step(newEvent);
+        Step();
     }
 
     return true;
@@ -367,18 +351,16 @@ bool CScript::Run()
 // Continues the execution of current program.
 // Returns true when execution is finished.
 
-bool CScript::Continue(const Event &event)
+bool CScript::Continue()
 {
-    if( m_botProg == 0 )  return true;
+    if (m_botProg == nullptr)  return true;
     if ( !m_bRun )  return true;
 
-    m_event = event;
-
     if ( m_bStepMode )  // step by step mode?
     {
         if ( m_bContinue )  // instuction "move", "goto", etc. ?
         {
-            if ( m_botProg->Run(m_object, 0) )
+            if ( m_botProg->Run(this, 0) )
             {
                 m_botProg->GetError(m_error, m_cursor1, m_cursor2);
                 if ( m_cursor1 < 0 || m_cursor1 > m_len ||
@@ -399,19 +381,14 @@ bool CScript::Continue(const Event &event)
                     GetError(s);
                     m_main->GetDisplayText()->DisplayText(s.c_str(), m_object, 10.0f, Ui::TT_ERROR);
                 }
-                m_pause->SetPause(PAUSE_EDITOR);  // gives pause
                 return true;
             }
-            if ( !m_bContinue )
-            {
-                m_pause->SetPause(PAUSE_EDITOR);  // gives pause
-            }
         }
 
         return false;
     }
 
-    if ( m_botProg->Run(m_object, m_ipf) )
+    if ( m_botProg->Run(this, m_ipf) )
     {
         m_botProg->GetError(m_error, m_cursor1, m_cursor2);
         if ( m_cursor1 < 0 || m_cursor1 > m_len ||
@@ -441,19 +418,13 @@ bool CScript::Continue(const Event &event)
 // Continues the execution of current program.
 // Returns true when execution is finished.
 
-bool CScript::Step(const Event &event)
+bool CScript::Step()
 {
-    if( m_botProg == 0 )  return true;
+    if (m_botProg == nullptr)  return true;
     if ( !m_bRun )  return true;
     if ( !m_bStepMode )  return false;
 
-    // ??? m_engine->SetPause(false);
-    // TODO: m_app StepSimulation??? m_engine->StepSimulation(0.01f);  // advance of 10ms
-    // ??? m_engine->SetPause(true);
-
-    m_event = event;
-
-    if ( m_botProg->Run(m_object, 0) )  // step mode
+    if ( m_botProg->Run(this, 0) )  // step mode
     {
         m_botProg->GetError(m_error, m_cursor1, m_cursor2);
         if ( m_cursor1 < 0 || m_cursor1 > m_len ||
@@ -476,11 +447,6 @@ bool CScript::Step(const Event &event)
         }
         return true;
     }
-
-    if ( m_bContinue )  // instuction "move", "goto", etc. ?
-    {
-        m_pause->ClearPause();  // removes the pause
-    }
     return false;
 }
 
@@ -490,18 +456,13 @@ void CScript::Stop()
 {
     if ( !m_bRun )  return;
 
-    if( m_botProg != 0 )
+    m_taskExecutor->StopForegroundTask();
+
+    if (m_botProg != nullptr)
     {
         m_botProg->Stop();
     }
 
-    if ( m_primaryTask != 0 )
-    {
-        m_primaryTask->Abort();
-        delete m_primaryTask;
-        m_primaryTask = 0;
-    }
-
     m_bRun = false;
 }
 
@@ -528,7 +489,7 @@ bool CScript::GetCursor(int &cursor1, int &cursor2)
 
     cursor1 = cursor2 = 0;
 
-    if( m_botProg == 0 )  return false;
+    if (m_botProg == nullptr)  return false;
     if ( !m_bRun )  return false;
 
     m_botProg->GetRunPos(funcName, cursor1, cursor2);
@@ -553,7 +514,7 @@ void PutList(const char *baseName, bool bArray, CBotVar *var, Ui::CList *list, i
     const char  *p;
     int         index, type;
 
-    if ( var == 0 && baseName[0] != 0 )
+    if ( var == nullptr && baseName[0] != 0 )
     {
         sprintf(buffer, "%s = null;", baseName);
         list->SetItemName(rankList++, buffer);
@@ -561,9 +522,9 @@ void PutList(const char *baseName, bool bArray, CBotVar *var, Ui::CList *list, i
     }
 
     index = 0;
-    while ( var != 0 )
+    while ( var != nullptr )
     {
-        var->Maj(NULL, false);
+        var->Maj(nullptr, false);
         pStatic = var->GetStaticVar();  // finds the static element
 
         bs = pStatic->GetName();  // variable name
@@ -638,14 +599,14 @@ void CScript::UpdateList(Ui::CList* list)
     const char  *progName, *funcName;
     int         total, select, level, cursor1, cursor2, rank;
 
-    if( m_botProg == 0 )  return;
+    if (m_botProg == nullptr) return;
 
     total  = list->GetTotal();
     select = list->GetSelect();
 
     list->Flush();  // empty list
     m_botProg->GetRunPos(progName, cursor1, cursor2);
-    if ( progName == 0 )  return;
+    if ( progName == nullptr )  return;
 
     level = 0;
     rank  = 0;
@@ -669,58 +630,69 @@ void CScript::UpdateList(Ui::CList* list)
 
 // Colorize the text according to syntax.
 
-void CScript::ColorizeScript(Ui::CEdit* edit)
+void CScript::ColorizeScript(Ui::CEdit* edit, int rangeStart, int rangeEnd)
 {
-    CBotToken*  bt;
-    CBotString  bs;
-    const char* token;
-    int         error, type, cursor1, cursor2;
-    Gfx::FontHighlight color;
+    if (rangeEnd > edit->GetMaxChar())
+        rangeEnd = edit->GetMaxChar();
 
-    edit->ClearFormat();
+    edit->SetFormat(rangeStart, rangeEnd, Gfx::FONT_HIGHLIGHT_COMMENT); // anything not processed is a comment
 
-    bt = CBotToken::CompileTokens(edit->GetText(), error);
-    while ( bt != 0 )
+    // NOTE: Images are registered as index in some array, and that can be 0 which normally ends the string!
+    std::string text = std::string(edit->GetText(), edit->GetMaxChar());
+    text = text.substr(rangeStart, rangeEnd-rangeStart);
+
+    int error;
+    CBotToken* bt = CBotToken::CompileTokens(text.c_str(), error);
+    while ( bt != nullptr )
     {
-        bs = bt->GetString();
-        token = bs;
-        type = bt->GetType();
+        CBotString bs = bt->GetString();
+        const char* token = bs;
+        int type = bt->GetType();
 
-        cursor1 = bt->GetStart();
-        cursor2 = bt->GetEnd();
-        color = Gfx::FONT_HIGHLIGHT_NONE;
-        if ( type >= TokenKeyWord && type < TokenKeyWord+100 )
-        {
-            color = Gfx::FONT_HIGHLIGHT_TOKEN;
-        }
-        if ( type >= TokenKeyDeclare && type < TokenKeyDeclare+100 )
+        int cursor1 = bt->GetStart();
+        int cursor2 = bt->GetEnd();
+
+        if (cursor1 < 0 || cursor2 < 0 || cursor1 == cursor2 || type == 0) { bt = bt->GetNext(); continue; } // seems to be a bug in CBot engine (how does it even still work? D:)
+
+        cursor1 += rangeStart;
+        cursor2 += rangeStart;
+
+        Gfx::FontHighlight color = Gfx::FONT_HIGHLIGHT_NONE;
+        if ((type == TokenTypVar || (type >= TokenKeyWord && type < TokenKeyWord+100)) && IsType(token)) // types (basic types are TokenKeyWord, classes are TokenTypVar)
         {
             color = Gfx::FONT_HIGHLIGHT_TYPE;
         }
-        if ( type >= TokenKeyVal && type < TokenKeyVal+100 )
+        else if (type == TokenTypVar && IsFunction(token)) // functions
+        {
+            color = Gfx::FONT_HIGHLIGHT_TOKEN;
+        }
+        else if (type == TokenTypVar && strcmp(token, "this") == 0) // this
+        {
+            color = Gfx::FONT_HIGHLIGHT_THIS;
+        }
+        else if (type >= TokenKeyWord && type < TokenKeyWord+100) // builtin keywords
+        {
+            color = Gfx::FONT_HIGHLIGHT_KEYWORD;
+        }
+        else if (type >= TokenKeyDeclare && type < TokenKeyDeclare+100) // TODO: no idea :P seems to never happen
+        {
+            color = Gfx::FONT_HIGHLIGHT_TYPE;
+        }
+        else if (type >= TokenKeyVal && type < TokenKeyVal+100) // true, false, null, nan
         {
             color = Gfx::FONT_HIGHLIGHT_CONST;
         }
-        if ( type == TokenTypVar )
+        else if (type == TokenTypDef) // constants (object types etc.)
         {
-            if ( IsType(token) )
-            {
-                color = Gfx::FONT_HIGHLIGHT_TYPE;
-            }
-            else if ( IsFunction(token) )
-            {
-                color = Gfx::FONT_HIGHLIGHT_TOKEN;
-            }
+            color = Gfx::FONT_HIGHLIGHT_CONST;
         }
-        if ( type == TokenTypDef )
+        else if (type == TokenTypString || type == TokenTypNum) // string literals and numbers
         {
-            color =Gfx::FONT_HIGHLIGHT_CONST;
+            color = Gfx::FONT_HIGHLIGHT_STRING;
         }
 
-        if ( cursor1 < cursor2 && color != Gfx::FONT_HIGHLIGHT_NONE )
-        {
-            edit->SetFormat(cursor1, cursor2, color);
-        }
+        assert(cursor1 < cursor2);
+        edit->SetFormat(cursor1, cursor2, color);
 
         bt = bt->GetNext();
     }
@@ -745,7 +717,7 @@ int SearchToken(char* script, const char* token)
     for ( i=0 ; i<lScript-lToken ; i++ )
     {
         p = strstr(script+i, token);
-        if ( p != 0 )
+        if ( p != nullptr )
         {
             found[iFound++] = p-script;
             if ( iFound >= 100 )  break;
@@ -786,9 +758,7 @@ void InsertToken(char* script, int pos, const char* token)
 
 bool CScript::IntroduceVirus()
 {
-    int     i, start, iFound;
-    int     found[11*2];
-    char*   newScript;
+    if (m_script == nullptr) return false;
 
     const char*   names[11*2] =
     {
@@ -805,10 +775,11 @@ bool CScript::IntroduceVirus()
         "EnergyCell",   "InFront",
     };
 
-    iFound = 0;
-    for ( i=0 ; i<11 ; i++ )
+    int iFound = 0;
+    int     found[11*2];
+    for ( int i=0 ; i<11 ; i++ )
     {
-        start = SearchToken(m_script, names[i*2]);
+        int start = SearchToken(m_script.get(), names[i*2]);
         if ( start != -1 )
         {
             found[iFound++] = i*2;
@@ -817,18 +788,17 @@ bool CScript::IntroduceVirus()
     }
     if ( iFound == 0 )  return false;
 
-    i = (rand()%(iFound/2))*2;
-    start = found[i+1];
+    int i = (rand()%(iFound/2))*2;
+    int start = found[i+1];
     i     = found[i+0];
 
-    newScript = new char[m_len+strlen(names[i+1])+1];
-    strcpy(newScript, m_script);
-    delete[] m_script;
-    m_script = newScript;
+    auto newScript = MakeUniqueArray<char>(m_len + strlen(names[i+1]) + 1);
+    strcpy(newScript.get(), m_script.get());
+    m_script = std::move(newScript);
 
-    DeleteToken(m_script, start, strlen(names[i]));
-    InsertToken(m_script, start, names[i+1]);
-    m_len = strlen(m_script);
+    DeleteToken(m_script.get(), start, strlen(names[i]));
+    InsertToken(m_script.get(), start, names[i+1]);
+    m_len = strlen(m_script.get());
     Compile();  // recompile with the virus
 
     return true;
@@ -911,7 +881,7 @@ void CScript::New(Ui::CEdit* edit, const char* name)
 
     edit->SetCursor(cursor2, cursor1);
     edit->ShowSelect();
-    edit->SetFocus(true);
+    m_interface->SetFocus(edit);
 
     sf = m_main->GetScriptFile();
     if ( sf[0] != 0 )  // Load an empty program specific?
@@ -972,7 +942,7 @@ void CScript::New(Ui::CEdit* edit, const char* name)
             cursor2 = cursor1;
             edit->SetCursor(cursor2, cursor1);
             edit->ShowSelect();
-            edit->SetFocus(true);
+            m_interface->SetFocus(edit);
         }
     }
 
@@ -1008,8 +978,7 @@ bool CScript::ReadScript(const char* filename)
 
     if (!CResourceManager::Exists(filename))  return false;
 
-    delete[] m_script;
-    m_script = nullptr;
+    m_script.reset();
 
     edit = m_interface->CreateEdit(Math::Point(0.0f, 0.0f), Math::Point(0.0f, 0.0f), 0, EVENT_EDIT9);
     edit->SetMaxChar(Ui::EDITSTUDIOMAX);
@@ -1024,18 +993,16 @@ bool CScript::ReadScript(const char* filename)
 
 bool CScript::WriteScript(const char* filename)
 {
-    Ui::CEdit*  edit;
-
     if ( m_script == nullptr )
     {
         CResourceManager::Remove(filename);
         return false;
     }
 
-    edit = m_interface->CreateEdit(Math::Point(0.0f, 0.0f), Math::Point(0.0f, 0.0f), 0, EVENT_EDIT9);
+    Ui::CEdit* edit = m_interface->CreateEdit(Math::Point(0.0f, 0.0f), Math::Point(0.0f, 0.0f), 0, EVENT_EDIT9);
     edit->SetMaxChar(Ui::EDITSTUDIOMAX);
     edit->SetAutoIndent(m_engine->GetEditIndentMode());
-    edit->SetText(m_script);
+    edit->SetText(m_script.get());
     edit->WriteText(filename);
     m_interface->DeleteControl(EVENT_EDIT9);
     return true;
@@ -1052,10 +1019,9 @@ bool CScript::ReadStack(FILE *file)
     fRead(&m_ipf, sizeof(int), 1, file);
     fRead(&m_errMode, sizeof(int), 1, file);
 
-    if ( m_botProg == 0 )  return false;
+    if (m_botProg == nullptr) return false;
     if ( !m_botProg->RestoreState(file) )  return false;
 
-    m_object->SetRunScript(this);
     m_bRun = true;
     m_bContinue = false;
     return true;
@@ -1082,7 +1048,7 @@ bool CScript::Compare(CScript* other)
 {
     if ( m_len != other->m_len )  return false;
 
-    return ( strcmp(m_script, other->m_script) == 0 );
+    return ( strcmp(m_script.get(), other->m_script.get()) == 0 );
 }
 
 
@@ -1097,4 +1063,3 @@ char* CScript::GetFilename()
 {
     return m_filename;
 }
-
