@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,22 +20,21 @@
 
 #include "sound/oalsound/alsound.h"
 
+#include "common/make_unique.h"
+
 #include <algorithm>
 #include <iomanip>
 
 #include <boost/filesystem.hpp>
 
 ALSound::ALSound()
+    : m_enabled(false),
+      m_audioVolume(1.0f),
+      m_musicVolume(1.0f),
+      m_channelsLimit(2048),
+      m_device{},
+      m_context{}
 {
-    m_enabled = false;
-    m_audioVolume = 1.0f;
-    m_musicVolume = 1.0f;
-    m_currentMusic = nullptr;
-    m_eye.LoadZero();
-    m_lookat.LoadZero();
-    m_previousMusic.fadeTime = 0.0f;
-    m_previousMusic.music = nullptr;
-    m_channels_limit = 2048;
 }
 
 
@@ -53,29 +52,17 @@ void ALSound::CleanUp()
         StopAll();
         StopMusic();
 
-        for (auto channel : m_channels)
-        {
-            delete channel.second;
-        }
+        m_channels.clear();
 
-        delete m_currentMusic;
+        m_currentMusic.reset();
 
-        for (auto item : m_oldMusic)
-        {
-            delete item.music;
-        }
+        m_oldMusic.clear();
 
-        delete m_previousMusic.music;
+        m_previousMusic.music.reset();
 
-        for (auto item : m_sounds)
-        {
-            delete item.second;
-        }
+        m_sounds.clear();
 
-        for (auto item : m_music)
-        {
-            delete item.second;
-        }
+        m_music.clear();
 
         m_enabled = false;
 
@@ -93,14 +80,14 @@ bool ALSound::Create()
         return true;
 
     GetLogger()->Info("Opening audio device...\n");
-    m_device = alcOpenDevice(NULL);
+    m_device = alcOpenDevice(nullptr);
     if (!m_device)
     {
         GetLogger()->Error("Could not open audio device!\n");
         return false;
     }
 
-    m_context = alcCreateContext(m_device, NULL);
+    m_context = alcCreateContext(m_device, nullptr);
     if (!m_context)
     {
         GetLogger()->Error("Could not create audio context!\n");
@@ -156,12 +143,12 @@ int ALSound::GetMusicVolume()
 }
 
 
-bool ALSound::Cache(Sound sound, const std::string &filename)
+bool ALSound::Cache(SoundType sound, const std::string &filename)
 {
-    Buffer *buffer = new Buffer();
+    auto buffer = MakeUnique<Buffer>();
     if (buffer->LoadFromFile(filename, sound))
     {
-        m_sounds[sound] = buffer;
+        m_sounds[sound] = std::move(buffer);
         return true;
     }
     return false;
@@ -169,29 +156,29 @@ bool ALSound::Cache(Sound sound, const std::string &filename)
 
 bool ALSound::CacheMusic(const std::string &filename)
 {
-    if (m_music.find("music/"+filename) == m_music.end())
+    if (m_music.find(filename) == m_music.end())
     {
-        Buffer *buffer = new Buffer();
-        if (buffer->LoadFromFile("music/"+filename, static_cast<Sound>(-1)))
+        auto buffer = MakeUnique<Buffer>();
+        if (buffer->LoadFromFile(filename, static_cast<SoundType>(-1)))
         {
-            m_music["music/"+filename] = buffer;
+            m_music[filename] = std::move(buffer);
             return true;
         }
     }
     return false;
 }
 
-bool ALSound::IsCached(Sound sound)
+bool ALSound::IsCached(SoundType sound)
 {
     return m_sounds.find(sound) != m_sounds.end();
 }
 
 bool ALSound::IsCachedMusic(const std::string &filename)
 {
-    return m_music.find("music/"+filename) != m_music.end();
+    return m_music.find(filename) != m_music.end();
 }
 
-int ALSound::GetPriority(Sound sound)
+int ALSound::GetPriority(SoundType sound)
 {
     if ( sound == SOUND_FLYh   ||
         sound == SOUND_FLY    ||
@@ -240,12 +227,12 @@ int ALSound::GetPriority(Sound sound)
 }
 
 
-bool ALSound::SearchFreeBuffer(Sound sound, int &channel, bool &bAlreadyLoaded)
+bool ALSound::SearchFreeBuffer(SoundType sound, int &channel, bool &alreadyLoaded)
 {
     int priority = GetPriority(sound);
 
     // Seeks a channel used which sound is stopped.
-    for (auto it : m_channels)
+    for (auto& it : m_channels)
     {
         if (it.second->IsPlaying())
         {
@@ -259,31 +246,30 @@ bool ALSound::SearchFreeBuffer(Sound sound, int &channel, bool &bAlreadyLoaded)
         it.second->SetPriority(priority);
         it.second->Reset();
         channel = it.first;
-        bAlreadyLoaded = it.second->IsLoaded();
+        alreadyLoaded = it.second->IsLoaded();
         return true;
     }
 
     // just add a new channel if we dont have any
     if (m_channels.size() == 0)
     {
-        Channel *chn = new Channel();
+        auto chn = MakeUnique<Channel>();
         // check if we channel ready to play music, if not report error
         if (chn->IsReady())
         {
             chn->SetPriority(priority);
             chn->Reset();
-            m_channels[1] = chn;
             channel = 1;
-            bAlreadyLoaded = false;
+            m_channels[channel] = std::move(chn);
+            alreadyLoaded = false;
             return true;
         }
-        delete chn;
         GetLogger()->Error("Could not open channel to play sound!\n");
         return false;
     }
 
     // Assigns new channel within limit
-    if (m_channels.size() < m_channels_limit)
+    if (m_channels.size() < m_channelsLimit)
     {
         auto it = m_channels.end();
         it--;
@@ -292,18 +278,17 @@ bool ALSound::SearchFreeBuffer(Sound sound, int &channel, bool &bAlreadyLoaded)
         {
             if (m_channels.find(i) == m_channels.end())
             {
-                Channel *chn = new Channel();
+                auto chn = MakeUnique<Channel>();
                 // check if channel is ready to play music, if not destroy it and seek free one
                 if (chn->IsReady())
                 {
                     chn->SetPriority(priority);
                     chn->Reset();
-                    m_channels[++i] = chn;
+                    m_channels[i] = std::move(chn);
                     channel = i;
-                    bAlreadyLoaded = false;
+                    alreadyLoaded = false;
                     return true;
                 }
-                delete chn;
                 GetLogger()->Debug("Could not open additional channel to play sound!\n");
                 break;
             }
@@ -311,7 +296,7 @@ bool ALSound::SearchFreeBuffer(Sound sound, int &channel, bool &bAlreadyLoaded)
     }
 
     int lowerOrEqual = -1;
-    for (auto it : m_channels)
+    for (auto& it : m_channels)
     {
         if (it.second->GetPriority() < priority)
         {
@@ -337,13 +322,13 @@ bool ALSound::SearchFreeBuffer(Sound sound, int &channel, bool &bAlreadyLoaded)
 }
 
 
-int ALSound::Play(Sound sound, float amplitude, float frequency, bool bLoop)
+int ALSound::Play(SoundType sound, float amplitude, float frequency, bool loop)
 {
-    return Play(sound, m_eye, amplitude, frequency, bLoop);
+    return Play(sound, m_eye, amplitude, frequency, loop);
 }
 
 
-int ALSound::Play(Sound sound, const Math::Vector &pos, float amplitude, float frequency, bool bLoop)
+int ALSound::Play(SoundType sound, const Math::Vector &pos, float amplitude, float frequency, bool loop)
 {
     if (!m_enabled)
     {
@@ -356,46 +341,45 @@ int ALSound::Play(Sound sound, const Math::Vector &pos, float amplitude, float f
     }
 
     int channel;
-    bool bAlreadyLoaded = false;
-    if (!SearchFreeBuffer(sound, channel, bAlreadyLoaded))
+    bool alreadyLoaded = false;
+    if (!SearchFreeBuffer(sound, channel, alreadyLoaded))
     {
         return -1;
     }
 
-    if (!bAlreadyLoaded)
+    if (!alreadyLoaded)
     {
-        if (!m_channels[channel]->SetBuffer(m_sounds[sound]))
+        if (!m_channels[channel]->SetBuffer(m_sounds[sound].get()))
         {
             m_channels[channel]->SetBuffer(nullptr);
             return -1;
         }
     }
 
-    m_channels[channel]->SetPosition(pos);
-    m_channels[channel]->SetVolumeAtrib(1.0f);
+    Channel* chn = m_channels[channel].get();
+
+    chn->SetPosition(pos);
+    chn->SetVolumeAtrib(1.0f);
 
     // setting initial values
-    m_channels[channel]->SetStartAmplitude(amplitude);
-    m_channels[channel]->SetStartFrequency(frequency);
-    m_channels[channel]->SetChangeFrequency(1.0f);
-    m_channels[channel]->ResetOper();
-    m_channels[channel]->SetFrequency(frequency);
-    m_channels[channel]->SetVolume(powf(amplitude * m_channels[channel]->GetVolumeAtrib(), 0.2f) * m_audioVolume);
-    m_channels[channel]->SetLoop(bLoop);
+    chn->SetStartAmplitude(amplitude);
+    chn->SetStartFrequency(frequency);
+    chn->SetChangeFrequency(1.0f);
+    chn->ResetOper();
+    chn->SetFrequency(frequency);
+    chn->SetVolume(powf(amplitude * chn->GetVolumeAtrib(), 0.2f) * m_audioVolume);
+    chn->SetLoop(loop);
 
-    if (!m_channels[channel]->Play())
+    if (!chn->Play())
     {
-        m_channels_limit = m_channels.size() - 1;
-        GetLogger()->Debug("Changing channel limit to %u.\n", m_channels_limit);
-        auto it = m_channels.find(channel);
-        Channel *ch = it->second;
-        m_channels.erase(it);
-        delete ch;
+        m_channelsLimit = m_channels.size() - 1;
+        GetLogger()->Debug("Changing channel limit to %u.\n", m_channelsLimit);
+        m_channels.erase(channel);
 
         return -1;
     }
 
-    return channel | ((m_channels[channel]->GetId() & 0xffff) << 16);
+    return channel | ((chn->GetId() & 0xffff) << 16);
 }
 
 
@@ -475,7 +459,7 @@ bool ALSound::StopAll()
         return false;
     }
 
-    for (auto channel : m_channels)
+    for (auto& channel : m_channels)
     {
         channel.second->Stop();
         channel.second->ResetOper();
@@ -485,18 +469,18 @@ bool ALSound::StopAll()
 }
 
 
-bool ALSound::MuteAll(bool bMute)
+bool ALSound::MuteAll(bool mute)
 {
     if (!m_enabled)
     {
         return false;
     }
 
-    for (auto it : m_channels)
+    for (auto& it : m_channels)
     {
         if (it.second->IsPlaying())
         {
-            it.second->Mute(bMute);
+            it.second->Mute(mute);
         }
     }
 
@@ -513,7 +497,7 @@ void ALSound::FrameMove(float delta)
 
     float progress;
     float volume, frequency;
-    for (auto it : m_channels)
+    for (auto& it : m_channels)
     {
         if (!it.second->IsPlaying())
         {
@@ -566,24 +550,24 @@ void ALSound::FrameMove(float delta)
             }
         }
     }
-    
-    std::list<OldMusic> toRemove;
-    
-    for (auto& it : m_oldMusic)
+
+    auto it = m_oldMusic.begin();
+    while (it != m_oldMusic.end())
     {
-        if (it.currentTime >= it.fadeTime)
+        if (it->currentTime >= it->fadeTime)
         {
-            delete it.music;
-            toRemove.push_back(it);
+            it = m_oldMusic.erase(it);
         }
         else
         {
-            it.currentTime += delta;
-            it.music->SetVolume(((it.fadeTime-it.currentTime) / it.fadeTime) * m_musicVolume);
+            it->currentTime += delta;
+            it->music->SetVolume(((it->fadeTime-it->currentTime) / it->fadeTime) * m_musicVolume);
+            ++it;
         }
     }
-    
-    if (m_previousMusic.fadeTime > 0.0f) {
+
+    if (m_previousMusic.fadeTime > 0.0f)
+    {
         if (m_previousMusic.currentTime >= m_previousMusic.fadeTime)
         {
             m_previousMusic.music->Pause();
@@ -594,9 +578,6 @@ void ALSound::FrameMove(float delta)
             m_previousMusic.music->SetVolume(((m_previousMusic.fadeTime-m_previousMusic.currentTime) / m_previousMusic.fadeTime) * m_musicVolume);
         }
     }
-    
-    for (auto it : toRemove)
-        m_oldMusic.remove(it);
 }
 
 
@@ -613,71 +594,52 @@ void ALSound::SetListener(const Math::Vector &eye, const Math::Vector &lookat)
 }
 
 
-bool ALSound::PlayMusic(int rank, bool bRepeat, float fadeTime)
-{
-    std::stringstream filename;
-    filename << "music" << std::setfill('0') << std::setw(3) << rank << ".ogg";
-    return PlayMusic(filename.str(), bRepeat, fadeTime);
-}
-
-
-bool operator<(const OldMusic & l, const OldMusic & r)
-{
-    return l.currentTime < r.currentTime;
-}
-
-
-bool operator==(const OldMusic & l, const OldMusic & r)
-{
-    return l.currentTime == r.currentTime;
-}
-
-
-bool ALSound::PlayMusic(const std::string &filename, bool bRepeat, float fadeTime)
+bool ALSound::PlayMusic(const std::string &filename, bool repeat, float fadeTime)
 {
     if (!m_enabled)
     {
         return false;
     }
 
-    Buffer *buffer;
+    Buffer *buffer = nullptr;
 
     // check if we have music in cache
-    if (m_music.find("music/"+filename) == m_music.end())
+    if (m_music.find(filename) == m_music.end())
     {
         GetLogger()->Debug("Music %s was not cached!\n", filename.c_str());
-        /* TODO: if (!boost::filesystem::exists("music/"+filename))
+        /* TODO: if (!boost::filesystem::exists(filename))
         {
             GetLogger()->Debug("Requested music %s was not found.\n", filename.c_str());
             return false;
         } */
 
-        buffer = new Buffer();
-        if (!buffer->LoadFromFile("music/"+filename, static_cast<Sound>(-1)))
+        auto newBuffer = MakeUnique<Buffer>();
+        buffer = newBuffer.get();
+        if (!newBuffer->LoadFromFile(filename, static_cast<SoundType>(-1)))
         {
             return false;
         }
-        m_music["music/"+filename] = buffer;
+        m_music[filename] = std::move(newBuffer);
     }
     else
     {
         GetLogger()->Debug("Music loaded from cache\n");
-        buffer = m_music["music/"+filename];
+        buffer = m_music[filename].get();
     }
 
     if (m_currentMusic)
     {
         OldMusic old;
-        old.music = m_currentMusic;
+        old.music = std::move(m_currentMusic);
         old.fadeTime = fadeTime;
         old.currentTime = 0.0f;
-        m_oldMusic.push_back(old);
+        m_oldMusic.push_back(std::move(old));
     }
 
-    m_currentMusic = new Channel();
+    m_currentMusic = MakeUnique<Channel>();
     m_currentMusic->SetBuffer(buffer);
     m_currentMusic->SetVolume(m_musicVolume);
-    m_currentMusic->SetLoop(bRepeat);
+    m_currentMusic->SetLoop(repeat);
     m_currentMusic->Play();
 
     return true;
@@ -688,24 +650,22 @@ bool ALSound::PlayPauseMusic(const std::string &filename, bool repeat)
 {
     if (m_previousMusic.fadeTime > 0.0f)
     {
-		if(m_currentMusic)
-		{
-			OldMusic old;
-			old.music = m_currentMusic;
-			old.fadeTime = 2.0f;
-			old.currentTime = 0.0f;
-			m_oldMusic.push_back(old);
-			m_currentMusic = nullptr;
-		}
+        if (m_currentMusic != nullptr)
+        {
+            OldMusic old;
+            old.music = std::move(m_currentMusic);
+            old.fadeTime = 2.0f;
+            old.currentTime = 0.0f;
+            m_oldMusic.push_back(std::move(old));
+        }
     }
     else
     {
-        if (m_currentMusic)
+        if (m_currentMusic != nullptr)
         {
-            m_previousMusic.music = m_currentMusic;
+            m_previousMusic.music = std::move(m_currentMusic);
             m_previousMusic.fadeTime = 2.0f;
             m_previousMusic.currentTime = 0.0f;
-            m_currentMusic = nullptr;
         }
     }
     return PlayMusic(filename, repeat);
@@ -718,12 +678,11 @@ void ALSound::StopPauseMusic()
     {
         StopMusic();
 
-        m_currentMusic = m_previousMusic.music;
-        m_previousMusic.music = nullptr;
-        if(m_currentMusic != nullptr)
+        m_currentMusic = std::move(m_previousMusic.music);
+        if (m_currentMusic != nullptr)
         {
             m_currentMusic->SetVolume(m_musicVolume);
-            if(m_previousMusic.currentTime >= m_previousMusic.fadeTime)
+            if (m_previousMusic.currentTime >= m_previousMusic.fadeTime)
             {
                 m_currentMusic->Play();
             }
@@ -735,7 +694,7 @@ void ALSound::StopPauseMusic()
 
 bool ALSound::RestartMusic()
 {
-    if (!m_enabled || !m_currentMusic)
+    if (!m_enabled || m_currentMusic == nullptr)
     {
         return false;
     }
@@ -748,24 +707,22 @@ bool ALSound::RestartMusic()
 
 void ALSound::StopMusic(float fadeTime)
 {
-    if (!m_enabled || !m_currentMusic)
+    if (!m_enabled || m_currentMusic == nullptr)
     {
         return;
     }
 
     OldMusic old;
-    old.music = m_currentMusic;
+    old.music = std::move(m_currentMusic);
     old.fadeTime = fadeTime;
     old.currentTime = 0.0f;
-    m_oldMusic.push_back(old);
-    
-    m_currentMusic = nullptr;
+    m_oldMusic.push_back(std::move(old));
 }
 
 
 bool ALSound::IsPlayingMusic()
 {
-    if (!m_enabled || !m_currentMusic)
+    if (!m_enabled || m_currentMusic == nullptr)
     {
         return false;
     }
@@ -776,7 +733,7 @@ bool ALSound::IsPlayingMusic()
 
 void ALSound::SuspendMusic()
 {
-    if (!m_enabled || !m_currentMusic)
+    if (!m_enabled || m_currentMusic == nullptr)
     {
         return;
     }
