@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,16 +25,12 @@
 #pragma once
 
 #include "sound/sound.h"
-#include "common/logger.h"
-
-#include "sound/oalsound/check.h"
 
 #include <string>
 #include <vector>
 #include <array>
 
 #include <al.h>
-#include <sndfile.h>
 
 class Buffer
 {
@@ -42,16 +38,16 @@ public:
     Buffer();
     ~Buffer();
 
-    bool LoadFromFile(std::string, Sound);
+    bool LoadFromFile(std::string, SoundType);
     bool IsLoaded();
 
-    Sound GetSoundType();
+    SoundType GetSoundType();
     ALuint GetBuffer();
     float GetDuration();
 
 private:
     ALuint m_buffer;
-    Sound m_sound;
+    SoundType m_sound;
     bool m_loaded;
     float m_duration;
 };
