@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,27 +17,11 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// check.h
-
 #pragma once
 
 #include "common/logger.h"
 
 #include <al.h>
-#include <alc.h>
-
-static ALenum CODE = AL_NO_ERROR;
-
-inline bool alCheck()
-{
-    CODE = alGetError();
-    return CODE != AL_NO_ERROR;
-}
-
-inline ALenum alGetCode()
-{
-    ALenum ret = CODE;
-    CODE = AL_NO_ERROR;
-    return ret;
-}
 
+bool CheckOpenALError();
+ALenum GetOpenALErrorCode();
