@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,35 +21,43 @@
 #include "ui/displayinfo.h"
 
 #include "app/app.h"
+#include "app/pausemanager.h"
 
 #include "common/misc.h"
 #include "common/restext.h"
+#include "common/settings.h"
 #include "common/stringutils.h"
 
 #include "graphics/core/light.h"
+
 #include "graphics/engine/engine.h"
 #include "graphics/engine/lightman.h"
 #include "graphics/engine/particle.h"
 
+#include "level/robotmain.h"
+
 #include "object/object.h"
-#include "object/objman.h"
-#include "object/robotmain.h"
+#include "object/object_manager.h"
+
+#include "object/interface/movable_object.h"
+
 #include "object/motion/motion.h"
 #include "object/motion/motiontoto.h"
 
 #include "script/cbottoken.h"
 
-#include "ui/interface.h"
-#include "ui/button.h"
-#include "ui/slider.h"
-#include "ui/edit.h"
-#include "ui/group.h"
-#include "ui/window.h"
+#include "ui/controls/button.h"
+#include "ui/controls/edit.h"
+#include "ui/controls/group.h"
+#include "ui/controls/interface.h"
+#include "ui/controls/slider.h"
+#include "ui/controls/window.h"
 
 #include <string.h>
 
 
-namespace Ui {
+namespace Ui
+{
 // Object's constructor.
 
 CDisplayInfo::CDisplayInfo()
@@ -61,7 +69,7 @@ CDisplayInfo::CDisplayInfo()
     m_main      = CRobotMain::GetInstancePointer();
     m_interface = m_main->GetInterface();
     m_camera    = m_main->GetCamera();
-    m_pause     = CPauseManager::GetInstancePointer();
+    m_pause     = m_engine->GetPauseManager();
 
     m_bInfoMaximized = true;
     m_bInfoMinimized = false;
@@ -70,7 +78,11 @@ CDisplayInfo::CDisplayInfo()
     m_infoFinalDim = m_infoActualPos = m_infoNormalDim = Math::Point(1.00f, 1.00f);
 
     m_lightSuppl = -1;
-    m_toto = 0;
+    m_toto = nullptr;
+    m_bSoluce = false;
+    m_bEditLock = false;
+    m_infoCamera = Gfx::CAM_TYPE_NULL;
+    m_index = -1;
 }
 
 // Object's destructor.
@@ -87,7 +99,6 @@ bool CDisplayInfo::EventProcess(const Event &event)
     Ui::CWindow*        pw;
     Ui::CEdit*          edit;
     Ui::CSlider*        slider;
-    CMotionToto*    toto;
 
     if ( event.type == EVENT_FRAME )
     {
@@ -97,24 +108,21 @@ bool CDisplayInfo::EventProcess(const Event &event)
 
     if ( event.type == EVENT_MOUSE_MOVE )
     {
-        if ( m_toto != 0 )
+        if ( m_toto != nullptr )
         {
-            toto = static_cast<CMotionToto*>(m_toto->GetMotion());
-            if ( toto != 0 )
-            {
-                toto->SetMousePos(event.mousePos);
-            }
+            assert(m_toto->Implements(ObjectInterfaceType::Movable));
+            CMotionToto* toto = static_cast<CMotionToto*>(dynamic_cast<CMovableObject*>(m_toto)->GetMotion());
+            assert(toto != nullptr);
+            toto->SetMousePos(event.mousePos);
         }
     }
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw != 0 )
+    if ( pw != nullptr )
     {
         if ( event.type == pw->GetEventTypeClose() )
         {
-            Event newEvent = event;
-            newEvent.type = EVENT_OBJECT_INFOOK;
-            m_event->AddEvent(newEvent);
+            m_event->AddEvent(Event(EVENT_OBJECT_INFOOK));
         }
 
         if ( event.type == EVENT_SATCOM_HUSTON )
@@ -147,7 +155,7 @@ bool CDisplayInfo::EventProcess(const Event &event)
              event.type == EVENT_HYPER_NEXT )
         {
             edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-            if ( edit != 0 )
+            if ( edit != nullptr )
             {
                 edit->HyperGo(event.type);
                 HyperUpdate();
@@ -156,52 +164,52 @@ bool CDisplayInfo::EventProcess(const Event &event)
 
         if ( event.type == EVENT_HYPER_SIZE1 )  // size 1?
         {
-            m_main->SetFontSize(9.0f);
+            CSettings::GetInstancePointer()->SetFontSize(9.0f);
             slider = static_cast<Ui::CSlider*>(pw->SearchControl(EVENT_STUDIO_SIZE));
-            if ( slider != 0 )  slider->SetVisibleValue((m_main->GetFontSize()-9.0f)/15.0f);
+            if ( slider != nullptr )  slider->SetVisibleValue((CSettings::GetInstancePointer()->GetFontSize()-9.0f)/15.0f);
             ViewDisplayInfo();
         }
         if ( event.type == EVENT_HYPER_SIZE2 )  // size 2?
         {
-            m_main->SetFontSize(14.0f);
+            CSettings::GetInstancePointer()->SetFontSize(14.0f);
             slider = static_cast<Ui::CSlider*>(pw->SearchControl(EVENT_STUDIO_SIZE));
-            if ( slider != 0 )  slider->SetVisibleValue((m_main->GetFontSize()-9.0f)/15.0f);
+            if ( slider != nullptr )  slider->SetVisibleValue((CSettings::GetInstancePointer()->GetFontSize()-9.0f)/15.0f);
             ViewDisplayInfo();
         }
         if ( event.type == EVENT_HYPER_SIZE3 )  // size 3?
         {
-            m_main->SetFontSize(19.0f);
+            CSettings::GetInstancePointer()->SetFontSize(19.0f);
             slider = static_cast<Ui::CSlider*>(pw->SearchControl(EVENT_STUDIO_SIZE));
-            if ( slider != 0 )  slider->SetVisibleValue((m_main->GetFontSize()-9.0f)/15.0f);
+            if ( slider != nullptr )  slider->SetVisibleValue((CSettings::GetInstancePointer()->GetFontSize()-9.0f)/15.0f);
             ViewDisplayInfo();
         }
         if ( event.type == EVENT_HYPER_SIZE4 )  // size 4?
         {
-            m_main->SetFontSize(24.0f);
+            CSettings::GetInstancePointer()->SetFontSize(24.0f);
             slider = static_cast<Ui::CSlider*>(pw->SearchControl(EVENT_STUDIO_SIZE));
-            if ( slider != 0 )  slider->SetVisibleValue((m_main->GetFontSize()-9.0f)/15.0f);
+            if ( slider != nullptr )  slider->SetVisibleValue((CSettings::GetInstancePointer()->GetFontSize()-9.0f)/15.0f);
             ViewDisplayInfo();
         }
 
         if ( event.type == EVENT_STUDIO_SIZE )  // size?
         {
             slider = static_cast<Ui::CSlider*>(pw->SearchControl(EVENT_STUDIO_SIZE));
-            if ( slider == 0 )  return false;
-            m_main->SetFontSize(9.0f+slider->GetVisibleValue()*15.0f);
+            if ( slider == nullptr )  return false;
+            CSettings::GetInstancePointer()->SetFontSize(9.0f+slider->GetVisibleValue()*15.0f);
             ViewDisplayInfo();
         }
 
         if ( event.type == EVENT_HYPER_COPY )  // copy ?
         {
             edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-            if ( edit != 0 )
+            if ( edit != nullptr )
             {
                 edit->Copy();
             }
         }
 
-        if ( ( event.type == EVENT_MOUSE_BUTTON_DOWN && event.mouseButton.button == MOUSE_BUTTON_LEFT )||
-             ( event.type == EVENT_MOUSE_BUTTON_UP && event.mouseButton.button == MOUSE_BUTTON_LEFT ))
+        if ((event.type == EVENT_MOUSE_BUTTON_DOWN || event.type == EVENT_MOUSE_BUTTON_UP) &&
+             event.GetData<MouseButtonEventData>()->button == MOUSE_BUTTON_LEFT)
         {
             UpdateCopyButton();
         }
@@ -232,7 +240,7 @@ bool CDisplayInfo::EventProcess(const Event &event)
             }
 //?         m_main->SetEditFull(m_bInfoMaximized);
             pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-            if ( pw != 0 )
+            if ( pw != nullptr )
             {
                 pw->SetMaximized(m_bInfoMaximized);
                 pw->SetMinimized(m_bInfoMinimized);
@@ -258,7 +266,7 @@ bool CDisplayInfo::EventProcess(const Event &event)
             }
 //?         m_main->SetEditFull(m_bInfoMaximized);
             pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-            if ( pw != 0 )
+            if ( pw != nullptr )
             {
                 pw->SetMaximized(m_bInfoMaximized);
                 pw->SetMinimized(m_bInfoMinimized);
@@ -302,26 +310,26 @@ void CDisplayInfo::HyperUpdate()
     bool            bEnable;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit == 0 )  return;
+    if ( edit == nullptr )  return;
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_HOME));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         bEnable = edit->HyperTest(EVENT_HYPER_HOME);
         button->SetState(STATE_ENABLE, bEnable);
     }
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_PREV));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         bEnable = edit->HyperTest(EVENT_HYPER_PREV);
         button->SetState(STATE_ENABLE, bEnable);
     }
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_NEXT));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         bEnable = edit->HyperTest(EVENT_HYPER_NEXT);
         button->SetState(STATE_ENABLE, bEnable);
@@ -339,7 +347,6 @@ void CDisplayInfo::StartDisplayInfo(std::string filename, int index, bool bSoluc
     Ui::CEdit*          edit;
     Ui::CButton*        button;
     Ui::CSlider*        slider;
-    CMotionToto*        toto;
 
     m_index = index;
     m_bSoluce = bSoluce;
@@ -348,7 +355,7 @@ void CDisplayInfo::StartDisplayInfo(std::string filename, int index, bool bSoluc
     if ( m_bEditLock )  // edition running program?
     {
         pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW3));
-        if ( pw != 0 )
+        if ( pw != nullptr )
         {
             pw->ClearState(STATE_ENABLE);  // CStudio inactive
         }
@@ -356,15 +363,17 @@ void CDisplayInfo::StartDisplayInfo(std::string filename, int index, bool bSoluc
 
     m_main->SetEditLock(true, false);
     m_main->SetEditFull(false);
-    m_bInitPause = m_pause->GetPauseType();
-    m_pause->SetPause(PAUSE_SATCOM);
+    m_satcomPause = m_pause->ActivatePause(PAUSE_SATCOM);
     m_infoCamera = m_camera->GetType();
     m_camera->SetType(Gfx::CAM_TYPE_INFO);
 
+    pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW6));
+    if (pw != nullptr) pw->ClearState(STATE_VISIBLE);
+
     pos = m_infoActualPos = m_infoFinalPos;
     dim = m_infoActualDim = m_infoFinalDim;
     pw = m_interface->CreateWindows(pos, dim, 4, EVENT_WINDOW4);
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
 //? pw->SetClosable(true);
 //? GetResource(RES_TEXT, RT_DISINFO_TITLE, res);
 //? pw->SetName(res);
@@ -374,7 +383,7 @@ void CDisplayInfo::StartDisplayInfo(std::string filename, int index, bool bSoluc
 //? m_main->SetEditFull(m_bInfoMaximized);
 
     edit = pw->CreateEdit(pos, dim, 0, EVENT_EDIT1);
-    if ( edit == 0 )  return;
+    if ( edit == nullptr )  return;
     edit->SetState(STATE_SHADOW);
     edit->SetMultiFont(true);
     edit->SetMaxChar(10000);
@@ -384,7 +393,7 @@ void CDisplayInfo::StartDisplayInfo(std::string filename, int index, bool bSoluc
     edit->HyperHome(filename.c_str());
     edit->SetEditCap(false);  // just to see!
     edit->SetHighlightCap(false);
-    edit->SetFocus(true);
+    m_interface->SetFocus(edit);
 
     ViewDisplayInfo();
 
@@ -420,7 +429,7 @@ void CDisplayInfo::StartDisplayInfo(std::string filename, int index, bool bSoluc
     button->SetState(STATE_SHADOW);
     slider = pw->CreateSlider(pos, dim, 0, EVENT_STUDIO_SIZE);
     slider->SetState(STATE_SHADOW);
-    slider->SetVisibleValue((m_main->GetFontSize()-9.0f)/15.0f);
+    slider->SetVisibleValue((CSettings::GetInstancePointer()->GetFontSize()-9.0f)/15.0f);
     button = pw->CreateButton(pos, dim, 61, EVENT_HYPER_COPY);
     button->SetState(STATE_SHADOW);
     HyperUpdate();
@@ -439,15 +448,14 @@ void CDisplayInfo::StartDisplayInfo(std::string filename, int index, bool bSoluc
     m_particle->SetFrameUpdate(Gfx::SH_WORLD, false);  // particles break into world
 
     m_toto = SearchToto();
-    if ( m_toto != 0 )
+    if ( m_toto != nullptr )
     {
         m_toto->SetDrawFront(true);
 
-        toto = static_cast<CMotionToto*>(m_toto->GetMotion());
-        if ( toto != 0 )
-        {
-            toto->StartDisplayInfo();
-        }
+        assert(m_toto->Implements(ObjectInterfaceType::Movable));
+        CMotionToto* toto = static_cast<CMotionToto*>(dynamic_cast<CMovableObject*>(m_toto)->GetMotion());
+        assert(toto != nullptr);
+        toto->StartDisplayInfo();
     }
 
     light.type    = Gfx::LIGHT_DIRECTIONAL;
@@ -477,7 +485,7 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     wdim.y = 420.0f / 480.0f;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw != 0 )
+    if ( pw != nullptr )
     {
         pw->SetPos(wpos);
         pw->SetDim(wdim);
@@ -489,14 +497,14 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     dim.x = 48.0f / 640.0f;
     dim.y = 48.0f / 480.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_HUSTON));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.y -= (48.0f + 4.0f) / 480.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_SAT));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
@@ -510,21 +518,21 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
 //? }
     pos.y -= (48.0f + 4.0f) / 480.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_LOADING));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.y -= (48.0f + 4.0f) / 480.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_PROG));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.y -= (48.0f + 4.0f) / 480.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_SOLUCE));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
@@ -535,7 +543,7 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     dim.x = (48.0f - 10.0f) / 640.0f;
     dim.y = 24.0f / 480.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_OBJECT_INFOOK));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
@@ -546,7 +554,7 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     dim.x = 462.0f / 640.0f;
     dim.y = 358.0f / 480.0f;
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit != 0 )
+    if ( edit != nullptr )
     {
         edit->SetPos(pos);
         edit->SetDim(dim);
@@ -557,21 +565,21 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     dim.x = 32.0f / 640.0f;
     dim.y = 32.0f / 480.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_PREV));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.x += 35.0f / 640.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_NEXT));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.x += 35.0f / 640.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_HOME));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
@@ -579,28 +587,28 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
 
     pos.x += 50.0f / 640.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_SIZE1));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.x += 35.0f / 640.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_SIZE2));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.x += 35.0f / 640.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_SIZE3));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
     }
     pos.x += 35.0f / 640.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_SIZE4));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
@@ -608,7 +616,7 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     pos.x += 35.0f / 640.0f;
     dim.x = 18.0f / 640.0f;
     slider = static_cast<Ui::CSlider*>(pw->SearchControl(EVENT_STUDIO_SIZE));
-    if ( slider != 0 )
+    if ( slider != nullptr )
     {
         slider->SetPos(pos);
         slider->SetDim(dim);
@@ -616,7 +624,7 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     pos.x += 50.0f / 640.0f;
     dim.x = 32.0f / 640.0f;
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_COPY));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetPos(pos);
         button->SetDim(dim);
@@ -627,7 +635,7 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     dim.x = 48.0f / 640.0f;
     dim.y = 40.0f / 480.0f;
     group = static_cast<Ui::CGroup*>(pw->SearchControl(EVENT_LABEL2));  // symbol SatCom
-    if ( group != 0 )
+    if ( group != nullptr )
     {
         group->SetPos(pos);
         group->SetDim(dim);
@@ -638,7 +646,7 @@ void CDisplayInfo::AdjustDisplayInfo(Math::Point wpos, Math::Point wdim)
     dim.x = 20.0f / 640.0f;
     dim.y = 20.0f / 480.0f;
     group = static_cast<Ui::CGroup*>(pw->SearchControl(EVENT_LABEL3));  // symbol stand-by
-    if ( group != 0 )
+    if ( group != nullptr )
     {
         group->SetPos(pos);
         group->SetDim(dim);
@@ -654,7 +662,7 @@ void CDisplayInfo::ChangeIndexButton(int index)
     char*       filename;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
 
     if ( m_index != -1 )
     {
@@ -663,7 +671,7 @@ void CDisplayInfo::ChangeIndexButton(int index)
     m_index = index;
 
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit != 0 )
+    if ( edit != nullptr )
     {
         filename = m_main->GetDisplayInfoName(m_index);
         edit->ReadText(filename);
@@ -696,10 +704,10 @@ void CDisplayInfo::UpdateIndexButton()
     };
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_HUSTON));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetState(STATE_CHECK, m_index==SATCOM_HUSTON);
         filename = m_main->GetDisplayInfoName(SATCOM_HUSTON);
@@ -707,7 +715,7 @@ void CDisplayInfo::UpdateIndexButton()
     }
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_SAT));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetState(STATE_CHECK, m_index==SATCOM_SAT);
         filename = m_main->GetDisplayInfoName(SATCOM_SAT);
@@ -723,7 +731,7 @@ void CDisplayInfo::UpdateIndexButton()
 //? }
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_LOADING));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetState(STATE_CHECK, m_index==SATCOM_LOADING);
         filename = m_main->GetDisplayInfoName(SATCOM_LOADING);
@@ -731,7 +739,7 @@ void CDisplayInfo::UpdateIndexButton()
     }
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_PROG));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetState(STATE_CHECK, m_index==SATCOM_PROG);
         filename = m_main->GetDisplayInfoName(SATCOM_PROG);
@@ -739,7 +747,7 @@ void CDisplayInfo::UpdateIndexButton()
     }
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_SATCOM_SOLUCE));
-    if ( button != 0 )
+    if ( button != nullptr )
     {
         button->SetState(STATE_CHECK, m_index==SATCOM_SOLUCE);
         filename = m_main->GetDisplayInfoName(SATCOM_SOLUCE);
@@ -747,7 +755,7 @@ void CDisplayInfo::UpdateIndexButton()
     }
 
     group = static_cast<Ui::CGroup*>(pw->SearchControl(EVENT_LABEL1));
-    if ( group != 0 )
+    if ( group != nullptr )
     {
         if ( m_index == -1 )
         {
@@ -767,16 +775,8 @@ void CDisplayInfo::UpdateIndexButton()
         }
     }
 
-#if 0
-    button = (CButton*)pw->SearchControl(EVENT_HYPER_COPY);
-    if ( button != 0 )
-    {
-        button->SetState(STATE_VISIBLE, m_index==SATCOM_LOADING);
-    }
-#endif
-
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit != 0 )
+    if ( edit != nullptr )
     {
 //?     edit->SetHiliteCap(m_index==SATCOM_LOADING);
         edit->SetHighlightCap(true);
@@ -797,13 +797,13 @@ void CDisplayInfo::UpdateCopyButton()
 //? if ( m_index != SATCOM_LOADING )  return;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
 
     button = static_cast<Ui::CButton*>(pw->SearchControl(EVENT_HYPER_COPY));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
 
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit == 0 )  return;
+    if ( edit == nullptr )  return;
 
     edit->GetCursor(c1, c2);
     button->SetState(STATE_ENABLE, c1!=c2);
@@ -815,26 +815,29 @@ void CDisplayInfo::UpdateCopyButton()
 void CDisplayInfo::StopDisplayInfo()
 {
     Ui::CWindow*        pw;
-    CMotionToto*    toto;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
 
     m_interface->DeleteControl(EVENT_WINDOW4);
 
     if ( m_bEditLock )  // editing running program?
     {
         pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW3));
-        if ( pw != 0 )
+        if ( pw != nullptr )
         {
             pw->SetState(STATE_ENABLE);  // CStudio operating
         }
     }
     else
     {
-        m_pause->SetPause(m_bInitPause);
         m_main->SetEditLock(false, false);
+
+        pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW6));
+        if (pw != nullptr) pw->SetState(STATE_VISIBLE);
     }
+    m_pause->DeactivatePause(m_satcomPause);
+    m_satcomPause = nullptr;
     m_camera->SetType(m_infoCamera);
 
     m_engine->SetDrawWorld(true);  // draws all on the interface
@@ -843,13 +846,12 @@ void CDisplayInfo::StopDisplayInfo()
     m_particle->FlushParticle(Gfx::SH_FRONT);
     m_particle->FlushParticle(Gfx::SH_INTERFACE);
 
-    if ( m_toto != 0 )
+    if ( m_toto != nullptr )
     {
-        toto = static_cast<CMotionToto*>(m_toto->GetMotion());
-        if ( toto != 0 )
-        {
-            toto->StopDisplayInfo();
-        }
+        assert(m_toto->Implements(ObjectInterfaceType::Movable));
+        CMotionToto* toto = static_cast<CMotionToto*>(dynamic_cast<CMovableObject*>(m_toto)->GetMotion());
+        assert(toto != nullptr);
+        toto->StopDisplayInfo();
     }
 
     m_light->DeleteLight(m_lightSuppl);
@@ -865,10 +867,10 @@ void CDisplayInfo::SetPosition(int pos)
     Ui::CEdit*          edit;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
 
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit == 0 )  return;
+    if ( edit == nullptr )  return;
 
     edit->SetFirstLine(pos);
 }
@@ -881,10 +883,10 @@ int CDisplayInfo::GetPosition()
     Ui::CEdit*          edit;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return 0;
+    if ( pw == nullptr )  return 0;
 
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit == 0 )  return 0;
+    if ( edit == nullptr )  return 0;
 
     return edit->GetFirstLine();
 }
@@ -900,13 +902,13 @@ void CDisplayInfo::ViewDisplayInfo()
     Math::IntPoint       dim;
 
     pw = static_cast<Ui::CWindow*>(m_interface->SearchControl(EVENT_WINDOW4));
-    if ( pw == 0 )  return;
+    if ( pw == nullptr )  return;
 
     edit = static_cast<Ui::CEdit*>(pw->SearchControl(EVENT_EDIT1));
-    if ( edit == 0 )  return;
+    if ( edit == nullptr )  return;
 
     dim = m_engine->GetWindowSize();
-    edit->SetFontSize(m_main->GetFontSize()/(dim.x / 640.0f));
+    edit->SetFontSize(CSettings::GetInstancePointer()->GetFontSize()/(dim.x / 640.0f));
 }
 
 // Returns the object human.
