@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,26 +17,23 @@
  * along with this program. If not, see http://gnu.org/licenses
  */
 
-// displaytext.h
-
 #pragma once
 
-
 #include "common/event.h"
-#include "common/global.h"
-
-#include "sound/sound.h"
-
+#include "common/error.h"
 
+#include <array>
 
 class CObject;
-class CSound;
+class CSoundInterface;
 
-namespace Gfx {
+namespace Gfx
+{
 class CEngine;
 }
 
-namespace Ui {
+namespace Ui
+{
 
 class CInterface;
 
@@ -90,11 +87,15 @@ protected:
     Ui::CInterface*     m_interface;
     CSoundInterface*         m_sound;
 
-    bool            m_bExist[MAXDTLINE];
-    float           m_time[MAXDTLINE];
-    Math::Vector        m_visitGoal[MAXDTLINE];
-    float           m_visitDist[MAXDTLINE];
-    float           m_visitHeight[MAXDTLINE];
+    struct TextLine
+    {
+        bool exist = false;
+        float time = 0.0f;
+        Math::Vector visitGoal;
+        float visitDist = 0.0f;
+        float visitHeight = 0.0f;
+    };
+    std::array<TextLine, MAXDTLINE> m_textLines;
 
     bool            m_bHide;
     bool            m_bEnable;
@@ -103,4 +104,3 @@ protected:
 
 
 } // namespace Ui
-
