@@ -1,7 +1,7 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
- * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,46 +23,50 @@
 #include "CBot/CBotDll.h"
 
 #include "app/app.h"
+#include "app/pausemanager.h"
+
+#include "common/event.h"
+#include "common/logger.h"
+#include "common/misc.h"
+#include "common/settings.h"
 
 #include "common/resources/resourcemanager.h"
 
-#include "common/event.h"
-#include "common/misc.h"
+#include "level/robotmain.h"
+#include "level/player_profile.h"
 
 #include "graphics/engine/camera.h"
 #include "graphics/engine/engine.h"
 
-#include "object/brain.h"
 #include "object/object.h"
 
+#include "object/interface/programmable_object.h"
+
 #include "script/cbottoken.h"
 #include "script/script.h"
 
 #include "sound/sound.h"
 
-#include "ui/check.h"
-#include "ui/control.h"
-#include "ui/color.h"
-#include "ui/compass.h"
-#include "ui/edit.h"
-#include "ui/group.h"
-#include "ui/image.h"
-#include "ui/interface.h"
-#include "ui/key.h"
-#include "ui/label.h"
-#include "ui/list.h"
-#include "ui/map.h"
-#include "ui/shortcut.h"
-#include "ui/target.h"
-#include "ui/window.h"
+#include "ui/controls/check.h"
+#include "ui/controls/color.h"
+#include "ui/controls/control.h"
+#include "ui/controls/edit.h"
+#include "ui/controls/group.h"
+#include "ui/controls/image.h"
+#include "ui/controls/interface.h"
+#include "ui/controls/key.h"
+#include "ui/controls/label.h"
+#include "ui/controls/list.h"
+#include "ui/controls/map.h"
+#include "ui/controls/shortcut.h"
+#include "ui/controls/target.h"
+#include "ui/controls/window.h"
 
 #include <stdio.h>
 
 
-const int MAX_FNAME = 255;
-
-
-namespace Ui {
+namespace Ui
+{
 
 
 // Object's constructor.
@@ -76,7 +80,11 @@ CStudio::CStudio()
     m_main      = CRobotMain::GetInstancePointer();
     m_interface = m_main->GetInterface();
     m_camera    = m_main->GetCamera();
-    m_pause     = CPauseManager::GetInstancePointer();
+    m_pause     = m_engine->GetPauseManager();
+    m_settings  = CSettings::GetInstancePointer();
+
+    m_program = nullptr;
+    m_script = nullptr;
 
     m_bEditMaximized = false;
     m_bEditMinimized = false;
@@ -85,8 +93,8 @@ CStudio::CStudio()
     m_bRealTime = true;
     m_bRunning  = false;
     m_fixInfoTextTime = 0.0f;
-    m_helpFilename[0] = 0;
     m_dialog = SD_NULL;
+    m_editCamera = Gfx::CAM_TYPE_NULL;
 }
 
 // Object's destructor.
@@ -122,9 +130,7 @@ bool CStudio::EventProcess(const Event &event)
 
     if ( event.type == pw->GetEventTypeClose() )
     {
-        Event newEvent = event;
-        newEvent.type = EVENT_STUDIO_OK;
-        m_event->AddEvent(newEvent);
+        m_event->AddEvent(Event(EVENT_STUDIO_OK));
     }
 
     if ( event.type == EVENT_STUDIO_EDIT )  // text modifief?
@@ -134,7 +140,7 @@ bool CStudio::EventProcess(const Event &event)
 
     if ( event.type == EVENT_STUDIO_LIST )  // list clicked?
     {
-        m_main->StartDisplayInfo(const_cast<char *>(m_helpFilename.c_str()), -1); // TODO change to std::string when RobotMain changes
+        m_main->StartDisplayInfo(m_helpFilename, -1);
     }
 
     if ( event.type == EVENT_STUDIO_NEW )  // new?
@@ -172,7 +178,7 @@ bool CStudio::EventProcess(const Event &event)
     {
         slider = static_cast< CSlider* >(pw->SearchControl(EVENT_STUDIO_SIZE));
         if ( slider == nullptr )  return false;
-        m_main->SetFontSize(9.0f+slider->GetVisibleValue()*15.0f);
+        m_settings->SetFontSize(9.0f+slider->GetVisibleValue()*15.0f);
         ViewEditScript();
     }
 
@@ -207,9 +213,7 @@ bool CStudio::EventProcess(const Event &event)
     {
         if ( m_script->IsRunning() )
         {
-            Event newEvent = event;
-            newEvent.type = EVENT_OBJECT_PROGSTOP;
-            m_event->AddEvent(newEvent);  // stop
+            m_event->AddEvent(Event(EVENT_OBJECT_PROGSTOP));
         }
         else
         {
@@ -217,9 +221,7 @@ bool CStudio::EventProcess(const Event &event)
             {
                 SetInfoText("", false);
 
-                Event newEvent = event;
-                newEvent.type = EVENT_OBJECT_PROGSTART;
-                m_event->AddEvent(newEvent);  // start
+                m_event->AddEvent(Event(EVENT_OBJECT_PROGSTART));
             }
             else
             {
@@ -240,23 +242,23 @@ bool CStudio::EventProcess(const Event &event)
 
     if ( event.type == EVENT_STUDIO_STEP )  // step?
     {
-        m_script->Step(event);
+        m_script->Step();
     }
 
     if ( event.type == EVENT_WINDOW3 )  // window is moved?
     {
         m_editActualPos = m_editFinalPos = pw->GetPos();
         m_editActualDim = m_editFinalDim = pw->GetDim();
-        m_main->SetWindowPos(m_editActualPos);
-        m_main->SetWindowDim(m_editActualDim);
+        m_settings->SetWindowPos(m_editActualPos);
+        m_settings->SetWindowDim(m_editActualDim);
         AdjustEditScript();
     }
     if ( event.type == pw->GetEventTypeReduce() )
     {
         if ( m_bEditMinimized )
         {
-            m_editFinalPos = m_main->GetWindowPos();
-            m_editFinalDim = m_main->GetWindowDim();
+            m_editFinalPos = m_settings->GetWindowPos();
+            m_editFinalDim = m_settings->GetWindowDim();
             m_bEditMinimized = false;
             m_bEditMaximized = false;
         }
@@ -281,8 +283,8 @@ bool CStudio::EventProcess(const Event &event)
     {
         if ( m_bEditMaximized )
         {
-            m_editFinalPos = m_main->GetWindowPos();
-            m_editFinalDim = m_main->GetWindowDim();
+            m_editFinalPos = m_settings->GetWindowPos();
+            m_editFinalDim = m_settings->GetWindowDim();
             m_bEditMinimized = false;
             m_bEditMaximized = false;
         }
@@ -345,10 +347,26 @@ bool CStudio::EventFrame(const Event &event)
     if ( pw == nullptr )  return false;
 
     edit = static_cast< CEdit* >(pw->SearchControl(EVENT_STUDIO_EDIT));
-    if ( edit == 0 )  return false;
+    if ( edit == nullptr )  return false;
 
     list = static_cast< CList* >(pw->SearchControl(EVENT_STUDIO_LIST));
-    if ( list == 0 )  return false;
+    if ( list == nullptr )  return false;
+
+    if (m_script->IsRunning() && (!m_script->GetStepMode() || m_script->IsContinue()))
+    {
+        if (m_editorPause != nullptr)
+        {
+            m_pause->DeactivatePause(m_editorPause);
+            m_editorPause = nullptr;
+        }
+    }
+    else
+    {
+        if (m_editorPause == nullptr)
+        {
+            m_editorPause = m_pause->ActivatePause(PAUSE_EDITOR);
+        }
+    }
 
     if ( !m_script->IsRunning() && m_bRunning )  // stop?
     {
@@ -359,9 +377,7 @@ bool CStudio::EventFrame(const Event &event)
         GetResource(RES_TEXT, RT_STUDIO_PROGSTOP, res);
         SetInfoText(res, false);
 
-        Event newEvent = event;
-        newEvent.type = EVENT_OBJECT_PROGSTOP;
-        m_event->AddEvent(newEvent);  // stop
+        m_event->AddEvent(Event(EVENT_OBJECT_PROGSTOP));
     }
 
     if ( m_script->IsRunning() && !m_bRunning )  // starting?
@@ -456,7 +472,7 @@ void CStudio::SearchToken(CEdit* edit)
             }
             if ( level > 0 )
             {
-                m_helpFilename[0] = 0;
+                m_helpFilename = "";
                 SetInfoText("", true);
                 return;
             }
@@ -557,7 +573,6 @@ void CStudio::StartEditScript(CScript *script, std::string name, Program* progra
 
     m_main->SetEditLock(true, true);
     m_main->SetEditFull(false);
-    m_bInitPause = m_pause->GetPauseType();
     m_main->SetSpeed(1.0f);
     m_editCamera = m_camera->GetType();
     m_camera->SetType(Gfx::CAM_TYPE_EDIT);
@@ -566,8 +581,11 @@ void CStudio::StartEditScript(CScript *script, std::string name, Program* progra
     m_bRealTime = m_bRunning;
     m_script->SetStepMode(!m_bRealTime);
 
-    pos = m_editFinalPos = m_editActualPos = m_main->GetWindowPos();
-    dim = m_editFinalDim = m_editActualDim = m_main->GetWindowDim();
+    pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW6));
+    if (pw != nullptr) pw->ClearState(STATE_VISIBLE);
+
+    pos = m_editFinalPos = m_editActualPos = m_settings->GetWindowPos();
+    dim = m_editFinalDim = m_editActualDim = m_settings->GetWindowDim();
     pw = m_interface->CreateWindows(pos, dim, 8, EVENT_WINDOW3);
     if (pw == nullptr)
         return;
@@ -627,7 +645,7 @@ void CStudio::StartEditScript(CScript *script, std::string name, Program* progra
     button->SetState(STATE_SHADOW);
     slider = pw->CreateSlider(pos, dim, 0, EVENT_STUDIO_SIZE);
     slider->SetState(STATE_SHADOW);
-    slider->SetVisibleValue((m_main->GetFontSize()-9.0f)/15.0f);
+    slider->SetVisibleValue((m_settings->GetFontSize()-9.0f)/15.0f);
     pw->CreateGroup(pos, dim, 19, EVENT_LABEL1);  // SatCom logo
     button = pw->CreateButton(pos, dim, 128+57, EVENT_STUDIO_TOOL);
     button->SetState(STATE_SHADOW);
@@ -649,12 +667,12 @@ void CStudio::StartEditScript(CScript *script, std::string name, Program* progra
     button = pw->CreateButton(pos, dim, 64+29, EVENT_STUDIO_STEP);
     button->SetState(STATE_SHADOW);
 
-    if(!m_program->runnable)
+    if (!m_program->runnable)
     {
         GetResource(RES_TEXT, RT_PROGRAM_EXAMPLE, res);
         SetInfoText(res, false);
     }
-    else if(m_program->readOnly)
+    else if (m_program->readOnly)
     {
         GetResource(RES_TEXT, RT_PROGRAM_READONLY, res);
         SetInfoText(res, false);
@@ -874,7 +892,7 @@ bool CStudio::StopEditScript(bool bCancel)
     if ( !bCancel && !m_script->IsRunning() )
     {
         edit = static_cast< CEdit* >(pw->SearchControl(EVENT_STUDIO_EDIT));
-        if ( edit != 0 )
+        if ( edit != nullptr )
         {
             if ( !m_script->GetScript(edit) )  // compile
             {
@@ -889,7 +907,11 @@ bool CStudio::StopEditScript(bool bCancel)
 
     m_interface->DeleteControl(EVENT_WINDOW3);
 
-    m_pause->SetPause(m_bInitPause);
+    pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW6));
+    if (pw != nullptr) pw->SetState(STATE_VISIBLE);
+
+    m_pause->DeactivatePause(m_editorPause);
+    m_editorPause = nullptr;
     m_sound->MuteAll(false);
     m_main->SetEditLock(false, true);
     m_camera->SetType(m_editCamera);
@@ -909,7 +931,7 @@ void CStudio::SetInfoText(std::string text, bool bClickable)
     if ( pw == nullptr )  return;
 
     CList* list = static_cast< CList* >(pw->SearchControl(EVENT_STUDIO_LIST));
-    if ( list == 0 )  return;
+    if ( list == nullptr )  return;
 
     list->Flush();  // just text
     list->SetItemName(0, text.c_str());
@@ -945,10 +967,10 @@ void CStudio::ViewEditScript()
     if ( pw == nullptr )  return;
 
     edit = static_cast< CEdit* >(pw->SearchControl(EVENT_STUDIO_EDIT));
-    if ( edit == 0 )  return;
+    if ( edit == nullptr )  return;
 
     dim = m_engine->GetWindowSize();
-    edit->SetFontSize(m_main->GetFontSize()/(dim.x/640.0f));
+    edit->SetFontSize(m_settings->GetFontSize()/(dim.x/640.0f));
 }
 
 
@@ -958,26 +980,25 @@ void CStudio::UpdateFlux()
 {
     if ( m_bRunning )
     {
-#if 1
         if ( m_bRealTime )  // run?
         {
-            m_pause->ClearPause();
-            m_sound->MuteAll(false);
+            m_pause->DeactivatePause(m_editorPause);
+            m_editorPause = nullptr;
         }
         else    // step by step?
         {
-            m_pause->SetPause(PAUSE_EDITOR);
-            m_sound->MuteAll(true);
+            if (m_editorPause == nullptr)
+            {
+                m_editorPause = m_pause->ActivatePause(PAUSE_EDITOR);
+            }
         }
-#else
-        m_pause->ClearPause();
-        m_sound->MuteAll(false);
-#endif
     }
     else    // stop?
     {
-        m_pause->SetPause(PAUSE_EDITOR);
-        m_sound->MuteAll(true);
+        if (m_editorPause == nullptr)
+        {
+            m_editorPause = m_pause->ActivatePause(PAUSE_EDITOR);
+        }
     }
 }
 
@@ -993,7 +1014,7 @@ void CStudio::UpdateButtons()
     if ( pw == nullptr )  return;
 
     edit = static_cast< CEdit* >(pw->SearchControl(EVENT_STUDIO_EDIT));
-    if ( edit == 0 )  return;
+    if ( edit == nullptr )  return;
 
     if ( m_bRunning )
     {
@@ -1010,47 +1031,47 @@ void CStudio::UpdateButtons()
 
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_CLONE));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, m_program->runnable && !m_bRunning);
 
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_COMPILE));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, m_program->runnable && !m_bRunning);
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_RUN));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetIcon(m_bRunning?8:21);  // stop/run
     button->SetState(STATE_ENABLE, m_program->runnable || m_bRunning);
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_REALTIME));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetIcon(m_bRealTime?64+22:64+21);
     button->SetState(STATE_ENABLE, (!m_bRunning || !m_script->IsContinue()));
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_STEP));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, (m_bRunning && !m_bRealTime && !m_script->IsContinue()));
 
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_NEW));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, !m_program->readOnly);
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_OPEN));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, !m_program->readOnly);
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_UNDO));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, !m_program->readOnly);
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_CUT));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, !m_program->readOnly);
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_PASTE));
-    if ( button == 0 )  return;
+    if ( button == nullptr )  return;
     button->SetState(STATE_ENABLE, !m_program->readOnly);
 }
 
@@ -1088,8 +1109,8 @@ void CStudio::StartDialog(StudioDialog type)
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW5));
     if ( pw != nullptr )  pw->ClearState(STATE_ENABLE);
 
-    pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW6));
-    if ( pw != nullptr )  pw->ClearState(STATE_ENABLE);
+    //pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW6));
+    //if ( pw != nullptr )  pw->ClearState(STATE_VISIBLE);
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW7));
     if ( pw != nullptr )  pw->ClearState(STATE_ENABLE);
@@ -1100,8 +1121,8 @@ void CStudio::StartDialog(StudioDialog type)
     if ( m_dialog == SD_OPEN ||
          m_dialog == SD_SAVE )
     {
-        pos = m_main->GetIOPos();
-        dim = m_main->GetIODim();
+        pos = m_settings->GetIOPos();
+        dim = m_settings->GetIODim();
     }
 //? pw = m_interface->CreateWindows(pos, dim, 8, EVENT_WINDOW9);
     pw = m_interface->CreateWindows(pos, dim, m_dialog==SD_OPEN?14:13, EVENT_WINDOW9);
@@ -1162,7 +1183,7 @@ void CStudio::StartDialog(StudioDialog type)
         UpdateDialogList();
         UpdateDialogPublic();
         UpdateDialogAction();
-        
+
         if ( m_dialog == SD_SAVE )
         {
             SetFilenameField(pe, m_script->GetFilename());
@@ -1170,7 +1191,7 @@ void CStudio::StartDialog(StudioDialog type)
         }
 
         pe->SetCursor(999, 0);  // selects all
-        pe->SetFocus(true);
+        m_interface->SetFocus(pe);
     }
 
     m_main->SetSatComLock(true);  // impossible to use the SatCom
@@ -1203,8 +1224,8 @@ void CStudio::StopDialog()
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW5));
     if ( pw != nullptr )  pw->SetState(STATE_ENABLE);
 
-    pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW6));
-    if ( pw != nullptr )  pw->SetState(STATE_ENABLE);
+    //pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW6));
+    //if ( pw != nullptr )  pw->SetState(STATE_VISIBLE);
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW7));
     if ( pw != nullptr )  pw->SetState(STATE_ENABLE);
@@ -1245,7 +1266,7 @@ void CStudio::AdjustDialog()
         ddim.x = wdim.x-20.0f/640.0f;
         ddim.y = 20.0f/480.0f;
         pla = static_cast< CLabel* >(pw->SearchControl(EVENT_DIALOG_LABEL1));
-        if ( pla != 0 )
+        if ( pla != nullptr )
         {
             pla->SetPos(ppos);
             pla->SetDim(ddim);
@@ -1255,7 +1276,7 @@ void CStudio::AdjustDialog()
         ddim.y = nli*18.0f/480.0f+9.0f/480.0f;
         ppos.y = wpos.y+wdim.y-48.0f/480.0f-ddim.y;
         pli = static_cast< CList* >(pw->SearchControl(EVENT_DIALOG_LIST));
-        if ( pli != 0 )
+        if ( pli != nullptr )
         {
             pli->SetPos(ppos);
             pli->SetDim(ddim);
@@ -1269,7 +1290,7 @@ void CStudio::AdjustDialog()
         ddim.x = 50.0f/640.0f;
         ddim.y = 20.0f/480.0f;
         pla = static_cast< CLabel* >(pw->SearchControl(EVENT_DIALOG_LABEL2));
-        if ( pla != 0 )
+        if ( pla != nullptr )
         {
             pla->SetPos(ppos);
             pla->SetDim(ddim);
@@ -1279,7 +1300,7 @@ void CStudio::AdjustDialog()
         ppos.y = wpos.y+36.0f/480.0f;
         ddim.x = wdim.x-170.0f/640.0f;
         pe = static_cast< CEdit* >(pw->SearchControl(EVENT_DIALOG_EDIT));
-        if ( pe != 0 )
+        if ( pe != nullptr )
         {
             pe->SetPos(ppos);
             pe->SetDim(ddim);
@@ -1296,7 +1317,7 @@ void CStudio::AdjustDialog()
         ddim.x = 50.0f/640.0f;
         ddim.y = 16.0f/480.0f;
         pla = static_cast< CLabel* >(pw->SearchControl(EVENT_DIALOG_LABEL3));
-        if ( pla != 0 )
+        if ( pla != nullptr )
         {
             pla->SetPos(ppos);
             pla->SetDim(ddim);
@@ -1306,7 +1327,7 @@ void CStudio::AdjustDialog()
         ppos.y = wpos.y+12.0f/480.0f;
         ddim.x = 70.0f/640.0f;
         pc = static_cast< CCheck* >(pw->SearchControl(EVENT_DIALOG_CHECK1));
-        if ( pc != 0 )
+        if ( pc != nullptr )
         {
             pc->SetPos(ppos);
             pc->SetDim(ddim);
@@ -1314,7 +1335,7 @@ void CStudio::AdjustDialog()
 
         ppos.x += 80.0f/640.0f;
         pc = static_cast< CCheck* >(pw->SearchControl(EVENT_DIALOG_CHECK2));
-        if ( pc != 0 )
+        if ( pc != nullptr )
         {
             pc->SetPos(ppos);
             pc->SetDim(ddim);
@@ -1325,7 +1346,7 @@ void CStudio::AdjustDialog()
         ddim.x = 90.0f/640.0f;
         ddim.y = 23.0f/480.0f;
         pb = static_cast< CButton* >(pw->SearchControl(EVENT_DIALOG_OK));
-        if ( pb != 0 )
+        if ( pb != nullptr )
         {
             pb->SetPos(ppos);
             pb->SetDim(ddim);
@@ -1333,7 +1354,7 @@ void CStudio::AdjustDialog()
 
         ppos.y -= 26.0f/480.0f;
         pb = static_cast< CButton* >(pw->SearchControl(EVENT_DIALOG_CANCEL));
-        if ( pb != 0 )
+        if ( pb != nullptr )
         {
             pb->SetPos(ppos);
             pb->SetDim(ddim);
@@ -1355,8 +1376,8 @@ bool CStudio::EventDialog(const Event &event)
     {
         wpos = pw->GetPos();
         wdim = pw->GetDim();
-        m_main->SetIOPos(wpos);
-        m_main->SetIODim(wdim);
+        m_settings->SetIOPos(wpos);
+        m_settings->SetIODim(wdim);
         AdjustDialog();
     }
 
@@ -1374,20 +1395,20 @@ bool CStudio::EventDialog(const Event &event)
 
         if ( event.type == EVENT_DIALOG_CHECK1 )  // private?
         {
-            m_main->SetIOPublic(false);
+            m_settings->SetIOPublic(false);
             UpdateDialogPublic();
             UpdateDialogList();
         }
         if ( event.type == EVENT_DIALOG_CHECK2 )  // public?
         {
-            m_main->SetIOPublic(true);
+            m_settings->SetIOPublic(true);
             UpdateDialogPublic();
             UpdateDialogList();
         }
     }
 
     if ( event.type == EVENT_DIALOG_OK ||
-         (event.type == EVENT_KEY_DOWN && event.key.key == KEY(RETURN)) )
+         (event.type == EVENT_KEY_DOWN && event.GetData<KeyEventData>()->key == KEY(RETURN)) )
     {
         if ( m_dialog == SD_OPEN )
         {
@@ -1403,7 +1424,7 @@ bool CStudio::EventDialog(const Event &event)
     }
 
     if ( event.type == EVENT_DIALOG_CANCEL ||
-         (event.type == EVENT_KEY_DOWN && event.key.key == KEY(ESCAPE)) ||
+         (event.type == EVENT_KEY_DOWN && event.GetData<KeyEventData>()->key == KEY(ESCAPE)) ||
          event.type == pw->GetEventTypeClose() )
     {
         StopDialog();
@@ -1424,7 +1445,7 @@ void CStudio::UpdateChangeList()
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW9));
     if ( pw == nullptr )  return;
     pl = static_cast< CList* >(pw->SearchControl(EVENT_DIALOG_LIST));
-    if ( pl == 0 )  return;
+    if ( pl == nullptr )  return;
     pe = static_cast< CEdit* >(pw->SearchControl(EVENT_DIALOG_EDIT));
     if ( pe == nullptr )  return;
 
@@ -1432,7 +1453,7 @@ void CStudio::UpdateChangeList()
     name = name.substr(0, name.find_first_of("\t"));
     SetFilenameField(pe, name);
     pe->SetCursor(999, 0);  // selects all
-    pe->SetFocus(true);
+    m_interface->SetFocus(pe);
 
     UpdateDialogAction();
 }
@@ -1440,11 +1461,13 @@ void CStudio::UpdateChangeList()
 void CStudio::SetFilenameField(CEdit* edit, const std::string& filename)
 {
     std::string name = filename;
-    if(name.length() > static_cast<unsigned int>(edit->GetMaxChar())) {
-        if(name.substr(name.length()-4) == ".txt")
+    if (name.length() > static_cast<unsigned int>(edit->GetMaxChar()))
+    {
+        if (name.substr(name.length()-4) == ".txt")
             name = name.substr(0, name.length()-4);
-        if(name.length() > static_cast<unsigned int>(edit->GetMaxChar())) {
-            CLogger::GetInstancePointer()->Warn("Tried to load too long filename!\n");
+        if (name.length() > static_cast<unsigned int>(edit->GetMaxChar()))
+        {
+            GetLogger()->Warn("Tried to load too long filename!\n");
             name = name.substr(0, edit->GetMaxChar());  // truncates according to max length
         }
     }
@@ -1461,7 +1484,7 @@ void CStudio::UpdateChangeEdit()
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW9));
     if ( pw == nullptr )  return;
     pl = static_cast< CList* >(pw->SearchControl(EVENT_DIALOG_LIST));
-    if ( pl == 0 )  return;
+    if ( pl == nullptr )  return;
 
     pl->SetSelect(-1);
 
@@ -1524,19 +1547,19 @@ void CStudio::UpdateDialogPublic()
     if ( pw == nullptr )  return;
 
     pc = static_cast< CCheck* >(pw->SearchControl(EVENT_DIALOG_CHECK1));
-    if ( pc != 0 )
+    if ( pc != nullptr )
     {
-        pc->SetState(STATE_CHECK, !m_main->GetIOPublic());
+        pc->SetState(STATE_CHECK, !m_settings->GetIOPublic());
     }
 
     pc = static_cast< CCheck* >(pw->SearchControl(EVENT_DIALOG_CHECK2));
-    if ( pc != 0 )
+    if ( pc != nullptr )
     {
-        pc->SetState(STATE_CHECK, m_main->GetIOPublic());
+        pc->SetState(STATE_CHECK, m_settings->GetIOPublic());
     }
 
     pl = static_cast< CLabel* >(pw->SearchControl(EVENT_DIALOG_LABEL1));
-    if ( pl != 0 )
+    if ( pl != nullptr )
     {
         // GetResource(RES_TEXT, RT_IO_LIST, name); // TODO: unused?
         pl->SetName(SearchDirectory(false).c_str(), false);
@@ -1558,11 +1581,12 @@ void CStudio::UpdateDialogList()
     if ( pl == nullptr )  return;
     pl->Flush();
 
-    if(!CResourceManager::DirectoryExists(SearchDirectory(false)))
+    if (!CResourceManager::DirectoryExists(SearchDirectory(false)))
         return;
-    
+
     std::vector<std::string> programs = CResourceManager::ListFiles(SearchDirectory(false));
-    for(auto& prog : programs) {
+    for (auto& prog : programs)
+    {
         std::ostringstream temp;
         TimeToAscii(CResourceManager::GetLastModificationTime(SearchDirectory(false) + prog), time);
         temp << prog << '\t' << CResourceManager::GetFileSize(SearchDirectory(false) + prog) << "  \t" << time;
@@ -1575,21 +1599,21 @@ void CStudio::UpdateDialogList()
 std::string CStudio::SearchDirectory(bool bCreate)
 {
     std::string dir;
-    if ( m_main->GetIOPublic() )
+    if ( m_settings->GetIOPublic() )
     {
-        dir = std::string(m_main->GetPublicDir()) + "/";
+        dir = "program";
     }
     else
     {
-        dir = std::string(m_main->GetSavegameDir()) + "/" + std::string(m_main->GetGamerName()) + "/program/";
+        dir = m_main->GetPlayerProfile()->GetSaveFile("program");
     }
 
     if ( bCreate )
     {
-        if(!CResourceManager::DirectoryExists(dir))
+        if (!CResourceManager::DirectoryExists(dir))
             CResourceManager::CreateDirectory(dir);
     }
-    return dir;
+    return dir+"/";
 }
 
 // Reads a new program.
@@ -1611,7 +1635,7 @@ bool CStudio::ReadProgram()
     if ( filename[0] == 0 )  return false;
 
     p = strstr(filename, ".txt");
-    if ( p == 0 || p != filename+strlen(filename)-4 )
+    if ( p == nullptr || p != filename+strlen(filename)-4 )
     {
         strcat(filename, ".txt");
     }
@@ -1649,7 +1673,7 @@ bool CStudio::WriteProgram()
     if ( filename[0] == 0 )  return false;
 
     p = strstr(filename, ".txt");
-    if ( p == 0 || p != filename+strlen(filename)-4 )
+    if ( p == nullptr || p != filename+strlen(filename)-4 )
     {
         strcat(filename, ".txt");
     }
@@ -1660,7 +1684,7 @@ bool CStudio::WriteProgram()
     if ( pw == nullptr )  return false;
     pe = static_cast< CEdit* >(pw->SearchControl(EVENT_STUDIO_EDIT));
     if ( pe == nullptr )  return false;
-    
+
     if ( !pe->WriteText(std::string(dir)) )  return false;
 
     m_script->SetFilename(filename);
@@ -1668,4 +1692,3 @@ bool CStudio::WriteProgram()
 }
 
 }
-
